/*= R&S UPV Audio Analyzer ==================================================*/
/* LabWindows/CVI 7.1 Instrument Driver
/* Original Release: January 2005
/* By: Petr Marcalik & Martin Krcmar
    /*
/* Should you have any technical questions please contact the hotline of
/* Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
/*
/* e-mail: CustomerSupport@rohde-schwarz.com
/*
/* Modification History:
/*   see file ChangeLog
/*===========================================================================*/

#include <visa.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <float.h>
#include <math.h>
#include <locale.h>
#include "rsupv.h"

/*= DEFINES =================================================================*/

#define RSUPV_REVISION              "Rev 3.4.0, 04/2020, CVI 2017" /* Instrument driver revision */
#define BUFFER_SIZE                 2048L                       /* File I/O buffer size */
#define NUMERIC_PRECISION           1e-12                       /* Precision of compare operations */
#if !defined (NAN)
#define NAN                         9.91e37                     /* Not A Number */
#endif
#define INF                         9.9e37                      /* Positive INFinity (INF) */
#define NINF                        -9.9e37                     /* Negative INFinity (NINF) */
#define RSUPV_DEFAULT_TIMEOUT       10000                       /* Default Timeout value in ms (10 sec) */
#define RSUPV_IEEE_ERROR_BITS       0x4                        /* AND mask for ESR checking */
#define RSUPV_CHECK_BEFORE_READ     VI_FALSE                    /* Apply checking before read operations? VI_TRUE or VI_FALSE */
#define RSUPV_MAX_OPTIONS           30                          /* Maximum number of installed options */

/*= Useful Macros ===========================================================*/

/* Error checking macro */
#define CHECKERR(fCal) if (rsupv_status = (fCal), rsupv_status < VI_SUCCESS) \
    return rsupv_status; else

/* Error checking macro with clean up (rsupv_init only) */
#define CHECKERR_CU(fCal) if (rsupv_status = (fCal), rsupv_status < VI_SUCCESS) \
    return rsupv_initCleanUp (rmSession, instrSession, rsupv_status); else

/* Check presency of instrument's option (when error checking is enabled) */
#define CHECKOPT(option) \
    if (rsupv_checkOption(instrSession, option) < VI_SUCCESS) \
        return RSUPV_ERROR_INSTRUMENT_OPTION;

/* Check instrument model (when error checking is enabled) */
#define CHECKINSTR(instr) \
    if (rsupv_checkInstrument(instrSession, instr) < VI_SUCCESS) \
        return RSUPV_ERROR_INSTRUMENT_NSUP_MODEL;

/* Is the result of scanning (sscanf) function OK? If not, return error code. */
#define CHECKSCAN(fCal) if ((fCal) < 0) \
    return VI_ERROR_INSTR_INTERPRETING_RESPONSE; else

/* Error checking macro with jump to the ERROR: label */
#define GOCHECKERR(fCal) if (rsupv_status = (fCal), rsupv_status < VI_SUCCESS) \
                            goto ERROR; else

/* Checking window parameter */
/* #define CHECKWINDOW if (rsupv_invalidViInt32Range (window, RSUPV_SCREEN_A, RSUPV_SCREEN_B)) return VI_ERROR_PARAMETER2; */

/* Checking NULL pointer in argument */
#define CHECKNULL(arg, number) if ( !arg ) return VI_ERROR_PARAMETER##number

/* Checking ViInt32 argument */
#define CHECKVIINT32(arg, min, max, errornumber) if ( rsupv_invalidViInt32Range (arg,min,max)) return VI_ERROR_PARAMETER##errornumber
/* Checking ViReal64 argument */
#define CHECKVIREAL64(arg, min, max, errornumber) if ( rsupv_invalidViReal64Range (arg,min,max)) return VI_ERROR_PARAMETER##errornumber

/* Remap parameter error code */
#define REMAP(par_in, par_out) if (rsupv_status == VI_ERROR_PARAMETER##par_in)\
                                    rsupv_status = VI_ERROR_PARAMETER##par_out;\
                                if (rsupv_status < VI_SUCCESS)\
                                    return rsupv_status; else

/* Remap parameter (higher than 8) error code */
#define REMAPINSTR(par_in, par_out) if (rsupv_status == VI_ERROR_PARAMETER##par_in)\
                                      rsupv_status = VI_ERROR_INSTR_PARAMETER##par_out;\
                                  if (rsupv_status < VI_SUCCESS)\
                                      return rsupv_status; else

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

static ViString instrList[] = {"UPV","UPP",VI_NULL};

/* --- Generator --- */

static ViString  offOnArr[] = {"OFF","ON",NULL};
static ViString  unitsArr[] = {"PCT","PCTFRM","PCTFS","PCTON","PCTPPR","PCTUI",
                               "PCTVVR","BITS","CYC","DPCT","DPCTHZ","DPCTV",
                               "DPCTW","DB","DBFS","DBM","DBON","DBR","DBU",
                               "DBUI","DBV","DDEG","DEC","DEG","DEGFRM","DHZ",
                               "DRAD","DS","DV","DW","FFR","FS","HEX","HZ",
                               "LSBS","MLT","NS","OCT","OHM","PPR","PPMUI",
                               "RAD","S","TOCT","UI","UIR","UIS","V","VVON",
                               "VVR","VPP","W","MV","UV","MW","UW","MVPP","UVPP",
                               "DMV","DUV","FSBIT","KHZ","DKHZ","MS","US","KCYC",
                               "MCYC","DMS","DUS","MIN","KOHM","DBMG","CORR","POL",VI_NULL};
static ViString  genInstrumentArr[] = {"ANLG","DIG","I2SB","IMP","U2CH", "HDMI", VI_NULL};
static ViString  genChannelArr[] = {"OFF","CH1","CH2","CH2I","CH2P","MULT,", VI_NULL};
static ViString  genOutputTypeArr[] = {"UNB","BAL","CTES", VI_NULL};
static ViString  genImpedanceArr[] = {"R10","R200","R600","R25", VI_NULL};
static ViString  genCommonArr[] = {"FLO","GRO", VI_NULL};
static ViString  genBandwidthArr[] = {"B22","B40","B80","AUTO", VI_NULL};
static ViString  genVoltRangeArr[] = {"AUTO","FIX", VI_NULL};
static ViString  genSourceModeArr[] = {"JITT","COMM", VI_NULL};
static ViString  genSyncToArr[] = {"INTC","AINP","AUX","SINP","SINV","EDAR","ECLK","EICL","HINT","HAUX", VI_NULL};
static ViString  genSampleFrequencyArr[] = {"F32","F44","F48","F88","F96","F176","F192",
                                            "VAL","SYNC",VI_NULL};
static ViString  genSyncOutputArr[] = {"ADO","ADIN","AXIN","GSYP","JRFP","AXO","SYIN","ICL", VI_NULL};
static ViString  genSyncOutTypeArr[] = {"WCL","BCL", VI_NULL};
static ViString  genAuxOutputArr[] = {"AOUT","AINP","AINR","RGEN","AUX", VI_NULL};
static ViString  genRefGenDataArr[] = {"AZER","AONE", VI_NULL};
static ViString  genPhaseToRefArr[] = {"OFF","VAL", VI_NULL};
static ViString  genUnbalOutputArr[] = {"AOUT","AINP", VI_NULL};
static ViString  genCableSimArr[] = {"OFF","SIML", VI_NULL};
static ViString  genFunctionArr[] = {"SIN","STER","MULT","BURS","S2P","MDIS","DFD","DIM",
                                     "RAND","ARB","POL","MOD","DC","PLAY","PLYA","SQU","CHIR", "UNIV","LIPS",VI_NULL};
static ViString  genModeArr[] = {"EQU","DEF","IEC268","IEC118","AM","FM", VI_NULL};
static ViString  genPdfArr[] = {"GAUS","TRI","RECT", VI_NULL};
static ViString  genSweepCtrlArr[] = {"OFF","ASW","ALIS","MSW","MLIS", VI_NULL};
static ViString  genNextStepArr[] = {"ASYN","DWEL","LIST", VI_NULL};
static ViString  genAxisArr[] = {"OFF","VOLT","FREQ","ONT","INT","PHAS", VI_NULL};
static ViString  genSpacingArr[] = {"LINP","LINS","LOGP","LOGS", VI_NULL};
static ViString  genSpacingModeArr[] = {"ATR","USER", VI_NULL};
static ViString  genCrestFactorArr[] = {"MIN","VAL","DPH", VI_NULL};
static ViString  genFilterArr[] = {"OFF","UFIL1","UFIL2","UFIL3","UFIL4","UFIL5","UFIL6",
                                   "UFIL7","UFIL8","UFIL9","AWE","CCIR","CARM","CCIU","CCIT",
                                   "CMES","DCN","DEMP17","DEMP5015","DEMP50","DEMP75","IECT",
                                   "JITT","URUM","WRUM","PEMP17","PEMP5015","PEMP50","PEMP75","HP22",
                                   "HP400","LP22","LP30","LP80", "AES17", "CWE", VI_NULL};
static ViString  genFreqModeArr[] = {"FQPH","FQFQ", VI_NULL};
static ViString  genVoltModeArr[] = {"VLRT","VLVL", VI_NULL};
static ViString  genSquareSineArr[] = {"DIMA","DIMB", "DIMS", VI_NULL};
static ViString  genDIMBandwidthArr[] = {"F30","F100", VI_NULL};
static ViString  genDomainArr[] = {"FREQ","TIME", VI_NULL};
static ViString  genShapeArr[] = {"WHIT","PINK","TOCT","FILE", VI_NULL};
static ViString  genI2SSyncToArr[] = {"INT","EXTM","EXTW", VI_NULL};
static ViString  genI2SwordLengthArr[] = {"W16","W24","W32", VI_NULL};
static ViString  genI2SSampleFrequencyArr[] = {"F08","F11","F16","F22","F32",
                                               "F44","F48","F88","F96","F176",
                                               "F192","F384","VAL", VI_NULL};
static ViString  genI2SMClkRatioArr[] = {"M64","M96","M128","M192","M256","M384","M512", VI_NULL};
static ViString  genI2SformatArr[] = {"SI2S","USER", VI_NULL};
static ViString  genI2SfsyncShapeArr[] = {"SQU","BITP", VI_NULL};
static ViString  genI2SfsyncPolarityArr[] = {"LFTL","LFTH","POS","NEG", VI_NULL};
static ViString  genLoopChannelArr[] = {"OFF","CH1","CH2","STER","CROS", VI_NULL};
static ViString  genAmplitudeVariationArr[] = {"OFF","SIN","BURS", VI_NULL};
static ViString  genPlayChannelArr[] = {"MLEF","MRIG","STER", VI_NULL};
static ViString  genPlayModeArr[] = {"TOC","TOS","TIC","TIS", VI_NULL};
static ViString  measurementModeArr[] = {"OFF","ON","WAI", VI_NULL};
static ViString balancedImpedanceArr[] = {"R5","R15","R10","R30","R200","R150","R600","R25","R50","USER",VI_NULL};
static ViString restartArr[] = {"OFF","AUTO","ONCE",VI_NULL}; 
static ViString  HaltArr [] = {"STAR","VAL","MUTE",VI_NULL};
static ViString  SweepcontrolArr[] = {"FREQ","VOLT",VI_NULL};
/* --- Analyzer --- */

static ViString  anInstrumentArr[] = {"ANLG","DIG","I2SB", "A8CH", "A16CH", "U2CH", "U8CH","DIGB","HDMI",VI_NULL};
static ViString  anChannelArr[] = {"CH1","CH2","CH1A","CH2I","CH1I","BOTH","MULT", VI_NULL};
static ViString  anBandwidthArr[] = {"B22","B40","B80","B250", VI_NULL};
static ViString  anPreFilterArr[] = {"OFF","UFIL1","UFIL2","UFIL3","UFIL4","UFIL5","UFIL6",
                                      "UFIL7","UFIL8","UFIL9","AWE","CARM","CCIU","CCIR",
                                      "CCIT","CMES","DCN","DEMP17","DEMP5015","DEMP50","DEMP75",
                                      "IECT","JITT","URUM","WRUM","PEMP17","PEMP5015","PEMP50","PEMP75","HP22",
                                      "HP400","LP22","LP30","LP80","AES17", "CWE", VI_NULL};
static ViString  anStartConditionArr[] = {"AUTO","TIM","TCH","CH1F","CH2F","CH1R","CH2R",
                                          "CH1L","CH2L","CH1T","CH2T","CH1E","CH2E", "FREQ", "RAP", "LEV",
                                          "TRIG","EDG", VI_NULL};
static ViString  anCouplingArr[] = {"AC","DC", VI_NULL};
static ViString  anChInputArr[] = {"BAL","GEN1","GEN2", VI_NULL};
static ViString  anChImpedArr[] = {"R300","R600","R200K","R110","R10K","R20K","USER", VI_NULL};
static ViString  anChCommonArr[] = {"FLO","GRO", VI_NULL};
static ViString  anRangeArr[] = {"AUTO","FIX","LOW", VI_NULL};
static ViString  anMeasModeArr[] = {"ADAT","JPH","CINP", VI_NULL};
static ViString  anJitterRefArr[] = {"INTC","REFP", VI_NULL};
static ViString  anSampleFreqArr[] = {"F32","F44","F48","F88","F96","F176","F192","VAL","AUTO","CHST", VI_NULL};
static ViString  anDigitalAudioInputArr[] = {"DBAL","DUNB","OPT","INT", VI_NULL};
static ViString  anFunctionArr[] = {"OFF","RMS","RMSS","PEAK","QPE","DC","THD",
                                    "THDN","MDIS","DFD","WAF","POL","FFT","FILT",
                                    "WAV","COH","RUBB","PROT","THIR",
                                    "SN","NOCT","REC", "DIM", "PESQ", "PEAQ","POLQA", "LIPS", "PLUG", VI_NULL};
static ViString  anNotchGainArr[] = {"OFF","DB0","DB12","DB30", VI_NULL};
static ViString  anNotchFrequencyArr[] = {"VAL","GENT", VI_NULL};
static ViString  anFilterArr[] = {"OFF","UFIL1","UFIL2","UFIL3","UFIL4","UFIL5",
                                  "UFIL6","UFIL7","UFIL8","UFIL9","AWE","CCIR",
                                  "CARM","CCIU","CCIT","CMES","DCN","DEMP17",
                                  "DEMP5015","DEMP50","DEMP75","IECT","JITT",
                                  "URUM","WRUM","PEMP17","PEMP5015","PEMP50","PEMP75","HP22",
                                  "HP400","LP22","LP30","LP80", "AES17", "CWE", VI_NULL};
static ViString  anLevelMeasTimeArr[] = {"AFAS","AUTO","VAL","GENT","SFAS","FAST","SLOW","FIX", VI_NULL};
static ViString  anControlArr[] = {"OFF","ASW","ALIS","MSW","MLIS", VI_NULL};
static ViString  anSpacingArr[] = {"LINP","LINS","LOGP","LOGS", VI_NULL};
static ViString  anPeakMeasModeArr[] = {"PPE","NPE","PTOP","PABS", VI_NULL};
static ViString  anIntervalTimeArr[] = {"SFAS","FAST","SLOW","VAL","FIX", VI_NULL};
static ViString  anDynamicModeArr[] = {"FAST","PREC", VI_NULL};
static ViString  anThdFundamentalArr[] = {"AUTO","VAL","GENT", VI_NULL};
static ViString  anThdMeasurementModeArr[] = {"SEL","DEV","DODD","DALL","LSEL",
                                              "LDEV","LDOD","LDAL", VI_NULL};
static ViString  anThdnMeasurementModeArr[] = {"THDN","SNDR","NOIS","LTHD","LNOI", VI_NULL};
static ViString  anThdnRejectionArr[] = {"NARR","WIDE", VI_NULL};
static ViString  anThdnRejectBandwidthArr[] = {"WIDE","MED","NARR", VI_NULL};
static ViString  anDfdMeasurementModeArr[] = {"D2_268","D3_268","D2_118","D3_118", VI_NULL};
static ViString  anFftSizeArr[] = {"S512","S1K","S2K","S4K","S8K","S16K","S32K","S64K",
                                    "S128K","S256K",VI_NULL};
static ViString  anFftWindowArr[] = {"HANN","RECT","BLAC","RIF1","RIF2","RIF3","HAMM","FLAT",VI_NULL};
static ViString  anFftAverageModeArr[] = {"OFF","EXP","NORM", VI_NULL};
static ViString  anLevelMonitorArr[] = {"OFF","LRMS","DC","PEAK", VI_NULL};
static ViString  anInputMonitorArr[] = {"OFF","IPE","PHAS","DIG", VI_NULL};
static ViString  anFrequencytoPhaseArr[] = {"OFF","FREQ","FQPH","FQGR","SFR","FQS", VI_NULL};
static ViString  anMeasurementTimeArr[] = {"FAST","PREC", VI_NULL};
static ViString  anFormatPhaseArr[] = {"POS","POSN","NEG","RAD","RADB","RADN","INF",VI_NULL};
static ViString  anReferenceArr[] = {"CH1S","CH2S","STOR","CH1M","CH2M","GENT","VAL","DIG",VI_NULL};
static ViString  anFreqReferenceArr[] = {"CH1S","CH2S","STOR","CH1M","CH2M","GENT","VAL","SOUR","DIG",
                                         "MREF","CH3S","CH4S","CH5S","CH6S","CH7S","CH8S","CH9S","CH10S",
                                         "CH11S","CH12S","CH13S","CH14S","CH15S","CH16S",VI_NULL};
static ViString  anPhaseReferenceArr[] = {"STOR","VAL","GENT", VI_NULL};
static ViString  anRMSBandwidthArr[] = {"PPCT1","PPCT3","POCT12","PTOC","PFAS","PFIX",
                                        "SPCT1","SPCT3","SOCT12","STOC","SFAS","SFIX", VI_NULL};
static ViString  anRMSFrequencyModeArr[] = {"FIX","GENT","CH1F","CH2F","AUTO", "REFF", VI_NULL};
static ViString  anSNMeasModeArr[] = {"RMS","QPE","PPE","NPE","PTOP","PABS", VI_NULL};
static ViString  anNOCTMeasModeArr[] = {"OCT1","OCT3","OCT6","OCT12","OCT24", VI_NULL};
static ViString  anRECTriggerSlopeArr[] = {"RIS","FALL", VI_NULL};
static ViString  anWavMeasModeArr[] = {"STAN","COMP","USAM", VI_NULL};
static ViString  anWavTriggerSourceArr[] = {"CH1","CH2","MAN", "GENB","TRGC", VI_NULL};
static ViString  anWavTriggerSlopeArr[] = {"RIS","FALL", VI_NULL};
static ViString  ComPortArr[] = {"COM1","COM2","COM3","COM4", "AUTO",VI_NULL};
static ViString  anCouplingModeArr[] = {"TRAC","SPL", VI_NULL};
static ViString  anAccordingToArr[] = {"PP862","PPSI862","PPSP862","MP8621","MPSI8621","MP8622","MPSI8622","MPSP8622", VI_NULL};
static ViString  anPESQMeasurementModeArr[] = {"DUT","OFFL", VI_NULL};
static ViString  anPEAQMeasurementModeArr[] = {"DUT","OFFL", VI_NULL};
static ViString  anPEAQMeasurementModelArr[] = {"BAS","ADV", VI_NULL};
static ViString  FnctSettlingArr[] = {"OFF","EXP","FLAT","AVER",VI_NULL};
static ViString  MonitoredChannelArr[] = {"OFF","CH1","CH2","CH3","CH4","CH5","CH6","CH7",
                                          "CH8","CH9","CH10","CH11","CH12","CH13","CH14",
                                          "CH15","CH16",VI_NULL};
/* --- I2S Board --- */

static ViString  I2SWordLengthArr[] = {"W16","W24","W32", VI_NULL};
static ViString  I2SFormatArr[] = {"SI2S","USER", VI_NULL};
static ViString  I2SFsyncSlopeArr[] = {"LFTF","LFTR", VI_NULL};
static ViString  I2SInputArr[] = {"DSUB","INT", VI_NULL};
static ViString  I2SSampleFreqArr[] = {"F08","F11","F16","F22","F32","F44","F48",
                                       "F88","F96","F176","F192","F384","VAL","AUTO", VI_NULL};
/* --- HDMI --- */  
static ViString audioHDMISourceArr[] = {"SINK","SARC","DIU","DIO", VI_NULL}; 
static ViString audioHDMIFormatArr[] = {"PCM2","PCM8","DBD","DBDP","DTS","DBTH","DTSH","DTSM", VI_NULL}; 
static ViString audioHDMICodingArr[] = {"PCM","AUT", VI_NULL}; 

    
/* --- USI Dual Channel --- */
static ViString  dualClockArr[] = {"CONT","GAT", VI_NULL};
static ViString  dualSyncArr[] = {"ICL","EMAS", "EFSY", "EFAM", "EFBC", VI_NULL};
static ViString  dualFirstBitArr[] = {"MSB","LSB", VI_NULL};
static ViString  dualSlopeArr[] = {"RIS","FALL", VI_NULL};
static ViString  dualWidthArr[] = {"ONEB","ONES", "SQU", "VAL", VI_NULL};
static ViString  dualVoltageArr[] = {"MV0900","MV1200", "MV1500", "MV1800", "MV2500", "MV3300", VI_NULL};
static ViString  dualCodingArr[] = {"NONE","ALAW","ULAW", VI_NULL};

/* --- User Defined Filters --- */

static ViString  filterTypeArr[] = {"LPAS","HPAS","BPAS","BSTO","NOTC",
                                    "TOCT","OCT","FILE", VI_NULL};
static ViString  fitlerOrderArr[] = {"N4","N8", VI_NULL};
static ViString  auxSignalSourceArr[] = {"INP","MON","FUNC","GEN","DC",VI_NULL};
static ViString  auxChannelArr[] = {"STER","CH1","CH2", VI_NULL};
static ViString  auxAnalogAuxOutArr[] = {"DC","AUDM1", VI_NULL};
static ViString  calibrationArr[] = {"CLDG","CAGE","CANL","CDPH", "B48P", "B48S", VI_NULL};
static ViString  auxTriggerInputModeArr[] = {"MSIN","MCON","TRIG", VI_NULL};
static ViString  auxTriggerInputEdgeArr[] = {"RIS","FALL", VI_NULL};
static ViString  auxTriggerOutputModeArr[] = {"MEAS","AUXC", VI_NULL};

/* --- Switcher --- */

static ViString  switcherTrackingArr[] = {"OFF","BVSA","OVSI","ALL", VI_NULL};

/* --- Protocol Analysis --- */

static ViString  protViewModeArr[] = {"BINT","BIN", VI_NULL};
static ViString  protDisplayModeArr[] = {"AUT","PROF","CONS", VI_NULL};
static ViString  protHighlightArr[] = {"NOTH","FOUT","BETW","FST", VI_NULL};
static ViString  protPersistenceArr[] = {"SHOR","LONG","FOR", VI_NULL};
static ViString  protCodingModeArr[] = {"AUT","PROF","CONS","FILE", VI_NULL};
static ViString  protValidityArr[] = {"CH1A","NONE", VI_NULL};
static ViString  protChannelsArr[] = {"CH2I","SPL", VI_NULL};

/* --- Display --- */

static ViString  displayDataListFilterType[] = {"ALL","LIMU","LIML","LIMB","HARM","PEAK", VI_NULL};
static ViString  displayTraceArr[] = {"A","B", VI_NULL};
static ViString  displayTraceUpdateTypeArr[] = {"ALIV","HOLD", VI_NULL};
static ViString  displayYSourceArr[] = {"OFF","FUNC1","FUNC2","FREQ1","FREQ2",
                                        "LMRM1","LMRM2","LMDC1","LMDC2","LMPK1","LMPK2",
                                        "INPP1","INPP2","PHAS","GRO",
                                        "FILEA","FILEB","LEV1","LEV2","PHAS1","PHAS2",
                                        "FFTL1","FFTL2","FFTP1","FFTP2","FFTP21","PEMO","DEL",
                                        "REFS","DEGS","DROP",VI_NULL};
static ViString  displayReferenceArr[] = {"MEAS","VAL","MAX","XCUR","OCUR","REF997",
                                          "REF1000","CH1M","CH2M","GENT","FILE","IFIL",
                                          "HOLD","NOIS", VI_NULL};
static ViString  displayXSourceArr[] = {"SWE","HOLD","MAN","LRMS","LDC","LPE","FREQ", VI_NULL};
static ViString  displayXAxisArr[] = {"TIME","VOLT","FREQ","PHAS","VDIG","JPK", VI_NULL};
static ViString  displayCursorTypeArr[] = {"OCUR","XCUR", VI_NULL};
static ViString  displayCursorStateArr[] = {"OFF","ACT","INAC", VI_NULL};
static ViString  displayCursorModeArr[] = {"VA","VB","VAB","HA","HB", VI_NULL};
static ViString  displayCursorPositionModeArr[] = {"PIX","POIN","PEAK","HARM", VI_NULL};
static ViString  displayStoreTraceAsArr[] = {"TRCL","EQUL","SWPL","LLIS","DSEL", VI_NULL};
static ViString  displaySpacingArr[] = {"LIN","LOG", VI_NULL};
static ViString  displayScalingArr[] = {"AUTO","MAN", VI_NULL};
static ViString  displayTraceUnitArr[] = {"A","B","X", VI_NULL};
static ViString  displayLimitTypeArr[] = {"LIMU","LIML", VI_NULL};
static ViString  displayLimitSourceArr[] = {"VAL","FILE","IFIL","HOLD", VI_NULL};
static ViString  displaySetCursorToArr[] = {"MAX","MIN","MRKA","MRKB", VI_NULL};
static ViString  displayMarkerModeArr[] = {"OFF","FIX","TRKM", VI_NULL};
static ViString displayMultichannelYSourceArr[] = {"OFF","FUNC","FREQ","PHAS","GRO","LMRM","LMDC","LMPK","INPP","FILEA","FILEB", "FFTL","FFTP","FFTR", "LEV", VI_NULL};
static ViString displayNormalizationArr[] = {"OFF","VAL","OCUR","XCUR", VI_NULL};

/* --- Measurement --- */

static ViString  MeasControlArr[] = {"STAR","STOP","SING","CONT", VI_NULL};
static ViString  dataSetArr[] = {"AX","AY","BX","BY", VI_NULL};
static ViString  measModifierArr[] = {"MAX", "MIN", VI_NULL};

/* --- Utilities ---*/

static ViString  freeBuffersArr[] = {"STR","DATA", VI_NULL};
static ViString  hcopySourceArr[] = {"WIND","GRAP", VI_NULL};
static ViString  hcopyDestinationArr[] = {"PRIN","FILE","CLIP", VI_NULL};
static ViString  hcopyOrientationArr[] = {"PORT","LAND", VI_NULL};
static ViString  hcopyStoreModeArr[] = {"NEW","OVER","INCR", VI_NULL};

static ViString selStatusRegArr[] = {"",":UND",":OVER",":MEAS"};
static ViString statusRegArr[] = {"OPER:EVEN","OPER:COND","QUES:EVEN","QUES:COND",
                                  "QUES:UND:EVEN","QUES:UND:COND","QUES:OVER:EVEN",
                                  "QUES:OVER:COND","QUES:MEAS:EVEN","QUES:MEAS:COND",
                                  "XQUE:EVEN","XQUE:COND","QUE"};

static ViString refinementArr[] = {"N1","N2","N4","N8", VI_NULL};
static ViString connectionArr [] = {"COM","USB", VI_NULL}; 
static ViString anDigitalAligmentArr [] = {"RIS","FALL","CH1H","CH1L","CH1R","CH1F", VI_NULL};
static ViString anDigitalChModeArr [] = {"MONO","STER", VI_NULL}; 
static ViString anDigitalClockSourceArr [] = {"INT","EXT", VI_NULL}; 
static ViString anDigitalDwnsmlpFactorArr [] = {"M16","M32","M64","M128","M256", VI_NULL};

static ViString anDigitalPOLQABandArr[] = {"NARR","WIDE", VI_NULL};      

/* --- HDMI ---*/ 

static ViString genHDMIcolorDepthArr[] = {"D08","D10","D12", VI_NULL};
static ViString genHDMIVideoContentArr[] = {"MON","LIPS","BERT","PATT", VI_NULL};

static ViString genHDMIVideoFmtFreqArr[] = {"HZ23","HZ24","HZ25","HZ29","HZ30","HZ50","HZ59","HZ60","HZ100","HZ119","HZ120",VI_NULL};   

static ViString genHDMIVideoFmtResolArr[] = {"P640x480_4x3","P720x576_4x3","P720x576_16x9","I720x576_4x3","I720x576_16x9",
                                             "P720x480_4x3","P720x480_16x9","I720x480_4x3","I720x480_16x9","P1280x720_16x9","P1920x1080_16x9","I1920x1080_16x9", VI_NULL};
    
static ViString genHDMIVideoSourceArr[] = {"INT","AXIN", VI_NULL}; 
static ViString audioHDMIsinkARCArr[] = {"LOOP","DAG", VI_NULL};      

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
typedef struct  rsupv_instrOption
{
   ViChar   optionName[10];
   ViInt32  optionRev;
}  rsupv_tinstrOption;

struct rsupv_statusDataRanges
{
    ViBoolean   errorChecking;
    rsupv_tinstrOption  instrOption[RSUPV_MAX_OPTIONS];
    ViInt32     instrModel;
    ViInt32     instrPorts;
    ViChar      instrDriverRevision[256];
	ViBoolean   optionChecking;
};

typedef struct rsupv_statusDataRanges *rsupv_instrRange;

typedef struct  rsupv_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rsupv_tStringValPair;


/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
void rsupv_delay (ViReal64 numberOfSeconds);
ViStatus rsupv_itemToConst (ViChar **array, ViString stringItem, ViPInt32 itemNo);
ViBoolean rsupv_invalidViBooleanRange (ViBoolean val);
ViBoolean rsupv_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rsupv_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean rsupv_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViBoolean rsupv_invalidViString (ViString val, ViUInt32 maxLength);
ViStatus rsupv_waitOPC (ViSession instrSession, ViPString command, ViPString instrResp, ViInt32 timeout);
ViStatus rsupv_initCleanUp (ViSession openRMSession, ViPSession openInstrSession, ViStatus currentStatus);
ViStatus rsupv_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*------ INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE ------*/
/*****************************************************************************/

ViStatus rsupv_checkStatus (ViSession instrSession);
ViStatus rsupv_sendCmd (ViSession instrSession, ViString cmd);
ViStatus rsupv_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode);
ViStatus rsupv_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus rsupv_setViInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode);
ViStatus rsupv_getViInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus rsupv_setViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 value,
                           ViUInt32 min,
                           ViUInt32 max,
                           ViStatus parameterErrorCode);
ViStatus rsupv_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus rsupv_setViUInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViUInt32 value,
                           ViUInt32 min,
                           ViUInt32 max,
                           ViStatus parameterErrorCode);
ViStatus rsupv_getViUInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus rsupv_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViStatus parameterErrorCode);
ViStatus rsupv_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViStatus parameterErrorCode);
ViStatus rsupv_setViBooleanWindow (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViBoolean value,
                           ViStatus parameterErrorCode);
ViStatus rsupv_getViBooleanWindow (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViBoolean *value,
                           ViStatus parameterErrorCode);
ViStatus rsupv_setViReal64 (ViSession vi,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode);
ViStatus rsupv_setViReal64Unit (ViSession vi,
                                ViString cmd,
                                ViReal64 value,
                                ViInt32 unit,
                                ViReal64 min,
                                ViReal64 max,
                                ViStatus parameterErrorCode);
ViStatus rsupv_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViStatus parameterErrorCode);
ViStatus rsupv_getViReal64Unit (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32 *unit,
                            ViStatus parameterErrorCode);
ViStatus rsupv_setViReal64Window (ViSession instrSession,
                            ViString cmd,
                            ViInt32  window,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode);
ViStatus rsupv_setViReal64WindowUnit (ViSession instrSession,
                            ViString cmd,
                            ViInt32  window,
                            ViReal64 value,
                            ViInt32  unit,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode);
ViStatus rsupv_getViReal64Window (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViStatus parameterErrorCode);
ViStatus rsupv_getViReal64WindowUnit (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViInt32  *unit,
                            ViStatus parameterErrorCode);
ViStatus rsupv_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus rsupv_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViInt32 buffer_size,
                            ViStatus parameterErrorCode);
ViStatus rsupv_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus rsupv_getViStringQuoted (ViSession instrSession,
                                  ViString cmd,
                                  ViString quotes,
                                  ViInt32  arraySize,
                                  ViChar   value[],
                                  ViStatus parameterErrorValue);
ViStatus rsupv_setViStringWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus rsupv_getViStringWindow (ViSession instrSession,
                                ViString cmd,
                                ViInt32     window,
                                ViString value,
                                ViInt32 buffer_size,
                                ViStatus parameterErrorCode);
ViStatus rsupv_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsupv_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsupv_setEnumWindow (ViSession instrSession,
                        ViString cmd,
                        ViInt32     window,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsupv_getEnumWindow (ViSession instrSession,
                        ViString cmd,
                        ViInt32     window,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsupv_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode);
ViStatus rsupv_setViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode);
ViStatus rsupv_setViReal64ArrayWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode);
ViStatus rsupv_getViReal64ArrayWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode);
ViStatus rsupv_getViReal64BinArray (ViSession instrSession,
                            ViString    cmd,
                            ViReal64    *value,
                            ViInt32     maxCount,
                            ViInt32     *retCount,
                            ViStatus    parameterErrorCode);
ViStatus rsupv_checkIndexInList (ViSession instrSession,
                                 ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError);
ViStatus rsupv_getIndexFromString (ViSession instrSession,
                                   ViString string,
                                   ViString *list,
                                   ViInt32 *index);
ViStatus rsupv_readDataUnknownLength(ViSession instrSession,
                                     ViChar **outBuffer,
                                     ViUInt32 *count);
ViStatus rsupv_checkBeforeRead (ViSession instrSession);
void rsupv_SWAP_32(unsigned char *data, long points);
void rsupv_coerceToNearest (void *table, ViPReal64 value, ViPInt16 index);
ViStatus rsupv_checkOption (ViSession instrSession, ViString option);
ViStatus rsupv_checkInstrument (ViSession instrSession, ViInt32 instrument);
ViStatus rsupv_traceCommandSelect (ViSession instrSession, ViInt32 channel,
                                   ViString traceName, ViChar buffer[],
                                   ViInt32 *offset);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_init (ViRsrc resourceName, ViBoolean IDQuery,
                               ViBoolean resetDevice, ViPSession instrSession)
{
    ViStatus            rsupv_status = VI_SUCCESS;
    ViUInt16            interface = VI_INTF_GPIB;
    ViSession           rmSession = 0;
    ViChar              buffer[BUFFER_SIZE];
    rsupv_instrRange   instrPtr;

    /*- Check input parameter ranges ----------------------------------------*/
    if (rsupv_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rsupv_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    CHECKERR( viOpenDefaultRM (&rmSession));

    if ((rsupv_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG,
                                 VI_NULL, instrSession)) < 0)
        {
        viClose (rmSession);
        return rsupv_status;
        }

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR_CU( viGetAttribute (*instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr == NULL)
        instrPtr = malloc (sizeof (struct rsupv_statusDataRanges));

    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_USER_DATA, (ViAttrState)instrPtr));

    /* --- Interface specific operations --- */
    CHECKERR_CU( viGetAttribute (*instrSession, VI_ATTR_INTF_TYPE, &interface));

    if (interface == VI_INTF_ASRL) /* ASRL */
        {
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n'));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR));
        /* Changed to VI_ASRL_END_NONE due to Agilent VISA */
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE));
        CHECKERR_CU( viFlush (*instrSession, VI_ASRL_IN_BUF));
        }
    else /* GPIB, RSIB */
        {
        /* VI_ATTR_SEND_END_EN will stay with this attribute set to VI_TRUE */
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));
        }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, RSUPV_DEFAULT_TIMEOUT));
    CHECKERR_CU( viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, BUFFER_SIZE));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    /* --- Interface specific operations --- */

#if 0

    switch (interface)
        {
        case VI_INTF_ASRL: /* Serial Interface. Go to remote mode. */
            CHECKERR_CU( viWrite (*instrSession, "@REM\n", 5, VI_NULL));
            break;

#ifdef VI_INTF_TCPIP

        case VI_INTF_TCPIP: /* RSIB Interface. Go to remote mode. */
            CHECKERR_CU( viSetAttribute (*instrSession, RS_ATTR_RSIB_SRE, 1));
            break;

#endif
        }

#endif

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery)
    {
        CHECKERR_CU( viQueryf (*instrSession, "*IDN?\n", "%*[^,],%[^,],%*[\r\n]", buffer));

        /* Check if instrument is supported or not */

        buffer[3] = '\0';
        rsupv_status = rsupv_getIndexFromString (*instrSession, buffer, instrList, &instrPtr->instrModel);
        if (rsupv_status != VI_SUCCESS)
            return rsupv_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }
    else
    {
		instrPtr->instrModel = RSUPV_INSTR_UPV;
    }

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice)
        CHECKERR_CU( rsupv_reset (*instrSession));
    else  /*- Send Default Instrument Setup ---------------------------------*/
        CHECKERR_CU( rsupv_defaultInstrSetup (*instrSession));

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Generator Output State
 * Purpose:  This function sets the state of generator output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorOutputState (ViSession instrSession,
                                                 ViBoolean state)
{
    return rsupv_setViBoolean (instrSession,
                               "OUTP",
                               state,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Generator Output State
 * Purpose:  This function queries the state of generator output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorOutputState (ViSession instrSession,
                                                 ViBoolean *state)
{
    return rsupv_getViBoolean (instrSession,
                               "OUTP?",
                               state,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator Instrument
 * Purpose:  This function selects one of the two generators (analog or
 *           digital).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorInstrument(
    ViSession   instrSession,
    ViInt32 instrument
)
{
    switch (instrument)
    {
        case RSUPV_GEN_I2S:
            CHECKOPT ("B41");
            break;
        case RSUPV_GEN_U2C:
            CHECKOPT ("B42");
            break;
        case RSUPV_GEN_IMPAIRMENT:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
        case RSUPV_GEN_HDMI:
            CHECKINSTR (RSUPV_INSTR_UPP);
            break;
    }
    
    return rsupv_setEnum (instrSession,
                             "INST1:SEL",
                             instrument,
                             genInstrumentArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Generator I2S
 * Purpose:  This function configures I2S generator's common parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorI2S (ViSession instrSession,
                                               ViInt32 syncTo,
                                               ViInt32 wordLength,
                                               ViInt32 sampleFrequency,
                                               ViReal64 variableSampleFrequency,
                                               ViInt32 MClkRatio,
                                               ViInt32 audioBits, ViInt32 format,
                                               ViInt32 fsyncShape,
                                               ViInt32 fsyncPolarity,
                                               ViInt32 wordOffset)
{
    ViStatus    rsupv_status;
    
    CHECKOPT ("B41");

    CHECKERR (rsupv_SetI2SGeneratorSyncTo(instrSession, syncTo));
    
    rsupv_status = rsupv_SetI2SGeneratorWordLength (instrSession, wordLength);
    REMAP (2, 3);

    rsupv_status = rsupv_SetI2SGeneratorSampleFrequency(instrSession, sampleFrequency, variableSampleFrequency);
    REMAP (2, 4);
    REMAP (3, 5);

    rsupv_status = rsupv_SetI2SGeneratorMClkRatio(instrSession, MClkRatio);
    REMAP (2, 6);

    rsupv_status = rsupv_SetI2SGeneratorAudioBits(instrSession, audioBits);
    REMAP (2, 7);

    rsupv_status = rsupv_SetI2SGeneratorFormat(instrSession, format);
    REMAP (2, 8);

    if (format == RSUPV_GEN_I2S_FORMAT_USER)
    {
        rsupv_status = rsupv_SetI2SGeneratorFsyncShape(instrSession, fsyncShape);
        REMAPINSTR (2, 9);

        rsupv_status = rsupv_SetI2SGeneratorFsyncPolarity(instrSession, fsyncPolarity);
        REMAPINSTR (2, 10);

        rsupv_status = rsupv_SetI2SGeneratorWordOffset(instrSession, wordOffset);
        REMAPINSTR (2, 11);
    }

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set I2S Generator Sync To
 * Purpose:  This function selects the source to which the I2S generator is 
 *           synchronized.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SGeneratorSyncTo (ViSession instrSession,
                                               ViInt32 syncTo)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                             "SOUR:DIG:SYNC:TO",
                             syncTo,
                             genI2SSyncToArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Generator Sync To
 * Purpose:  This function returns the source to which the I2S generator is 
 *           synchronized.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorSyncTo (ViSession instrSession,
                                               ViInt32 *syncTo)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                             "SOUR:DIG:SYNC:TO?",
                             syncTo,
                             genI2SSyncToArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Generator Word Length
 * Purpose:  This function selects the word length of a data frame for each 
 *           channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SGeneratorWordLength (ViSession instrSession,
                                                    ViInt32 wordLength)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                             "OUTP:WORD",
                             wordLength,
                             genI2SwordLengthArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Generator Word Length
 * Purpose:  This function returns the word length of a data frame for each 
 *           channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorWordLength (ViSession instrSession,
                                                   ViInt32 *wordLength)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                             "OUTP:WORD?",
                             wordLength,
                             genI2SwordLengthArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Generator Sample Frequency
 * Purpose:  This function is used to set the sampling frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SGeneratorSampleFrequency (ViSession instrSession,
                                                        ViInt32 sampleFrequency,
                                                        ViReal64 variableSampleFrequency)
{
    ViStatus rsupv_status;
    CHECKOPT ("B41");
    
    if (sampleFrequency == RSUPV_GEN_I2S_SAMPLE_FREQ_384)
    {
        CHECKINSTR(RSUPV_INSTR_UPV);
    }
    CHECKERR (rsupv_setEnum (instrSession,
                             "OUTP:SAMP:MODE",
                             sampleFrequency,
                             genI2SSampleFrequencyArr,
                             VI_ERROR_PARAMETER2));

    if (sampleFrequency == RSUPV_GEN_I2S_SAMPLE_FREQ_VAL)
    {
        CHECKERR (rsupv_setViReal64 (instrSession,
                                     "OUTP:SAMP:FREQ",
                                     variableSampleFrequency,
                                     6.75e3, 410e3,
                                     VI_ERROR_PARAMETER3));
    }

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Get I2S Generator BClk Frequency
 * Purpose:  This function queries the bit clock frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorBClkFrequency (ViSession instrSession,
                                                      ViReal64 *BClkFreq)
{
     return rsupv_getViReal64 (instrSession,
                              "OUTP:BCLK:FREQ?",
                              BClkFreq,
                              VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get I2S Generator Sample Frequency
 * Purpose:  This function returns the sampling frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorSampleFrequency (ViSession instrSession,
                                                        ViInt32 *sampleFrequency,
                                                        ViReal64 *variableSampleFrequency)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B41");
    
    CHECKERR (rsupv_getEnum (instrSession,
                             "OUTP:SAMP:MODE?",
                             sampleFrequency,
                             genI2SSampleFrequencyArr,
                             VI_ERROR_PARAMETER2));

    CHECKERR (rsupv_getViReal64 (instrSession,
                                 "OUTP:SAMP:FREQ?",
                                 variableSampleFrequency,
                                 VI_ERROR_PARAMETER3));

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Set I2S Generator MClk Ratio
 * Purpose:  This function selects the ratio between MCLK and FSYNC.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SGeneratorMClkRatio (ViSession instrSession,
                                                  ViInt32 MClkRatio)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                             "OUTP:MCLK",
                             MClkRatio,
                             genI2SMClkRatioArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Generator MClk Ratio
 * Purpose:  This function returns the ratio between MCLK and FSYNC.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorMClkRatio (ViSession instrSession,
                                                  ViInt32 *MClkRatio)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                             "OUTP:MCLK?",
                             MClkRatio,
                             genI2SMClkRatioArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Generator Audio Bits
 * Purpose:  Resolution of audio data within a frame.
 *           If the wordwidth is reduced, the values of the audio samples are
 *           rounded off to the specified wordwidth. Unused data bits are
 *           encoded as zeroes in the digital audio signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SGeneratorAudioBits (ViSession instrSession,
                                                  ViInt32 audioBits)
{
    CHECKOPT ("B41");
    
    return rsupv_setViInt32 (instrSession,
                              "OUTP:AUD",
                              audioBits,
                              8, 32,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Audio Bits
 * Purpose:  This function returns the audio bits.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorAudioBits (ViSession instrSession,
                                                  ViInt32 *audioBits)
{
    CHECKOPT ("B41");
    
    return rsupv_getViInt32 (instrSession,
                              "OUTP:AUD?",
                              audioBits,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set I2S Generator Format
 * Purpose:  This function selects the I2S generator function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SGeneratorFormat (ViSession instrSession,
                                               ViInt32 format)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                             "OUTP:FORM",
                             format,
                             genI2SformatArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Generator Format
 * Purpose:  This function selects the I2S generator function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorFormat (ViSession instrSession,
                                               ViInt32 *format)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                             "OUTP:FORM?",
                             format,
                             genI2SformatArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Generator Fsync Shape
 * Purpose:  This function selects the FSYNC signal form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SGeneratorFsyncShape (ViSession instrSession,
                                                   ViInt32 fsyncShape)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                             "OUTP:FSH",
                             fsyncShape,
                             genI2SfsyncShapeArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Generator Fsync Shape
 * Purpose:  This function returns the FSYNC signal form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorFsyncShape (ViSession instrSession,
                                                   ViInt32 *fsyncShape)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                             "OUTP:FSH?",
                             fsyncShape,
                             genI2SfsyncShapeArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Generator Fsync Polarity
 * Purpose:  This function selects the FSYNC signal polarity.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SGeneratorFsyncPolarity (ViSession instrSession,
                                                      ViInt32 fsyncPolarity)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                             "OUTP:POL",
                             fsyncPolarity,
                             genI2SfsyncPolarityArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Generator Fsync Polarity
 * Purpose:  This function returns the FSYNC signal polarity.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorFsyncPolarity (ViSession instrSession,
                                                      ViInt32 *fsyncPolarity)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                             "OUTP:POL?",
                             fsyncPolarity,
                             genI2SfsyncPolarityArr,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Generator Word Offset
 * Purpose:  This function sets the the number of SCK clock cycles by which 
 *           the Fsync signal transmission is delayed relative to the audio data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SGeneratorWordOffset (ViSession instrSession,
                                                   ViInt32 wordOffset)
{
    ViStatus rsupv_status;
    ViInt32 min, max, word_length;
    
    CHECKOPT ("B41");
    
    rsupv_status = rsupv_GetI2SGeneratorWordLength (instrSession, &word_length);
    
    switch (word_length)
    {
        case RSUPV_GEN_I2S_WLENGTH_16:
            min = -16; max = 15;
        break;
        case RSUPV_GEN_I2S_WLENGTH_24:
            min = -24; max = 23;    
        break;
        default:
            min = -32; max = 31;
    }
    
    return rsupv_setViInt32 (instrSession,
                              "OUTP:WRDO",
                              wordOffset,
                              min, max,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Generator Word Offset
 * Purpose:  This function gets the the number of SCK clock cycles by which 
 *           the Fsync signal transmission is delayed relative to the audio data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SGeneratorWordOffset (ViSession instrSession,
                                                   ViInt32 *wordOffset)
{
    CHECKOPT ("B41");
    
    return rsupv_getViInt32 (instrSession,
                              "OUTP:WRDO?",
                              wordOffset,
                              VI_ERROR_PARAMETER2);    
}

    
    
/*****************************************************************************
 * Function: Configure USI Generator Clock And Frequency
 * Purpose:  This function configures clock and mixed sample frequency of USI 
 *           generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIGeneratorClockAndFrequency (ViSession instrSession,
                                                                ViInt32 clock,
                                                                ViBoolean mixedSamplingFrequency,
                                                                ViInt32 ratio)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIGeneratorClock (instrSession, clock));
    
    rsupv_status = rsupv_SetUSIGeneratorMixedSamplingFrequencyState (instrSession, mixedSamplingFrequency);
    REMAP (2,3);
    
    if (mixedSamplingFrequency)
    {
        rsupv_status = rsupv_SetUSIGeneratorRatio (instrSession, ratio);
        REMAP (2,4);
    }
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure USI Generator Frame
 * Purpose:  This function configures generator frame parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIGeneratorFrame (ViSession instrSession,
                                                    ViInt32 samples,
                                                    ViInt32 numberOfSlots)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIGeneratorSamplesPerFrame (instrSession, samples));
    
    rsupv_status = rsupv_SetUSIGeneratorNumberOfSlots (instrSession, numberOfSlots);
    REMAP (2,3);
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure USI Generator Slot
 * Purpose:  This function configures slot layout. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIGeneratorSlot (ViSession instrSession,
                                                   ViInt32 firstBit,
                                                   ViInt32 slotLength,
                                                   ViInt32 audioBits,
                                                   ViInt32 leadBits)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIGeneratorFirstBit (instrSession, firstBit));
    
    rsupv_status = rsupv_SetUSIGeneratorSlotLength (instrSession, slotLength);
    REMAP (2,3);
    
    rsupv_status = rsupv_SetUSIGeneratorAudioBits (instrSession, audioBits);
    REMAP (2,4);
    
    rsupv_status = rsupv_SetUSIGeneratorLeadBits (instrSession, leadBits);
    REMAP (2,5);
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure USI Generator Fsync
 * Purpose:  This function configures Frame Synchronization parameters. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIGeneratorFsync (ViSession instrSession,
                                                    ViInt32 fsyncWidth,
                                                    ViInt32 variableFsyncWidth,
                                                    ViInt32 fsyncOffset,
                                                    ViInt32 fsyncSlope)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIGeneratorFsyncWidth (instrSession, fsyncWidth, variableFsyncWidth));
    
    rsupv_status = rsupv_SetUSIGeneratorFsyncOffset (instrSession, fsyncOffset);
    REMAP (2,4);
    
    rsupv_status = rsupv_SetUSIGeneratorFsyncSlope (instrSession, fsyncSlope);
    REMAP (2,5);
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure USI Generator SlClk
 * Purpose:  This function configures slot clock parameters. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIGeneratorSlClk (ViSession instrSession,
                                                    ViInt32 slClkWidth,
                                                    ViInt32 variableSlClkWidth,
                                                    ViInt32 slClkOffset,
                                                    ViInt32 slClkSlope)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIGeneratorSlClkWidth (instrSession, slClkWidth, variableSlClkWidth));
    
    rsupv_status = rsupv_SetUSIGeneratorSlClkOffset (instrSession, slClkOffset);
    REMAP (2,4);
    
    rsupv_status = rsupv_SetUSIGeneratorSlClkSlope (instrSession, slClkSlope);
    REMAP (2,5);
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure USI Generator Jitter
 * Purpose:  This function configures bit clock and master clock jitter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIGeneratorJitter (ViSession instrSession,
                                                     ViReal64 BClkJitterFrequency,
                                                     ViReal64 BClkJitterAmplitude,
                                                     ViReal64 MClkJitterFrequency,
                                                     ViReal64 MClkJitterAmplitude)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIGeneratorBClkJitterFrequency (instrSession, BClkJitterFrequency));
    
    rsupv_status = rsupv_SetUSIGeneratorBClkJitterAmplitude (instrSession, BClkJitterAmplitude);
    REMAP (2,3);

    rsupv_status = rsupv_SetUSIGeneratorMClkJitterFrequency (instrSession, MClkJitterFrequency);
    REMAP (2,4);
    
    rsupv_status = rsupv_SetUSIGeneratorMClkJitterAmplitude (instrSession, MClkJitterAmplitude);
    REMAP (2,5);
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set USI Generator Clock
 * Purpose:  This function selects continuous clock or gated clock operating 
 *           mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorClock(
    ViSession   instrSession,
    ViInt32 clock
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "OUTP:CLOC",
                          clock,
                          dualClockArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator Clock
 * Purpose:  This function queries clock operating mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorClock(
    ViSession   instrSession,
    ViInt32*    clock
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "OUTP:CLOC?",
                          clock,
                          dualClockArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Generator Mixed Sampling Frequency State
 * Purpose:  This function enables or disables use of mixed sampling 
 *           frequency. Used to produce data streams of varying sampling rates 
 *           in both generator channels. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorMixedSamplingFrequencyState(
    ViSession   instrSession,
    ViBoolean   mixedSamplingFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_setViBoolean (instrSession,
                               "OUTP:MSAM",
                               mixedSamplingFrequency,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator Mixed Sampling Frequency State
 * Purpose:  This function queries state of mixed sampling frequency. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorMixedSamplingFrequencyState(
    ViSession   instrSession,
    ViBoolean*  mixedSamplingFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_getViBoolean (instrSession,
                               "OUTP:MSAM?",
                               mixedSamplingFrequency,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Generator Ratio
 * Purpose:  This function selects down-sampling factor for 2nd generator 
 *           channel with regard to the sampling rate.
 *           
 *           Remote-control command(s):
 *           OUTPut:RATio
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorRatio(
    ViSession   instrSession,
    ViInt32 ratio
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "OUTP:RAT",
                              ratio,
                              2, 6,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator Ratio
 * Purpose:  This function queries down-sampling factor for 2nd generator 
 *           channel with regard to the sampling rate.
 *           
 *           Remote-control command(s):
 *           OUTPut:RATio?
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorRatio(
    ViSession   instrSession,
    ViInt32*    ratio
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "OUTP:RAT?",
                              ratio,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator Samples Per Frame
 * Purpose:  This function selects the number of samples of a generator 
 *           channel per frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorSamplesPerFrame(
    ViSession   instrSession,
    ViInt32 samples
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "OUTP:SPFR",
                              samples,
                              1, 32,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator Samples Per Frame
 * Purpose:  This function queries the number of samples of a generator 
 *           channel per frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorSamplesPerFrame(
    ViSession   instrSession,
    ViInt32*    samples
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "OUTP:SPFR?",
                              samples,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator Number Of Slots
 * Purpose:  This function selects number of slots per frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorNumberOfSlots(
    ViSession   instrSession,
    ViInt32 numberOfSlots
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "OUTP:NOSL",
                              numberOfSlots,
                              1, 256,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator Number Of Slots
 * Purpose:  This function queries number of slots per frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorNumberOfSlots(
    ViSession   instrSession,
    ViInt32*    numberOfSlots
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "OUTP:NOSL?",
                              numberOfSlots,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator Tx Data
 * Purpose:  This function defines which signal is output on the individual 
 *           slots of the selected data line. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorTxData(
    ViSession   instrSession,
    ViInt32 dataLink,
    ViString    signalType
)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKOPT ("B42");
    CHECKVIINT32 (dataLink, 1, 2, 2);
    
    sprintf (buffer, "OUTP:TXD%ld", dataLink);
    
    return rsupv_setViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    signalType,
                                    VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get USI Generator Tx Data
 * Purpose:  This function queries which signal is output on the individual 
 *           slots of the selected data line.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorTxData (ViSession instrSession,
                                                ViInt32 dataLink,
                                                ViInt32 arraySize,
                                                ViChar signalType[])
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKOPT ("B42");
    CHECKVIINT32 (dataLink, 1, 2, 2);
    
    sprintf (buffer, "OUTP:TXD%ld?", dataLink);
    
    return rsupv_getViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    arraySize,
                                    signalType,
                                    VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set USI Generator Sync To
 * Purpose:  This function selects the source to which the USI dual channel 
 *           generator is synchronized.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorSyncTo(
    ViSession   instrSession,
    ViInt32 syncTo
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "SOUR:SYNC:TO",
                          syncTo,
                          dualSyncArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator Sync To
 * Purpose:  This function queries the source to which the USI dual channel 
 *           generator is synchronized.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorSyncTo(
    ViSession   instrSession,
    ViInt32*    syncTo
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "SOUR:SYNC:TO?",
                          syncTo,
                          dualSyncArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Generator First Bit
 * Purpose:  This function defines the sequence in which the data bits are 
 *           transmitted.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorFirstBit(
    ViSession   instrSession,
    ViInt32 firstBit
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "OUTP:FBIT",
                          firstBit,
                          dualFirstBitArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator First Bit
 * Purpose:  This function the sequence in which the data bits are 
 *           transmitted.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorFirstBit(
    ViSession   instrSession,
    ViInt32*    firstBit
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "OUTP:FBIT?",
                          firstBit,
                          dualFirstBitArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Generator Slot Length
 * Purpose:  This function sets slot length of a slot including gaps (0 bits).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorSlotLength(
    ViSession   instrSession,
    ViInt32 slotLength
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "OUTP:SLTL",
                              slotLength,
                              8, 256,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator Slot Length
 * Purpose:  This function queries slot length of a slot including gaps (0 
 *           bits).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorSlotLength(
    ViSession   instrSession,
    ViInt32*    slotLength
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "OUTP:SLTL?",
                              slotLength,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator Audio Bits
 * Purpose:  This function sets slot length of the audio data within a slot 
 *           (sub-frame). Unused data bits are coded as zeros in the digital 
 *           audio signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorAudioBits(
    ViSession   instrSession,
    ViInt32 audioBits
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "OUTP:AUD",
                              audioBits,
                              8, 32,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator Audio Bits
 * Purpose:  This function queries slot length of the audio data within a slot 
 *           (sub-frame).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorAudioBits(
    ViSession   instrSession,
    ViInt32*    audioBits
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                             "OUTP:AUD?",
                             audioBits,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator Lead Bits
 * Purpose:  This function defines the offset of the audio bits within a slot 
 *           relative to the start of the slot.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorLeadBits(
    ViSession   instrSession,
    ViInt32 leadBits
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "OUTP:LBIT",
                              leadBits,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator Lead Bits
 * Purpose:  This function queries the offset of the audio bits within a slot 
 *           relative to the start of the slot.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorLeadBits(
    ViSession   instrSession,
    ViInt32*    leadBits
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "OUTP:LBIT?",
                              leadBits,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator Sample Frequency
 * Purpose:  This function is used to set the sampling frequency. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorSampleFrequency (ViSession instrSession,
                                                        ViInt32 sampleFrequency,
                                                        ViReal64 variableSampleFrequency,
                                                        ViInt32 units)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    CHECKERR (rsupv_setEnum (instrSession,
                             "OUTP:SAMP:MODE",
                             sampleFrequency,
                             genI2SSampleFrequencyArr,
                             VI_ERROR_PARAMETER2));

    if (sampleFrequency == RSUPV_SAMPLE_FREQ_VAL)
    {
        CHECKERR (rsupv_setViReal64Unit (instrSession,
                                     "OUTP:SAMP:FREQ",
                                     variableSampleFrequency,
                                     units,
                                     0.0, 0.0,
                                     VI_ERROR_PARAMETER3));
    }

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Get USI Generator Sample Frequency
 * Purpose:  This function is used to query the sampling frequency. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorSampleFrequency (ViSession instrSession,
                                                        ViInt32 *sampleFrequency,
                                                        ViReal64 *variableSampleFrequency,
                                                        ViInt32 *units)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    CHECKERR (rsupv_getEnum (instrSession,
                             "OUTP:SAMP:MODE?",
                             sampleFrequency,
                             genI2SSampleFrequencyArr,
                             VI_ERROR_PARAMETER2));

    if (*sampleFrequency == RSUPV_SAMPLE_FREQ_VAL)
    {
        CHECKERR (rsupv_getViReal64Unit (instrSession,
                                 "OUTP:SAMP:FREQ?",
                                 variableSampleFrequency,
                                 units,
                                 VI_ERROR_PARAMETER3));
    }

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Get USI Generator Fsync Frequency
 * Purpose:  This function queries Frame Sync frequency which corresponds to 
 *           the sampling frequncy divided by the samples/frame factor. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorFsyncFrequency (ViSession instrSession,
                                                       ViReal64 *fsyncFrequency)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "OUTP:FSYN:FREQ?",
                              fsyncFrequency,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator BClk Frequency
 * Purpose:  This function queries bit clock frequency. Is calculated from the 
 *           product of channel length * no of slots * sample freq / 
 *           samples/frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorBClkFrequency (ViSession instrSession,
                                                      ViReal64 *BClkFrequency)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "OUTP:BCLK:FREQ?",
                              BClkFrequency,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Generator Fsync Width
 * Purpose:  This function selects width of the FSYNC pulse
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorFsyncWidth(
    ViSession   instrSession,
    ViInt32 fsyncWidth,
    ViInt32 variableFsyncWidth
)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    CHECKERR (rsupv_setEnum (instrSession,
                             "OUTP:FWID",
                             fsyncWidth,
                             dualWidthArr,
                             VI_ERROR_PARAMETER2));

    if (fsyncWidth == RSUPV_DUAL_FWIDTH_VALUE)
    {
        CHECKERR (rsupv_setViInt32 (instrSession,
                                    "OUTP:FWID:VAL",
                                    variableFsyncWidth,
                                    0, 0,
                                    VI_ERROR_PARAMETER3));
    }

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Get USI Generator Fsync Width
 * Purpose:  This function queries width of the FSYNC pulse
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorFsyncWidth(
    ViSession   instrSession,
    ViInt32*    fsyncWidth,
    ViInt32*    variableFsyncWidth
)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    CHECKERR (rsupv_getEnum (instrSession,
                             "OUTP:FWID?",
                             fsyncWidth,
                             dualWidthArr,
                             VI_ERROR_PARAMETER2));
    if (*fsyncWidth == RSUPV_DUAL_FWIDTH_VALUE)
    {
        CHECKERR (rsupv_getViInt32 (instrSession,
                                "OUTP:FWID:VAL?",
                                variableFsyncWidth,
                                VI_ERROR_PARAMETER3));
    }

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Set USI Generator Fsync Offset
 * Purpose:  This function determines the offset of the Fsync pulse relative 
 *           to the reference point (start of the first slot).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorFsyncOffset(
    ViSession   instrSession,
    ViInt32 fsyncOffset
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "OUTP:FOFF",
                              fsyncOffset,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator Fsync Offset
 * Purpose:  This function queries the offset of the Fsync pulse relative to 
 *           the reference point (start of the first slot).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorFsyncOffset(
    ViSession   instrSession,
    ViInt32*    fsyncOffset
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "OUTP:FOFF?",
                              fsyncOffset,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator Fsync Slope
 * Purpose:  This function selects polarity of the Fsync pulse.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorFsyncSlope(
    ViSession   instrSession,
    ViInt32 fsyncSlope
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "OUTP:FSL",
                          fsyncSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator Fsync Slope
 * Purpose:  This function queries polarity of the Fsync pulse.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorFsyncSlope(
    ViSession   instrSession,
    ViInt32*    fsyncSlope
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "OUTP:FSL?",
                          fsyncSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Generator BClk Slope
 * Purpose:  This function selects bit clock polarity.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorBClkSlope(
    ViSession   instrSession,
    ViInt32 bclkSlope
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "OUTP:BSL",
                          bclkSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator BClk Slope
 * Purpose:  This function queries bit clock polarity.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorBClkSlope(
    ViSession   instrSession,
    ViInt32*    bclkSlope
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "OUTP:BSL?",
                          bclkSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Generator SlClk Width
 * Purpose:  This function selects slot clock pulse width.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorSlClkWidth(
    ViSession   instrSession,
    ViInt32 slclkWidth,
    ViInt32 variableSlClkWidth
)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    if (slclkWidth == RSUPV_DUAL_FWIDTH_SLOT)
        return VI_ERROR_PARAMETER2;
    
    CHECKERR (rsupv_setEnum (instrSession,
                             "OUTP:SLCW",
                             slclkWidth,
                             dualWidthArr,
                             VI_ERROR_PARAMETER2));

    if (slclkWidth == RSUPV_DUAL_FWIDTH_VALUE)
    {
        CHECKERR (rsupv_setViInt32 (instrSession,
                                    "OUTP:SLCW:VAL",
                                    variableSlClkWidth,
                                    0, 0,
                                    VI_ERROR_PARAMETER3));
    }

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Get USI Generator SlClk Width
 * Purpose:  This function queries slot clock pulse width.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorSlClkWidth(
    ViSession   instrSession,
    ViInt32*    slclkWidth,
    ViInt32*    variableSlClkWidth
)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    CHECKERR (rsupv_getEnum (instrSession,
                             "OUTP:SLCW?",
                             slclkWidth,
                             dualWidthArr,
                             VI_ERROR_PARAMETER2));

    if (*slclkWidth == RSUPV_DUAL_FWIDTH_VALUE)
    {
        CHECKERR (rsupv_getViInt32 (instrSession,
                                    "OUTP:SLCW:VAL?",
                                    variableSlClkWidth,
                                    VI_ERROR_PARAMETER3));
    }
    return rsupv_status;    
}

/*****************************************************************************
 * Function: Set USI Generator SlClk Offset
 * Purpose:  This function determines the slot clock (SlClk) offset relative 
 *           to the start of the slot.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorSlClkOffset(
    ViSession   instrSession,
    ViInt32 slclkOffset
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "OUTP:SLCO",
                              slclkOffset,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator SlClk Offset
 * Purpose:  This function queries the slot clock (SlClk) offset relative to 
 *           the start of the slot.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorSlClkOffset(
    ViSession   instrSession,
    ViInt32*    slclkOffset
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "OUTP:SLCO?",
                              slclkOffset,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator SlClk Slope
 * Purpose:  This function selects slot clock (SlClk) pulse polarity.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorSlClkSlope(
    ViSession   instrSession,
    ViInt32 slclkSlope
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "OUTP:SLCS",
                          slclkSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator SlClk Slope
 * Purpose:  This function queries slot clock (SlClk) pulse polarity.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorSlClkSlope(
    ViSession   instrSession,
    ViInt32*    slclkSlope
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "OUTP:SLCS?",
                          slclkSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Generator MClkRatio
 * Purpose:  This function defines relationship between the master clock 
 *           (MCLK) and the FrameSync (FSYNC).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorMClkRatio(
    ViSession   instrSession,
    ViInt32 ratio
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "OUTP:MRAT",
                              ratio,
                              16, 768,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator MClkRatio
 * Purpose:  This function queries relationship between the master clock 
 *           (MCLK) and the FrameSync (FSYNC).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorMClkRatio(
    ViSession   instrSession,
    ViInt32*    ratio
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "OUTP:MRAT?",
                              ratio,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator BClk Jitter Frequency
 * Purpose:  This function sets jitter oscillator frequency. The generator 
 *           output signals (TX_BCLK_OUT, TX_FSYNC_OUT, TX_SLCLK_OUT, TX_DATA1 
 *           to 4) can be jittered sinusoidally.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorBClkJitterFrequency(
    ViSession   instrSession,
    ViReal64    bclkJitterFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_setViReal64 (instrSession,
                              "OUTP:BCL:JITF",
                              bclkJitterFrequency,
                              100.0e-3, 110.0e6,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator BClk Jitter Frequency
 * Purpose:  This function queries jitter oscillator frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorBClkJitterFrequency(
    ViSession   instrSession,
    ViReal64*   bclkJitterFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "OUTP:BCL:JITF?",
                              bclkJitterFrequency,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator BClk Jitter Amplitude
 * Purpose:  This function sets jitter oscillator's peak amplitude. The 
 *           generator output signals (TX_BCLK_OUT, TX_FSYNC_OUT, TX_SLCLK_OUT, 
 *           TX_DATA1 to 4) can be jittered sinusoidally.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorBClkJitterAmplitude(
    ViSession   instrSession,
    ViReal64    bclkJitterAmplitude
)
{
    CHECKOPT ("B42");
    return rsupv_setViReal64 (instrSession,
                              "OUTP:BCL:JITA",
                              bclkJitterAmplitude,
                              0.0, 2.5,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator BClk Jitter Amplitude
 * Purpose:  This function queries jitter oscillator's peak amplitude.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorBClkJitterAmplitude(
    ViSession   instrSession,
    ViReal64*   bclkJitterAmplitude
)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "OUTP:BCL:JITA?",
                              bclkJitterAmplitude,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator MClk Jitter Frequency
 * Purpose:  The TX_MCLK_OUT generator signal can be jittered sinusoidally. 
 *           The jitter oscillator frequency is set with this function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorMClkJitterFrequency(
    ViSession   instrSession,
    ViReal64    mclkJitterFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_setViReal64 (instrSession,
                              "OUTP:MCL:JITF",
                              mclkJitterFrequency,
                              100.0e-3, 110.0e6,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator MClk Jitter Frequency
 * Purpose:  This function queries jitter oscillator frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorMClkJitterFrequency(
    ViSession   instrSession,
    ViReal64*   mclkJitterFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "OUTP:MCL:JITF?",
                              mclkJitterFrequency,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator MClk Jitter Amplitude
 * Purpose:  The TX_MCLK_OUT generator signal can be jittered sinusoidally. 
 *           This function determines the jitter oscillator peak amplitude.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorMClkJitterAmplitude(
    ViSession   instrSession,
    ViReal64    mclkJitterAmplitude
)
{
    CHECKOPT ("B42");
    return rsupv_setViReal64 (instrSession,
                              "OUTP:MCL:JITA",
                              mclkJitterAmplitude,
                              0.0, 2.5,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Generator MClk Jitter Amplitude
 * Purpose:  This function queries jitter oscillator's peak amplitude.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorMClkJitterAmplitude(
    ViSession   instrSession,
    ViReal64*   mclkJitterAmplitude
)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "OUTP:MCL:JITA?",
                              mclkJitterAmplitude,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Generator Logic Voltage
 * Purpose:  This function sets the generator logic voltage. Changing the 
 *           voltage level causes the sample to be reloaded. The sample can be 
 *           attached and disconnected and put back into operation during 
 *           operation with this mechanism.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorLogicVoltage(
    ViSession   instrSession,
    ViInt32 logicVoltage
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "OUTP:LOGV",
                          logicVoltage,
                          dualVoltageArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator Logic Voltage
 * Purpose:  This function queries the generator logic voltage.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorLogicVoltage(
    ViSession   instrSession,
    ViInt32*    logicVoltage
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "OUTP:LOGV?",
                          logicVoltage,
                          dualVoltageArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Generator Coding
 * Purpose:  This function sets generator data coding regulation. A Law and 
 *           micro Law quantification characteristics are available for 
 *           telephony applications.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIGeneratorCoding(
    ViSession   instrSession,
    ViInt32 coding
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "OUTP:COD",
                          coding,
                          dualCodingArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Generator Coding
 * Purpose:  This function queries generator data coding regulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIGeneratorCoding(
    ViSession   instrSession,
    ViInt32*    coding
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "OUTP:COD?",
                          coding,
                          dualCodingArr,
                          VI_ERROR_PARAMETER2);    
}

    
    
/*****************************************************************************
 * Function: USI Generator Resync
 * Purpose:  Using this function triggers renewed synchronisation of the 
 *           generator. If this is synchronised externally, this function must 
 *           be called once after applying the external clock signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_USIGeneratorResync(
    ViSession   instrSession
)
{
    CHECKOPT ("B42");
    return rsupv_sendCmd (instrSession, "OUTP:RESY ONCE");
}

/* HDMI  Generator */

/*****************************************************************************
 * Function: Configure HDMI Generator Audio
 * Purpose: This function configures the HDMI generator audio parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureHDMIGeneratorAudio (ViSession instrSession,
                                                     ViInt32 audioFormat,
                                                     ViInt32 channel,
                                                     ViInt32 syncTo,
                                                     ViInt32 sampleFrequency,
                                                     ViReal64 variableSampleFrequency)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetHDMIGeneratorAudioFormat (instrSession, audioFormat));
    
    rsupv_status = rsupv_SetDigitalGeneratorChannelMode (instrSession, channel);
    REMAP (2,3);
    
    rsupv_status = rsupv_SetDigitalGeneratorSyncTo (instrSession, syncTo);
    REMAP (2,4);
    
    rsupv_status = rsupv_SetDigitalGeneratorSampleFrequency (instrSession, sampleFrequency, variableSampleFrequency);
    REMAP (2,5);   
    REMAP (3,6);
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure HDMI Generator Video
 * Purpose: This function configures the HDMI generator video parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureHDMIGeneratorVideo (ViSession instrSession,
                                                     ViInt32 source,
                                                     ViInt32 formatResolution,
                                                     ViInt32 formatFrequency,
                                                     ViInt32 colorDepth)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetHDMIGeneratorVideoSource (instrSession, source));
    
    rsupv_status = rsupv_SetHDMIGeneratorVideoFormatResolution (instrSession, formatResolution);
    REMAP (2,3);
    
    rsupv_status = rsupv_SetHDMIGeneratorVideoFormatFrequency (instrSession, formatFrequency);
    REMAP (2,4);
    
    rsupv_status = rsupv_SetHDMIGeneratorVideoColorDepth (instrSession, colorDepth);
    REMAP (2,5);
    
    return rsupv_status;
}


/*****************************************************************************
 * Function: Set HDMI Generator Audio Format
 * Purpose: This function selects the HDMI layout of the supplied audio
 *          data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIGeneratorAudioFormat (ViSession instrSession,
                                                   ViInt32 audioFormat)

{
    CHECKOPT ("B4");
    return rsupv_setEnum (instrSession,
                        "SOUR:HDMI:AUD:FORM",
                        audioFormat,
                        audioHDMIFormatArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Generator Audio Format
 * Purpose:  This function returns the HDMI layout of the supplied audio
 *           data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorAudioFormat (ViSession instrSession,
                                                   ViInt32* audioFormat)

{
    CHECKOPT ("B4");
    return rsupv_getEnum (instrSession,
                        "SOUR:HDMI:AUD:FORM?",
                        audioFormat,
                        audioHDMIFormatArr,
                        VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Set HDMI Generator Audio Sink ARC
 * Purpose: This function selects the HDMI layout of the supplied audio
 *          data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIGeneratorAudioSinkARC (ViSession instrSession,
                                                   ViInt32 sinkARC)

{
    CHECKOPT ("B4");
    return rsupv_setEnum (instrSession,
                        "SOUR:HDMI:SARC",
                        sinkARC,
                        audioHDMIsinkARCArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Generator Audio Sink ARC
 * Purpose:  This function returns the HDMI layout of the supplied audio
 *           data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorAudioSinkARC (ViSession instrSession,
                                                   ViInt32* sinkARC)

{
    CHECKOPT ("B4");
    return rsupv_getEnum (instrSession,
                        "SOUR:HDMI:SARC?",
                        sinkARC,
                        audioHDMIsinkARCArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Generator Audio Info Frame
 * Purpose:  The query retrieves in the hexadecimal format the image of the
 *           binary data structure.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorAudioInfoFrame (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar audioInfoFrame[])
{
    ViStatus rsupv_status = 0;

    CHECKOPT ("B4");
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SOUR:HDMI:AUD:INF:STR?",
                                 audioInfoFrame,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}   

    
/*****************************************************************************
 * Function: Set HDMI Generator Channels Mode
 * Purpose:  This function configures active generator channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIGeneratorChannelsMode (ViSession instrSession,
                                                ViInt32 channel,
                                                 ViBoolean genChannels)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKOPT ("B4");
    
    CHECKVIINT32 (channel, 1, 8, 2);
    
    sprintf (buffer, "OUTP:MCH%ld", channel);
    
    return rsupv_setViBoolean (instrSession,
                               buffer,
                               genChannels,
                               VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get HDMI Generator Channels Mode
 * Purpose:  This function queries the state of active generator channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorChannelsMode (ViSession instrSession,
                                                ViInt32 channel,
                                                 ViBoolean *genChannels)
{
    ViChar buffer[BUFFER_SIZE];

    
    CHECKOPT ("B4");
    CHECKVIINT32 (channel, 1, 8, 2);
    
    sprintf (buffer, "OUTP:MCH%ld?", channel);
    
    return rsupv_getViBoolean (instrSession,
                                buffer,
                                genChannels,
                               VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get HDMI Generator Video Format No 
 * Purpose:  This function returns the Video identification code of the
 *           generated video signal according to CEA-861. The ID code is
 *           copied to the AVI Info Frame of the HDMI generator signal during
 *           the format selection.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoFormatNo (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar formatNo[])
{
    ViStatus rsupv_status = 0;
    CHECKOPT ("B4");
    CHECKERR (rsupv_getViString (instrSession,
                                 "SOUR:HDMI:VID:FORM?",
                                 formatNo,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set HDMI Generator Video Color Depth
 * Purpose:  This function selects  the color depth of the picture data in
 *           bit.  
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIGeneratorVideoColorDepth (ViSession instrSession,
                                                   ViInt32 colorDepth)

{
   
    CHECKOPT ("B4");
    return rsupv_setEnum (instrSession,
                        "SOUR:HDMI:VID:COL:DEPT",
                        colorDepth,
                        genHDMIcolorDepthArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Generator Video Color Depth
 * Purpose:  This function returns the color depth of the picture data in
 *           bit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoColorDepth (ViSession instrSession,
                                                   ViInt32* colorDepth)

{
    
    CHECKOPT ("B4");
    return rsupv_getEnum (instrSession,
                        "SOUR:HDMI:VID:COL:DEPT?",
                        colorDepth,
                        genHDMIcolorDepthArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set HDMI Generator Video Content
 * Purpose:  This function selects generated test pattern.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIGeneratorVideoContent (ViSession instrSession,
                                                   ViInt32 content)

{
   
    CHECKOPT ("B4");
    return rsupv_setEnum (instrSession,
                        "SOUR:HDMI:VID:CONT",
                        content,
                        genHDMIVideoContentArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Generator Video Content
 * Purpose:  This function returns generated test pattern.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoContent (ViSession instrSession,
                                                   ViInt32* content)

{
    CHECKOPT ("B4");
    return rsupv_getEnum (instrSession,
                        "SOUR:HDMI:VID:CONT?",
                        content,
                        genHDMIVideoContentArr,
                        VI_ERROR_PARAMETER2);
}
     
/*****************************************************************************
 * Function: Get HDMI Generator Video Color String 
 * Purpose:  This function retrieves in the hexadecimal format the image of
 *           the binary data structure.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoColorString (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar colorString[])
{
    CHECKOPT ("B4");
    return rsupv_getViStringQuoted (instrSession,
                              "SOUR:HDMI:VID:COL:STR?",
                              "'",
                              bufferSize,
                              colorString,
                              VI_ERROR_PARAMETER3);    
}   

/*****************************************************************************
 * Function: Get HDMI Generator Video AVI
 * Purpose:  This function returns an image of the binary data structure from
 *           the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoAVI (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar avi[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SOUR:HDMI:AVI:STR?",
                                 avi,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    return rsupv_status;
}   

/*****************************************************************************
 * Function: Get HDMI Generator Video SPD
 * Purpose:  This function returns an image of the binary data structure from
 *           the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoSPD (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar spd[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SOUR:HDMI:SPD:STR?",
                                 spd,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}   

/*****************************************************************************
 * Function: Get HDMI Generator Video E-EDID
 * Purpose:  This function returns an image of the binary data structure from
 *           the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoEEDID (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar eedid[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SOUR:HDMI:EED:STR?",
                                 eedid,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}


/*****************************************************************************
 * Function: Set HDMI Generator Video Format Frequency
 * Purpose:  This function sets the vertical video frequency. Corresponds to
 *           the VSYNC frequency. The values available for selection are
 *           oriented to the previously selected resolution according to CEA-861.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIGeneratorVideoFormatFrequency (ViSession instrSession,
                                                   ViInt32 formatFrequency)
{
   
    CHECKOPT ("B4");
    return rsupv_setEnum (instrSession,
                        "SOUR:HDMI:VID:FREQ",
                        formatFrequency,
                        genHDMIVideoFmtFreqArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Generator Video Format Frequency
 * Purpose:  This function gets the vertical video frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoFormatFrequency (ViSession instrSession,
                                                   ViInt32* formatFrequency)
{
    CHECKOPT ("B4");
    return rsupv_getEnum (instrSession,
                        "SOUR:HDMI:VID:FREQ?",
                        formatFrequency,
                        genHDMIVideoFmtFreqArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set HDMI Generator Video Format Resolution
 * Purpose:  This function selects the horizontal and vertical resolution as
 *           well as the aspect ratio of the created video format according
 *           to the CEA-861 standard.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIGeneratorVideoFormatResolution (ViSession instrSession,
                                                   ViInt32 formatResolution)
{
    CHECKOPT ("B4");
   
    return rsupv_setEnum (instrSession,
                        "SOUR:HDMI:VID:RES",
                        formatResolution,
                        genHDMIVideoFmtResolArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Generator Video Format Resolution
 * Purpose:  This function returns the horizontal and vertical resolution.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoFormatResolution (ViSession instrSession,
                                                   ViInt32* formatResolution)
{
    CHECKOPT ("B4");
    return rsupv_getEnum (instrSession,
                        "SOUR:HDMI:VID:RES?",
                        formatResolution,
                        genHDMIVideoFmtResolArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set HDMI Generator Video Source
 * Purpose:  This function selects the video source.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIGeneratorVideoSource (ViSession instrSession,
                                                   ViInt32 source)
{
    CHECKOPT ("B4");
    return rsupv_setEnum (instrSession,
                        "SOUR:HDMI:VID:SOUR",
                        source,
                        genHDMIVideoSourceArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Generator Video Source
 * Purpose:  This function returns the video source.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoSource (ViSession instrSession,
                                                   ViInt32* source)
{
    CHECKOPT ("B4");
    return rsupv_getEnum (instrSession,
                        "SOUR:HDMI:VID:SOUR?",
                        source,
                        genHDMIVideoSourceArr,
                        VI_ERROR_PARAMETER2);
}



/*****************************************************************************
 * Function: Get HDMI Generator Video CEC
 * Purpose:  The query retrieves in the hexadecimal format the image of the
 *           binary data structure.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIGeneratorVideoCEC (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar cec[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
    CHECKERR (rsupv_getViString (instrSession,
                                 "SOUR:HDMI:CEC:STR?",
                                 cec,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Generator Reference
 * Purpose:  This function sets reference value for the relative voltage and
 *           frequency units.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorReference (ViSession instrSession,
    ViReal64    refVoltage,
                                                     ViInt32 voltageUnits,
                                                     ViReal64 refFrequency,
                                                     ViInt32 frequencyUnits)
{

    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetGeneratorRefVoltage(instrSession, refVoltage, voltageUnits));

    rsupv_status = rsupv_SetGeneratorRefFrequency(instrSession, refFrequency, frequencyUnits);
    REMAP (2, 3);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Get Generator Instrument
 * Purpose:  This function queries selected generator instrument (analog or
 *           digital).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorInstrument (ViSession instrSession,
                                       ViInt32 *instrument)
{
    return rsupv_getEnum (instrSession,
                             "INST1:SEL?",
                             instrument,
                             genInstrumentArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Channel Mode
 * Purpose:  This function is used to select the signal-carrying XLR plugs in
 *           the bottom right-hand part of the R&S UPV front panel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorChannelMode(
    ViSession   instrSession,
    ViInt32 channel
)
{
    return rsupv_setEnum (instrSession,
                             "OUTP:CHAN",
                             channel,
                             genChannelArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Channel Mode
 * Purpose:  This function is used to return how the signal-carrying XLR
 *           plugs in the bottom right-hand part of the R&S UPV front panel
 *           are configured.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorChannelMode(
    ViSession   instrSession,
    ViInt32*    channel
)
{
    return rsupv_getEnum (instrSession,
                          "OUTP:CHAN?",
                          channel,
                          genChannelArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Add Impairment
 * Purpose:  This function sets whether impairments (jitter or a common-mode 
 *           signal) are to be added to the output signal of the digital audio 
 *           generators.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorAddImpairment (ViSession instrSession,
                                                          ViBoolean addImpairment)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViBoolean (instrSession,
                               "SOUR:IMP",
                               addImpairment,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Digital Generator Add Impairment
 * Purpose:  This function gets whether impairments (jitter or a common-mode 
 *           signal) are to be added to the output signal of the digital audio 
 *           generators.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorAddImpairment (ViSession instrSession,
                                                          ViBoolean *addImpairment)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViBoolean (instrSession,
                               "SOUR:IMP?",
                               addImpairment,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Digital Generator Channel Mode
 * Purpose:  This function is used to select the left and right-hand digital
 *           audio channels if 'Audio Data' is selected as the 'Source Mode'.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorChannelMode(
    ViSession   instrSession,
    ViInt32 channel
)
{
    return rsupv_setEnum (instrSession,
                             "OUTP:CHAN",
                             channel,
                             genChannelArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Channel Mode
 * Purpose:  This function is used to return how the left and right-hand
 *           digital audio channels are configured.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorChannelMode(
    ViSession   instrSession,
    ViInt32*    channel
)
{
    return rsupv_getEnum (instrSession,
                          "OUTP:CHAN?",
                          channel,
                          genChannelArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Ref Voltage
 * Purpose:  This function sets reference value for the relative voltage
 *           units.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorRefVoltage (ViSession instrSession,
                                                ViReal64 refVoltage,
                                                ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT:REF",
                              refVoltage,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Ref Voltage
 * Purpose:  This function returns reference value for the relative voltage
 *           units.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorRefVoltage (ViSession instrSession,
                                                ViReal64 *refVoltage,
                                                ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:VOLT:REF?",
                              refVoltage,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Ref Frequency
 * Purpose:  This function sets reference value for the relative frequency
 *           units.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorRefFrequency (ViSession instrSession,
                                                  ViReal64 refFrequency,
                                                  ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:FREQ:REF",
                              refFrequency,
                              units,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Ref Frequency
 * Purpose:  This function returns reference value for the relative frequency
 *           units.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorRefFrequency (ViSession instrSession,
                                                  ViReal64 *refFrequency,
                                                  ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:FREQ:REF?",
                              refFrequency,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Generator Analog
 * Purpose:  This function configures analog generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorAnalog (ViSession instrSession,
    ViInt32 channel,
    ViInt32 outputType,
    ViInt32 impedance,
    ViInt32 common,
    ViInt32 bandwidth,
    ViInt32 voltRange,
    ViReal64 maxVoltage,
    ViInt32 units)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetGeneratorChannelMode(instrSession, channel));

    rsupv_status = rsupv_SetGeneratorOutputType(instrSession, outputType);
    REMAP (2, 3);

    if (rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPV) == VI_SUCCESS)
    {
        if (outputType == RSUPV_GEN_OUTPUT_BAL)
        {
            rsupv_status = rsupv_SetGeneratorImpedance(instrSession, impedance);
            REMAP (2, 4);
        }
        rsupv_status = rsupv_SetGeneratorCommon(instrSession, common);
        REMAP (2, 5);
    }

    rsupv_status = rsupv_SetGeneratorBandwidth(instrSession, bandwidth);
    REMAP (2, 6);

    rsupv_status = rsupv_SetGeneratorVoltageRange(instrSession, voltRange);
    REMAP (2, 7);

    rsupv_status = rsupv_SetGeneratorMaxVoltage(instrSession, maxVoltage, units);
    REMAP (2, 8);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Generator Output Type
 * Purpose:  This function configures the generator output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorOutputType(
    ViSession   instrSession,
    ViInt32 outputType
)
{
    return rsupv_setEnum (instrSession,
                             "OUTP:TYPE",
                             outputType,
                             genOutputTypeArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get  Generator Output Type
 * Purpose:  This function returns the generator output type.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorOutputType(
    ViSession   instrSession,
    ViInt32*    outputType
)
{
    return rsupv_getEnum (instrSession,
                          "OUTP:TYPE?",
                          outputType,
                          genOutputTypeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Impedance
 * Purpose:  This function sets the output impedance.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorImpedance(
    ViSession   instrSession,
    ViInt32 impedance
)
{
    if (impedance == RSUPV_GEN_IMPEDANCE_25)
    {
        CHECKINSTR (RSUPV_INSTR_UPP);
    }
    else
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
    return rsupv_setEnum (instrSession,
                             "OUTP:IMP",
                             impedance,
                             genImpedanceArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Impedance
 * Purpose:  This function returns the output impedance.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorImpedance(
    ViSession   instrSession,
    ViInt32*    impedance
)
{
    return rsupv_getEnum (instrSession,
                          "OUTP:IMP?",
                          impedance,
                          genImpedanceArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Common
 * Purpose:  This function selects the generator ground.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorCommon(
    ViSession   instrSession,
    ViInt32     common
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                             "OUTP:LOW",
                             common,
                             genCommonArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Common
 * Purpose:  This function returns the generator ground.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorCommon(
    ViSession   instrSession,
    ViInt32*    common
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "OUTP:LOW?",
                          common,
                          genCommonArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Bandwidth
 * Purpose:  If the universal generator is selected, this function selects the
 *           frequency ranges.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorBandwidth(
    ViSession   instrSession,
    ViInt32 bandwidth
)
{
    return rsupv_setEnum (instrSession,
                             "OUTP:BAND:MODE",
                             bandwidth,
                             genBandwidthArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Bandwidth
 * Purpose:  If the universal generator is selected, this function returns the
 *           frequency ranges.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorBandwidth(
    ViSession   instrSession,
    ViInt32*    bandwidth
)
{
    return rsupv_getEnum (instrSession,
                          "OUTP:BAND:MODE?",
                          bandwidth,
                          genBandwidthArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Voltage Range
 * Purpose:  This function selects the generator level control.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorVoltageRange(
    ViSession   instrSession,
    ViInt32 voltRange
)
{
    return rsupv_setEnum (instrSession,
                             "SOUR:VOLT:RANG",
                             voltRange,
                             genVoltRangeArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Voltage Range
 * Purpose:  This function returns the generator level control.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorVoltageRange(
    ViSession   instrSession,
    ViInt32*    voltRange
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:VOLT:RANG?",
                          voltRange,
                          genVoltRangeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Max Voltage
 * Purpose:  This function sets maximum value for the output voltage.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMaxVoltage (ViSession instrSession,
                                                ViReal64 maxVoltage,
                                                ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT:MAX",
                              maxVoltage,
                                  units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Max Voltage
 * Purpose:  This function returns maximum value for the output voltage.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMaxVoltage(
    ViSession   instrSession,
    ViReal64*   maxVoltage,
    ViInt32*    units
)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:VOLT:MAX?",
                              maxVoltage,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Generator Digital
 * Purpose:  This function configures digital generator's common parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorDigital (ViSession instrSession,
                                                   ViInt32 syncTo,
                                                   ViInt32 sampleFreq,
                                                   ViReal64 sampleFreqValue,
                                                   ViInt32 syncOutput,
                                                   ViReal64 intClkFreq,
                                                   ViInt32 syncOutType,
                                                   ViInt32 auxOutput)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetDigitalGeneratorSyncTo(instrSession, syncTo));

    rsupv_status = rsupv_SetDigitalGeneratorSampleFrequency(instrSession, sampleFreq, sampleFreqValue);
    REMAP (2, 3);
    REMAP (3, 4);

    if (rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPV) == VI_SUCCESS)
    {
        rsupv_status = rsupv_SetDigitalGeneratorSyncOutput(instrSession, syncOutput);
        REMAP (2, 5);
        if ((syncTo != RSUPV_GEN_SYNC_INT_CLK) && (syncOutput == RSUPV_GEN_SYNC_OUTP_INT_CLK))
        {
            rsupv_status = rsupv_SetDigitalGeneratorInternalClockFrequency(instrSession, intClkFreq);
            REMAP (2, 6);
        }
        if (syncOutput != RSUPV_GEN_SYNC_OUTP_SYNC_IN)
        {
            rsupv_status = rsupv_SetDigitalGeneratorSyncOutType(instrSession, syncOutType);
            REMAP (2, 7);
        }
        rsupv_status = rsupv_SetDigitalGeneratorAuxOutput(instrSession, auxOutput);
        REMAP (2, 8);

    }


    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Generator Digital Audio Ref Gen
 * Purpose:  This function configures the audio data for reference generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorDigitalAudioRefGen (ViSession instrSession,
    ViInt32 refGenData,
    ViInt32 phaseToRef,
                                                              ViReal64 framePhaseValue,
                                                              ViInt32 units)
{
    ViStatus rsupv_status;
    CHECKINSTR (RSUPV_INSTR_UPV);

    CHECKERR (rsupv_SetDigitalGeneratorRefGeneratorData(instrSession, refGenData));

    rsupv_status = rsupv_SetDigitalGeneratorPhaseToRef(instrSession, phaseToRef);
    REMAP (2, 3);

    if (phaseToRef == RSUPV_GEN_PHASE_REF_VAL)
    {
    rsupv_status = rsupv_SetDigitalGeneratorFramePhase(instrSession, framePhaseValue, units);
    REMAP (2, 4);
    REMAP (3, 5);
    }

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Digital Generator Source Mode
 * Purpose:  This function determines whether audio data or impairments are
 *           generated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorSourceMode(
    ViSession   instrSession,
    ViInt32 sourceMode
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                             "SOUR:DIG:SRCM",
                             sourceMode - 1,
                             genSourceModeArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Source Mode
 * Purpose:  This function returns source mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorSourceMode(
    ViSession   instrSession,
    ViInt32*    sourceMode
)
{
    ViStatus rsupv_status = VI_SUCCESS;
    CHECKINSTR (RSUPV_INSTR_UPV);
        
    rsupv_status = rsupv_getEnum (instrSession,
                          "SOUR:DIG:SRCM?",
                          sourceMode,
                          genSourceModeArr,
                          VI_ERROR_PARAMETER2);
    *sourceMode = *sourceMode +1;
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Digital Generator Sync To
 * Purpose:  This function selects the source to which the digital audio
 *           generator is synchronized.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorSyncTo(
    ViSession   instrSession,
    ViInt32 syncTo
)
{
    if (rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPV) == VI_SUCCESS)
    {
        CHECKVIINT32 (syncTo, 0, 4, 2);
    }
    else
    {
        if ((syncTo >= 2) && (syncTo <= 4))
            return RSUPV_ERROR_INSTRUMENT_NSUP_MODEL;
    }
    return rsupv_setEnum (instrSession,
                             "SOUR:SYNC:TO",
                             syncTo,
                             genSyncToArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Sync To
 * Purpose:  This function returns the source to which the digital audio
 *           generator is synchronized.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorSyncTo(
    ViSession   instrSession,
    ViInt32*    syncTo
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:SYNC:TO?",
                          syncTo,
                          genSyncToArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Sample Frequency
 * Purpose:  This function sets the clock rate of the digital audio signal for
 *           the generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorSampleFrequency(
    ViSession   instrSession,
    ViInt32     sampleFrequency,
    ViReal64    variableSampleFrequency
)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_setEnum (instrSession,
                             "OUTP:SAMP:MODE",
                             sampleFrequency,
                             genSampleFrequencyArr,
                             VI_ERROR_PARAMETER2));

    if (sampleFrequency == RSUPV_GEN_SAMPLE_FREQ_VAL)
        {
        CHECKERR (rsupv_setViReal64 (instrSession,
                                     "OUTP:SAMP:FREQ",
                                     variableSampleFrequency,
                                     30.0e3, 200.0e3,
                                     VI_ERROR_PARAMETER3));
        }

    return rsupv_status;
}

/*****************************************************************************
 * Function: Get Digital Generator Sample Frequency
 * Purpose:  This function returns the clock rate of the digital audio signal
 *           for the generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorSampleFrequency(
    ViSession   instrSession,
    ViInt32*    sampleFrequency,
    ViReal64*   variableSampleFrequency
)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_getEnum (instrSession,
                             "OUTP:SAMP:MODE?",
                             sampleFrequency,
                             genSampleFrequencyArr,
                             VI_ERROR_PARAMETER2));

    CHECKERR (rsupv_getViReal64 (instrSession,
                                 "OUTP:SAMP:FREQ?",
                                 variableSampleFrequency,
                                 VI_ERROR_PARAMETER3));

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Digital Generator Sync Output
 * Purpose:  This function selects the source for the digital audio sync
 *           output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorSyncOutput(
    ViSession   instrSession,
    ViInt32 syncOutput
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                             "OUTP:DIG:SYNC:OUTP",
                             syncOutput,
                             genSyncOutputArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Sync Output
 * Purpose:  This function returns the source for the digital audio sync
 *           output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorSyncOutput(
    ViSession   instrSession,
    ViInt32*    syncOutput
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "OUTP:DIG:SYNC:OUTP?",
                          syncOutput,
                          genSyncOutputArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Internal Clock Frequency
 * Purpose:  This function sets the (word-clock) frequency of the internal
 *           clock generator if the internal clock generator is used as an
 *           independent clock source at the digital audio sync output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorInternalClockFrequency(
    ViSession   instrSession,
    ViReal64    intClkFreq
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64 (instrSession,
                              "OUTP:DIG:INTC",
                              intClkFreq,
                              1.0e3, 250.0e3,
                              VI_ERROR_PARAMETER2);

}

/*****************************************************************************
 * Function: Get Digital Generator Internal Clock Frequency
 * Purpose:  This function returns the (word-clock) frequency of the internal
 *           clock generator if the internal clock generator is used as an
 *           independent clock source at the digital audio sync output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorInternalClockFrequency(
    ViSession   instrSession,
    ViReal64*   intClkFreq
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "OUTP:DIG:INTC?",
                              intClkFreq,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Sync Out Type
 * Purpose:  This function selects the signal of the digital audio sync
 *           output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorSyncOutType(
    ViSession   instrSession,
    ViInt32 syncOutType
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                             "OUTP:DIG:SYNC:TYPE",
                             syncOutType,
                             genSyncOutTypeArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Sync Out Type
 * Purpose:  This function returns the signal of the digital audio sync
 *           output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorSyncOutType(
    ViSession   instrSession,
    ViInt32*    syncOutType
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "OUTP:DIG:SYNC:TYPE?",
                          syncOutType,
                          genSyncOutTypeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Aux Output
 * Purpose:  This function selects the source for the digital audio auxiliary
 *           output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorAuxOutput(
    ViSession   instrSession,
    ViInt32 auxOutput
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                             "OUTP:DIG:AUX:OUT",
                             auxOutput,
                             genAuxOutputArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Aux Output
 * Purpose:  This function returns the source for the digital audio auxiliary
 *           output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorAuxOutput(
    ViSession   instrSession,
    ViInt32*    auxOutput
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "OUTP:DIG:AUX:OUT?",
                          auxOutput,
                          genAuxOutputArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Ref Generator Data
 * Purpose:  If "Audio Ref Gen" is selected as the "Aux Output" source, this
 *           function configures the audio data for this reference generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorRefGeneratorData(
    ViSession   instrSession,
    ViInt32 refGenData
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "SOUR:DIG:REF",
                          refGenData,
                          genRefGenDataArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Ref Generator Data
 * Purpose:  If "Audio Ref Gen" is selected as the "Aux Output" source, this
 *           function returns the audio data for this reference generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorRefGeneratorData(
    ViSession   instrSession,
    ViInt32*    refGenData
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SOUR:DIG:REF?",
                          refGenData,
                          genRefGenDataArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Phase To Ref
 * Purpose:  If "Audio Ref Gen" is selected as the "Aux Output" source, it is
 *           then possible to vary the frame phase offset of the digital audio
 *           signal between the front output and the digital audio AUX output
 *           at the rear of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorPhaseToRef(
    ViSession   instrSession,
    ViInt32 phaseToRef
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "SOUR:DIG:PHAS",
                          phaseToRef,
                          genPhaseToRefArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Phase To Ref
 * Purpose:  This function returns the Phase to Ref state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorPhaseToRef(
    ViSession   instrSession,
    ViInt32*    phaseToRef
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SOUR:DIG:PHAS?",
                          phaseToRef,
                          genPhaseToRefArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Frame Phase
 * Purpose:  This function sets value for the desired frame phase shift.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorFramePhase (ViSession instrSession,
                                                       ViReal64 framePhase,
                                                       ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:DIG:FRAM",
                              framePhase,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Frame Phase
 * Purpose:  This function returns value for the desired frame phase shift.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorFramePhase (ViSession instrSession,
                                                       ViReal64 *framePhase,
                                                       ViInt32 *units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:DIG:FRAM?",
                              framePhase,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Audio Bits
 * Purpose:  Wordwidth used by the digital audio generator for the Source Mode
 *           "Audio Data".
 *           If the wordwidth is reduced, the values of the audio samples are
 *           rounded off to the specified wordwidth. Unused data bits are
 *           encoded as zeroes in the digital audio signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorAudioBits(
    ViSession   instrSession,
    ViInt32 audioBits
)
{
    return rsupv_setViInt32 (instrSession,
                              "OUTP:AUD",
                              audioBits,
                              8, 24,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Audio Bits
 * Purpose:  This function returns the audio bits.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorAudioBits(
    ViSession   instrSession,
    ViInt32*    audioBits
)
{
    return rsupv_getViInt32 (instrSession,
                              "OUTP:AUD?",
                              audioBits,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
/* Function: Set Digital Generator Balanced Impedance
/* Purpose:  This function sets the balanced impedance for the digital
/*           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorBalancedImpedance (ViSession instrSession,
                                                     ViInt32 balancedImpedance)
{
   
   return rsupv_setEnum (instrSession,
                         "OUTP:IMP:BAL",
                          balancedImpedance,
                          balancedImpedanceArr,
                          VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
/* Function: Get Digital Generator Balanced Impedance
/* Purpose:  This function queries the balanced impedance for the digital
/*           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorBalancedImpedance (ViSession instrSession,
                                                     ViInt32 *balancedImpedance)
{
   return rsupv_getEnum (instrSession,
                         "OUTP:IMP:BAL?",
                          balancedImpedance,
                          balancedImpedanceArr,
                          VI_ERROR_PARAMETER2); 
}
/*****************************************************************************
/* Function: Set Digital Generator Unbalanced Impedance
/* Purpose:  This function sets the unbalanced impedance for the digital
/*           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorUnbalancedImpedance (ViSession instrSession,
                                                       ViInt32 unbalancedImpedance)
{
    return rsupv_setEnum (instrSession,
                         "OUTP:IMP:UNB",
                          unbalancedImpedance,
                          balancedImpedanceArr,
                          VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
/* Function: Get Digital Generator Unbalanced Impedance
/* Purpose:  This function queries the unbalanced impedance for the digital
/*           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorUnbalancedImpedance (ViSession instrSession,
                                                       ViInt32 *unbalancedImpedance)
{
   return rsupv_getEnum (instrSession,
                         "OUTP:IMP:UNB?",
                          unbalancedImpedance,
                          balancedImpedanceArr,
                          VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Digital Generator Balanced Amplitude
 * Purpose:  This function sets the pulse amplitude of the digital audio
 *           signal at the BAL (XLR) output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorBalancedAmplitude (ViSession instrSession,
                                                              ViReal64 balancedAmplitude,
                                                              ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "OUTP:SIGN:BAL:LEV",
                              balancedAmplitude,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Balanced Amplitude
 * Purpose:  This function returns the pulse amplitude of the digital audio
 *           signal at the BAL (XLR) output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorBalancedAmplitude (ViSession instrSession,
                                                              ViReal64 *balancedAmplitude,
                                                              ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "OUTP:SIGN:BAL:LEV?",
                              balancedAmplitude,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Unbalanced Amplitude
 * Purpose:  This function sets the pulse amplitude of the digital audio
 *           signal at the UNBAL (BNC) output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorUnbalancedAmplitude (ViSession instrSession,
                                                                ViReal64 unbalancedAmplitude,
                                                                ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "OUTP:SIGN:LEV",
                              unbalancedAmplitude,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Unbalanced Amplitude
 * Purpose:  This function returns the pulse amplitude of the digital audio
 *           signal at the UNBAL (BNC) output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorUnbalancedAmplitude(
    ViSession   instrSession,
    ViReal64*   unbalAmpl,
    ViInt32*    units
)
{
    return rsupv_getViReal64Unit (instrSession,
                              "OUTP:SIGN:LEV?",
                              unbalAmpl,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Unbalanced Output
 * Purpose:  This function selects unbalanced output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorUnbalancedOutput(
    ViSession   instrSession,
    ViInt32 unbalOutput
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "OUTP:DIG:UNB:OUTP",
                          unbalOutput,
                          genUnbalOutputArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Unbalanced Output
 * Purpose:  This function returns unbalanced output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorUnbalancedOutput(
    ViSession   instrSession,
    ViInt32*    unbalOutput
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "OUTP:DIG:UNB:OUTP?",
                          unbalOutput,
                          genUnbalOutputArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Cable Simulation
 * Purpose:  This function checks the effect of attenuated input signals
 *           along long cables when tests are performed on digital audio
 *           receivers.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorCableSimulation(
    ViSession   instrSession,
    ViInt32 cableSim
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "OUTP:DIG:CSIM",
                          cableSim,
                          genCableSimArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Generator Cable Simulation
 * Purpose:  This function returns the effect of attenuated input signals
 *           along long cables when tests are performed on digital audio
 *           receivers.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorCableSimulation(
    ViSession   instrSession,
    ViInt32*    cableSim
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "OUTP:DIG:CSIM?",
                          cableSim,
                          genCableSimArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Generator Channels 
 * Purpose:  This function configures active generator channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalGeneratorChannels (ViSession instrSession,
                                                ViInt32 channel,
                                                 ViBoolean genChannels)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKINSTR (RSUPV_INSTR_UPP);
    CHECKVIINT32 (channel, 1, 8, 2);
    
    sprintf (buffer, "OUTP:MCH%ld", channel);
    
    return rsupv_setViBoolean (instrSession,
                               buffer,
                               genChannels,
                               VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Digital Generator Channels 
 * Purpose:  This function queries the state of active generator channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalGeneratorChannels (ViSession instrSession,
                                                ViInt32 channel,
                                                 ViBoolean *genChannels)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKINSTR (RSUPV_INSTR_UPP);
    CHECKVIINT32 (channel, 1, 8, 2);
    
    sprintf (buffer, "OUTP:MCH%ld?", channel);
    
    return rsupv_getViBoolean (instrSession,
                                buffer,
                                genChannels,
                               VI_ERROR_PARAMETER3);    
}
    


/*****************************************************************************
 * Function: Configure Generator Mod Dist
 * Purpose:  This function generates a test signal for the intermodulation
 *           measurement in accordance with SMPTE and for the modulation factor
 *           analysis in accordance with IEC 268-3.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorModDist (ViSession instrSession,
                                          ViReal64 upperFreq,
                                          ViInt32 upperFreqUnits,
                                          ViReal64 lowerFreq,
                                          ViInt32 lowerFreqUnits,
                                          ViReal64 voltCh21,
                                          ViReal64 totalVoltage,
                                          ViInt32 voltageUnits)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetGeneratorModDistUpperFrequency(instrSession, upperFreq, upperFreqUnits));

    rsupv_status = rsupv_SetGeneratorModDistLowerFrequency(instrSession, lowerFreq, lowerFreqUnits);
    REMAP (2, 3);
    REMAP (3, 4);

    rsupv_status = rsupv_SetGeneratorModDistLevelRatio (instrSession, voltCh21);
    REMAP (2, 5);

    rsupv_status = rsupv_SetGeneratorTotalVoltage(instrSession, totalVoltage, voltageUnits);
    REMAP (2, 6);
        REMAP (3, 7);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Generator DFD
 * Purpose:  This function generates a test signal for the intermodulation
 *           measurement using the differential-tone method in accordance with
 *           IEC 118 or 268.

 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorDFD (ViSession instrSession, ViInt32 mode,
                                      ViReal64 diffFreq, ViInt32 diffFreqUnits,
                                      ViReal64 meanFreq, ViInt32 meanFreqUnits,
                                      ViReal64 upperFreq, ViInt32 upperFreqUnits,
                                      ViReal64 voltageValue,
                                      ViInt32 voltageUnits)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetGeneratorFunctionMode(instrSession, mode));

    rsupv_status = rsupv_SetGeneratorDFDDiffFrequency(instrSession, diffFreq, diffFreqUnits);
    REMAP (2, 3);
    REMAP (3, 4);

    if (mode == RSUPV_GEN_MODE_IEC_268)
    {
        rsupv_status = rsupv_SetGeneratorDFDMeanFrequency(instrSession, meanFreq, meanFreqUnits);
        REMAP (2, 5);
        REMAP (3, 6);
    }
    else
    {
        rsupv_status = rsupv_SetGeneratorFrequency (instrSession, 1,
                                                    upperFreq, upperFreqUnits);
        REMAP (3, 7);
        REMAP (4, 8);
    }

    rsupv_status = rsupv_SetGeneratorTotalVoltage(instrSession, voltageValue, voltageUnits);
        REMAPINSTR (2, 9);
    REMAPINSTR (3,10);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Generator DIM
 * Purpose:  This function generates a test signal for measuring the Dynamic
 *           InterModulation distortions in accordance with IEC 268-3. The R&S
 *           UPV-B3 option is required to generate this signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorDIM (ViSession instrSession,
                                      ViInt32 squareSine, ViInt32 bandwidth,
                                      ViReal64 voltageValue,
                                      ViInt32 voltageUnits)
{
    ViStatus    rsupv_status;
    CHECKINSTR (RSUPV_INSTR_UPV);

    CHECKERR (rsupv_SetGeneratorDIMSquareToSine(instrSession, squareSine));

    rsupv_status = rsupv_SetGeneratorDIMBandwidth(instrSession, bandwidth);
    REMAP (2,3);

    rsupv_status = rsupv_SetGeneratorTotalVoltage(instrSession, voltageValue, voltageUnits);
    REMAP (2,4);
    REMAP (3,5);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Generator Arbitrary
 * Purpose:  This function generates a freely definable curve shape which is
 *           read out from a file as a sequence of samples. The maximum length
 *           is 16384 samples. When output, the curve is repeated continuously
 *           in an unbroken sequence. The individual samples are output at the
 *           sampling rate selected in the generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorArbitrary (ViSession instrSession,
                                                     ViString shapeFile,
                                            ViReal64 voltPeak, ViInt32 units)
{

    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetGeneratorRandomShapeFile(instrSession, shapeFile));

    rsupv_status = rsupv_SetGeneratorTotalVoltage(instrSession, voltPeak, units);
    REMAP (2,3);
    REMAP (3,4);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Generator FM Modulation
 * Purpose:  This function generates a sinewave signal which can be modulated
 *           with respect to frequency according to a sinusoidal function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorFMModulation (ViSession instrSession,
    ViReal64    modFreq,
                                               ViInt32 modFreqUnits,
    ViReal64    carrierFreq,
                                               ViInt32 carrierFreqUnits,
    ViReal64    deviation,
                                               ViReal64 carrierVolt,
                                               ViInt32 carrierVoltUnits)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetGeneratorFunctionMode (instrSession, RSUPV_GEN_MODE_FM));

    CHECKERR (rsupv_SetGeneratorModulationFrequency(instrSession, modFreq, modFreqUnits));

    rsupv_status = rsupv_SetGeneratorModulationCarrierFrequency(instrSession, carrierFreq, carrierFreqUnits);
    REMAP (2,3);
    REMAP (3,4);

    rsupv_status = rsupv_SetGeneratorModulationDeviation(instrSession, deviation);
        REMAP (2,5);

    rsupv_status = rsupv_SetGeneratorModulationCarrierVoltage(instrSession, carrierVolt, carrierVoltUnits);
    REMAP (2,6);
    REMAP (3,7);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Generator Function
 * Purpose:  This function selects generator function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorFunction(
    ViSession   instrSession,
    ViInt32 function
)
{
    switch (function)
    {
        case RSUPV_GEN_FUNC_SINE2_BURST:
        case RSUPV_GEN_FUNC_DIM:
        case RSUPV_GEN_FUNC_MODULATION:
        case RSUPV_GEN_FUNC_PLYA:
        case RSUPV_GEN_FUNC_SQUARE:
        case RSUPV_GEN_FUNC_CHIRP:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }
    return rsupv_setEnum (instrSession,
                         "SOUR:FUNC:SHAP",
                          function,
                          genFunctionArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Function
 * Purpose:  This function returns generator function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorFunction(
    ViSession   instrSession,
    ViInt32*    function
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:FUNC:SHAP?",
                          function,
                          genFunctionArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Function Mode
 * Purpose:  This function selects generator mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorFunctionMode(
    ViSession   instrSession,
    ViInt32     mode
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "SOUR:FUNC:MODE",
                          mode,
                          genModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Function Mode
 * Purpose:  This function returns generator mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorFunctionMode(
    ViSession   instrSession,
    ViInt32*    mode
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SOUR:FUNC:MODE?",
                          mode,
                          genModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Play Scale Pk To FS
 * Purpose:  This function enables optional scaling up of samples in order to 
 *           obtain full control.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPlayScalePkToFS (ViSession instrSession,
                                                     ViBoolean scalePkToFS)
{
    return rsupv_setViBoolean (instrSession,
                               "SOUR:PLAY:SCAL",
                               scalePkToFS,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Generator Play Scale Pk To FS
 * Purpose:  This function returns optional scaling up of samples in order to 
 *           obtain full control.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorPlayScalePkToFS (ViSession instrSession,
                                                     ViBoolean *scalePkToFS)
{
    return rsupv_getViBoolean (instrSession,
                               "SOUR:PLAY:SCAL?",
                               scalePkToFS,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
/* Function: Set Generator Play Restart
/* Purpose:  This function Defines the play mode for the WAV file after a
/* change of the generator settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPlayRestart (ViSession instrSession,
                                        ViInt32 restart)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:PLAY:REST",
                          restart,
                          restartArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
/* Function: get Generator Play Restart
/* Purpose:  This function queries the selected play mode for the WAV file after a
/* change of the generator settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorPlayRestart (ViSession instrSession,
                                        ViInt32 *restart)
{
   return rsupv_getEnum (instrSession,
                         "SOUR:PLAY:REST?",
                          restart,
                          restartArr,
                          VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Generator Play Channel
 * Purpose:  This function enables a single WAV channel to be output
 *           as a mono signal on both generator channels.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPlayChannel (ViSession instrSession,
                                                 ViInt32 channel)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:PLAY:CHAN",
                          channel,
                          genPlayChannelArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Generator Play Channel
 * Purpose:  This function returns a single WAV channel to be output
 *           as a mono signal on both generator channels.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorPlayChannel (ViSession instrSession,
                                                 ViInt32 *channel)
{
    return rsupv_getEnum (instrSession,
                         "SOUR:PLAY:CHAN?",
                          channel,
                          genPlayChannelArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator Play Mode
 * Purpose:  This function defines the playback mode of the loaded WAV file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPlayMode (ViSession instrSession,
                                              ViInt32 mode)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:PLAY:MODE",
                          mode,
                          genPlayModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Generator Play Mode
 * Purpose:  This function returns the playback mode of the loaded WAV file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorPlayMode (ViSession instrSession,
                                              ViInt32 *mode)
{
    return rsupv_getEnum (instrSession,
                         "SOUR:PLAY:MODE?",
                          mode,
                          genPlayModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator Play Tim
 * Purpose:  This function sets the playback time.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPlayTime (ViSession instrSession,
                                              ViReal64 time)
{
    return rsupv_setViReal64 (instrSession,
                              "SOUR:PLAY:TIME",
                              time,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Generator Play Tim
 * Purpose:  This function gets the playback time.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorPlayTime (ViSession instrSession,
                                              ViReal64 *time)
{
    return rsupv_getViReal64 (instrSession,
                              "SOUR:PLAY:TIME?",
                              time,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator Play Tim
 * Purpose:  This function gets the playback time.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPlayShapeFile (ViSession instrSession,
                                                   ViString shapeFile)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:ARB",
                                    "'",
                                    shapeFile,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator Play Analyzer Loop Channel
 * Purpose:  This function selects the analyzer channel whose signal is to be 
 *           looped back to the generator and to be superimposed on the noise 
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPlayAnalyzerLoopChannel (ViSession instrSession,
                                                             ViInt32 loopChannel)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKOPT ("K6");    

    return rsupv_setEnum (instrSession,
                         "SOUR:LOOP:CHAN",
                          loopChannel,
                          genLoopChannelArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Play Analyzer Loop Channel
 * Purpose:  This function returns the analyzer channel whose signal is to be 
 *           looped back to the generator and to be superimposed on the noise 
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorPlayAnalyzerLoopChannel (ViSession instrSession,
                                                             ViInt32 *loopChannel)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKOPT ("K6");    

    return rsupv_getEnum (instrSession,
                         "SOUR:LOOP:CHAN?",
                          loopChannel,
                          genLoopChannelArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator Play Analyzer Loop Gain
 * Purpose:  This function sets the gain for loopback of the analyzer signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPlayAnalyzerLoopGain (ViSession instrSession,
                                                          ViReal64 loopGain,
                                                          ViInt32 units)
{
    CHECKOPT ("K6");    

    if (units == RSUPV_UNIT_MLT)
        CHECKVIREAL64 (loopGain, 0.0, 1000.0, 2);
    if (units == RSUPV_UNIT_DB)
        CHECKVIREAL64 (loopGain, -240.0, 60.0, 2);
    
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:LOOP:GAIN",
                              loopGain,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator Play Analyzer Loop Gain
 * Purpose:  This function gets the gain for loopback of the analyzer signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorPlayAnalyzerLoopGain (ViSession instrSession,
                                                          ViReal64 *loopGain,
                                                          ViInt32 *units)
{
    CHECKOPT ("K6");    

    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:LOOP:GAIN?",
                              loopGain,
                              units,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator Dither
 * Purpose:  This function activates/deactivates dither function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDither(
    ViSession   instrSession,
    ViBoolean   dither
)
{
    return rsupv_setViBoolean (instrSession,
                               "SOUR:SIN:DITH:STAT",
                               dither,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Dither
 * Purpose:  This function returns dither function state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDither(
    ViSession   instrSession,
    ViBoolean*  dither
)
{
    return rsupv_getViBoolean (instrSession,
                               "SOUR:SIN:DITH:STAT?",
                               dither,
                               VI_ERROR_PARAMETER2);

}

/*****************************************************************************
 * Function: Set Generator Dither Value
 * Purpose:  This function sets dither value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDitherValue (ViSession instrSession,
                                        ViReal64 value, ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:SIN:DITH",
                              value,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Dither Value
 * Purpose:  This function returns dither value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDitherValue (ViSession instrSession,
                                        ViReal64 *value, ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:SIN:DITH?",
                              value,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator PDF
 * Purpose:  This function selects the distribution function for amplitude.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPDF(
    ViSession   instrSession,
    ViInt32 pdf
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
        
    return rsupv_setEnum (instrSession,
                         "SOUR:RAND:PDF",
                          pdf,
                          genPdfArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator PDF
 * Purpose:  This function returns the distribution function for amplitude.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorPDF(
    ViSession   instrSession,
    ViInt32*    pdf
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SOUR:RAND:PDF?",
                          pdf,
                          genPdfArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sweep Ctrl
 * Purpose:  This function activates and configures or deactivates the
 *           generator sweep system.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepCtrl(
    ViSession   instrSession,
    ViInt32 sweepCtrl
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:SWE:CONT",
                          sweepCtrl,
                          genSweepCtrlArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sweep Ctrl
 * Purpose:  This function returns the generator sweep system.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepCtrl(
    ViSession   instrSession,
    ViInt32*    sweepCtrl
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:SWE:CONT?",
                          sweepCtrl,
                          genSweepCtrlArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sweep Next Step
 * Purpose:  This function selects the sweep stepping.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepNextStep(
    ViSession   instrSession,
    ViInt32 nextStep
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:SWE:NEXT",
                          nextStep,
                          genNextStepArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sweep Next Step
 * Purpose:  This function returns the sweep stepping.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepNextStep(
    ViSession   instrSession,
    ViInt32*    nextStep
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:SWE:NEXT?",
                          nextStep,
                          genNextStepArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sweep Dwell Time
 * Purpose:  This function sets the dwell time value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepDwellTime(
    ViSession   instrSession,
    ViReal64    dwell,
    ViInt32     units
)
{   return rsupv_setViReal64Unit (instrSession,
                              "SOUR:SWE:DWEL",
                              dwell,
                              units,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sweep Dwell Time
 * Purpose:  This function returns the dwell time value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepDwellTime(
    ViSession   instrSession,
    ViReal64*   dwell,
    ViInt32*    units
)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:SWE:DWEL?",
                              dwell,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sweep Dwell File
 * Purpose:  This function selects the file containing the list of dwell
 *           times.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepDwellFile(
    ViSession   instrSession,
    ViString    dwellFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:DWEL",
                                    "'",
                                    dwellFile,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sweep X Axis
 * Purpose:  This function selects the sweep parameter (with one-dimensional
 *           sweeps) or the first sweep parameter (with two-dimensional
 *           sweeps).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepXAxis(
    ViSession   instrSession,
    ViInt32 xAxis
)
{
    switch (xAxis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }
    
    return rsupv_setEnum (instrSession,
                         "SOUR:SWE:XAX",
                          xAxis,
                          genAxisArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sweep X Axis
 * Purpose:  This function returns the sweep parameter (with one-dimensional
 *           sweeps) or the first sweep parameter (with two-dimensional
 *           sweeps).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepXAxis(
    ViSession   instrSession,
    ViInt32*    xAxis
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:SWE:XAX?",
                          xAxis,
                          genAxisArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sweep Z Axis
 * Purpose:  This function selects the second sweep parameter for the
 *           two-dimensional sweep. Used both to activate and to deactivate
 *           the two-dimensional sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepZAxis(
    ViSession   instrSession,
    ViInt32 zAxis
)
{
    switch (zAxis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }
    return rsupv_setEnum (instrSession,
                         "SOUR:SWE:ZAX",
                          zAxis,
                          genAxisArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sweep Z Axis
 * Purpose:  This function returns the second sweep parameter for the
 *           two-dimensional sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepZAxis(
    ViSession   instrSession,
    ViInt32*    zAxis
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:SWE:ZAX?",
                          zAxis,
                          genAxisArr,
                          VI_ERROR_PARAMETER2);
}
/*****************************************************************************
 * Function: Set Generator Sweep Halt
 * Purpose:  This function sets the sweeps halt with the halt value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepHalt (ViSession instrSession, ViInt32 axis,
                                      ViInt32 halt, ViReal64 haltValue,
                                      ViInt32 units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];
    

    CHECKVIINT32 (axis, RSUPV_GEN_SWEEP_AXIS_OFF, RSUPV_GEN_SWEEP_AXIS_PHASE, 2);
    CHECKERR (rsupv_checkIndexInList(instrSession, axis, genAxisArr, VI_ERROR_PARAMETER2));
    CHECKVIINT32 (halt, RSUPV_GEN_SWEEP_START, RSUPV_GEN_SWEEP_MUTE, 3);
    CHECKERR (rsupv_checkIndexInList(instrSession, halt, HaltArr, VI_ERROR_PARAMETER3));
    
    sprintf (buffer, "SOUR:SWE:%s:HALT %s", genAxisArr[axis], HaltArr[halt]);
    CHECKERR (rsupv_sendCmd (instrSession, buffer));
    
    if (halt == RSUPV_GEN_SWEEP_VALUE)
        sprintf (buffer, "SOUR:SWE:%s:HALT:VAL %.12f %s ", genAxisArr[axis], haltValue, unitsArr[units]);
    
    return rsupv_sendCmd (instrSession, buffer);    

}

/*****************************************************************************
 * Function: Get Generator Sweep Halt
 * Purpose:  This function returns the sweeps halt with the halt value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepHalt (ViSession instrSession, ViInt32 axis,
                                      ViInt32 *halt, ViReal64 *haltValue,
                                      ViInt32 *units)
{   
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];

    sprintf (buffer, "SOURce:SWEep:%s:HALT?", genAxisArr[axis]);
    
    CHECKERR (rsupv_getEnum (instrSession,
                              buffer,
                              halt,
                              HaltArr,
                              VI_ERROR_PARAMETER3));
   
    if ((haltValue || units) && (*halt== RSUPV_GEN_SWEEP_VALUE))
    {
        ViInt32 l_units;
        ViReal64 l_value;
        
        sprintf (buffer, "SOUR:SWE:%s:HALT:VAL?", genAxisArr[axis]);

        CHECKERR (rsupv_getViReal64Unit (instrSession,
                                       buffer,
                                       &l_value,
                                       &l_units,
                                       VI_ERROR_PARAMETER4));
        if (haltValue) *haltValue = l_value;
        if (units) *units = l_units;
    }
    return rsupv_status;
}
/*****************************************************************************
 * Function: Set Generator Sweep Spacing
 * Purpose:  This function selects the spacing type (logarithmic or linear).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepSpacing(
    ViSession   instrSession,
    ViInt32 axis,
    ViInt32 spacing
)
{
    ViChar  buffer[BUFFER_SIZE];

    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }

    if (axis == RSUPV_GEN_SWEEP_AXIS_PHASE)
        CHECKVIINT32 (spacing, RSUPV_GEN_SWEEP_SPAC_LIN_POINT, RSUPV_GEN_SWEEP_SPAC_LIN_STEP, 3);

    sprintf (buffer, "SOUR:SWE:%s:SPAC %s", genAxisArr[axis], genSpacingArr[spacing]);

    return rsupv_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Generator Sweep Spacing
 * Purpose:  This function returns the spacing type (logarithmic or linear).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepSpacing(
    ViSession   instrSession,
    ViInt32 axis,
    ViInt32*    spacing
)
{
    ViChar  buffer[BUFFER_SIZE];

    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }

    sprintf (buffer, "SOUR:SWE:%s:SPAC?", genAxisArr[axis]);

    return rsupv_getEnum (instrSession,
                          buffer,
                          spacing,
                          genSpacingArr,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Generator Sweep Start
 * Purpose:  This function sets the start value for the sweep parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepStart (ViSession instrSession, ViInt32 axis,
                                       ViReal64 start, ViInt32 units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];

    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }
    CHECKERR (rsupv_checkIndexInList(instrSession, units, unitsArr, VI_ERROR_PARAMETER4));
    sprintf (buffer, "SOUR:SWE:%s:STAR %.12f %s", genAxisArr[axis], start, unitsArr[units]);

    return rsupv_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Generator Sweep Start
 * Purpose:  This function returns the start value for the sweep parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepStart (ViSession instrSession, ViInt32 axis,
                                       ViReal64 *start, ViInt32 *units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];
    ViChar  result[25];

    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }

    sprintf (buffer, "SOUR:SWE:%s:STAR?", genAxisArr[axis]);

    CHECKERR (rsupv_getViString (instrSession,
                              buffer,
                              result,
                              25,
                              VI_ERROR_PARAMETER3));
                              
    sscanf (result, "%le %s", start, buffer);
    CHECKERR (rsupv_getIndexFromString (instrSession, buffer, unitsArr, units));
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Generator Sweep Stop
 * Purpose:  This function sets the stop value for the sweep parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepStop(
    ViSession   instrSession,
    ViInt32 axis,
    ViReal64 stop, ViInt32 units
)

{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];
    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }

    CHECKERR (rsupv_checkIndexInList(instrSession, units, unitsArr, VI_ERROR_PARAMETER4));
    sprintf (buffer, "SOUR:SWE:%s:STOP %.12f %s", genAxisArr[axis], stop, unitsArr[units]);

    return rsupv_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Generator Sweep Stop
 * Purpose:  This function returns the stop value for the sweep parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepStop(
    ViSession   instrSession,
    ViInt32 axis,
    ViReal64*   stop, ViInt32 *units
)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];
    ViChar  result[25];
    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }

    sprintf (buffer, "SOUR:SWE:%s:STOP?", genAxisArr[axis]);

    CHECKERR (rsupv_getViString (instrSession,
                              buffer,
                              result,
                              25,
                              VI_ERROR_PARAMETER3));
                              
    sscanf (result, "%le %s", stop, buffer);
    CHECKERR (rsupv_getIndexFromString (instrSession, buffer, unitsArr, units));
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Generator Sweep Points
 * Purpose:  This function sets the number of sweep points for the sweep
 *           parameter
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepPoints(
    ViSession   instrSession,
    ViInt32 axis,
    ViInt32 points
)
{
    ViChar  buffer[BUFFER_SIZE];
    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }

    sprintf (buffer, "SOUR:SWE:%s:POIN %ld", genAxisArr[axis], points);

    return rsupv_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Generator Sweep Points
 * Purpose:  This function returns the number of sweep points for the sweep
 *           parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepPoints(
    ViSession   instrSession,
    ViInt32 axis,
    ViInt32*    points
)
{
    ViChar  buffer[BUFFER_SIZE];
    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }

    sprintf (buffer, "SOUR:SWE:%s:POIN?", genAxisArr[axis]);

    return rsupv_getViInt32 (instrSession,
                              buffer,
                              points,
                              VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Generator Sweep Steps
 * Purpose:  This function sets the (linear or logarithmic) step size for the
 *           sweep parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSweepSteps (ViSession instrSession, ViInt32 axis,
                                       ViReal64 steps, ViInt32 units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];
    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }

    CHECKERR (rsupv_checkIndexInList(instrSession, units, unitsArr, VI_ERROR_PARAMETER4));

    sprintf (buffer, "SOUR:SWE:%s:STEP %.12f %s", genAxisArr[axis], steps, unitsArr[units]);

    return rsupv_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Generator Sweep Steps
 * Purpose:  This function returns the (linear or logarithmic) step size for
 *           the sweep parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSweepSteps (ViSession instrSession, ViInt32 axis,
                                       ViReal64 *steps, ViInt32 *units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];
    ViChar  result[25];
    switch (axis)
    {
        case RSUPV_GEN_SWEEP_AXIS_ON_TIME:
        case RSUPV_GEN_SWEEP_AXIS_INTERVAL:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
    }

    sprintf (buffer, "SOUR:SWE:%s:STEP?", genAxisArr[axis]);

    CHECKERR (rsupv_getViString (instrSession,
                              buffer,
                              result,
                              25,
                              VI_ERROR_PARAMETER3));
                              
    sscanf (result, "%le %s", steps, buffer);
    CHECKERR (rsupv_getIndexFromString (instrSession, buffer, unitsArr, units));
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Generator Tot Volt File
 * Purpose:  This function sets the file name for the list of the level sweep
 *           parameter (level sweep list).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorTotVoltFile(
    ViSession   instrSession,
    ViString    voltFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:VOLT",
                                    "'",
                                    voltFile,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Freq File
 * Purpose:  This function the file name for the list of the frequency sweep
 *           parameter (frequency sweep list).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorFreqFile(
    ViSession   instrSession,
    ViString    freqFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:FREQ",
                                    "'",
                                    freqFile,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Phase File
 * Purpose:  This function sets the file name for the phase sweep list.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorPhaseFile (ViSession instrSession,
                                               ViString phaseFile)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:PHAS",
                                    "'",
                                    phaseFile,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator On Time File
 * Purpose:  This function sets the file name for the list of the
 *           burst-duration sweep (burst-duration sweep list).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorOnTimeFile(
    ViSession   instrSession,
    ViString    onTimeFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:ONT",
                                    "'",
                                    onTimeFile,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Interval File
 * Purpose:  This function sets the file name for the list of the
 *           burst-interval sweep (burst-interval sweep list).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorIntervalFile(
    ViSession   instrSession,
    ViString    intervalFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:INT",
                                    "'",
                                    intervalFile,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Frequency Spacing
 * Purpose:  This function selects the frequency spacing. The frequency of
 *           each sinewave line is an integer multiple of this frequency
 *           spacing.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorFrequencySpacing(
    ViSession   instrSession,
    ViInt32 spacing
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:RAND:SPAC:MODE",
                          spacing,
                          genSpacingModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Frequency Spacing
 * Purpose:  This function returns the frequency spacing. The frequency of
 *           each sinewave line is an integer multiple of this frequency
 *           spacing.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorFrequencySpacing(
    ViSession   instrSession,
    ViInt32*    spacing
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:RAND:SPAC:MODE?",
                          spacing,
                          genSpacingModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Frequency Spacing Value
 * Purpose:  This function sets the frequency spacing.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorFrequencySpacingValue (ViSession instrSession,
                                                  ViReal64 frequencySpacingValue,
                                                  ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:RAND:SPAC:FREQ",
                              frequencySpacingValue,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Frequency Spacing Value
 * Purpose:  This function returns the frequency spacing.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorFrequencySpacingValue (ViSession instrSession,
                                                  ViReal64 *frequencySpacingValue,
                                                  ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:RAND:SPAC:FREQ?",
                              frequencySpacingValue,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Crest Factor
 * Purpose:  The ratio of peak value to RMS (crest factor) of the signal can
 *           be influenced using the phase of the individual sinewave lines.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorCrestFactor(
    ViSession   instrSession,
    ViInt32 crestFactor
)
{
    if (crestFactor == RSUPV_GEN_CR_FACTOR_VAL)
        CHECKOPT ("B6");
    
    return rsupv_setEnum (instrSession,
                         "SOUR:VOLT:CRES:MODE",
                          crestFactor,
                          genCrestFactorArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Crest Factor
 * Purpose:  This function returns ratio of peak value to RMS (crest factor).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorCrestFactor(
    ViSession   instrSession,
    ViInt32*    crestFactor
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:VOLT:CRES:MODE?",
                          crestFactor,
                          genCrestFactorArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Crest Factor Value
 * Purpose:  This function sets the ratio of peak value to RMS of the signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorCrestFactorValue(
    ViSession   instrSession,
    ViReal64    crestFactor
)
{
    CHECKOPT ("B6");
    
    return rsupv_setViReal64 (instrSession,
                              "SOUR:VOLT:CRES",
                              crestFactor,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Crest Factor Value
 * Purpose:  This function returns the ratio of peak value to RMS of the
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorCrestFactorValue(
    ViSession   instrSession,
    ViReal64*   crestFactor
)
{
    CHECKOPT ("B6");
    
    return rsupv_getViReal64 (instrSession,
                              "SOUR:VOLT:CRES?",
                              crestFactor,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Channel Phase Ratio
 * Purpose:  This function sets phase of Channel 2 relative to Channel 1 (Freq
 *           Mode F&Ph).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorChannelPhaseRatio(
    ViSession   instrSession,
    ViReal64    phaseCh21
)
{
    return rsupv_setViReal64 (instrSession,
                              "SOUR:PHAS",
                              phaseCh21,
                              -360.0, 360.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Channel Phase Ratio
 * Purpose:  This function returns phase of Channel 2 relative to Channel 1
 *           (Freq Mode F&Ph).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorChannelPhaseRatio(
    ViSession   instrSession,
    ViReal64*   phaseCh21
)
{
    return rsupv_getViReal64 (instrSession,
                              "SOUR:PHAS?",
                              phaseCh21,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Channel Voltage Ratio
 * Purpose:  This function sets ratio of the level of both channels relative
 *           to Channel 1 (Volt Mode Ch2/1).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorChannelVoltageRatio(
    ViSession   instrSession,
    ViReal64    voltCh21
)
{
    return rsupv_setViReal64 (instrSession,
                              "SOUR:VOLT:RAT",
                              voltCh21,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Channel Voltage Ratio
 * Purpose:  This function returns ratio of the level of both channels
 *           relative to Channel 1 (Volt Mode Ch2/1).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorChannelVoltageRatio(
    ViSession   instrSession,
    ViReal64*   voltCh21
)
{
    return rsupv_getViReal64 (instrSession,
                              "SOUR:VOLT:RAT?",
                              voltCh21,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Frequency
 * Purpose:  This function sets frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorFrequency (ViSession instrSession,
    ViInt32 channel,
                                               ViReal64 frequency, ViInt32 units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 17, 2);
    CHECKERR (rsupv_checkIndexInList(instrSession, units, unitsArr, VI_ERROR_PARAMETER4));

    sprintf (buffer, "SOUR:FREQ%ld %.12f %s", channel, frequency, unitsArr[units]);

    return rsupv_sendCmd (instrSession, buffer);

}

/*****************************************************************************
 * Function: Get Generator Frequency
 * Purpose:  This function returns frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorFrequency (ViSession instrSession,
    ViInt32 channel,
                                               ViReal64 *frequency,
                                               ViInt32 *units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];
    ViChar  result[25];

    CHECKVIINT32 (channel, 1, 17, 2);

    sprintf (buffer, "SOUR:FREQ%ld?", channel);

    CHECKERR (rsupv_getViString (instrSession,
                              buffer,
                              result,
                              25,
                              VI_ERROR_PARAMETER3));
                              
    sscanf (result, "%le %s", frequency, buffer);
    CHECKERR (rsupv_getIndexFromString (instrSession, buffer, unitsArr, units));
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Generator Voltage
 * Purpose:  This function sets voltage.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorVoltage (ViSession instrSession,
                                             ViInt32 channel, ViReal64 voltage,
                                             ViInt32 units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];

    CHECKERR (rsupv_checkIndexInList(instrSession, units, unitsArr, VI_ERROR_PARAMETER4));

    CHECKVIINT32 (channel, 1, 17, 2);

    sprintf (buffer, "SOUR:VOLT%ld %.12f %s", channel, voltage, unitsArr[units]);

    return rsupv_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Generator Voltage
 * Purpose:  This function returns voltage.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorVoltage (ViSession instrSession,
                                             ViInt32 channel, ViReal64 *voltage,
                                             ViInt32 *units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];
    ViChar  result[25];

    CHECKVIINT32 (channel, 1, 17, 2);

    sprintf (buffer, "SOUR:VOLT%ld?", channel);

    CHECKERR (rsupv_getViString (instrSession,
                              buffer,
                              result,
                              25,
                              VI_ERROR_PARAMETER3));
                              
    sscanf (result, "%le %s", voltage, buffer);
    CHECKERR (rsupv_getIndexFromString (instrSession, buffer, unitsArr, units));
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Generator Total Voltage
 * Purpose:  Used to enter the peak value of the noise signal. The maximum
 *           value depends on the set DC offset. In the analog generator, the
 *           maximum total voltage also depends on the settings for Output Type
 *           and Max Voltage in the Generator Config panel. The peak value and
 *           RMS are interlinked by means of the crest factor. If the noise is
 *           defined in the time domain, the selected distribution function for
 *           amplitude determines the crest factor.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorTotalVoltage (ViSession instrSession,
                                                  ViReal64 voltageValue,
                                                  ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT:TOT",
                              voltageValue,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Total Voltage
 * Purpose:  This function returns total (peak) voltage.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorTotalVoltage (ViSession instrSession,
                                                  ViReal64 *voltageValue,
                                                  ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:VOLT:TOT?",
                              voltageValue,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Voltage RMS
 * Purpose:  This function sets the RMS of the multi-sinewave signal (only in
 *           the analog generator).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorVoltageRMS (ViSession instrSession,
                                                ViReal64 voltageRMS,
                                                ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT:RMS",
                              voltageRMS,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Voltage RMS
 * Purpose:  This function returns the RMS of the multi-sinewave signal (only
 *           in the analog generator).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorVoltageRMS (ViSession instrSession,
                                                ViReal64 *voltageRMS,
                                                ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:VOLT:RMS?",
                              voltageRMS,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Filter
 * Purpose:  This function selects the filter which can be set up in the
 *           analyzer and generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorFilter(
    ViSession   instrSession,
    ViInt32 filter
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:FILT",
                          filter,
                          genFilterArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Filter
 * Purpose:  This function returns the filter which can be set up in the
 *           analyzer and generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorFilter(
    ViSession   instrSession,
    ViInt32*    filter
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:FILT?",
                          filter,
                          genFilterArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Equalizer
 * Purpose:  This function activates/deactivates equalizer function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorEqualizer(
    ViSession   instrSession,
    ViBoolean   equalizer
)
{
    return rsupv_setViBoolean (instrSession,
                               "SOUR:VOLT:EQU:STAT",
                               equalizer,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Equalizer
 * Purpose:  This function returns equalizer function state
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorEqualizer(
    ViSession   instrSession,
    ViBoolean*  equalizer
)
{
    return rsupv_getViBoolean (instrSession,
                               "SOUR:VOLT:EQU:STAT?",
                               equalizer,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Equalizer File
 * Purpose:  This function selects the precorrector file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorEqualizerFile(
    ViSession   instrSession,
    ViString    equalFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:OEQ",
                                    "'",
                                    equalFile,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Equalizer Coupling
 * Purpose:  The equalizer can be selected for both channels together (Track)
 *           or separately from each other (Split).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorEqualizerCoupling(
    ViSession   instrSession,
    ViInt32 equalizer
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:VOLT:EQU:CHAN",
                          equalizer,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Equalizer Coupling
 * Purpose:  This function returns the Equalizer coupling.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorEqualizerCoupling(
    ViSession   instrSession,
    ViInt32*    equalizer
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:VOLT:EQU:CHAN?",
                          equalizer,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);
}        
     
/*****************************************************************************
 * Function: Set Generator DC Offset
 * Purpose:  This function activates/deactivates DC offset function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDCOffset(
    ViSession   instrSession,
    ViBoolean   dcOffset
)
{
    return rsupv_setViBoolean (instrSession,
                               "SOUR:VOLT:OFFS:STAT",
                               dcOffset,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator DC Offset
 * Purpose:  This function returns DC offset function state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDCOffset(
    ViSession   instrSession,
    ViBoolean*  dcOffset
)
{
    return rsupv_getViBoolean (instrSession,
                               "SOUR:VOLT:OFFS:STAT?",
                               dcOffset,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator DC Offset Value
 * Purpose:  This function reduces the maximum possible AC voltage.
 *           A DC offset is superimposed on the generator signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDCOffsetValue(
    ViSession   instrSession,
    ViReal64    dcOffset,
    ViInt32     units
)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT:OFFS",
                              dcOffset,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator DC Offset Value
 * Purpose:  This function returns the maximum possible AC voltage.
 *           A DC offset is superimposed on the generator signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDCOffsetValue (ViSession instrSession,
                                          ViReal64 *DCOffset, ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:VOLT:OFFS?",
                              DCOffset,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator DC Offset Channel Value
 * Purpose:  This function reduces the maximum possible AC voltage.
 *           A DC offset is superimposed on the generator signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDCOffsetChannelValue(
    ViSession   instrSession,
    ViInt32     channel,
    ViReal64    dcOffset,
    ViInt32     units
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "SOUR:VOLT:OFFS%ld", channel);

    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  dcOffset,
                              units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator DC Offset Channel Value 
 * Purpose:  This function returns the maximum possible AC voltage.
 *           A DC offset is superimposed on the generator signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDCOffsetChannelValue (ViSession instrSession,
                    ViInt32 channel, ViReal64 *DCOffset, ViInt32 *units)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "SOUR:VOLT:OFFS%ld?", channel);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  DCOffset,
                            units,
                                  VI_ERROR_PARAMETER3);
}    
     
/*****************************************************************************
 * Function: Set Generator DC Offset Coupling
 * Purpose:  The DC Offset can be selected for both channels together (Track)
 *           or separately from each other (Split).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDCOffsetCoupling(
    ViSession   instrSession,
    ViInt32 DCOffset
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:VOLT:OFFS:CHAN",
                          DCOffset,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator DC Offset Coupling
 * Purpose:  This function returns the Equalizer coupling.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDCOffsetCoupling(
    ViSession   instrSession,
    ViInt32*    DCOffset
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:VOLT:OFFS:CHAN?",
                          DCOffset,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);
}        
        
    
/*****************************************************************************
 * Function: Set Generator Sine Low Distortion
 * Purpose:  In the analog generator, the very low-noise and low-distortion
 *           generator (R&S UPV-B1 option) can be used to generate a sinewave
 *           signal instead of the universal generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSineLowDistortion(
    ViSession   instrSession,
    ViBoolean   lowDist
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViBoolean (instrSession,
                               "SOUR:LOWD",
                               lowDist,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sine Low Distortion
 * Purpose:  This function returns the low-distortion generator state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSineLowDistortion(
    ViSession   instrSession,
    ViBoolean*  lowDist
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViBoolean (instrSession,
                               "SOUR:LOWD?",
                               lowDist,
                               VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Set Generator All Channel Sine
 * Purpose:  This function activates/deactivates channel-based sine wave
 *           state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorAllChannelSine(
    ViSession   instrSession,
    ViBoolean   allChannelSine
)
{
    CHECKINSTR (RSUPV_INSTR_UPP);
    return rsupv_setViBoolean (instrSession,
                               "SOUR:ACHS:STAT",
                               allChannelSine,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator All Channel Sine
 * Purpose:  This function returns the channel-based sine wave state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorAllChannelSine(
    ViSession   instrSession,
    ViBoolean*  allChannelSine
)
{
    CHECKINSTR (RSUPV_INSTR_UPP);
    return rsupv_getViBoolean (instrSession,
                               "SOUR:ACHS:STAT?",
                               allChannelSine,
                               VI_ERROR_PARAMETER2);
}
     
/*****************************************************************************
 * Function: Set Generator Sine Frequency
 * Purpose:  This function sets the frequency of the channel-based sine wave
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSineFrequency(
    ViSession   instrSession,
    ViReal64    sineFrequency,
    ViInt32     units
)
{
    CHECKINSTR (RSUPV_INSTR_UPP);
    return rsupv_setViReal64Unit (instrSession,
                                    "SOUR:ACHS:FREQ",
                                    sineFrequency,
                                    units,
                                    0.0, 0.0,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sine Frequency
 * Purpose:  This function gets the frequency of the channel-based sine wave
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSineFrequency(
    ViSession   instrSession,
    ViReal64*   sineFrequency,
    ViInt32*    units
)
{
    CHECKINSTR (RSUPV_INSTR_UPP);   
    return rsupv_getViReal64Unit (instrSession,
                                    "SOUR:ACHS:FREQ?",
                                    sineFrequency,
                                    units,
                                    VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Set Generator Sine Voltage
 * Purpose:  This function sets the level of the higher-level sine wave.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSineVoltage(
    ViSession   instrSession,
    ViReal64    sineVoltage,
    ViInt32     units
)

{
    CHECKINSTR (RSUPV_INSTR_UPP);

    return rsupv_setViReal64Unit (instrSession,
                                    "SOUR:ACHS:VOLT",
                                    sineVoltage,
                                    units,
                                    0.0, 0.0,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sine Voltage
 * Purpose:  This function gets the level of the higher-level sine wave.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSineVoltage(
    ViSession   instrSession,
    ViReal64*   sineVoltage,
    ViInt32*    units
)
{
        
    CHECKINSTR (RSUPV_INSTR_UPP);
    return rsupv_getViReal64Unit (instrSession,
                                    "SOUR:ACHS:VOLT?",
                                    sineVoltage,
                                    units,
                                    VI_ERROR_PARAMETER2);
}

    
/*****************************************************************************
 * Function: Set Generator Stereo Frequency Mode
 * Purpose:  This function determines the frequency and phase setting for
 *           Channel 1 and 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorStereoFrequencyMode(
    ViSession   instrSession,
    ViInt32 freqMode
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:FREQ:SEL",
                          freqMode,
                          genFreqModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Stereo Frequency Mode
 * Purpose:  This function returns the frequency and phase setting for Channel
 *           1 and 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorStereoFrequencyMode(
    ViSession   instrSession,
    ViInt32*    freqMode
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:FREQ:SEL?",
                          freqMode,
                          genFreqModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Stereo Voltage Mode
 * Purpose:  This function determines the level setting for Channel 1 and 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorStereoVoltageMode(
    ViSession   instrSession,
    ViInt32 voltMode
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:VOLT:SEL",
                          voltMode,
                          genVoltModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Stereo Voltage Mode
 * Purpose:  This function returns the level setting for Channel 1 and 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorStereoVoltageMode(
    ViSession   instrSession,
    ViInt32*    voltMode
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:VOLT:SEL?",
                          voltMode,
                          genVoltModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Stereo Frequency Ch2
 * Purpose:  This function sets frequency of Channel 2 (Freq Mode Ch1&2).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorStereoFrequencyCh2 (ViSession instrSession,
                                               ViReal64 frequencyCh2,
                                               ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:FREQ:CH2S",
                              frequencyCh2,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Stereo Frequency Ch2
 * Purpose:  This function returns frequency of Channel 2 (Freq Mode Ch1&2).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorStereoFrequencyCh2 (ViSession instrSession,
                                               ViReal64 *frequencyCh2,
                                               ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:FREQ:CH2S?",
                              frequencyCh2,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Stereo Voltage Ch2
 * Purpose:  This function sets level of Channel 2 (Volt Mode Ch1&2).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorStereoVoltageCh2 (ViSession instrSession,
                                             ViReal64 voltageCh2, ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT:CH2S",
                              voltageCh2,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Stereo Voltage Ch2
 * Purpose:  This function returns level of Channel 2 (Volt Mode Ch1&2).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorStereoVoltageCh2 (ViSession instrSession,
                                             ViReal64 *voltageCh2,
                                             ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:VOLT:CH2S?",
                              voltageCh2,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Stereo Channel Filter
 * Purpose:  This function selects the channel filter. The filter can be
 *           selected for both channels together (Track) or separately from
 *           each other (Split).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorStereoChannelFilterCoupling(
    ViSession   instrSession,
    ViInt32 filter
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:FILT:CHAN",
                          filter,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Stereo Channel Filter
 * Purpose:  This function returns the channel filter type.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorStereoChannelFilterCoupling(
    ViSession   instrSession,
    ViInt32*    filter
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:FILT:CHAN?",
                          filter,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);
}    
     

/*****************************************************************************
 * Function: Set Generator Stereo Channel Filter
 * Purpose:  This function selects the filter which can be set up in the
 *           analyzer and generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorStereoChannelFilter(
    ViSession   instrSession,
    ViInt32 filter
)
{
    return rsupv_setEnum (instrSession,
                         "SOURce:STEReo2:FILTer",
                          filter,
                          genFilterArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Stereo Channel Filter
 * Purpose:  This function returns the filter which can be set up in the
 *           analyzer and generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorStereoChannelFilter(
    ViSession   instrSession,
    ViInt32*    filter
)
{
    return rsupv_getEnum (instrSession,
                          "SOURce:STEReo2:FILTer?",
                          filter,
                          genFilterArr,
                          VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Set Generator Stereo Equalizer
 * Purpose:  This function activates/deactivates equalizer function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorStereoEqualizer(
    ViSession   instrSession,
    ViBoolean   equalizer
)
{
    return rsupv_setViBoolean (instrSession,
                               "SOURce:VOLTage:EQUalize",
                               equalizer,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Stereo Equalizer
 * Purpose:  This function returns equalizer function state
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorStereoEqualizer(
    ViSession   instrSession,
    ViBoolean*  equalizer
)
{
    return rsupv_getViBoolean (instrSession,
                               "SOURce:VOLTage:EQUalize?",
                               equalizer,
                               VI_ERROR_PARAMETER2);
}   
    

/*****************************************************************************
 * Function: Set Generator Sweep Dwell File
 * Purpose:  This function selects the equalizer file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorStereoEqualizerFile(
    ViSession   instrSession,
    ViString    equalFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    //"MMEMory:LOAD:STEReo2:OEQualize",
                                    "MMEMory:LOAD:OEQualize",
                                    "'",
                                    equalFile,
                                    VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Set Generator Multisine No of Sine
 * Purpose:  This function sets the number of sinewave tones of the
 *           multi-sinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultisineNoOfSine(
    ViSession   instrSession,
    ViInt32 noofSine
)
{
    return rsupv_setViInt32 (instrSession,
                              "SOUR:MULT:COUN",
                              noofSine,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Multisine No of Sine
 * Purpose:  This function returns the number of sinewave tones of the
 *           multi-sinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultisineNoOfSine(
    ViSession   instrSession,
    ViInt32*    noofSine
)
{
    return rsupv_getViInt32 (instrSession,
                              "SOUR:MULT:COUN?",
                              noofSine,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Multisine Phase No i
 * Purpose:  This function sets the start phase of sinewave number of the
 *           multisinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultisinePhaseNoi (ViSession instrSession,
                                              ViInt32 number, ViReal64 phaseNo_i,
                                              ViInt32 units)
{
    ViStatus rsupv_status;
    ViChar  buffer[BUFFER_SIZE];

    CHECKERR (rsupv_checkIndexInList(instrSession, units, unitsArr, VI_ERROR_PARAMETER4));

    sprintf (buffer, "SOUR:PHAS%ld %.12f %s", number, phaseNo_i, unitsArr[units]);

    return rsupv_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Generator Multisine Phase No i
 * Purpose:  This function returns the start phase of sinewave number of the
 *           multisinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultisinePhaseNoi (ViSession instrSession,
    ViInt32 number,
                                              ViReal64 *phaseNo_i,
                                              ViInt32 *units)
{
    ViChar  buffer[BUFFER_SIZE];

    sprintf (buffer, "SOUR:PHAS%ld?", number);

    return rsupv_getViReal64Unit (instrSession,
                              buffer,
                              phaseNo_i,
                              units,
                              VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Generator Multisine Total Gain
 * Purpose:  This function sets a gain factor which can be used to vary the
 *           total level of the multi-sinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultisineTotalGain(
    ViSession   instrSession,
    ViReal64    totalGain
)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT:TOT:GAIN",
                              totalGain,
                              RSUPV_UNIT_DB,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Multisine Total Gain
 * Purpose:  This function returns a gain factor which can be used to vary the
 *           total level of the multi-sinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultisineTotalGain(
    ViSession   instrSession,
    ViReal64*   totalGain
)
{
    return rsupv_getViReal64 (instrSession,
                              "SOUR:VOLT:TOT:GAIN?",
                              totalGain,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sine Burst Low Level
 * Purpose:  This function sets the level of the sinewave during the burst
 *           pause.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSineBurstLowLevel (ViSession instrSession,
                                              ViReal64 lowLevel, ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT:LOWL",
                              lowLevel,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sine Burst Low Level
 * Purpose:  This function returns the level of the sinewave during the burst
 *           pause.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSineBurstLowLevel (ViSession instrSession,
                                              ViReal64 *lowLevel, ViInt32 *units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:VOLT:LOWL?",
                              lowLevel,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sine Burst On Time
 * Purpose:  This function sets the burst duration.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSineBurstOnTime (ViSession instrSession,
                                            ViReal64 onTime, ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:ONT",
                              onTime,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sine Burst On Time
 * Purpose:  This function returns the burst duration.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSineBurstOnTime (ViSession instrSession,
                                            ViReal64 *onTime, ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:ONT?",
                              onTime,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sine Burst Interval
 * Purpose:  This function sets the repetition rate of the burst signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSineBurstInterval (ViSession instrSession,
                                              ViReal64 interval, ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:INT",
                              interval,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sine Burst Interval
 * Purpose:  This function returns the repetition rate of the burst signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSineBurstInterval (ViSession instrSession,
                                              ViReal64 *interval, ViInt32 *units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:INT?",
                              interval,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Sine Burst on Delay
 * Purpose:  This function sets the delay time which must expire (after
 *           generator settings have been changed or a measurement has been
 *           started) before the generator can start the burst with the high
 *           level (Voltage).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSineBurstOnDelay (ViSession instrSession,
                                             ViReal64 burst_onDelay,
                                             ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:ONT:DEL",
                              burst_onDelay,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Sine Burst on Delay
 * Purpose:  This function returns the delay time which must expire (after
 *           generator settings have been changed or a measurement has been
 *           started) before the generator can start the burst with the high
 *           level (Voltage).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSineBurstOnDelay (ViSession instrSession,
                                             ViReal64 *burst_onDelay,
                                             ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:ONT:DEL?",
                              burst_onDelay,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Mod Dist Upper Frequency
 * Purpose:  This function sets the frequency of the high-frequency useful
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorModDistUpperFrequency (ViSession instrSession,
                                                  ViReal64 upperFrequency,
                                                  ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:FREQ1",
                              upperFrequency,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Mod Dist Upper Frequency
 * Purpose:  This function returns the frequency of the high-frequency useful
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorModDistUpperFrequency (ViSession instrSession,
                                                  ViReal64 *upperFrequency,
                                                  ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:FREQ1?",
                              upperFrequency,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Mod Dist Lower Frequency
 * Purpose:  This function sets the frequency of the low-frequency noise
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorModDistLowerFrequency (ViSession instrSession,
                                                  ViReal64 lowerFrequency,
                                                  ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:FREQ2",
                              lowerFrequency,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Mod Dist Lower Frequency
 * Purpose:  This function returns the frequency of the low-frequency noise
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorModDistLowerFrequency (ViSession instrSession,
                                                  ViReal64 *lowerFrequency,
                                                  ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:FREQ2?",
                              lowerFrequency,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Mod Dist Level Ratio
 * Purpose:  This function sets the level ratio of noise signal to useful
 *           signal as a dimension-less number.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorModDistLevelRatio(
    ViSession   instrSession,
    ViReal64    voltLFUF
)
{
    return rsupv_setViReal64 (instrSession,
                              "SOUR:VOLT:RAT",
                              voltLFUF,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Mod Dist Level Ratio
 * Purpose:  This function returns the level ratio of noise signal to useful
 *           signal as a dimension-less number.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorModDistLevelRatio(
    ViSession   instrSession,
    ViReal64*   voltLFUF
)
{
    return rsupv_getViReal64 (instrSession,
                              "SOUR:VOLT:RAT?",
                              voltLFUF,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Modulation Carrier Voltage
 * Purpose:  This function sets the level of the unmodulated sinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorModulationCarrierVoltage (ViSession instrSession,
                                                     ViReal64 carrierVoltage,
                                                     ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT2",
                              carrierVoltage,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Modulation Carrier Voltage
 * Purpose:  This function returns the level of the unmodulated sinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorModulationCarrierVoltage (ViSession instrSession,
                                                     ViReal64 *carrierVoltage,
                                                     ViInt32 *units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:VOLT2?",
                              carrierVoltage,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator DFD Mean Frequency
 * Purpose:  This function sets the center frequency (in the IEC 268 mode).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDFDMeanFrequency (ViSession instrSession,
                                             ViReal64 meanFrequency,
                                             ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:FREQ:MEAN",
                              meanFrequency,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator DFD Mean Frequency
 * Purpose:  This function returns the center frequency (in the IEC 268 mode).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDFDMeanFrequency (ViSession instrSession,
                                             ViReal64 *meanFrequency,
                                             ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:FREQ:MEAN?",
                              meanFrequency,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator DFD Diff Frequency
 * Purpose:  This function sets the frequency difference of the two sinewaves.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDFDDiffFrequency (ViSession instrSession,
                                             ViReal64 diffFrequency,
                                             ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:FREQ:DIFF",
                              diffFrequency,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator DFD Diff Frequency
 * Purpose:  This function returns the frequency difference of the two
 *           sinewaves.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDFDDiffFrequency (ViSession instrSession,
                                             ViReal64 *diffFrequency,
                                             ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:FREQ:DIFF?",
                              diffFrequency,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator DIM Square to Sine
 * Purpose:  This function selects the ratio between the frequencies of the
 *           squarewave signal and sinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDIMSquareToSine(
    ViSession   instrSession,
    ViInt32 squareSine
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "SOUR:DIM",
                          squareSine,
                          genSquareSineArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator DIM Square to Sine
 * Purpose:  This function returns the ratio between the frequencies of the
 *           squarewave signal and sinewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDIMSquareToSine(
    ViSession   instrSession,
    ViInt32*    squareSine
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SOUR:DIM?",
                          squareSine,
                          genSquareSineArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator DIM Bandwidth
 * Purpose:  This function selects the bandwidth (3 dB) of the first-order
 *           filter in order to limit the bandwidth of the squarewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorDIMBandwidth(
    ViSession   instrSession,
    ViInt32 bandwidth
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "SOUR:BAND",
                          bandwidth,
                          genDIMBandwidthArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator DIM Bandwidth
 * Purpose:  This function returns the bandwidth (3 dB) of the first-order
 *           filter in order to limit the bandwidth of the squarewave signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorDIMBandwidth(
    ViSession   instrSession,
    ViInt32*    bandwidth
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SOUR:BAND?",
                          bandwidth,
                          genDIMBandwidthArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Generator FM Modulation
 * Purpose:  This function generates a sinewave signal which can be modulated
 *           with respect to amplitude according to a sinusoidal function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureGeneratorAMModulation (ViSession instrSession,
                                                        ViReal64 modFreq,
                                               ViInt32 modFreqUnits,
                                                        ViReal64 carrierFreq,
                                               ViInt32 carrierFreqUnits,
                                               ViReal64 modDepth,
                                               ViReal64 carrierVolt,
                                               ViInt32 carrierVoltUnits)
{
    ViStatus    rsupv_status;

    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKERR (rsupv_SetGeneratorFunctionMode (instrSession, RSUPV_GEN_MODE_AM));

    CHECKERR (rsupv_SetGeneratorModulationFrequency(instrSession, modFreq, modFreqUnits));

    rsupv_status = rsupv_SetGeneratorModulationCarrierFrequency(instrSession, carrierFreq, carrierFreqUnits);
    REMAP (2,4);
    REMAP (3,5);

    rsupv_SetGeneratorModulationDepth(instrSession, modDepth);
    REMAP (2,6);

    rsupv_status = rsupv_SetGeneratorModulationCarrierVoltage(instrSession, carrierVolt, carrierVoltUnits);
    REMAP (2,7);
    REMAP (3,8);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Generator Modulation Frequency
 * Purpose:  This function sets the modulation frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorModulationFrequency (ViSession instrSession,
                                                ViReal64 modulationFrequency,
                                                ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:FREQ",
                              modulationFrequency,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Modulation Frequency
 * Purpose:  This function returns the modulation frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorModulationFrequency (ViSession instrSession,
                                                ViReal64 *modulationFrequency,
                                                ViInt32 *units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:FREQ?",
                              modulationFrequency,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Modulation Carrier Frequency
 * Purpose:  This function sets the carrier frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorModulationCarrierFrequency (ViSession instrSession,
                                                       ViReal64 carrierFrequency,
                                                       ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:FREQ2",
                              carrierFrequency,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Modulation Carrier Frequency
 * Purpose:  This function returns the carrier frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorModulationCarrierFrequency(
    ViSession   instrSession,
    ViReal64*   carrierFreq,
    ViInt32*    units
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:FREQ2?",
                              carrierFreq,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Modulation Deviation
 * Purpose:  With FM, this function sets the modulation deviation in %
 *           relative to the carrier frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorModulationDeviation (ViSession instrSession,
                                                ViReal64 deviation)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT1",
                              deviation,
                              RSUPV_UNIT_PCT,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Modulation Deviation
 * Purpose:  With FM, this function returns the modulation deviation in %
 *           relative to the carrier frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorModulationDeviation (ViSession instrSession,
                                                ViReal64 *deviation)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SOUR:VOLT1?",
                              deviation,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Modulation Depth
 * Purpose:  With AM, this function sets the modulation depth in % relative to
 *           the carrier amplitude.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorModulationDepth (ViSession instrSession,
                                            ViReal64 modulationDepth)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:VOLT1",
                              modulationDepth,
                              RSUPV_UNIT_PCT,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Modulation Depth
 * Purpose:  With AM, this function returns the modulation depth in % relative
 *           to the carrier amplitude.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorModulationDepth (ViSession instrSession,
                                            ViReal64 *modulationDepth)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SOUR:VOLT1?",
                              modulationDepth,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Random Domain
 * Purpose:  This function selects characteristics of the noise signal defined
 *           in the frequency or time domain.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorRandomDomain(
    ViSession   instrSession,
    ViInt32 domain
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "SOUR:RAND:DOM",
                          domain,
                          genDomainArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Random Domain
 * Purpose:  This function returns characteristics of the noise signal defined
 *           in the frequency or time domain.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorRandomDomain(
    ViSession   instrSession,
    ViInt32*    domain
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SOUR:RAND:DOM?",
                          domain,
                          genDomainArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Random Shape
 * Purpose:  This function defines the amplitude distribution of the signal
 *           spectrum within a selectable frequency domain.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorRandomShape(
    ViSession   instrSession,
    ViInt32 shape
)
{
    return rsupv_setEnum (instrSession,
                         "SOUR:RAND:SHAP",
                          shape,
                          genShapeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Random Shape
 * Purpose:  This function returns the amplitude distribution of the signal
 *           spectrum within a selectable frequency domain.
 *
 *           Remote-control command(s):
 *           SOURce:RANDom:SHAPe?
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorRandomShape(
    ViSession   instrSession,
    ViInt32*    shape
)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:RAND:SHAP?",
                          shape,
                          genShapeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Random Shape File
 * Purpose:  This function selects file containing the amplitude distribution.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorRandomShapeFile(
    ViSession   instrSession,
    ViString    shapeFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:LIST ARB,",
                                    "'",
                                    shapeFile,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Random Lower Frequency
 * Purpose:  This function defines the lower frequency limit of the noise
 *           signal if Shape is set to White or Pink.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorRandomLowerFrequency (ViSession instrSession,
                                                 ViReal64 lowerFrequency,
                                                 ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:RAND:FREQ:LOW",
                              lowerFrequency,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Random Lower Frequency
 * Purpose:  This function returns the lower frequency limit of the noise
 *           signal if Shape is set to White or Pink.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorRandomLowerFrequency (ViSession instrSession,
                                                 ViReal64 *lowerFrequency,
                                                 ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:RAND:FREQ:LOW?",
                              lowerFrequency,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Random Upper Frequency
 * Purpose:  This function defines the upper frequency limit of the noise
 *           signal if Shape is set to White or Pink.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorRandomUpperFrequency (ViSession instrSession,
                                                 ViReal64 upperFrequency,
                                                 ViInt32 units)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:RAND:FREQ:UPP",
                              upperFrequency,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Random Upper Frequency
 * Purpose:  This function returns the upper frequency limit of the noise
 *           signal if Shape is set to White or Pink.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorRandomUpperFrequency (ViSession instrSession,
                                                 ViReal64 *upperFrequency,
                                                 ViInt32 *units)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:RAND:FREQ:UPP?",
                              upperFrequency,
                              units,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Arbitrary Shape File
 * Purpose:  This function selects file containing the amplitude distribution.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorArbitraryShapeFile(
    ViSession   instrSession,
    ViString    shapeFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:LIST ARB,",
                                    "'",
                                    shapeFile,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Generator Amplitude Variation
 * Purpose:  This function selects the amplitude variation mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorAmplitudeVariation (ViSession instrSession,
                                                        ViInt32 amplitudeVariation)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "SOUR:AM:MODE",
                          amplitudeVariation,
                          genAmplitudeVariationArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Generator Amplitude Variation
 * Purpose:  This function returns the amplitude variation mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorAmplitudeVariation (ViSession instrSession,
                                                        ViInt32 *amplitudeVariation)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                         "SOUR:AM:MODE?",
                          amplitudeVariation,
                          genAmplitudeVariationArr,
                          VI_ERROR_PARAMETER2);     
}

/*****************************************************************************
 * Function: Set Generator AV Modulation Frequency
 * Purpose:  This function sets the modulation frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorAVModulationFrequency (ViSession instrSession,
                                                           ViReal64 modulationFrequency,
                                                           ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                              "SOUR:FREQ:AM",
                              modulationFrequency,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Generator AV Modulation Frequency
 * Purpose:  This function gets the modulation frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorAVModulationFrequency (ViSession instrSession,
                                                           ViReal64 *modulationFrequency,
                                                           ViInt32 *units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                              "SOUR:FREQ:AM?",
                              modulationFrequency,
                              units,
                              VI_ERROR_PARAMETER2);     
}

/*****************************************************************************
 * Function: Set Generator Variation
 * Purpose:  This function sets the value in % by which the amplitude of the 
 *           signal is reduced.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorVariation (ViSession instrSession,
                                               ViReal64 variation)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64 (instrSession,
                              "SOUR:VOLT:AM",
                              variation,
                              -100.0, 0.0,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Generator Variation
 * Purpose:  This function gets the value in % by which the amplitude of the 
 *           signal is reduced.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorVariation (ViSession instrSession,
                                               ViReal64 *variation)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SOUR:VOLT:AM?",
                              variation,
                              VI_ERROR_PARAMETER2);    
}
 
      
/*****************************************************************************
 * Function: Set Generator Multichannel Filter
 * Purpose:  This function selects the filter which can be set up in the
 *           analyzer and generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelFilter(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 filter
)
{
    ViChar  buffer[BUFFER_SIZE];
    
    CHECKINSTR (RSUPV_INSTR_UPP);

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:FILT", channel);


    return rsupv_setEnum (instrSession,
                          buffer,
                          filter,
                          genFilterArr,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel Filter
 * Purpose:  This function returns the filter which can be set up in the
 *           analyzer and generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelFilter(
    ViSession   instrSession,
        ViInt32 channel,
    ViInt32*    filter
)
{
    ViChar  buffer[BUFFER_SIZE];
    
    CHECKINSTR (RSUPV_INSTR_UPP);

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:FILT?", channel);


    return rsupv_getEnum (instrSession,
                          buffer,
                          filter,
                          genFilterArr,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Generator Multichannel All Chan Sine
 * Purpose:  This function activates/deactivates All Channel Settings
 *           function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelAllChanSine(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean allChanSine
)
{
    ViChar  buffer[BUFFER_SIZE];
       
    CHECKINSTR (RSUPV_INSTR_UPP);

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:ACHS:STAT", channel);


    return rsupv_setViBoolean (instrSession,
                          buffer,
                          allChanSine,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel All Chan Sine
 * Purpose:  This control returns all channel settings function state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelAllChanSine(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean *allChanSine
)
{
    ViChar  buffer[BUFFER_SIZE];
    
    CHECKINSTR (RSUPV_INSTR_UPP);

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:ACHS:STAT?", channel);


    return rsupv_getViBoolean (instrSession,
                          buffer,
                          allChanSine,
                          VI_ERROR_PARAMETER3);
}


/*****************************************************************************
 * Function: Set Generator Multichannel Gain
 * Purpose:  This function sets a percentage from the level of the
 *           higher-level sine wave is added to the overall signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelGain(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 gain
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP); 
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:GAIN", channel);


    return rsupv_setViReal64 (instrSession,
                                buffer,
                                gain,
                                0.0, 100.0,
                                VI_ERROR_PARAMETER3); 
}


/*****************************************************************************
 * Function: Get Generator Multichannel Gain
 * Purpose:  This function gets a percentage from the level of the
 *           higher-level sine wave.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelGain(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 *gain
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKINSTR (RSUPV_INSTR_UPP); 
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:GAIN?", channel);


    return rsupv_getViReal64 (instrSession,
                                buffer,
                                gain,
                                VI_ERROR_PARAMETER3); 
}


/*****************************************************************************
 * Function: Set Generator Multichannel Limit To FS
 * Purpose:  This function activates/deactivates limit overall signal to a
 *           maximum of 1 FS.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelLimitToFS(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean limitToFS
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKINSTR (RSUPV_INSTR_UPP);   
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:LIM", channel);


    return rsupv_setViBoolean (instrSession,
                          buffer,
                          limitToFS,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel Limit To FS
 * Purpose: This function returns limit overall signal state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelLimitToFS(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean *limitToFS
)
{
    ViChar  buffer[BUFFER_SIZE];
    
    CHECKINSTR (RSUPV_INSTR_UPP); 
    
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:LIM?", channel);


    return rsupv_getViBoolean (instrSession,
                          buffer,
                          limitToFS,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Generator Multichannel Total Gain
 * Purpose:  This function sets the gain upper limit. Value is calculated so
 *           that the level of the overallsignal cannot exceed 1 FS.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelTotalGain(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 totalGain
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP); 
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:TOT:GAIN", channel);


    return rsupv_setViReal64 (instrSession,
                                buffer,
                                totalGain,
                                0.0, 100.0,
                                VI_ERROR_PARAMETER3); 
}


/*****************************************************************************
 * Function: Get Generator Multichannel Total Gain
 * Purpose:  This function gets the upper limit gain value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelTotalGain(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 *totalGain
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP); 
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:TOT:GAIN?", channel);


    return rsupv_getViReal64 (instrSession,
                                buffer,
                                totalGain,
                                VI_ERROR_PARAMETER3); 
}


/*****************************************************************************
 * Function: Set Generator Multichannel Sine
 * Purpose:  This function activates/deactivates channel-based sine wave
 *           state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelSine(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean addToChannel
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 8, 2);
    CHECKINSTR (RSUPV_INSTR_UPP); 
    
    sprintf (buffer, "SOUR:MCH%ld:SINE:STAT", channel);


    return rsupv_setViBoolean (instrSession,
                          buffer,
                          addToChannel,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel Sine
 * Purpose: This function returns channel-based sine wave signal state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelSine(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean *addToChannel
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP); 
    
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:STAT?", channel);


    return rsupv_getViBoolean (instrSession,
                          buffer,
                          addToChannel,
                          VI_ERROR_PARAMETER3);
}


    
/*****************************************************************************
 * Function: Set Generator Multichannel Sine Frequency
 * Purpose:  This function sets the frequency of the channel-based sine wave
 *           signal
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelSineFrequency(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 sineFrequency,
    ViInt32     units 
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:FREQ", channel);


    return rsupv_setViReal64Unit (instrSession,
                                buffer,
                                sineFrequency,
                                units,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel Sine Frequency
 * Purpose:  This function gets the frequency of the channel-based sine wave
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelSineFrequency(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 *sineFrequency,
    ViInt32*    units
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:FREQ?", channel);


    return rsupv_getViReal64Unit (instrSession,
                                buffer,
                                sineFrequency,
                                units,
                                VI_ERROR_PARAMETER3); 
}   
  
/*****************************************************************************
 * Function: Set Generator Multichannel Sine Phase
 * Purpose:  This function sets the phase of the channel-based sine wave
 *           signal
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelSinePhase(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 sinePhase,
    ViInt32 units  
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP); 
    
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:PHAS", channel);


    return rsupv_setViReal64Unit (instrSession,
                                buffer,
                                sinePhase,
                                units,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel Sine Phase
 * Purpose:  This function gets the phase of the channel-based sine wave
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelSinePhase(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 *sinePhase,
    ViInt32 *units
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP); 
    
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:PHAS?", channel);


    return rsupv_getViReal64Unit (instrSession,
                                buffer,
                                sinePhase,
                                units,
                                VI_ERROR_PARAMETER3); 
}   


/*****************************************************************************
 * Function: Set Generator Multichannel Sine Voltage
 * Purpose:  This function sets the level of the channel-based sine wave signal
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelSineVoltage(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 sineVoltage,
    ViInt32  units 
)
{
    ViChar  buffer[BUFFER_SIZE];
        
    CHECKINSTR (RSUPV_INSTR_UPP); 
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:VOLT", channel);


    return rsupv_setViReal64Unit (instrSession,
                                buffer,
                                sineVoltage,
                                units,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel Sine Voltage
 * Purpose:  This function gets the level of the channel-based sine wave signal
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelSineVoltage(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64 *sineVoltage,
     ViInt32 *units
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 8, 2);
    CHECKINSTR (RSUPV_INSTR_UPP); 
    
    sprintf (buffer, "SOUR:MCH%ld:SINE:VOLT?", channel);


    return rsupv_getViReal64Unit (instrSession,
                                buffer,
                                sineVoltage,
                                units,
                                VI_ERROR_PARAMETER3); 
}   

/*****************************************************************************
 * Function: Set Generator Multichannel Sine Arbitrary
 * Purpose:  This function activates/deactivates channel-based sine arbitrary
 *           state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelSineArbitrary(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean arbitrary
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP); 
        
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:ARB:STAT", channel);


    return rsupv_setViBoolean (instrSession,
                          buffer,
                          arbitrary,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel Sine Arbitrary
 * Purpose:  This function returns arbitrary settings state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelSineArbitrary(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean *arbitrary
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:ARB:STAT?", channel);


    return rsupv_getViBoolean (instrSession,
                          buffer,
                          arbitrary,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Generator Multichannel Sine Volt Peak
 * Purpose:  A signal with random frequency and shape and the level specified
 *           in "Volt Peak" is added to the overall signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelSineVoltPeak(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64    voltPeak,
    ViInt32     units
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:ARB:VOLT", channel);

    
    return rsupv_setViReal64Unit (instrSession,
                                buffer,
                                voltPeak,
                                units,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel Sine Volt Peak
 * Purpose:  This function returns the "Volt Peak" value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelSineVoltPeak (ViSession instrSession,
        ViInt32 channel, ViReal64 *voltPeak, ViInt32 *units)
{
    
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:ARB:VOLT?", channel);
    
    return rsupv_getViReal64Unit (instrSession,
                              buffer,
                              voltPeak,
                              units,
                              VI_ERROR_PARAMETER3);
}


/*****************************************************************************
 * Function: Set Generator Multichannel Sine Arbitrary Filename
 * Purpose:  This function selects the Arbitrary waveform file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelSineArbitraryFilename (ViSession instrSession,
                                            ViInt32 channel, ViString arbitraryFile)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP); 

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "MMEM:LOAD:MCH%ld:SINE:ARB", channel);
    
    return rsupv_setViStringQuoted (instrSession,
                              buffer,
                              "'",
                              arbitraryFile,
                              VI_ERROR_PARAMETER3);
}
 
/*****************************************************************************
 * Function: Set Generator Multichannel Sine Equalizer
 * Purpose:  This function activates/deactivates equalizer function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelSineEqualizer(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean equalizer
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:EQU:STAT", channel);


    return rsupv_setViBoolean (instrSession,
                          buffer,
                          equalizer,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel Sine Equalizer
 * Purpose:  This function returns the equalizer function state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelSineEqualizer(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean *equalizer
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:EQU:STAT?", channel);


    return rsupv_getViBoolean (instrSession,
                          buffer,
                          equalizer,
                          VI_ERROR_PARAMETER3);
}

     
/*****************************************************************************
 * Function: Set Generator Multichannel Sine Equalizer Filename
 * Purpose:  This function selects the equalization file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelSineEqualizerFilename (ViSession instrSession,
                                            ViInt32 channel, ViString equalFile)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 8, 2);
    CHECKINSTR (RSUPV_INSTR_UPP);   

    sprintf (buffer, "MMEM:LOAD:MCH%ld:SINE:EQU", channel);
    
    return rsupv_setViStringQuoted (instrSession,
                              buffer,
                              "'",
                              equalFile,
                              VI_ERROR_PARAMETER3);
}
     

/*****************************************************************************
 * Function: Set Generator Multichannel DC Offset
 * Purpose:  This function activates/deactivates DC offset function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelDCOffset(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean dcOffset
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 8, 2);
    CHECKINSTR (RSUPV_INSTR_UPP); 

    sprintf (buffer, "SOUR:MCH%ld:SINE:OFFS:STAT", channel);


    return rsupv_setViBoolean (instrSession,
                          buffer,
                          dcOffset,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel DC Offset 
 * Purpose: This function returns DC offset function state.  
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelDCOffset(
    ViSession   instrSession,
    ViInt32 channel,
    ViBoolean *dcOffset
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:OFFS:STAT?", channel);


    return rsupv_getViBoolean (instrSession,
                          buffer,
                          dcOffset,
                          VI_ERROR_PARAMETER3);
}   
    
/*****************************************************************************
 * Function: Set Generator Multichannel DC Offset Value
 * Purpose:  This function sets DC offset on the generator signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorMultichannelDCOffsetValue(
    ViSession   instrSession,
    ViInt32 channel,
    ViReal64    dcOffset,
    ViInt32     units
)
{
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:OFFS:VOLT", channel);

    
    return rsupv_setViReal64Unit (instrSession,
                                buffer,
                                dcOffset,
                                units,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Generator Multichannel DC Offset Value
 * Purpose:  This function gets DC offset on the generator signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorMultichannelDCOffsetValue (ViSession instrSession,
        ViInt32 channel, ViReal64 *DCOffset, ViInt32 *units)
{
    
    ViChar  buffer[BUFFER_SIZE];
    CHECKINSTR (RSUPV_INSTR_UPP);   

    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "SOUR:MCH%ld:SINE:OFFS:VOLT?", channel);
    
    return rsupv_getViReal64Unit (instrSession,
                              buffer,
                              DCOffset,
                              units,
                              VI_ERROR_PARAMETER3);
}


/*****************************************************************************
 * Function: Set Generator Single Channel Track To Other Channel
 * Purpose:  This function activates/deactivates Track to other channels
 *           state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSingleChannelTrackToOtherChannel(
    ViSession   instrSession,
    ViBoolean   trackToOtherChannel
)
{
        
    CHECKINSTR (RSUPV_INSTR_UPP); 
    return rsupv_setViBoolean (instrSession,
                               "SOUR:SCHS:TTOC",
                               trackToOtherChannel,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Generator Single Channel Track To Other Channel
 * Purpose:  This function returns the Track to other channels state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSingleChannelTrackToOtherChannel(
    ViSession   instrSession,
    ViBoolean*  trackToOtherChannel
)
{
    CHECKINSTR (RSUPV_INSTR_UPP); 
        
    return rsupv_getViBoolean (instrSession,
                               "SOUR:SCHS:TTOC?",
                               trackToOtherChannel,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Generator Single Channel
 * Purpose:  This function sets the channel on the generator signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGeneratorSingleChannel(
    ViSession   instrSession,
    ViInt32     channel
)
{
    CHECKINSTR (RSUPV_INSTR_UPP); 
    return rsupv_setViInt32 (instrSession,
                            "SOUR:SCHS:CHAN",
                            channel,
                            1, 8,
                            VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Generator Single Channel
 * Purpose:  This function gets the channel on the generator signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetGeneratorSingleChannel(
    ViSession   instrSession,
    ViInt32*    channel
)
{
    CHECKINSTR (RSUPV_INSTR_UPP); 
    
    return rsupv_getViInt32 (instrSession,
                            "SOUR:SCHS:CHAN?",
                            channel,
                            VI_ERROR_PARAMETER2);
}


/*############################################################################
/*                               ANALYZER
/*############################################################################

/*****************************************************************************
 * Function: Select Analyzer Instrument
 * Purpose:  This function selects analyzer instrument type.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerInstrument(
    ViSession   instrSession,
    ViInt32     instrument
)
{
    ViBoolean is_upp =  rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPP) == VI_SUCCESS;
    switch (instrument)
    {
        case RSUPV_ANALYZER_I2S:
            if (is_upp)
            {
                if (!rsupv_checkOption (instrSession, "B4") && !rsupv_checkOption (instrSession, "B2"))
                    return RSUPV_ERROR_INSTRUMENT_OPTION;
            }
            else
            {
            CHECKOPT ("B41");
            }
            break;
        case RSUPV_ANALYZER_A16C:
            CHECKOPT ("B48");
            break;
        case RSUPV_ANALYZER_A8C:
            if (!is_upp)
            {
            CHECKOPT ("B48");
            }
            break;
        case RSUPV_ANALYZER_U8C:
        case RSUPV_ANALYZER_U2C:
            CHECKOPT ("B42");
            break;
        case RSUPV_ANALYZER_ANALOG:
        case RSUPV_ANALYZER_DIGBITS:
            break;

        case RSUPV_ANALYZER_HDMI:
            break;          
    }

    return rsupv_setEnum (instrSession,
                     "INST2:SEL",
                      instrument,
                      anInstrumentArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Instrument
 * Purpose:  This function returns analyzer instrument type.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerInstrument(
    ViSession   instrSession,
    ViInt32*    instrument
)
{
    return rsupv_getEnum (instrSession,
                     "INST2:SEL?",
                      instrument,
                      anInstrumentArr,
                      VI_ERROR_PARAMETER2);
}

/*############################################################################
/*                              I2S ANALYZER
/*############################################################################

/*****************************************************************************
 * Function: Set I2S Analyzer Word Length
 * Purpose:  This function sets the word length of a data frame for each channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SAnalyzerWordLength (ViSession instrSession,
                                                  ViInt32 wordLength)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                     "INP:WORD",
                      wordLength,
                      I2SWordLengthArr,
                      VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Analyzer Word Length
 * Purpose:  This function gets the word length of a data frame for each channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SAnalyzerWordLength (ViSession instrSession,
                                                  ViInt32 *wordLength)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                     "INP:WORD?",
                      wordLength,
                      I2SWordLengthArr,
                      VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Analyzer Format
 * Purpose:  This function selects the I2S analyzer format.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SAnalyzerFormat (ViSession instrSession,
                                              ViInt32 format)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                     "INP:FORM",
                      format,
                      I2SFormatArr,
                      VI_ERROR_PARAMETER2);     
}

/*****************************************************************************
 * Function: Get I2S Analyzer Format
 * Purpose:  This function returns the I2S analyzer format.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SAnalyzerFormat (ViSession instrSession,
                                              ViInt32 *format)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                     "INP:FORM?",
                      format,
                      I2SFormatArr,
                      VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Analyzer Fsync Slope
 * Purpose:  This function selects the polarity of the Fsync signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SAnalyzerFsyncSlope (ViSession instrSession,
                                                  ViInt32 fsyncSlope)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                     "INP:FSL",
                      fsyncSlope,
                      I2SFsyncSlopeArr,
                      VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Analyzer Fsync Slope
 * Purpose:  This function returns the polarity of the Fsync signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SAnalyzerFsyncSlope (ViSession instrSession,
                                                  ViInt32 *fsyncSlope)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                     "INP:FSL?",
                      fsyncSlope,
                      I2SFsyncSlopeArr,
                      VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Analyzer Word Offset
 * Purpose:  This function sets the number of the SCK cycles by which the 
 *           Fsync signal is analyzed with delay with respect to the audio
 *           data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SAnalyzerWordOffset (ViSession instrSession,
                                                  ViInt32 wordOffset)
{
    ViStatus rsupv_status;
    ViInt32 min, max, word_length;
    
    CHECKOPT ("B41");
    
    rsupv_status = rsupv_GetI2SAnalyzerWordLength (instrSession, &word_length);
    
    switch (word_length)
    {
        case RSUPV_ANALYZER_I2S_W16:
            min = -16; max = 15;
        break;
        case RSUPV_ANALYZER_I2S_W24:
            min = -24; max = 23;    
        break;
        default:
            min = -32; max = 31;
    }
    
    return rsupv_setViInt32 (instrSession,
                              "INP:WRDO",
                              wordOffset,
                              min, max,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Analyzer Word Offset
 * Purpose:  This function gets the number of the SCK cycles by which the 
 *           Fsync signal is analyzed with delay with respect to the audio
 *           data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SAnalyzerWordOffset (ViSession instrSession,
                                                  ViInt32 *wordOffset)
{
    CHECKOPT ("B41");
    
    return rsupv_getViInt32 (instrSession,
                              "INP:WRDO?",
                              wordOffset,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set I2S Analyzer Input
 * Purpose:  This function selects the data source for the I2S audio analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SAnalyzerInput (ViSession instrSession,
                                             ViInt32 input)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                     "INP:INP",
                      input,
                      I2SInputArr,
                      VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get I2S Analyzer Input
 * Purpose:  This function returns the data source for the I2S audio analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SAnalyzerInput (ViSession instrSession,
                                             ViInt32 *input)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                     "INP:INP?",
                      input,
                      I2SInputArr,
                      VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Configure USI Analyzer Clock And Frequency
 * Purpose:  This function configures clock and mixed sample frequency of USI 
 *           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIAnalyzerClockAndFrequency (ViSession instrSession,
                                                               ViInt32 clock,
                                                               ViBoolean mixedSamplingFrequency,
                                                               ViInt32 ratio)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIAnalyzerClock (instrSession, clock));
    
    rsupv_status = rsupv_SetUSIAnalyzerMixedSamplingFrequencyState (instrSession, mixedSamplingFrequency);
    REMAP (2,3);
    
    if (mixedSamplingFrequency)
    {
        rsupv_status = rsupv_SetUSIAnalyzerRatio (instrSession, ratio);
        REMAP (2,4);
    }
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure USI Analyzer Frame
 * Purpose:  This function configures analyzer frame parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIAnalyzerFrame (ViSession instrSession,
                                                   ViInt32 samples,
                                                   ViInt32 numberOfSlots)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIAnalyzerSamplesPerFrame (instrSession, samples));
    
    rsupv_status = rsupv_SetUSIAnalyzerNumberOfSlots (instrSession, numberOfSlots);
    REMAP (2,3);
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure USI Analyzer Slot
 * Purpose:  This function configures slot layout. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIAnalyzerSlot (ViSession instrSession,
                                                  ViInt32 firstBit,
                                                  ViInt32 slotLength,
                                                  ViInt32 audioBits,
                                                  ViInt32 leadBits)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIAnalyzerFirstBit (instrSession, firstBit));
    
    rsupv_status = rsupv_SetUSIAnalyzerSlotLength (instrSession, slotLength);
    REMAP (2,3);
    
    rsupv_status = rsupv_SetUSIAnalyzerAudioBits (instrSession, audioBits);
    REMAP (2,4);
    
    rsupv_status = rsupv_SetUSIAnalyzerLeadBits (instrSession, leadBits);
    REMAP (2,5);
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure USI Analyzer Fsync
 * Purpose:  This function configures Frame Synchronization parameters. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureUSIAnalyzerFsync (ViSession instrSession,
                                                   ViInt32 fsyncWidth,
                                                   ViInt32 variableFsyncWidth,
                                                   ViInt32 fsyncOffset,
                                                   ViInt32 fsyncSlope)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetUSIAnalyzerFsyncWidth (instrSession, fsyncWidth, variableFsyncWidth));
    
    rsupv_status = rsupv_SetUSIAnalyzerFsyncOffset (instrSession, fsyncOffset);
    REMAP (2,4);
    
    rsupv_status = rsupv_SetUSIAnalyzerFsyncSlope (instrSession, fsyncSlope);
    REMAP (2,5);
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set USI Analyzer Meas Channel State
 * Purpose:  This function configures measurement channels state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerMeasChannelState(
    ViSession   instrSession,
    ViInt32 measChannel,
    ViBoolean   state
)
{
    ViChar buffer[BUFFER_SIZE];
    CHECKOPT ("B42");
    
    CHECKVIINT32 (measChannel, 1, 8, 2);
    
    sprintf (buffer, "INP:MCH%ld", measChannel);
    
    return rsupv_setViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Meas Channel State
 * Purpose:  This function queries measurement channels state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerMeasChannelState(
    ViSession   instrSession,
    ViInt32 measChannel,
    ViBoolean*  state
)
{
    ViChar buffer[BUFFER_SIZE];
    CHECKOPT ("B42");
    
    CHECKVIINT32 (measChannel, 1, 8, 2);
    
    sprintf (buffer, "INP:MCH%ld?", measChannel);
    
    return rsupv_getViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set USI Analyzer Clock
 * Purpose:  This function sets continuous clock or gated clock operating 
 *           mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerClock(
    ViSession   instrSession,
    ViInt32 clock
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "INP:CLOC",
                          clock,
                          dualClockArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Clock
 * Purpose:  This function queries clock operating mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerClock(
    ViSession   instrSession,
    ViInt32*    clock
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "INP:CLOC?",
                          clock,
                          dualClockArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Analyzer Timeout
 * Purpose:  This function sets timeout of the measurement in Gated clock
 *           mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerTimeout (ViSession instrSession,
                                               ViReal64 timeout)
{
    CHECKOPT ("B42");
    return rsupv_setViReal64 (instrSession,
                              "INP:TIM",
                              timeout,
                              0.0, 0.5,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Timeout
 * Purpose:  This function queries timeout of the measurement in Gated clock 
 *           mode. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerTimeout (ViSession instrSession,
                                               ViReal64* timeout)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "INP:TIM?",
                              timeout,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Analyzer Mixed Sampling Frequency State
 * Purpose:  This function permits simultaneous analysis of data streams at 
 *           varying sampling rates.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerMixedSamplingFrequencyState(
    ViSession   instrSession,
    ViBoolean   mixedSamplingFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_setViBoolean (instrSession,
                               "INP:MSAM",
                               mixedSamplingFrequency,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Mixed Sampling Frequency State
 * Purpose:  This function queries state of mixed sampling frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerMixedSamplingFrequencyState(
    ViSession   instrSession,
    ViBoolean*  mixedSamplingFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_getViBoolean (instrSession,
                               "INP:MSAM?",
                               mixedSamplingFrequency,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Analyzer Ratio
 * Purpose:  This function selects up-sampling factor for the slower data 
 *           stream. The input data with the lower sampling rate is 
 *           characterised by correspondingly fewer entries in the Slots input 
 *           field.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerRatio(
    ViSession   instrSession,
    ViInt32 ratio
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "INP:RAT",
                              ratio,
                              2, 6,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Analyzer Ratio
 * Purpose:  This function queries up-sampling factor for the slower data 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerRatio(
    ViSession   instrSession,
    ViInt32*    ratio
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "INP:RAT?",
                              ratio,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Analyzer Samples Per Frame
 * Purpose:  This function defines the number of samples per frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerSamplesPerFrame(
    ViSession   instrSession,
    ViInt32 samples
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "INP:SPFR",
                              samples,
                              1, 32,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Analyzer Samples Per Frame
 * Purpose:  This function queries the number of samples per frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerSamplesPerFrame(
    ViSession   instrSession,
    ViInt32*    samples
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "INP:SPFR?",
                              samples,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Analyzer Number Of Slots
 * Purpose:  This function selects number of slots per frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerNumberOfSlots(
    ViSession   instrSession,
    ViInt32 numberOfSlots
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "INP:NOSL",
                              numberOfSlots,
                              1, 256,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Analyzer Number Of Slots
 * Purpose:  This function queries number of slots per frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerNumberOfSlots(
    ViSession   instrSession,
    ViInt32*    numberOfSlots
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "INP:NOSL?",
                              numberOfSlots,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Analyzer Measurement Source
 * Purpose:  This function selects the input data line for selected measuring 
 *           channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerMeasurementSource(
    ViSession   instrSession,
    ViInt32 measChannel,
    ViInt32 dataLink
)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKOPT ("B42");
    CHECKVIINT32 (measChannel, 1, 8, 2);
    CHECKVIINT32 (dataLink, 1, 4, 3);
    
    sprintf (buffer, "INP:MCHS%ld RX%ldD", measChannel, dataLink);
    
    return rsupv_sendCmd (instrSession, buffer);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Measurement Source
 * Purpose:  This function queries the input data line for selected measuring 
 *           channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerMeasurementSource(
    ViSession   instrSession,
    ViInt32 measChannel,
    ViInt32*    dataLink
)
{
    ViChar buffer[BUFFER_SIZE] = "";
    ViChar response[BUFFER_SIZE] = "";
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B42");
    CHECKVIINT32 (measChannel, 1, 8, 2);
    
    sprintf (buffer, "INP:MCHS%ld?", measChannel);
    CHECKERR (rsupv_getViString (instrSession,
                                 buffer,
                                 response,
                                 BUFFER_SIZE,
                                 0));
    
    sscanf (response,"RX%ld", dataLink);
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set USI Analyzer Slots
 * Purpose:  This function selects the audio frame slots to be analyzed for 
 *           selected measuring channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerSlots (ViSession instrSession,
                                             ViInt32 measChannel, ViString slots)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKOPT ("B42");
    CHECKVIINT32 (measChannel, 1, 8, 2);
    
    sprintf (buffer, "INP:SLOT%ld", measChannel);
    
    return rsupv_setViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    slots,
                                    VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Slots
 * Purpose:  This function queries the audio frame slots to be analyzed for 
 *           selected measuring channel. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerSlots (ViSession instrSession,
                                             ViInt32 measChannel, ViInt32 arraySize, ViChar slots[])
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKOPT ("B42");
    CHECKVIINT32 (measChannel, 1, 8, 2);
    
    sprintf (buffer, "INP:SLOT%ld?", measChannel);
    
    return rsupv_getViStringQuoted (instrSession,
                             buffer,
                             "'",
                             arraySize,
                             slots,
                             VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set USI Analyzer Sync To
 * Purpose:  This function selects the source to which the USI dual channel 
 *           analyzer is synchronized.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerSyncTo(
    ViSession   instrSession,
    ViInt32 syncTo
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "INP:SYNC",
                          syncTo,
                          dualSyncArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Sync To
 * Purpose:  This function queries the source to which the USI dual channel 
 *           analyzer is synchronized.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerSyncTo(
    ViSession   instrSession,
    ViInt32*    syncTo
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "INP:SYNC?",
                          syncTo,
                          dualSyncArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Analyzer First Bit
 * Purpose:  This function selects data bit receiving sequence.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerFirstBit(
    ViSession   instrSession,
    ViInt32 firstBit
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "INP:FBIT",
                          firstBit,
                          dualFirstBitArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer First Bit
 * Purpose:  This function queries data bit receiving sequence.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerFirstBit(
    ViSession   instrSession,
    ViInt32*    firstBit
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "INP:FBIT?",
                          firstBit,
                          dualFirstBitArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Analyzer Slot Length
 * Purpose:  This function selects slot length of a slot including gaps (0 
 *           bits).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerSlotLength(
    ViSession   instrSession,
    ViInt32 slotLength
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "INP:SLTL",
                              slotLength,
                              8, 256,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Analyzer Slot Length
 * Purpose:  This function queries slot length of a slot including gaps (0 
 *           bits).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerSlotLength(
    ViSession   instrSession,
    ViInt32*    slotLength
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "INP:SLTL?",
                              slotLength,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Analyzer Audio Bits
 * Purpose:  This function sets slot length of the audio data within a slot.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerAudioBits(
    ViSession   instrSession,
    ViInt32 audioBits
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "INP:AUD",
                              audioBits,
                              8, 32,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Analyzer Audio Bits
 * Purpose:  This function queries slot length of the audio data within a 
 *           slot.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerAudioBits(
    ViSession   instrSession,
    ViInt32*    audioBits
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                             "INP:AUD?",
                             audioBits,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Analyzer Lead Bits
 * Purpose:  This function sets the offset of the audio bits within a slot 
 *           relative to the start of the slot.
 *           If LSB is selected as the first bit, the setting for lead bits is 
 *           automatically corrected if the setting for audio bits is changed 
 *           so that the MSB is received at the same location. This has the 
 *           advantage that the resolution and not the amplitude of the 
 *           measured signal is changed if the audio bits are changed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerLeadBits(
    ViSession   instrSession,
    ViInt32 leadBits
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "INP:LBIT",
                              leadBits,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Analyzer Lead Bits
 * Purpose:  This function queries the offset of the audio bits within a slot 
 *           relative to the start of the slot.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerLeadBits(
    ViSession   instrSession,
    ViInt32*    leadBits
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "INP:LBIT?",
                              leadBits,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Analyzer Sample Frequency
 * Purpose:  This function is used to set the sample frequency. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerSampleFrequency (ViSession instrSession,
                                                       ViInt32 sampleFrequency,
                                                       ViReal64 variableSampleFrequency,
                                                       ViInt32 units)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    CHECKERR (rsupv_setEnum (instrSession,
                             "INP:SAMP:FREQ:MODE",
                             sampleFrequency,
                             genI2SSampleFrequencyArr,
                             VI_ERROR_PARAMETER2));

    if (sampleFrequency == RSUPV_SAMPLE_FREQ_VAL)
    {
        CHECKERR (rsupv_setViReal64Unit (instrSession,
                                     "INP:SAMP:FREQ",
                                     variableSampleFrequency,
                                     units,
                                     0.0, 0.0,
                                     VI_ERROR_PARAMETER3));
    }

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Get USI Analyzer Sample Frequency
 * Purpose:  This function is used to query the sampling frequency. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerSampleFrequency (ViSession instrSession,
                                                       ViInt32 *sampleFrequency,
                                                       ViReal64 *variableSampleFrequency,
                                                       ViInt32 *units)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    CHECKERR (rsupv_getEnum (instrSession,
                             "INP:SAMP:FREQ:MODE?",
                             sampleFrequency,
                             genI2SSampleFrequencyArr,
                             VI_ERROR_PARAMETER2));

    if (*sampleFrequency == RSUPV_SAMPLE_FREQ_VAL)
    {
        CHECKERR (rsupv_getViReal64Unit (instrSession,
                                     "INP:SAMP:FREQ?",
                                     variableSampleFrequency,
                                     units,
                                     VI_ERROR_PARAMETER3));
    }
    return rsupv_status;    
}

/*****************************************************************************
 * Function: Get USI Analyzer Fsync Frequency
 * Purpose:  This function queries Frame Sync frequency which corresponds to 
 *           the sampling frequncy divided by the samples/frame factor. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerFsyncFrequency(
    ViSession   instrSession,
    ViReal64*   fsyncFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "INP:FSYN:FREQ?",
                              fsyncFrequency,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer BClk Frequency
 * Purpose:  This function queries bit clock frequency. Is calculated from the 
 *           product of channel length * no of slots * sample freq / 
 *           samples/frame.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerBClkFrequency(
    ViSession   instrSession,
    ViReal64*   bclkFrequency
)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "INP:BCLK:FREQ?",
                              bclkFrequency,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Analyzer Fsync Width
 * Purpose:  This function selects width of the FSYNC pulse
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerFsyncWidth(
    ViSession   instrSession,
    ViInt32 fsyncWidth,
    ViInt32 variableFsyncWidth
)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    CHECKERR (rsupv_setEnum (instrSession,
                             "INP:FWID",
                             fsyncWidth,
                             dualWidthArr,
                             VI_ERROR_PARAMETER2));

    if (fsyncWidth == RSUPV_DUAL_FWIDTH_VALUE)
    {
        CHECKERR (rsupv_setViInt32 (instrSession,
                                    "INP:FWID:VAL",
                                    variableFsyncWidth,
                                    0, 0,
                                    VI_ERROR_PARAMETER3));
    }

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Get USI Analyzer Fsync Width
 * Purpose:  This function queries width of the FSYNC pulse
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerFsyncWidth(
    ViSession   instrSession,
    ViInt32*    fsyncWidth,
    ViInt32*    variableFsyncWidth
)
{
    ViStatus    rsupv_status;

    CHECKOPT ("B42");
    CHECKERR (rsupv_getEnum (instrSession,
                             "INP:FWID?",
                             fsyncWidth,
                             dualWidthArr,
                             VI_ERROR_PARAMETER2));

    if (*fsyncWidth == RSUPV_DUAL_FWIDTH_VALUE)
    {
        CHECKERR (rsupv_getViInt32 (instrSession,
                                    "INP:FWID:VAL?",
                                    variableFsyncWidth,
                                    VI_ERROR_PARAMETER3));
    }
    
    return rsupv_status;    
}

/*****************************************************************************
 * Function: Set USI Analyzer Fsync Offset
 * Purpose:  This function determines the offset of the Fsync pulse relative 
 *           to the reference point (start of the first slot).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerFsyncOffset(
    ViSession   instrSession,
    ViInt32 fsyncOffset
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "INP:FOFF",
                              fsyncOffset,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Analyzer Fsync Offset
 * Purpose:  This function queries the offset of the Fsync pulse relative to 
 *           the reference point (start of the first slot).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerFsyncOffset(
    ViSession   instrSession,
    ViInt32*    fsyncOffset
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "INP:FOFF?",
                              fsyncOffset,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Analyzer Fsync Slope
 * Purpose:  This function selects polarity of the Fsync pulse.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerFsyncSlope(
    ViSession   instrSession,
    ViInt32 fsyncSlope
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "INP:FSL",
                          fsyncSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Fsync Slope
 * Purpose:  This function queries polarity of the Fsync pulse.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerFsyncSlope(
    ViSession   instrSession,
    ViInt32*    fsyncSlope
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "INP:FSL?",
                          fsyncSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Analyzer BClk Slope
 * Purpose:  This function selects bit clock polarity.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerBClkSlope(
    ViSession   instrSession,
    ViInt32 bclkSlope
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "INP:BSL",
                          bclkSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer BClk Slope
 * Purpose:  This function queries bit clock polarity.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerBClkSlope(
    ViSession   instrSession,
    ViInt32*    bclkSlope
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "INP:BSL?",
                          bclkSlope,
                          dualSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Analyzer MClkRatio
 * Purpose:  This function defines relationship between the master clock 
 *           (MCLK) and the FrameSync (FSYNC).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerMClkRatio(
    ViSession   instrSession,
    ViInt32 ratio
)
{
    CHECKOPT ("B42");
    return rsupv_setViInt32 (instrSession,
                              "INP:MRAT",
                              ratio,
                              16, 768,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Analyzer MClkRatio
 * Purpose:  This function queries relationship between the master clock 
 *           (MCLK) and the FrameSync (FSYNC).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerMClkRatio(
    ViSession   instrSession,
    ViInt32*    ratio
)
{
    CHECKOPT ("B42");
    return rsupv_getViInt32 (instrSession,
                              "INP:MRAT?",
                              ratio,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Analyzer Sampling Delay
 * Purpose:  This function permits a delay of the RX_FSYNC_IN, RX_DATA1, 
 *           RX_DATA2, RX_DATA3 and RX_DATA4 input signal sampling times in 
 *           relation to the RX_BCLK_IN or RX_BCLK_OUT clocks. Positive values 
 *           delay the sampling to a later time. 
 *           This feature is used above all if TCO delays, in other words 
 *           response times of a device under test being operated in slave mode 
 *           at high frequencies, reach the size of a bit clock period. In 
 *           order to compensate for bit shifts when analyzing in the analyzer 
 *           these delay times are compensated by a later sampling point. 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerSamplingDelay(
    ViSession   instrSession,
    ViReal64    samplingDelay
)
{
    CHECKOPT ("B42");
    return rsupv_setViReal64 (instrSession,
                              "INP:SDEL",
                              samplingDelay*1e-12,
                              -10.0e-12, 10.0e-12,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get USI Analyzer Sampling Delay
 * Purpose:  This function queries a delay of the RX_FSYNC_IN, RX_DATA1, 
 *           RX_DATA2, RX_DATA3 and RX_DATA4 input signal sampling times in 
 *           relation to the RX_BCLK_IN or RX_BCLK_OUT clocks.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerSamplingDelay(
    ViSession   instrSession,
    ViReal64*   samplingDelay
)
{
    CHECKOPT ("B42");
    return rsupv_getViReal64 (instrSession,
                              "INP:SDEL?",
                              samplingDelay,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set USI Analyzer Logic Voltage
 * Purpose:  This function sets the analyzer logic voltage. Changing the 
 *           voltage level causes the sample to be reloaded. The sample can be 
 *           attached and disconnected and put back into operation during 
 *           operation with this mechanism.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerLogicVoltage(
    ViSession   instrSession,
    ViInt32 logicVoltage
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "INP:LOGV",
                          logicVoltage,
                          dualVoltageArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Logic Voltage
 * Purpose:  This function queries the analyzer logic voltage.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerLogicVoltage(
    ViSession   instrSession,
    ViInt32*    logicVoltage
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "INP:LOGV?",
                          logicVoltage,
                          dualVoltageArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set USI Analyzer Coding
 * Purpose:  This function sets analyzer data coding regulation. A Law and 
 *           micro Law quantification characteristics are available for 
 *           telephony applications.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetUSIAnalyzerCoding(
    ViSession   instrSession,
    ViInt32 coding
)
{
    CHECKOPT ("B42");
    return rsupv_setEnum (instrSession,
                          "INP:COD",
                          coding,
                          dualCodingArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get USI Analyzer Coding
 * Purpose:  This function queries analyzer data coding regulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetUSIAnalyzerCoding(
    ViSession   instrSession,
    ViInt32*    coding
)
{
    CHECKOPT ("B42");
    return rsupv_getEnum (instrSession,
                          "INP:COD?",
                          coding,
                          dualCodingArr,
                          VI_ERROR_PARAMETER2);    
}

 /* HDMI Analyzer */

/*****************************************************************************
 * Function: Configure HDMI Analyzer Audio
 * Purpose:  This function cofigures the Digital Bitstream Analyzer
 *           parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureHDMIAnalyzerAudio (ViSession instrSession,
                                                    ViInt32 input,
                                                    ViInt32 audioCoding)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetHDMIAnalyzerAudioInput (instrSession, input));
    
    rsupv_status = rsupv_SetHDMIAnalyzerAudioCoding (instrSession, audioCoding);
    REMAP (2,3);

    return rsupv_status;
}       

/*****************************************************************************
 * Function: Set HDMI Analyzer Audio Input
 * Purpose: This function selects the audio source. By default, the audio
 *          data are received and analyzed via the HDMI SINK socket in the
 *          HDMI analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIAnalyzerAudioInput (ViSession instrSession,
                                                   ViInt32 audioSource)

{
    CHECKOPT ("B4");
    
    return rsupv_setEnum (instrSession,
                        "SENS:HDMI:AUD:INP",
                        audioSource,
                        audioHDMISourceArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Analyzer Audio Input
 * Purpose:  This function queries the audio source.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerAudioInput (ViSession instrSession,
                                                   ViInt32* audioSource)

{
    CHECKOPT ("B4");
        
    return rsupv_getEnum (instrSession,
                        "SENS:HDMI:AUD:INP?",
                        audioSource,
                        audioHDMISourceArr,
                        VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Get HDMI Analyzer Audio Format
 * Purpose:  This function returns the HDMI layout of the supplied audio
 *           data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerAudioFormat (ViSession instrSession,
                                                   ViInt32* audioFormat)

{
    CHECKOPT ("B4");

    return rsupv_getEnum (instrSession,
                        "SENS:HDMI:AUD:FORM?",
                        audioFormat,
                        audioHDMIFormatArr,
                        VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Set HDMI Analyzer Audio Coding
 * Purpose: This function selects the decoder functionality.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIAnalyzerAudioCoding (ViSession instrSession,
                                                   ViInt32 audioCoding)

{
    CHECKOPT ("B4");
    return rsupv_setEnum (instrSession,
                        "SENS:HDMI:AUD:COD",
                        audioCoding,
                        audioHDMICodingArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Analyzer Audio Coding
 * Purpose:  This function returns the decoder functionality.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerAudioCoding (ViSession instrSession,
                                                   ViInt32* audioCoding)

{
    CHECKOPT ("B4");
        
    return rsupv_getEnum (instrSession,
                        "SENS:HDMI:AUD:COD?",
                        audioCoding,
                        audioHDMICodingArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Analyzer Audio Detected
 * Purpose:  This function returns  field in which the determined decoding
 *           method (e.g. Dolby/DTS) is displayed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerAudioDetected (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar detected[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
        
    CHECKERR (rsupv_getViString (instrSession,
                                 "SENS:HDMI:AUD:COD:DET:STR?",
                                 detected,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Get HDMI Analyzer Audio N Parameter
 * Purpose:  The parameters N and CTS are required by an HDMI load for the
 *           regeneration of the audio master clock. They describe a
 *           fractional-rational ratio of the audio master clock to the video
 *           pixel clock.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerAudioNParameter (ViSession instrSession,
                                                   ViInt32 *nParameter)
{
    CHECKOPT ("B4");
    return rsupv_getViInt32 (instrSession,
                              "SENS:HDMI:AUD:N?",
                              nParameter,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get HDMI Analyzer Audio N Parameter
 * Purpose:  The parameters N and CTS are required by an HDMI load for the
 *           regeneration of the audio master clock. They describe a
 *           fractional-rational ratio of the audio master clock to the video
 *           pixel clock.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerAudioCTSParameter (ViSession instrSession,
                                                   ViInt32 *CTS)
{
    CHECKOPT ("B4");
    return rsupv_getViInt32 (instrSession,
                              "SENS:HDMI:AUD:CTS?",
                              CTS,
                              VI_ERROR_PARAMETER2);    
}


    
/*****************************************************************************
 * Function: Set HDMI Analyzer Audio Channel
 * Purpose:  This function is used to select the active measurement channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIAnalyzerAudioChannel(
    ViSession   instrSession,
    ViInt32     channel
)
{
    CHECKOPT ("B4");
    return rsupv_setEnum (instrSession,
                     "INP:CHAN",
                      channel,
                      anChannelArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get HDMI Analyzer Audio Channel
 * Purpose:  This function returns the active measurement channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerAudioChannel(
    ViSession   instrSession,
    ViInt32*    channel
)
{
    CHECKOPT ("B4");
    return rsupv_getEnum (instrSession,
                     "INP:CHAN?",
                      channel,
                      anChannelArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set HDMI Analyzer Audio Meas Channel State
 * Purpose:  This function configures measurement channels state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHDMIAnalyzerAudioMeasChannelState(
    ViSession   instrSession,
    ViInt32 measChannel,
    ViBoolean   state
)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKOPT ("B4");
    
    CHECKVIINT32 (measChannel, 1, 8, 2);
    
    sprintf (buffer, "INP:MCH%ld", measChannel);
    
    return rsupv_setViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get HDMI Analyzer Audio Meas Channel State
 * Purpose:  This function queries measurement channels state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerAudioMeasChannelState(
    ViSession   instrSession,
    ViInt32 measChannel,
    ViBoolean*  state
)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKOPT ("B4");
    CHECKVIINT32 (measChannel, 1, 8, 2);
    
    sprintf (buffer, "INP:MCH%ld?", measChannel);
    
    return rsupv_getViBoolean (instrSession,
                               buffer,
                               state,
                               VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get HDMI Analyzer Video Format No 
 * Purpose:  This function returns the Video identification code of the
 *           supplied video signal according to CEA-861. The ID code is
 *           extracted directly from the AVI Info Frame of the connected HDMI
 *           source.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerVideoFormatNo (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar formatNo[])
{
    ViStatus rsupv_status = 0;

    CHECKOPT ("B4");
    CHECKERR (rsupv_getViString (instrSession,
                                 "SENS:HDMI:VID:FORM?",
                                 formatNo,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}
    
/*****************************************************************************
 * Function: Get HDMI Analyzer Video Color Depth
 * Purpose:  This function returns the Color depth of the supplied video
 *           format. This information is extracted from the General
 *           Control Packet of a connected HDMI source.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerVideoColorDepth (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar colorDepth[])
{
    ViStatus rsupv_status = 0;
    CHECKOPT ("B4");
        
    CHECKERR (rsupv_getViString (instrSession,
                                 "SENS:HDMI:VID:COL:DEPT?",
                                 colorDepth,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}


/*****************************************************************************
 * Function: Get HDMI Analyzer Audio Info Frame
 * Purpose:  This function returns an image of the binary data structure from
 *           the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerAudioInfoFrame (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar audioInfoFrame[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SENS:HDMI:AUD:INF:STR?",
                                 audioInfoFrame,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Get HDMI Analyzer Video HDCP State
 * Purpose:  This function indicates whether the A/V signal of a connected
 *           HDMI source is encrypted or not.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerVideoHDCPState (ViSession instrSession,
                                                  ViBoolean *HDCPState)
{
    CHECKOPT ("B4");
    return rsupv_getViBoolean (instrSession,
                               "SENSe:HDMI:HDCP?",
                               HDCPState,
                               VI_ERROR_PARAMETER2);    
}


/*****************************************************************************
 * Function: Get HDMI Analyzer Video Timings
 * Purpose:  This function returns an image of the binary data structure from
 *           the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerVideoTimings (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar timings[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SENS:HDMI:VID:TIM:STR?",
                                 timings,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}   

/*****************************************************************************
 * Function: Get HDMI Analyzer Video AVI
 * Purpose:  This function returns an image of the binary data structure from
 *           the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerVideoAVI (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar AVI[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SENS:HDMI:AVI:STR?",
                                 AVI,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}   

/*****************************************************************************
 * Function: Get HDMI Analyzer Video SPD
 * Purpose:  This function returns an image of the binary data structure from
 *           the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerVideoSPD (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar SPD[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SENS:HDMI:SPD:STR?",
                                 SPD,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}   

/*****************************************************************************
 * Function: Get HDMI Analyzer Video E-EDID
 * Purpose:  This function returns an image of the binary data structure from
 *           the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHDMIAnalyzerVideoEEDID (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar eedid[])
{
    ViStatus rsupv_status = 0;
    
    CHECKOPT ("B4");
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SENS:HDMI:EED:STR?",
                                 eedid,
                                 bufferSize,
                                 VI_ERROR_PARAMETER3));
    
    return rsupv_status;
}


/*****************************************************************************
 * Function: USI Analyzer Resync
 * Purpose:  This function causes a complete renewed synchronisation of the 
 *           analyzer. If the analyzer is synchronised by external signals in 
 *           slave mode the resync mechanismus should be performed once after 
 *           the application and synchronisation of the external cocks.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_USIAnalyzerResync(
    ViSession   instrSession
)
{
    CHECKOPT ("B42");
    return rsupv_sendCmd (instrSession, "INP:RESY ONCE");
}

/*****************************************************************************
 * Function: Set Analog Analyzer Channel Mode
 * Purpose:  This function is used to select the active XLR sockets in the
 *           bottom right-hand part of the R&S UPV front panel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalogAnalyzerChannelMode(
    ViSession   instrSession,
    ViInt32     channel
)
{
    return rsupv_setEnum (instrSession,
                     "INP:CHAN",
                      channel,
                      anChannelArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Analyzer Channel Mode
 * Purpose:  This function returns the active XLR sockets in the bottom
 *           right-hand part of the R&S UPV front panel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalogAnalyzerChannelMode(
    ViSession   instrSession,
    ViInt32*    channel
)
{
    return rsupv_getEnum (instrSession,
                     "INP:CHAN?",
                      channel,
                      anChannelArr,
                      VI_ERROR_PARAMETER2);
}
/*****************************************************************************
 * Function: Set Digital Analyzer Channel Mode
 * Purpose:  This function is used to select the active measurement channel
 *           in the Meas Mode "Audio Data". Used to select the audio data for
 *           the left-hand channel (Ch1), righthand channel (Ch2) or both
 *           channels.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalAnalyzerChannelMode(
    ViSession   instrSession,
    ViInt32     channel
)
{
    return rsupv_setEnum (instrSession,
                     "INP:CHAN",
                      channel,
                      anChannelArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Analyzer Channel Mode
 * Purpose:  This function returns the active measurement channel in the Meas
 *           Mode "Audio Data".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalAnalyzerChannelMode(
    ViSession   instrSession,
    ViInt32*    channel
)
{
    return rsupv_getEnum (instrSession,
                     "INP:CHAN?",
                      channel,
                      anChannelArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Analyzer Channel Mode
 * Purpose:  This function is used to select the active measurement channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SAnalyzerChannelMode (ViSession instrSession,
                                                   ViInt32 channelMode)
{
    CHECKOPT ("B41");
    
    return rsupv_setEnum (instrSession,
                     "INP:CHAN",
                      channelMode,
                      anChannelArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get I2S Analyzer Channel Mode
 * Purpose:  This function returns the active measurement channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SAnalyzerChannelMode (ViSession instrSession,
                                                   ViInt32 *channelMode)
{
    CHECKOPT ("B41");
    
    return rsupv_getEnum (instrSession,
                     "INP:CHAN?",
                      channelMode,
                      anChannelArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Bandwidth
 * Purpose:  This function selects bandwidth that determines the upper
 *           frequency limit for the measurement signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerBandwidth(
    ViSession   instrSession,
    ViInt32     bandwidth
)
{
    return rsupv_setEnum (instrSession,
                     "INP:BAND:MODE",
                      bandwidth,
                      anBandwidthArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Bandwidth
 * Purpose:  This function gets bandwidth that determines the upper frequency
 *           limit for the measurement signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerBandwidth(
    ViSession   instrSession,
    ViInt32*    bandwidth
)
{
    return rsupv_getEnum (instrSession,
                     "INP:BAND:MODE?",
                      bandwidth,
                      anBandwidthArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Prefilter
 * Purpose:  This function selects prefilter that affects all measurement
 *           functions. This means that prefiltering is possible without having
 *           to select the required filter for every single measurement
 *           function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPrefilter(
    ViSession   instrSession,
    ViInt32     preFilter
)
{
    return rsupv_setEnum (instrSession,
                     "INP:FILT",
                      preFilter,
                      anPreFilterArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Prefilter
 * Purpose:  This function gets prefilter that affects all measurement
 *           functions. This means that prefiltering is possible without having
 *           to select the required filter for every single measurement
 *           function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPrefilter(
    ViSession   instrSession,
    ViInt32*    preFilter
)
{
    return rsupv_getEnum (instrSession,
                     "INP:FILT?",
                      preFilter,
                      anPreFilterArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Start Condition
 * Purpose:  This function is used to define which conditions trigger a
 *           measurement to be started.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerStartCondition(
    ViSession   instrSession,
    ViInt32     startCondition
)
{
    return rsupv_setEnum (instrSession,
                     "TRIG:SOUR",
                      startCondition,
                      anStartConditionArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Start Condition
 * Purpose:  This function gets start conditions.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerStartCondition(
    ViSession   instrSession,
    ViInt32*    startCondition
)
{
    return rsupv_getEnum (instrSession,
                     "TRIG:SOUR?",
                      startCondition,
                      anStartConditionArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Start Delay
 * Purpose:  This function determines the time which must elapse between the
 *           above-mentioned events and restarting of the measurement so that,
 *           for example, a DUT has sufficient time to settle.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerStartDelay(
    ViSession   instrSession,
    ViReal64    startDelay
)
{
    return rsupv_setViReal64Unit (instrSession,
                              "TRIG:DEL",
                              startDelay,
                              RSUPV_UNIT_S,
                              0.0, 2000.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Start Delay
 * Purpose:  This function gets the time which must elapse between the
 *           above-mentioned events and restarting of the measurement so that,
 *           for example, a DUT has sufficient time to settle.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerStartDelay(
    ViSession   instrSession,
    ViReal64*   startDelay
)
{
    return rsupv_getViReal64Unit (instrSession,
                              "TRIG:DEL?",
                              startDelay,
                              VI_NULL,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Trigger Settling Mode
 * Purpose:  This function is Used to activate the settling measurement for
 *           the frequency/level monitoring routine in the case of external
 *           sweeps and external level/edge triggering, and to select the
 *           "settling" method.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTriggerSettlingMode (ViSession instrSession,
                                                        ViInt32 settling)
{
    if (settling == RSUPV_ANALYZER_TRIG_SETTLING_AVER)
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
   return rsupv_setEnum (instrSession,
                        "SENS:TRIG:SETT:MODE",
                        settling,
                        FnctSettlingArr,
                        VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Analyzer Trigger Settling Mode
 * Purpose:  This function queries the settling measurement for
 *           the frequency/level monitoring routine in the case of external
 *           sweeps and external level/edge triggering, and to select the
 *           "settling" method.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTriggerSettlingMode (ViSession instrSession,
                                                        ViInt32 *settling)
{
    return rsupv_getEnum (instrSession,
                        "SENS:TRIG:SETT:MODE?",
                         settling,
                         FnctSettlingArr,
                         VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Trigger Settling Count
 * Purpose:  This function Designates the number of measured values to be
 *           covered by the settling algorithm, i.e.
 *           it defines how many frequency or level measured values are to be
 *           used for settling.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTriggerSettlingCount (ViSession instrSession,
                                                         ViInt32 samples)
{
   return rsupv_setViInt32 (instrSession,
                            "SENS:TRIG:SETT:COUN",
                             samples,
                             0, 0,
                             VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Analyzer Trigger Settling Count
 * Purpose:  This function queries the number of measured values to be
 *           covered by the settling algorithm, i.e.
 *           it defines how many frequency or level measured values are to be
 *           used for settling.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTriggerSettlingCount (ViSession instrSession,
                                                         ViInt32 *samples)
{
   return rsupv_getViInt32 (instrSession,
                            "SENS:TRIG:SETT:COUN?",
                             samples,
                             VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Analyzer Trigger Settling Resolution
 * Purpose:  This function sets the settling trigger resolution.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTriggerSettlingResolution (ViSession instrSession,
                                                              ViReal64 resolution)
{
   return rsupv_setViReal64Unit (instrSession,
                                "SENS:TRIG:SETT:RES",
                                resolution,
                                RSUPV_UNIT_V,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Analyzer Trigger Settling Resolution
 * Purpose:  This function gets the settling trigger resolution.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTriggerSettlingResolution (ViSession instrSession,
                                                              ViReal64 *resolution)
{
   return rsupv_getViReal64Unit (instrSession,
                                "SENS:TRIG:SETT:RES?",
                                resolution,
                                VI_NULL,
                                VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Analyzer Trigger Settling Tolerance
 * Purpose:  This function sets the trigger settling tolerance.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTriggerSettlingTolerance (ViSession instrSession,
                                                             ViReal64 tolerance)
{
   return rsupv_setViReal64Unit (instrSession,
                                "SENS:TRIG:SETT:TOL",
                                tolerance,
                                RSUPV_UNIT_PCT,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER2);   
}

/*****************************************************************************
 * Function: Get Analyzer Trigger Settling Tolerance
 * Purpose:  This function gets the trigger settling tolerance.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTriggerSettlingTolerance (ViSession instrSession,
                                                             ViReal64 *tolerance)
{
   return rsupv_getViReal64Unit (instrSession,
                              "SENS:TRIG:SETT:TOL?",
                               tolerance,
                               VI_NULL,
                               VI_ERROR_PARAMETER2);  

}

/*****************************************************************************
 * Function: Set External Sweep Start Value
 * Purpose:  This function sets the start value for the recording interval
 * (in the case of external sweeps) or for the trigger range (in
 * the case of level triggering).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetExternalSweepStartValue (ViSession instrSession,
                                           ViInt32 sweepControl, ViReal64 start,
                                           ViInt32 units)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (sweepControl, RSUPV_ANALYZER_FREQ, RSUPV_ANALYZER_VOLT, 2);

    sprintf (buffer, "ARM:%s:STAR", SweepcontrolArr[sweepControl]);

    return rsupv_setViReal64Unit (instrSession, 
                              buffer,
                              start,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER3); 
}

/*****************************************************************************
 * Function: Get External Sweep Start Value
 * Purpose:  This function queries the start value for the recording interval
 * (in the case of external sweeps) or for the trigger range (in
 * the case of level triggering).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetExternalSweepStartValue (ViSession instrSession,
                                           ViInt32 sweepControl,
                                           ViReal64 *start, ViInt32 *units)
{
  
   ViChar  buffer[BUFFER_SIZE];

   CHECKVIINT32 (sweepControl, RSUPV_ANALYZER_FREQ, RSUPV_ANALYZER_VOLT, 2);

   sprintf (buffer, "ARM:%s:STARt?", SweepcontrolArr[sweepControl]);    
    

   return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  start,
                                  units,
                                  VI_ERROR_PARAMETER3);     
}

/*****************************************************************************
 * Function: Set External Sweep Stop Value
 * Purpose:  This function sets the stop value for the recording interval (in
 * the case of external sweeps) or for the trigger range (in the
 * case of level triggering).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetExternalSweepStopValue (ViSession instrSession,
                                          ViInt32 sweepControl,ViReal64 stop,
                                          ViInt32 units)
{
  
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (sweepControl, RSUPV_ANALYZER_FREQ, RSUPV_ANALYZER_VOLT, 2);

    sprintf (buffer, "ARM:%s:STOP", SweepcontrolArr[sweepControl]);

    return rsupv_setViReal64Unit (instrSession, 
                              buffer,
                              stop,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER3); 
}

/*****************************************************************************
 * Function: Get External Sweep Stop Value
 * Purpose:  This function gets the stop value for the recording interval (in
 *           the case of external sweeps) or for the trigger range (in the
 *           case of level triggering).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetExternalSweepStopValue (ViSession instrSession,
                                          ViInt32 sweepControl, ViReal64 *stop,
                                          ViInt32 *units)
{
   ViChar  buffer[BUFFER_SIZE];

   CHECKVIINT32 (sweepControl, RSUPV_ANALYZER_FREQ, RSUPV_ANALYZER_VOLT, 2);

   sprintf (buffer, "ARM:%s:STOP?", SweepcontrolArr[sweepControl]); 
    

   return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  stop,
                                  units,
                                  VI_ERROR_PARAMETER3);  
}

/*****************************************************************************
 * Function: Set External Sweep Minimum Level
 * Purpose:  This function sets the minimum level for the external frequency
 *           sweeps. If the signal is below this threshold, no triggering
 *           occurs.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetExternalSweepMinimumLevel (ViSession instrSession,
                                             ViReal64 minVolt)
{
   return rsupv_setViReal64 (instrSession,
                                    "ARM:LEV:MIN",
                                    minVolt,
                                    0.0, 0.0,
                                    VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get External Sweep Minimum Level
 * Purpose:  This function returns the minimum level for the external
 *           frequency sweeps.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetExternalSweepMinimumLevel (ViSession instrSession,
                                             ViReal64 *minVolt)
{
  return rsupv_getViReal64Unit (instrSession,
                                "ARM:LEV:MIN?",
                                 minVolt,
                                 VI_NULL,
                                 VI_ERROR_PARAMETER2);   
}

/*****************************************************************************
 * Function: Set External Sweep Variation
 * Purpose:  This function sets the relative change of frequency or level in
 * the form of a percentage with effect from which a new
 * measurement is triggered in the case of external sweeps. For
 * external level sweeps the value can also be entered in dB.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetExternalSweepVariation (ViSession instrSession,
                                          ViInt32 sweepControl,
                                          ViReal64 variation)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (sweepControl, RSUPV_ANALYZER_FREQ, RSUPV_ANALYZER_VOLT, 2);

    sprintf (buffer, "TRIG:%s:VAR", SweepcontrolArr[sweepControl]);

    return rsupv_setViReal64Unit (instrSession, 
                                  buffer,
                                  variation,
                                  RSUPV_UNIT_PCT,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get External Sweep Variation
 * Purpose:  This function gets the relative change of frequency or level in
 * the form of a percentage with effect from which a new
 * measurement is triggered in the case of external sweeps. For
 * external level sweeps the value can also be entered in dB.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetExternalSweepVariation (ViSession instrSession,
                                          ViInt32  sweepControl,
                                          ViReal64 *variation)
{
   ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (sweepControl, RSUPV_ANALYZER_FREQ, RSUPV_ANALYZER_VOLT, 2);

    sprintf (buffer, "TRIG:%s:VAR?", SweepcontrolArr[sweepControl]);

    return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  variation,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3); 
}
/*****************************************************************************
 * Function: Set Analyzer Coupling
 * Purpose:  This function selects the input coupling. AC or DC coupling can
 *           be selected. The coupling can be selected separately for both
 *           channels if "Channel 1&2" is set. If AC coupling is selected, the
 *           R&S UPV has a limit frequency (-3 dB) of approx. 2 Hz. DC coupling
 *           should be selected if input voltages with very low-frequency
 *           signal components are to be measured.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerCoupling(
    ViSession   instrSession,
    ViInt32     coupling
)
{
    return rsupv_setEnum (instrSession,
                     "INP:COUP",
                      coupling,
                      anCouplingArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Coupling
 * Purpose:  This function gets the input coupling. AC or DC coupling can be
 *           selected. The coupling can be selected separately for both
 *           channels if "Channel 1&2" is set. If AC coupling is selected, the
 *           R&S UPV has a limit frequency (-3 dB) of approx. 2 Hz. DC coupling
 *           should be selected if input voltages with very low-frequency
 *           signal components are to be measured.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerCoupling(
    ViSession   instrSession,
    ViInt32*    coupling
)
{
    return rsupv_getEnum (instrSession,
                     "INP:COUP?",
                      coupling,
                      anCouplingArr,
                      VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Channel Input
 * Purpose:  This function selects the signal source for Channel 1.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerChannelInput(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     channelInput
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "INP%ld:TYPE", channel);

    return rsupv_setEnum (instrSession,
                        buffer,
                        channelInput,
                        anChInputArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analyzer Channel Input
 * Purpose:  This function gets the signal source for Channel 1.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerChannelInput(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32*    channelInput
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "INP%ld:TYPE?", channel);

    return rsupv_getEnum (instrSession,
                        buffer,
                        channelInput,
                        anChInputArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Analyzer Channel Input Impedance
 * Purpose:  This function selects the input impedance; can only be selected
 *           if Input is set to "Balanced".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerChannelInputImpedance(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     channelImped
)
{
    ViChar  buffer[BUFFER_SIZE];
       
    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "INP%ld:IMP", channel);

    return rsupv_setEnum (instrSession,
                        buffer,
                        channelImped,
                        anChImpedArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analyzer Channel Input Impedance
 * Purpose:  This function gets the input impedance; can only be selected if
 *           Input is set to "Balanced".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerChannelInputImpedance(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32*    channelImped
)
{
    ViChar  buffer[BUFFER_SIZE];
        
    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "INP%ld:IMP?", channel);

    return rsupv_getEnum (instrSession,
                        buffer,
                        channelImped,
                        anChImpedArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Analyzer Channel Input Common
 * Purpose:  This function selects the potential reference of Pin 1 of the XLR
 *           inputs; can only be selected if Input is set to "Bal". "Float"
 *           must always be set if the analyzer inputs are disabled or
 *           connected to the internal generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerChannelInputCommon(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 channelCommon
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "INP%ld:COMM", channel);

    return rsupv_setEnum (instrSession,
                        buffer,
                        channelCommon,
                        anChCommonArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analyzer Channel Input Common
 * Purpose:  This function gets the potential reference of Pin 1 of the XLR
 *           inputs; can only be selected if Input is set to "Bal". "Float"
 *           must always be set if the analyzer inputs are disabled or
 *           connected to the internal generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerChannelInputCommon(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32*    channelCommon
)
{
    ViChar  buffer[BUFFER_SIZE];
    
    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "INP%ld:COMM?", channel);

    return rsupv_getEnum (instrSession,
                        buffer,
                        channelCommon,
                        anChCommonArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Analyzer Channel Input Range
 * Purpose:  This function selects the measuring range allows optimum
 *           adaptation to a voltage range required for a specific measurement
 *           task. When the selection window is opened, the user can toggle
 *           between 3 modes.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerChannelInputRange(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     range
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 16, 2);

    sprintf (buffer, "SENS:VOLT:RANG%ld:MODE", channel);

    return rsupv_setEnum (instrSession,
                        buffer,
                        range,
                        anRangeArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analyzer Channel Input Range
 * Purpose:  This function gets the measuring range allows optimum
 *           adaptation to a voltage range required for a specific measurement
 *           task.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerChannelInputRange(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32*    range
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 16, 2);

    sprintf (buffer, "SENS:VOLT:RANG%ld:MODE?", channel);

    return rsupv_getEnum (instrSession,
                        buffer,
                        range,
                        anRangeArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Analyzer Channel Input Range Value
 * Purpose:  This function sets the nominal range value when the Fix or Lower
 *           mode is selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerChannelInputRangeValue(
    ViSession   instrSession,
    ViInt32     channel,
    ViReal64    rangeValue
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 16, 2);

    sprintf (buffer, "SENS:VOLT:RANG%ld:VAL", channel);

    return rsupv_setViReal64 (instrSession,
                              buffer,
                              rangeValue,
                              0.018, 100.0,
                              VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analyzer Channel Input Range Value
 * Purpose:  This function gets the nominal range value when the Fix or Lower
 *           mode is selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerChannelInputRangeValue(
    ViSession   instrSession,
    ViInt32     channel,
    ViReal64*   rangeValue
)
{
    ViStatus rsupv_status;
    
    CHECKVIINT32 (channel, 1, 16, 2);

    rsupv_status = rsupv_getViReal64Window (instrSession, 
                                            "SENS:VOLT:RANG%ld:VAL?",
                                            channel,
                                            rangeValue,
                                            VI_ERROR_PARAMETER3);
    
                                    
    return rsupv_status;
    
}

/*****************************************************************************
 * Function: Set I2S Analyzer Sample Frequency
 * Purpose:  This function selects the clock rate of the I2S for the analyzer. 
 *           The sampling frequency corresponds to the frequency of the FSYNC 
 *           signal. All frequency information of generated audio data is with 
 *           reference to this setting.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetI2SAnalyzerSampleFrequency (ViSession instrSession,
                                                       ViInt32 sampleFrequency,
                                                       ViReal64 sampleFrequencyValue)
{
    ViStatus    rsupv_status = VI_SUCCESS;

    CHECKOPT ("B41");
    
    if ((rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPP) == VI_SUCCESS) &&
        (sampleFrequency == RSUPV_ANALYZER_I2S_SAMPLE_FREQ_F384))
        return RSUPV_ERROR_INSTRUMENT_NSUP_MODEL;
    CHECKERR (rsupv_setEnum (instrSession,
                            "INP:SAMP:FREQ:MODE",
                            sampleFrequency,
                            I2SSampleFreqArr,
                            VI_ERROR_PARAMETER2));

    if (sampleFrequency == RSUPV_ANALYZER_I2S_SAMPLE_FREQ_VALUE)
        {
        CHECKERR (rsupv_setViReal64Unit (instrSession,
                                    "INP:SAMP:FREQ",
                                    sampleFrequencyValue,
                                    RSUPV_UNIT_HZ,
                                    6.75e3, 410.0e3,
                                    VI_ERROR_PARAMETER3));
        }

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Get I2S Analyzer Sample Frequency
 * Purpose:  This function returns the clock rate of the I2S for the analyzer. 
 *           The sampling frequency corresponds to the frequency of the FSYNC 
 *           signal. All frequency information of generated audio data is with 
 *           reference to this setting.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetI2SAnalyzerSampleFrequency (ViSession instrSession,
                                                       ViInt32 *sampleFrequency,
                                                       ViReal64 *sampleFrequencyValue)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    
    CHECKOPT ("B41");
    
    CHECKERR (rsupv_getEnum (instrSession,
                            "INP:SAMP:FREQ:MODE?",
                            sampleFrequency,
                            I2SSampleFreqArr,
                            VI_ERROR_PARAMETER2));

    CHECKERR (rsupv_getViReal64Unit (instrSession,
                                "INP:SAMP:FREQ?",
                                sampleFrequencyValue,
                                VI_NULL,
                                VI_ERROR_PARAMETER3));

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Set Analyzer Ref Impedance
 * Purpose:  This function selects the reference value for impedance.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerRefImpedance(
    ViSession   instrSession,
    ViReal64    refImped
)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SENS:POW:REF:RES",
                              refImped,
                              RSUPV_UNIT_OHM,
                              100.0e-6, 100.0e3,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Ref Impedance
 * Purpose:  This function gets the reference value for impedance.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerRefImpedance(
    ViSession   instrSession,
    ViReal64*   refImped
)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SENS:POW:REF:RES?",
                              refImped,
                              VI_NULL,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Multichannel Analyzer Meas Channels
 * Purpose:  This function enable or disables desired channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMultichannelAnalyzerMeasChannels (ViSession instrSession,
                                                             ViInt32 channel,
                                                             ViBoolean channelState)
{
    ViChar buffer[BUFFER_SIZE];
    CHECKOPT ("B48");
    
    CHECKVIINT32 (channel, 1, 16, 2);
    
    sprintf (buffer, "INP:MCH%ld", channel);
    
    return rsupv_setViBoolean (instrSession,
                               buffer,
                               channelState,
                               VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Multichannel Analyzer Meas Channels
 * Purpose:  This function queries state of desired channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMultichannelAnalyzerMeasChannels (ViSession instrSession,
                                                             ViInt32 channel,
                                                             ViBoolean *channelState)
{
    ViChar buffer[BUFFER_SIZE];
    CHECKOPT ("B48");
    
    CHECKVIINT32 (channel, 1, 16, 2);
    
    sprintf (buffer, "INP:MCH%ld?", channel);
    
    return rsupv_getViBoolean (instrSession,
                               buffer,
                               channelState,
                               VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Multichannel Analyzer Coupling Mode
 * Purpose:  This function selects the input coupling mode. The coupling can
 *           be selected for all channels together (Track) or separately from
 *           each other (Split).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMultichannelAnalyzerCouplingMode (ViSession instrSession,
                                                             ViInt32 coupling)
{
    CHECKOPT ("B48");
    
    return rsupv_setEnum (instrSession,
                          "INP:COUP:CHAN",
                          coupling,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Multichannel Analyzer Coupling Mode
 * Purpose:  This function queries the input coupling mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMultichannelAnalyzerCouplingMode (ViSession instrSession,
                                                             ViInt32 *coupling)
{
    CHECKOPT ("B48");
    
    return rsupv_getEnum (instrSession,
                          "INP:COUP:CHAN?",
                          coupling,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Multichannel Analyzer Coupling
 * Purpose:  This function selects the input coupling. AC or DC coupling can
 *           be selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMultichannelAnalyzerCoupling (ViSession instrSession,
                                                         ViInt32 channel,
                                                         ViInt32 coupling)
{
    ViChar buffer[BUFFER_SIZE];
    CHECKOPT ("B48");
    
    CHECKVIINT32 (channel, 1, 16, 2);
    
    sprintf (buffer, "INP%ld:COUP", channel);
    
    return rsupv_setEnum (instrSession,
                          buffer,
                          coupling,
                          anCouplingArr,
                          VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Multichannel Analyzer Coupling
 * Purpose:  This function queries the input coupling.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMultichannelAnalyzerCoupling (ViSession instrSession,
                                                         ViInt32 channel,
                                                         ViInt32 *coupling)
{
    ViChar buffer[BUFFER_SIZE];
    CHECKOPT ("B48");
    
    CHECKVIINT32 (channel, 1, 16, 2);
    
    sprintf (buffer, "INP%ld:COUP?", channel);
    
    return rsupv_getEnum (instrSession,
                          buffer,
                          coupling,
                          anCouplingArr,
                          VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Multichannel Analyzer Channel Range
 * Purpose:  This function selects the measurement range mode. The range can
 *           be selected for all channels together (Track) or separately from
 *           each other (Split).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMultichannelAnalyzerChannelRange (ViSession instrSession,
                                                             ViInt32 coupling)
{
    CHECKOPT ("B48");
    
    return rsupv_setEnum (instrSession,
                          "INP:RANG:CHAN",
                          coupling,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Multichannel Analyzer Channel Range
 * Purpose:  This function queries the measurement range mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMultichannelAnalyzerChannelRange (ViSession instrSession,
                                                             ViInt32 *coupling)
{
    CHECKOPT ("B48");
    
    return rsupv_getEnum (instrSession,
                          "INP:RANG:CHAN?",
                          coupling,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Multichannel Analyzer Reference Channel
 * Purpose:  This function selects reference channel for phase measurements,
 *           channel-specific measurements such as common-channel selectivity
 *           and the like.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMultichannelAnalyzerReferenceChannel (ViSession instrSession,
                                                                 ViInt32 channel)
{
    ViChar buffer[BUFFER_SIZE];
    CHECKOPT ("B48");
    
    CHECKVIINT32 (channel, 1, 16, 2);
    
    sprintf (buffer, "SENS:REF:CHAN CH%ld", channel);
    
    return rsupv_sendCmd (instrSession, buffer);    
}

/*****************************************************************************
 * Function: Get Multichannel Analyzer Reference Channel
 * Purpose:  This function queries reference channel for phase measurements,
 *           channel-specific measurements such as common-channel selectivity
 *           and the like.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMultichannelAnalyzerReferenceChannel (ViSession instrSession,
                                                                 ViInt32 *channel)
{
    ViChar buffer[BUFFER_SIZE];
    ViStatus rsupv_status = VI_SUCCESS;
    CHECKOPT ("B48");
    
    rsupv_status = rsupv_getViString (instrSession,
                             "SENS:REF:CHAN?",
                             buffer,
                             BUFFER_SIZE,
                             0);
    
    if (channel != NULL)
        sscanf (buffer, "CH%ld", channel);
    else
        rsupv_status = VI_ERROR_PARAMETER2;
            
    return rsupv_status;
}


/*****************************************************************************
 * Function: Set Multichannel Analyzer Impedance Channel
 * Purpose:  This function selects the input impedance for all 8 channels
  *          together (Track) or separately for each channel (Split).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMultichannelAnalyzerImpedanceChannel (ViSession instrSession,
                                                            ViInt32 channel,
                                                             ViInt32 impedance)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKINSTR (RSUPV_INSTR_UPP); 
    CHECKVIINT32 (channel, 1, 8, 2);
    
    sprintf (buffer, "INP:MIMP%ld:CHAN %s", channel, anCouplingModeArr[impedance]);
    
    return rsupv_sendCmd (instrSession, buffer);    

}
  

/*****************************************************************************
 * Function: Get Multichannel Analyzer Impedance Channel
 * Purpose:  This function queries the measurement range mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMultichannelAnalyzerImpedanceChannel (ViSession instrSession,
                                                            ViInt32 channel,    
                                                             ViInt32 *impedance)
{
    ViChar  buffer[BUFFER_SIZE];
    
    CHECKINSTR (RSUPV_INSTR_UPP); 
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "INP:MIMP%ld:CHAN?", channel);

    return rsupv_getEnum (instrSession,
                          buffer,
                          impedance,
                          anCouplingModeArr,
                          VI_ERROR_PARAMETER2); 
 }
    

/*****************************************************************************
 * Function: Set Analyzer Channel Input Impedance
 * Purpose:  This function selects the input impedance; can only be selected
 *           if Input is set to "Balanced".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMultichannelAnalyzerChannelInputImpedance(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     channelImped
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKINSTR (RSUPV_INSTR_UPP); 
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "INP:MIMP%ld", channel);

    return rsupv_setEnum (instrSession,
                        buffer,
                        channelImped,
                        anChImpedArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analyzer Channel Input Impedance
 * Purpose:  This function gets the input impedance; can only be selected if
 *           Input is set to "Balanced".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMultichannelAnalyzerChannelInputImpedance(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32*    channelImped
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKINSTR (RSUPV_INSTR_UPP);
    CHECKVIINT32 (channel, 1, 8, 2);

    sprintf (buffer, "INP:MIMP%ld?", channel);

    return rsupv_getEnum (instrSession,
                        buffer,
                        channelImped,
                        anChImpedArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Multichannel Analyzer Trigger Channel
 * Purpose:  This function selects the trigger channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMultichannelAnalyzerTriggerChannel (ViSession instrSession,
                                                               ViInt32 channel)
{
    ViChar buffer[BUFFER_SIZE] = "";
    ViStatus    rsupv_status = VI_SUCCESS;
    
    CHECKOPT ("B48");
    
    if (channel == 0)
        strcpy (buffer, "TRIG:CHAN OFF");
    else
        sprintf (buffer, "TRIG:CHAN CH%ld", channel);
    
    CHECKERR (rsupv_sendCmd (instrSession, buffer));
    
    return rsupv_status;
    
}

/*****************************************************************************
 * Function: Get Multichannel Analyzer Trigger Channel
 * Purpose:  This function queries the trigger channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMultichannelAnalyzerTriggerChannel (ViSession instrSession,
                                                               ViInt32 *channel)
{
    ViChar buffer[BUFFER_SIZE];
    ViStatus rsupv_status = VI_SUCCESS;
    CHECKOPT ("B48");
    
    if (channel == NULL) return VI_ERROR_PARAMETER2;
    
    rsupv_status = rsupv_getViString (instrSession,
                             "TRIG:CHAN?",
                             buffer,
                             BUFFER_SIZE,
                             0);
    
    if (strncmp (buffer, "OFF", 3) != 0)
        sscanf (buffer, "CH%ld", channel);
    else
        *channel = 0;
            
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Digital Analyzer Meas Mode
 * Purpose:  This function specifies what is to be measured at the digital
 *           interfaces.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalAnalyzerMeasMode(
    ViSession   instrSession,
    ViInt32     measMode
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                        "SENS:DIG:MMOD",
                        measMode,
                        anMeasModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Analyzer Meas Mode
 * Purpose:  This function gets what is to be measured at the digital
 *           interfaces.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalAnalyzerMeasMode(
    ViSession   instrSession,
    ViInt32*    measMode
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                        "SENS:DIG:MMOD?",
                        measMode,
                        anMeasModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Analyzer Jitter Ref
 * Purpose:  This function selects the jitter reference clock (only available
 *           for jitter measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalAnalyzerJitterRef(
    ViSession   instrSession,
    ViInt32     jitterRef
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                        "SENS:DIG:JITT:REF",
                        jitterRef,
                        anJitterRefArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Analyzer Jitter Ref
 * Purpose:  This function gets the jitter reference clock (only available for
 *           jitter measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalAnalyzerJitterRef(
    ViSession   instrSession,
    ViInt32*    jitterRef
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                        "SENS:DIG:JITT:REF?",
                        jitterRef,
                        anJitterRefArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Sample Frequency
 * Purpose:  This function selects the clock rate of the digital audio signal
 *           for the analyzer. The sample frequency setting is used as a
 *           reference for all audio data frequency measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSampleFrequency(
    ViSession   instrSession,
    ViInt32     sampleFrequency,
    ViReal64    sampleFrequencyValue
)
{
    ViStatus rsupv_status;

    if (sampleFrequency == RSUPV_ANALYZER_SAMPLE_FREQ_CHSTAT)
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
    CHECKERR (rsupv_setEnum (instrSession,
                            "INP:SAMP:FREQ:MODE",
                            sampleFrequency,
                            anSampleFreqArr,
                            VI_ERROR_PARAMETER2));

    if (sampleFrequency == RSUPV_ANALYZER_SAMPLE_FREQ_VALUE)
        {
        CHECKERR (rsupv_setViReal64Unit (instrSession,
                                    "INP:SAMP:FREQ",
                                    sampleFrequencyValue,
                                    RSUPV_UNIT_HZ,
                                    30.0e3, 200.0e3,
                                    VI_ERROR_PARAMETER3));
        }

    return rsupv_status;
}

/*****************************************************************************
 * Function: Get Analyzer Sample Frequency
 * Purpose:  This function gets the clock rate of the digital audio signal for
 *           the analyzer. The sample frequency setting is used as a reference
 *           for all audio data frequency measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSampleFrequency(
    ViSession   instrSession,
    ViInt32*    sampleFrequency,
    ViReal64*   sampleFrequencyValue
)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    
    CHECKERR (rsupv_getEnum (instrSession,
                            "INP:SAMP:FREQ:MODE?",
                            sampleFrequency,
                            anSampleFreqArr,
                            VI_ERROR_PARAMETER2));

    CHECKERR (rsupv_getViReal64Unit (instrSession,
                                "INP:SAMP:FREQ?",
                                sampleFrequencyValue,
                                VI_NULL,
                                VI_ERROR_PARAMETER3));

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Digital Analyzer Audio Bits
 * Purpose:  This function sets the wordwidth used by the digital audio
 *           analyzer in the Measurement Mode "Audio Data".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalAnalyzerAudioBits(
    ViSession   instrSession,
    ViInt32     audioBits
)
{
    return rsupv_setViInt32 (instrSession,
                            "INP:AUD",
                            audioBits,
                            8, 24,
                            VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Analyzer Audio Bits
 * Purpose:  This function gets the wordwidth used by the digital audio
 *           analyzer in the Measurement Mode "Audio Data".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalAnalyzerAudioBits(
    ViSession   instrSession,
    ViInt32*    audioBits
)
{
    return rsupv_getViInt32 (instrSession,
                            "INP:AUD?",
                            audioBits,
                            VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Digital Analyzer Audio Input
 * Purpose:  This function selects the input interface for the digital audio
 *           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigitalAnalyzerAudioInput(
    ViSession   instrSession,
    ViInt32     digitalAudioInput
)
{
    return rsupv_setEnum (instrSession,
                        "INP:TYPE",
                        digitalAudioInput,
                        anDigitalAudioInputArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Digital Analyzer Audio Input
 * Purpose:  This function gets the input interface for the digital audio
 *           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigitalAnalyzerAudioInput(
    ViSession   instrSession,
    ViInt32*    digitalAudioInput
)
{
    return rsupv_getEnum (instrSession,
                        "INP:TYPE?",
                        digitalAudioInput,
                        anDigitalAudioInputArr,
                        VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Configure Dig Bitstream Analyzer
 * Purpose:  This function cofigures the Digital Bitstream Analyzer
 *           parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureDigBitstreamAnalyzer (ViSession instrSession,
                                                       ViInt32 clockSource,
                                                       ViInt32 channelMode,
                                                       ViInt32 alignment,
                                                       ViInt32 dwnsmplFact,
                                                       ViReal64 clockFrequency,
                                                       ViReal64 dutyCycle)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_SetDigBitstreamAnalyzerClockSource (instrSession, clockSource));
    
    rsupv_status = rsupv_SetDigBitstreamAnalyzerChannelMode (instrSession, channelMode);
    REMAP (2,3);

    rsupv_status = rsupv_SetDigBitstreamAnalyzerAlignment (instrSession, alignment);
    REMAP (2,4);
       
    rsupv_status = rsupv_SetDigBitstreamAnalyzerDownSamplingFactor (instrSession, dwnsmplFact);
    REMAP (2,5);
    
    rsupv_status = rsupv_SetDigBitstreamAnalyzerClockFrequency (instrSession, clockFrequency);
    REMAP (2,6);
    
    rsupv_status = rsupv_SetDigBitstreamAnalyzerDutyCycle (instrSession, dutyCycle);
    REMAP (2,7);
    
    return rsupv_status;
}      
       
/*****************************************************************************
 * Function: Set Dig Bitstream Analyzer Alignment
 * Purpose:  This function selects the relationship of clock and data to each
 *           other. Depending on whether a mono or stereo format is to be
 *           analyzed, different modes can be selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigBitstreamAnalyzerAlignment(
    ViSession   instrSession,
    ViInt32     alignment
)
{
    CHECKINSTR (RSUPV_INSTR_UPV); 
        
    return rsupv_setEnum (instrSession,
                        "INP:ALIG",
                        alignment,
                        anDigitalAligmentArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Dig Bitstream Analyzer Alignment
 * Purpose:  This function returns the relationship of clock and data to each
 *           other. Depending on whether a mono or stereo format is to be
 *           analyzed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigBitstreamAnalyzerAlignment(
    ViSession   instrSession,
    ViInt32*    alignment
)
{
    CHECKINSTR (RSUPV_INSTR_UPV); 
    
    return rsupv_getEnum (instrSession,
                        "INP:ALIG? ",
                        alignment,
                        anDigitalAligmentArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Dig Bitstream Analyzer Clock Source
 * Purpose:  This function selects the clock source for the bit stream
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigBitstreamAnalyzerClockSource(
    ViSession   instrSession,
    ViInt32     clockSource
)
{
    CHECKINSTR (RSUPV_INSTR_UPV); 
        
    return rsupv_setEnum (instrSession,
                        "INP:CLKS",
                        clockSource,
                        anDigitalClockSourceArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Dig Bitstream Analyzer Clock Source
 * Purpose:  This function returns the clock source for the bit stream
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigBitstreamAnalyzerClockSource(
    ViSession   instrSession,
    ViInt32*    clockSource
)
{
    CHECKINSTR (RSUPV_INSTR_UPV); 
    
    return rsupv_getEnum (instrSession,
                        "INP:CLKS? ",
                        clockSource,
                        anDigitalClockSourceArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Dig Bitstream Analyzer Channel Mode
 * Purpose:  This control selects the input format of the data lines.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigBitstreamAnalyzerChannelMode(
    ViSession   instrSession,
    ViInt32     channelMode
)
{
    CHECKINSTR (RSUPV_INSTR_UPV); 
        
    return rsupv_setEnum (instrSession,
                        "INP:CHM",
                        channelMode,
                        anDigitalChModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Dig Bitstream Analyzer Channel Mode
 * Purpose:  This control returns the input format of the data lines.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigBitstreamAnalyzerChannelMode(
    ViSession   instrSession,
    ViInt32*    channelMode
)
{
    CHECKINSTR (RSUPV_INSTR_UPV); 
    
    return rsupv_getEnum (instrSession,
                        "INP:CHM? ",
                        channelMode,
                        anDigitalChModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function Set Dig Bitstream Analyzer Clock Frequency
 * Purpose: If the Bitstream Analyzer is configured as clock master (Clock
 *          Source = Intern), this parameter defines the frequency of the
 *          output clock.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigBitstreamAnalyzerClockFrequency(
    ViSession   instrSession,
    ViReal64    clockFrequency
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);  
    return rsupv_setViReal64 (instrSession,
                              "INP:CLKF",
                              clockFrequency,
                              512.0, 12800.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Dig Bitstream Analyzer Clock Frequency
 * Purpose:  This function returns the frequency of the output clock.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigBitstreamAnalyzerClockFrequency(
    ViSession   instrSession,
    ViReal64*   clockFrequency
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);  
    return rsupv_getViReal64 (instrSession,
                              "INP:CLKF?",
                              clockFrequency,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function Set Dig Bitstream Analyzer Duty Cycle
 * Purpose: If the Bitstream Analyzer is configured as clock master (Clock
 *          Source = Intern), this parameter defines the duty cycle of the
 *          output clock.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigBitstreamAnalyzerDutyCycle(
    ViSession   instrSession,
    ViReal64    dutyCycle
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);  
    return rsupv_setViReal64 (instrSession,
                              "INP:DTYC",
                              dutyCycle,
                              10.0, 90.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Dig Bitstream Analyzer Duty Cycle
 * Purpose:  This function returns the duty cycle of the output clock.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigBitstreamAnalyzerDutyCycle(
    ViSession   instrSession,
    ViReal64*   dutyCycle 
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);  
    return rsupv_getViReal64 (instrSession,
                              "INP:DTYC?",
                              dutyCycle,
                              VI_ERROR_PARAMETER2);
}

       
/*****************************************************************************
 * Function: Set Dig Bitstream Analyzer Down Sampling Factor
 * Purpose:  This function defines the down-sampling factor. A filter bank is
 *           configured with the different factors depending on the defined
 *           value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDigBitstreamAnalyzerDownSamplingFactor(
    ViSession   instrSession,
    ViInt32     dwnsmlpFactor
)
{
    CHECKINSTR (RSUPV_INSTR_UPV); 
        
    return rsupv_setEnum (instrSession,
                        "INP:DWNF",
                        dwnsmlpFactor,
                        anDigitalDwnsmlpFactorArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Dig Bitstream Analyzer Down Sampling Factor
 * Purpose:  This function returns the down-sampling factor. A filter bank is
 *           configured with the different factors depending on the defined
 *           value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDigBitstreamAnalyzerDownSamplingFactor(
    ViSession   instrSession,
    ViInt32*    dwnsmlpFactor
)
{
    CHECKINSTR (RSUPV_INSTR_UPV); 
    
    return rsupv_getEnum (instrSession,
                        "INP:DWNF? ",
                        dwnsmlpFactor,
                        anDigitalDwnsmlpFactorArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Function
 * Purpose:  This function selects the analyzer function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFunction(
    ViSession   instrSession,
    ViInt32     function
)
{
    switch (function)
    {
        case RSUPV_ANALYZER_FUNC_QPE:
        case RSUPV_ANALYZER_FUNC_DIM:
        case RSUPV_ANALYZER_FUNC_REC:
        case RSUPV_ANALYZER_FUNC_NOCT:
        case RSUPV_ANALYZER_FUNC_RUBB:
        case RSUPV_ANALYZER_FUNC_PEAQ:
        case RSUPV_ANALYZER_FUNC_PESQ:
        case RSUPV_ANALYZER_FUNC_POLQA:
        case RSUPV_ANALYZER_FUNC_PLUGIN:
            CHECKINSTR (RSUPV_INSTR_UPV);
        break;
    }
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC",
                        function,
                        anFunctionArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Function
 * Purpose:  This function gets the analyzer function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFunction(
    ViSession   instrSession,
    ViInt32*    function
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC?",
                        function,
                        anFunctionArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Measurement Functions Settling
 * Purpose:  This function activates the settling measurement for results of
 *           measurement functions and to select the settling method. Results
 *           of measurement functions are valid only if the selected settling
 *           condition is fulfilled.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMeasurementFunctionsSettling (ViSession instrSession,
                                                ViInt32 fnctSettling)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:SETT:MODE",
                        fnctSettling,
                        FnctSettlingArr,
                        VI_ERROR_PARAMETER2);
    
}

/*****************************************************************************
 * Function: Get Measurement Functions Settling
 * Purpose:  This function queries the settling measurement for results of
 * measurement functions and returns the settling method.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMeasurementFunctionsSettling (ViSession instrSession,
                                                ViInt32 *fnctSettling)
{
   return rsupv_getEnum (instrSession,
                        "SENS:FUNC:SETT:MODE?",
                        fnctSettling,
                        FnctSettlingArr,
                        VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Measurement Function Settling Count
 * Purpose:  This function sets the number of measured values to be covered
 * by the settling algorithm, i.e. it defines how many measured
 * values are to be used in the settling measurement or for
 * averaging.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMeasurementFunctionSettlingCount (ViSession instrSession,
                                         ViInt32 samples)
{
   return rsupv_setViInt32 (instrSession,
                            "SENS:FUNC:SETT:COUN",
                             samples,
                             0, 0,
                             VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Measurement Function Settling Count
 * Purpose:  This function gets the number of measured values to be covered
 * by the settling algorithm, i.e. it defines how many measured
 * values are to be used in the settling measurement or for
 * averaging.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMeasurementFunctionSettlingCount (ViSession instrSession,
                                         ViInt32 *samples)
{
   return rsupv_getViInt32 (instrSession,
                            "SENS:FUNC:SETT:COUN?",
                             samples,
                             VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Measurement Function Settling Resolution
 * Purpose:  This function sets the settling resolution.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMeasurementFunctionSettlingResolution (ViSession instrSession,
                                                                  ViReal64 resolution)
{
   return rsupv_setViReal64Unit (instrSession,
                                "SENS:FUNC:SETT:RES",
                                resolution,
                                RSUPV_UNIT_V,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER2);
    
}

/*****************************************************************************
 * Function: Get Measurement Function Settling Resolution
 * Purpose:  This function gets the settling resolution.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMeasurementFunctionSettlingResolution (ViSession instrSession,
                                                                  ViReal64 *resolution)
{
   return rsupv_getViReal64Unit (instrSession,
                              "SENS:FUNC:SETT:RES?",
                               resolution,
                               VI_NULL,
                               VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Measurement Function Settling Tolerance
 * Purpose:  This function sets the settling tolerance.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMeasurementFunctionSettlingTolerance (ViSession instrSession,
                                                                 ViReal64 tolerance)
{
   return rsupv_setViReal64Unit (instrSession,
                                "SENS:FUNC:SETT:TOL",
                                tolerance,
                                RSUPV_UNIT_PCT,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Measurement Function Settling Tolerance
 * Purpose:  This function gets the settling tolerance.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMeasurementFunctionSettlingTolerance (ViSession instrSession,
                                                                 ViReal64 *tolerance)
{
   return rsupv_getViReal64Unit (instrSession,
                              "SENS:FUNC:SETT:TOL?",
                               tolerance,
                               VI_NULL,
                               VI_ERROR_PARAMETER2);  
}

/*****************************************************************************
 * Function: Set Measurement Function Settling Timeout
 * Purpose:  This function sets the settling timeout.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMeasurementFunctionSettlingTimeout (ViSession instrSession,
                                                               ViReal64 timeout)
{
   return rsupv_setViReal64Unit (instrSession,
                                "SENS:FUNC:SETT:TOUT",
                                timeout,
                                RSUPV_UNIT_S,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Measurement Function Settling Timeout
 * Purpose:  This function returns the settling timeout.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMeasurementFunctionSettlingTimeout (ViSession instrSession,
                                                               ViReal64 *timeout)
{
    return rsupv_getViReal64Unit (instrSession,
                              "SENS:FUNC:SETT:TOUT?",
                               timeout,
                               VI_NULL,
                               VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Aux Trigger Input Enable
 * Purpose:  This function sets the state of the aux trigger input.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxTriggerInputEnable (ViSession instrSession,
                                                  ViBoolean triggerInput)
{
    return rsupv_setViBoolean (instrSession,
                               "AUX:TRIG:INP:ENAB",
                               triggerInput,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Aux Trigger Input Enable
 * Purpose:  This function gets the state of the aux trigger input.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxTriggerInputEnable (ViSession instrSession,
                                                  ViBoolean *triggerInput)
{
    return rsupv_getViBoolean (instrSession,
                               "AUX:TRIG:INP:ENAB?",
                               triggerInput,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Aux Trigger Input Mode
 * Purpose:  This function selects the aux trigger input mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxTriggerInputMode (ViSession instrSession,
                                                ViInt32 triggerInputMode)
{
    return rsupv_setEnum (instrSession,
                          "AUX:TRIG:INP:MODE",
                          triggerInputMode,
                          auxTriggerInputModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Aux Trigger Input Mode
 * Purpose:  This function returns the aux trigger input mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxTriggerInputMode (ViSession instrSession,
                                                ViInt32 *triggerInputMode)
{
    return rsupv_getEnum (instrSession,
                          "AUX:TRIG:INP:MODE?",
                          triggerInputMode,
                          auxTriggerInputModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Aux Trigger Input Edge
 * Purpose:  This function selects whether you want the R&S UPV to react on 
 *           the rising or falling edge of the input signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxTriggerInputEdge (ViSession instrSession,
                                                ViInt32 triggerInputEdge)
{
    return rsupv_setEnum (instrSession,
                          "AUX:TRIG:INP:EDGE",
                          triggerInputEdge,
                          auxTriggerInputEdgeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Aux Trigger Input Edge
 * Purpose:  This function returns whether you want the R&S UPV to react on 
 *           the rising or falling edge of the input signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxTriggerInputEdge (ViSession instrSession,
                                                ViInt32 *triggerInputEdge)
{
    return rsupv_getEnum (instrSession,
                          "AUX:TRIG:INP:EDGE?",
                          triggerInputEdge,
                          auxTriggerInputEdgeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Aux Trigger Output Enable
 * Purpose:  This function sets the state of the aux trigger output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxTriggerOutputEnable (ViSession instrSession,
                                                   ViBoolean triggerOutput)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViBoolean (instrSession,
                               "AUX:TRIG:OUTP:ENAB",
                               triggerOutput,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Aux Trigger Output Enable
 * Purpose:  This function gets the state of the aux trigger output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxTriggerOutputEnable (ViSession instrSession,
                                                   ViBoolean *triggerOutput)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViBoolean (instrSession,
                               "AUX:TRIG:OUTP:ENAB?",
                               triggerOutput,
                               VI_ERROR_PARAMETER2);     
}

/*****************************************************************************
 * Function: Set Aux Trigger Output Mode
 * Purpose:  This function selects the aux trigger output mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxTriggerOutputMode (ViSession instrSession,
                                                 ViInt32 triggerOutputMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "AUX:TRIG:OUTP:MODE",
                          triggerOutputMode,
                          auxTriggerOutputModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Aux Trigger Output Mode
 * Purpose:  This function returns the aux trigger output mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxTriggerOutputMode (ViSession instrSession,
                                                 ViInt32 *triggerOutputMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "AUX:TRIG:OUTP:MODE?",
                          triggerOutputMode,
                          auxTriggerOutputModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Aux Trigger Output Frequency
 * Purpose:  This function sets the frequency of the DDS oscillator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxTriggerOutputFrequency (ViSession instrSession,
                                                      ViReal64 triggerOutputFrequency)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                                  "AUX:TRIG:OUTP:FREQ",
                                  triggerOutputFrequency,
                                  RSUPV_UNIT_S,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Aux Trigger Output Frequency
 * Purpose:  This function gets the frequency of the DDS oscillator.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxTriggerOutputFrequency (ViSession instrSession,
                                                      ViReal64 *triggerOutputFrequency)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                                  "AUX:TRIG:OUTP:FREQ?",
                                  triggerOutputFrequency,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Aux Trigger Output Edge
 * Purpose:  This function selects whether you want the R&S UPV to output a 
 *           high or low level during measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxTriggerOutputEdge (ViSession instrSession,
                                                 ViInt32 triggerOutputEdge)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "AUX:TRIG:OUTP:EDGE",
                          triggerOutputEdge,
                          auxTriggerInputEdgeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Aux Trigger Output Edge
 * Purpose:  This function returns whether you want the R&S UPV to output a 
 *           high or low level during measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxTriggerOutputEdge (ViSession instrSession,
                                                 ViInt32 *triggerOutputEdge)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "AUX:TRIG:OUTP:EDGE?",
                          triggerOutputEdge,
                          auxTriggerInputEdgeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Switcher State
 * Purpose:  This function sets the state of the switcher.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetSwitcherState (ViSession instrSession,
                                          ViBoolean switcherState)
{
    return rsupv_setViBoolean (instrSession,
                               "SWIT:STAT",
                               switcherState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Switcher State
 * Purpose:  This function gets the state of the switcher.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetSwitcherState (ViSession instrSession,
                                          ViBoolean *switcherState)
{
    return rsupv_getViBoolean (instrSession,
                               "SWIT:STAT?",
                               switcherState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Switcher Tracking
 * Purpose:  This function selects the switcher tracking.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetSwitcherTracking (ViSession instrSession,
                                             ViInt32 switcherTracking)
{
    return rsupv_setEnum (instrSession,
                          "SWIT:TRAC",
                          switcherTracking,
                          switcherTrackingArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Switcher Tracking
 * Purpose:  This function returns the switcher tracking.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetSwitcherTracking (ViSession instrSession,
                                             ViInt32 *switcherTracking)
{
    return rsupv_getEnum (instrSession,
                          "SWIT:TRAC?",
                          switcherTracking,
                          switcherTrackingArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Switcher Offset
 * Purpose:  This function selects the switcher offset.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetSwitcherOffset (ViSession instrSession,
                                           ViInt32 switcherOffset,
                                           ViInt32 offsetValue)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (switcherOffset)
    {
        case RSUPV_SWITCHER_OFFSET_BVSA:
            sprintf (buffer, "SWIT:OFFS:BVSA");
        break;
        case RSUPV_SWITCHER_OFFSET_OVSI:
            sprintf (buffer, "SWIT:OFFS:OVSI");
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViInt32 (instrSession, 
                              buffer,
                              offsetValue,
                             -128, 128,
                              VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Switcher Offset
 * Purpose:  This function returns the switcher offset.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetSwitcherOffset (ViSession instrSession,
                                           ViInt32 switcherOffset,
                                           ViInt32 *offsetValue)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (switcherOffset)
    {
        case RSUPV_SWITCHER_OFFSET_BVSA:
            sprintf (buffer, "SWIT:OFFS:BVSA?");
        break;
        case RSUPV_SWITCHER_OFFSET_OVSI:
            sprintf (buffer, "SWIT:OFFS:OVSI?");
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViInt32 (instrSession, 
                              buffer,
                              offsetValue,
                              VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Switcher Input
 * Purpose:  This function sets the channel number of the input switcher that 
 *           is to be connected to busbar A or B.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetSwitcherInput (ViSession instrSession,
                                          ViInt32 switcherInput,
                                          ViInt32 inputChannelNumber)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (switcherInput)
    {
        case RSUPV_SWITCHER_A:
            sprintf (buffer, "SWIT:INPA");
        break;
        case RSUPV_SWITCHER_B:
            sprintf (buffer, "SWIT:INPB");
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViInt32 (instrSession, 
                             buffer,
                             inputChannelNumber,
                             0, 0,
                             VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Switcher Input
 * Purpose:  This function gets the channel number of the input switcher that 
 *           is to be connected to busbar A or B.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetSwitcherInput (ViSession instrSession,
                                          ViInt32 switcherInput,
                                          ViInt32 *inputChannelNumber)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (switcherInput)
    {
        case RSUPV_SWITCHER_A:
            sprintf (buffer, "SWIT:INPA?");
        break;
        case RSUPV_SWITCHER_B:
            sprintf (buffer, "SWIT:INPB?");
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViInt32 (instrSession, 
                             buffer,
                             inputChannelNumber,
                             VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Switcher Output
 * Purpose:  This function sets the channel number of the output switcher 
 *           that is to be connected to busbar A or B.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetSwitcherOutput (ViSession instrSession,
                                           ViInt32 switcherOutput,
                                           ViInt32 outputChannelNumber)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (switcherOutput)
    {
        case RSUPV_SWITCHER_A:
            sprintf (buffer, "SWIT:OUTA");
        break;
        case RSUPV_SWITCHER_B:
            sprintf (buffer, "SWIT:OUTB");
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViInt32 (instrSession, 
                             buffer,
                             outputChannelNumber,
                             0, 0,
                             VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Switcher Output
 * Purpose:  This function gets the channel number of the output switcher 
 *           that is to be connected to busbar A or B.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetSwitcherOutput (ViSession instrSession,
                                           ViInt32 switcherOutput,
                                           ViInt32 *outputChannelNumber)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (switcherOutput)
    {
        case RSUPV_SWITCHER_A:
            sprintf (buffer, "SWIT:OUTA?");
        break;
        case RSUPV_SWITCHER_B:
            sprintf (buffer, "SWIT:OUTB?");
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViInt32 (instrSession, 
                             buffer,
                             outputChannelNumber,
                             VI_ERROR_PARAMETER3);     
}

/*****************************************************************************
 * Function: Set Switcher Connection
 * Purpose:  This function selects the switcher connection.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetSwitcherConnection (ViSession instrSession, ViInt32 connection)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
   
    return rsupv_setEnum (instrSession,
                          "SWIT:CONN",
                          connection,
                          connectionArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Switcher Connection
 * Purpose:  This function queries the switcher connection.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetSwitcherConnection (ViSession instrSession, ViInt32 *connection)
{

    CHECKINSTR (RSUPV_INSTR_UPV);
       
    return rsupv_getEnum (instrSession,
                          "SWIT:CONN?",
                          connection,
                          connectionArr,
                          VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Switcher Port
 * Purpose:  This function selects the COM port.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetSwitcherPort (ViSession instrSession, ViInt32 comPort)
{
   
    CHECKINSTR (RSUPV_INSTR_UPP);
   return rsupv_setEnum (instrSession,
                          "SWIT:COMP",
                          comPort,
                          ComPortArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Switcher Port
 * Purpose:  This function queries the selected COM port.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetSwitcherPort (ViSession instrSession, ViInt32 *comPort)
{
    CHECKINSTR (RSUPV_INSTR_UPP);
    return rsupv_getEnum (instrSession,
                          "SWIT:COMP?",
                          comPort,
                          ComPortArr,
                          VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Protocol Analysis State
 * Purpose:  This function sets the state of the protocol analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolAnalysisState (ViSession instrSession,
                                                  ViBoolean protocolAnalysisState)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViBoolean (instrSession,
                               "SENS8:FUNC",
                               protocolAnalysisState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Analysis State
 * Purpose:  This function gets the state of the protocol analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolAnalysisState (ViSession instrSession,
                                                  ViBoolean *protocolAnalysisState)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViBoolean (instrSession,
                               "SENS8:FUNC?",
                               protocolAnalysisState,
                               VI_ERROR_PARAMETER2);     
}

/*****************************************************************************
 * Function: Set Protocol Analysis Display State
 * Purpose:  This function sets the state of the protocol analyzer display.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolAnalysisDisplayState (ViSession instrSession,
                                                         ViBoolean protocolAnalysisDisplayState)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViBoolean (instrSession,
                               "SENS8:PROT:DISP",
                               protocolAnalysisDisplayState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Analysis Display State
 * Purpose:  This function gets the state of the protocol analyzer display.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolAnalysisDisplayState (ViSession instrSession,
                                                         ViBoolean *protocolAnalysisDisplayState)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViBoolean (instrSession,
                               "SENS8:PROT:DISP?",
                               protocolAnalysisDisplayState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Protocol Analysis Display Mode
 * Purpose:  This function sets the display format of the channel-status 
 *           information.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolAnalysisDisplayMode (ViSession instrSession,
                                                        ViInt32 displayMode)
{
    return rsupv_setEnum (instrSession,
                          "SENS8:PROT:MODE",
                          displayMode,
                          protDisplayModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Analysis Display Mode
 * Purpose:  This function gets the display format of the channel-status 
 *           information.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolAnalysisDisplayMode (ViSession instrSession,
                                                        ViInt32 *displayMode)
{
    return rsupv_getEnum (instrSession,
                          "SENS8:PROT:MODE?",
                          displayMode,
                          protDisplayModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Protocol Analysis Highlight
 * Purpose:  This function sets when channel-status information is to
 *           be highlighted.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolAnalysisHighlight (ViSession instrSession,
                                                      ViInt32 highlight)
{
    return rsupv_setEnum (instrSession,
                          "SENS8:PROT:HIGH",
                          highlight,
                          protHighlightArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Analysis Highlight
 * Purpose:  This function gets when channel-status information is to
 *           be highlighted.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolAnalysisHighlight (ViSession instrSession,
                                                      ViInt32 *highlight)
{
    return rsupv_getEnum (instrSession,
                          "SENS8:PROT:HIGH?",
                          highlight,
                          protHighlightArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Protocol Analysis Persistence
 * Purpose:  This function defines how long a highlight should stay.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolAnalysisPersistence (ViSession instrSession,
                                                        ViInt32 persistence)
{
    return rsupv_setEnum (instrSession,
                          "SENS8:PROT:PERS",
                          persistence,
                          protPersistenceArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Analysis Persistence
 * Purpose:  This function returns how long a highlight should stay.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolAnalysisPersistence (ViSession instrSession,
                                                        ViInt32 *persistence)
{
    return rsupv_getEnum (instrSession,
                          "SENS8:PROT:PERS?",
                          persistence,
                          protPersistenceArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Protocol Analysis View Mode
 * Purpose:  This function selects how the channel-status information should 
 *           appear.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolAnalysisViewMode (ViSession instrSession,
                                                     ViInt32 viewMode)
{
    return rsupv_setEnum (instrSession,
                          "SENS8:PROT:VIEW",
                          viewMode,
                          protViewModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Protocol Analysis View Mode
 * Purpose:  This function returns how the channel-status information should 
 *           appear.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolAnalysisViewMode (ViSession instrSession,
                                                     ViInt32 *viewMode)
{
    return rsupv_getEnum (instrSession,
                          "SENS8:PROT:VIEW?",
                          viewMode,
                          protViewModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Protocol Generator Coding Mode
 * Purpose:  This function sets the entry mask for channel status data to 
 *           Consumer or Professional.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolGeneratorCodingMode (ViSession instrSession,
                                                        ViInt32 codingMode)
{
    if ((rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPP) == VI_SUCCESS) &&
        (codingMode == RSUPV_PROTOCOL_CODING_MODE_AUTO))
        return RSUPV_ERROR_INSTRUMENT_NSUP_MODEL;
    return rsupv_setEnum (instrSession,
                          "SOUR:PROT:MODE",
                          codingMode,
                          protCodingModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Generator Coding Mode
 * Purpose:  This function gets the entry mask for channel status data to 
 *           Consumer or Professional.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolGeneratorCodingMode (ViSession instrSession,
                                                        ViInt32 *codingMode)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:PROT:MODE?",
                          codingMode,
                          protCodingModeArr,
                          VI_ERROR_PARAMETER2);    
}


/*****************************************************************************
 * Function: Set Protocol Generator Coding File
 * Purpose:  This function is used for entering the drive, path and file name
 *           of the protocol file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolGeneratorCodingFile (ViSession instrSession,
                                                        ViString protocolFile)
{
    return rsupv_setViStringQuoted (instrSession,
                              "SOUR:PROT:FILE",
                              "'",
                              protocolFile,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Protocol Generator CRC
 * Purpose:  This function sets the state of the cyclic redundancy check 
 *           character: A checksum is generated from the content of bytes 0 to 
 *           22 of the channel status data and transferred to byte 23 (Professional
 *           mode only).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolGeneratorCRC (ViSession instrSession,
                                                 ViBoolean CRC)
{
    CHECKINSTR (RSUPV_INSTR_UPV); 
    return rsupv_setViBoolean (instrSession,
                               "SOUR:PROT:CRC",
                               CRC,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Generator CRC
 * Purpose:  This function gets the state of the cyclic redundancy check 
 *           character.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolGeneratorCRC (ViSession instrSession,
                                                 ViBoolean *CRC)
{
    return rsupv_getViBoolean (instrSession,
                               "SOUR:PROT:CRC?",
                               CRC,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Protocol Generator Validity
 * Purpose:  This function defines the state of the validity bit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolGeneratorValidity (ViSession instrSession,
                                                      ViInt32 validity)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "SOUR:PROT:VAL",
                          validity,
                          protValidityArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Generator Validity
 * Purpose:  This function returns the state of the validity bit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolGeneratorValidity (ViSession instrSession,
                                                      ViInt32 *validity)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:PROT:VAL?",
                          validity,
                          protValidityArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Protocol Generator All Bits To Zero
 * Purpose:  This function sets all bits to zero. Exception: Bit 0 in byte 0, 
 *           which defines the coding mode as Consumer or Professional, is not 
 *           changed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolGeneratorAllBitsToZero (ViSession instrSession)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_sendCmd (instrSession, "SOUR:PROT:AZER ONCE");    
}

/*****************************************************************************
 * Function: Set Protocol Generator Channel Byte
 * Purpose:  This function selects the channel, byte and decimal value for the 
 *           byte selected in channel 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolGeneratorChannelByte (ViSession instrSession,
                                                         ViInt32 channel,
                                                         ViInt32 byte,
                                                         ViInt32 value)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (byte, 0, 3, 3);
    
    sprintf (buffer, "SOURce:PROTocol:CH%ld:BYTE%ld", channel, byte);
    
    return rsupv_setViInt32 (instrSession, 
                             buffer,
                             value,
                             0, 255,
                             VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Protocol Generator Channel Byte
 * Purpose:  This function returns decimal value for the byte selected in 
 *           channel 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolGeneratorChannelByte (ViSession instrSession,
                                                         ViInt32 channel,
                                                         ViInt32 byte,
                                                         ViInt32 *value)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (byte, 0, 3, 3);
    
    sprintf (buffer, "SOURce:PROTocol:CH%ld:BYTE%ld?", channel, byte);
    
    return rsupv_getViInt32 (instrSession, 
                             buffer,
                             value,
                             VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Protocol Generator Channels
 * Purpose:  This function selects whether parameters can be set separately or 
 *           for both channels in common.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetProtocolGeneratorChannels (ViSession instrSession,
                                                      ViInt32 channels)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "SOUR:PROT:CHAN",
                          channels,
                          protChannelsArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Generator Channels
 * Purpose:  This function returns whether parameters can be set separately or 
 *           for both channels in common.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolGeneratorChannels (ViSession instrSession,
                                                      ViInt32 *channels)
{
    return rsupv_getEnum (instrSession,
                          "SOUR:PROT:CHAN?",
                          channels,
                          protChannelsArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Display Screen
 * Purpose:  This function selects displayed screen.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayScreen (ViSession instrSession,
                                          ViInt32 screenNumber)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViInt32 (instrSession, 
                             "DISP:SCR",
                             screenNumber,
                             1, 5,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Display Screen
 * Purpose:  This function queries displayed screen.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayScreen (ViSession instrSession,
                                          ViInt32 *screenNumber)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViInt32 (instrSession, 
                             "DISP:SCR?",
                             screenNumber,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Display Channel
 * Purpose:  This function selects the Channel, which is shown in Trace A or
 *           B of the Display Panel of Graphic Subsystem Subsys.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayChannel (ViSession instrSession,
                                           ViInt32 subsystemNumber,
                                           ViInt32 subsystem, ViInt32 trace,
                                           ViInt32 channelNumber)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (channelNumber, 1, 16, 5);
    CHECKVIINT32 (trace, 0, 1, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:CHAN CH%ld", subsystemNumber, displayTraceArr[trace], channelNumber);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:CHAN CH%ld", subsystemNumber, displayTraceArr[trace], channelNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:CHAN CH%ld", displayTraceArr[trace], channelNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:CHAN CH%ld", subsystemNumber, displayTraceArr[trace], channelNumber);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:CHAN CH%ld", subsystemNumber, displayTraceArr[trace], channelNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Display Channel
 * Purpose:  This function queries the Channel, which is shown in Trace A or
 *           B of the Display Panel of Graphic Subsystem Subsys.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayChannel (ViSession instrSession,
                                           ViInt32 subsystemNumber,
                                           ViInt32 subsystem, ViInt32 trace,
                                           ViInt32 *channelNumber)
{
    ViChar buffer[BUFFER_SIZE] = "";
    ViChar response [20] = "";
    ViStatus rsupv_status = VI_SUCCESS;
    
    CHECKVIINT32 (trace, 0, 1, 4);
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:CHAN?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:CHAN?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:CHAN?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:CHAN?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:CHAN?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    CHECKERR (rsupv_getViString (instrSession, buffer, response, 20, 0));
    sscanf (response, "CH%ld", channelNumber);
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Display Show Min Max
 * Purpose:  This function selects whether minimum and maximum curves will be
 *           displayed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayShowMinMax (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViBoolean showMinMax)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:MINM", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:MINM", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:MINM", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:MINM", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               showMinMax,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Show Min Max
 * Purpose:  This function queries whether minimum and maximum curves will be 
 *           displayed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayShowMinMax (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViBoolean *showMinMax)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:MINM?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:MINM?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:MINM?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:MINM?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               showMinMax,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Data List Filter
 * Purpose:  This function selects the data list filter of displayed 
 *           measurement values for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayDataListFilter (ViSession instrSession,
                                                  ViInt32 subsystem,
                                                  ViInt32 subsystemNumber,
                                                  ViInt32 dataListFilterType)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:DLIS:FILT", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:DLIS:FILT");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:DLIS:FILT", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:DLIS:FILT");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:DLIS:FILT", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:DLIS:FILT", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          dataListFilterType,
                          displayDataListFilterType,
                          VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Display Data List Filter
 * Purpose:  This function returns the data list filter of displayed 
 *           measurement values for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayDataListFilter (ViSession instrSession,
                                                  ViInt32 subsystem,
                                                  ViInt32 subsystemNumber,
                                                  ViInt32 *dataListFilterType)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:DLIS:FILT?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:DLIS:FILT?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:DLIS:FILT?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:DLIS:FILT?");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:DLIS:FILT?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:DLIS:FILT?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          dataListFilterType,
                          displayDataListFilterType,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Scan Offset
 * Purpose:  This function sets the scan offset. For those graphic systems 
 *           that allow multiscan recording (FFT Monitor and Sweep Graph1 to 4) 
 *           the desired scan can be selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayScanOffset (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 scanOffset)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:SCAN", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:SCAN");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            CHECKVIINT32 (scanOffset, 0, 2, 4);
            sprintf (buffer, "DISP:FFT%ld:SCAN", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            if (scanOffset != 0)
                return VI_ERROR_PARAMETER4;
            sprintf (buffer, "DISP:WAV:SCAN");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            CHECKVIINT32 (scanOffset, 0, 2, 4);
            sprintf (buffer, "DISP:BAR%ld:SCAN", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViInt32 (instrSession, 
                             buffer,
                             scanOffset,
                             -2,19,
                             VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Scan Offset
 * Purpose:  This function gets the scan offset. For those graphic systems 
 *           that allow multiscan recording (FFT Monitor and Sweep Graph1 to 4) 
 *           the desired scan can be selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayScanOffset (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 *scanOffset)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:SCAN?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:SCAN?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:SCAN?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:SCAN?");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:SCAN?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViInt32 (instrSession, 
                             buffer,
                             scanOffset,
                             VI_ERROR_PARAMETER4);    
}
  
/*****************************************************************************
 * Function: Set Display Max Channels
 * Purpose:  This function sets the number of simultaneously displayed
 *           channels in the Numeric Display for Multichannel Instruments.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayMaxChannels (ViSession instrSession,
                                          ViInt32 maxChannelDisp)
{
    return rsupv_setViInt32 (instrSession, 
                             "SYST:MAXC",
                             maxChannelDisp,
                             1, 16,
                             VI_ERROR_PARAMETER2);
}
  
/*****************************************************************************
 * Function: Get Display Max Channels
 * Purpose:  This function gets the number of simultaneously displayed
 *           channels in the Numeric Display for Multichannel Instruments.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayMaxChannels (ViSession instrSession,
                                          ViInt32 *maxChannelDisp)
{
    return rsupv_getViInt32 (instrSession, 
                             "SYST:MAXC?",
                             maxChannelDisp,
                             VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Set Display Sweep Y-Source
 * Purpose:  This function selects which sweep measurement results are to be 
 *           collected as TRACE A or TRACE B and displayed graphically on the 
 *           left-hand or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplaySweepYSource (ViSession instrSession,
                                                ViInt32 subsystemNumber,
                                                ViInt32 trace, ViInt32 ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
   
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKVIINT32 (subsystemNumber, 1, 4, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
    CHECKVIINT32 (ySource, RSUPV_DISP_Y_SRC_OFF, RSUPV_DISP_Y_SRC_FILEB, 4);

    sprintf (buffer, "DISP:SWE%ld:%s:YSO", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Sweep Y-Source
 * Purpose:  This function returns which sweep measurement results are to be 
 *           collected as TRACE A or TRACE B and displayed graphically on the 
 *           left-hand or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplaySweepYSource (ViSession instrSession,
                                                ViInt32 subsystemNumber,
                                                ViInt32 trace, ViInt32 *ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKVIINT32 (subsystemNumber, 1, 4, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
            
    sprintf (buffer, "DISP:SWE%ld:%s:YSO?", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER4);     
}

/*****************************************************************************
 * Function: Set Display Sweep Multichannel Y Source
 * Purpose:  This function selects which sweep measurement results are to be
 *           collected as TRACE A or TRACE B and displayed graphically on the
 *           left-hand or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplaySweepMultichannelYSource (ViSession instrSession,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViInt32 ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (subsystemNumber, 1, 4, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
            
    sprintf (buffer, "DISP:SWE%ld:%s:YSO", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          ySource,
                          displayMultichannelYSourceArr,
                          VI_ERROR_PARAMETER4);     
}

/*****************************************************************************
 * Function: Get Display Sweep Multichannel Y Source
 * Purpose:  This function returns which sweep measurement results are to be
 *           collected as TRACE A or TRACE B and displayed graphically on the
 *           left-hand or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplaySweepMultichannelYSource (ViSession instrSession,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViInt32 *ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (subsystemNumber, 1, 4, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
            
    sprintf (buffer, "DISP:SWE%ld:%s:YSO?", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          ySource,
                          displayMultichannelYSourceArr,
                          VI_ERROR_PARAMETER4);     
}

/*****************************************************************************
 * Function: Set Display FFT Graph Y-Source
 * Purpose:  This function selects which measurement results obtained with the 
 *           analyzer measurement function FFT are to be collected as TRACE A 
 *           or TRACE B and displayed graphically on the left-hand or right-hand 
 *           Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayFFTGraphYSource (ViSession instrSession,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViInt32 ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
    
    CHECKVIINT32 (ySource, RSUPV_DISP_Y_SRC_OFF, RSUPV_DISP_Y_SRC_FFTP21, 4);
    if ((ySource > RSUPV_DISP_Y_SRC_OFF) && (ySource < RSUPV_DISP_Y_SRC_FILEA) ||
        (ySource > RSUPV_DISP_Y_SRC_FILEB) && (ySource < RSUPV_DISP_Y_SRC_FFTL1)) 
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "DISP:FFT%ld:%s:YSO", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display FFT Graph Y-Source
 * Purpose:  This function returns which measurement results obtained with the 
 *           analyzer measurement function FFT are to be collected as TRACE A 
 *           or TRACE B and displayed graphically on the left-hand or right-hand 
 *           Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayFFTGraphYSource (ViSession instrSession,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViInt32 *ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
            
    sprintf (buffer, "DISP:FFT%ld:%s:YSO?", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display FFT Graph Multichannel Y Source
 * Purpose:  This function selects which measurement results obtained with
 *           the analyzer measurement function FFT are to be collected as
 *           TRACE A or TRACE B and displayed graphically on the left-hand or
 *           right-hand Y axis
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayFFTGraphMultichannelYSource (ViSession instrSession,
                                                      ViInt32 subsystemNumber,
                                                      ViInt32 trace,
                                                      ViInt32 ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
            
    sprintf (buffer, "DISP:FFT%ld:%s:YSO", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          ySource,
                          displayMultichannelYSourceArr,
                          VI_ERROR_PARAMETER4);     
}

/*****************************************************************************
 * Function: Get Display FFT Graph Multichannel Y Source
 * Purpose:  This function returns which measurement results obtained with
 *           the analyzer measurement function FFT are to be collected as
 *           TRACE A or TRACE B and displayed graphically on the left-hand or
 *           right-hand Y axis
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayFFTGraphMultichannelYSource (ViSession instrSession,
                                                      ViInt32 subsystemNumber,
                                                      ViInt32 trace,
                                                      ViInt32 *ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
            
    sprintf (buffer, "DISP:FFT%ld:%s:YSO?", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          ySource,
                          displayMultichannelYSourceArr,
                          VI_ERROR_PARAMETER4);     
}

/*****************************************************************************
 * Function: Set Display Waveform Y-Source
 * Purpose:  This function selects which measurement results obtained with the 
 *           additional measurement function Waveform are to be collected as 
 *           TRACE A or TRACE B and displayed graphically on the left-hand or 
 *           right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayWaveformYSource (ViSession instrSession,
                                                   ViInt32 trace,
                                                   ViInt32 ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 2);
    CHECKVIINT32 (ySource, RSUPV_DISP_Y_SRC_OFF, RSUPV_DISP_Y_SRC_LEV2, 3);
    if ((ySource > RSUPV_DISP_Y_SRC_OFF) && (ySource < RSUPV_DISP_Y_SRC_FILEA)) 
        return VI_ERROR_PARAMETER3;
            
    sprintf (buffer, "DISP:WAV:%s:YSO", displayTraceArr[trace]);
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Display Waveform Y-Source
 * Purpose:  This function returns which measurement results obtained with the 
 *           additional measurement function Waveform are to be collected as 
 *           TRACE A or TRACE B and displayed graphically on the left-hand or 
 *           right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayWaveformYSource (ViSession instrSession,
                                                   ViInt32 trace,
                                                   ViInt32 *ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 2);
            
    sprintf (buffer, "DISP:WAV:%s:YSO?", displayTraceArr[trace]);
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Display Waveform Multichannel Y Source
 * Purpose:  This function selects which measurement results obtained with
 *           the additional measurement function Waveform are to be collected
 *           as TRACE A or TRACE B and displayed graphically on the left-hand
 *           or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayWaveformMultichannelYSource (ViSession instrSession,
                                                      ViInt32 trace,
                                                      ViInt32 ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 2);
            
    sprintf (buffer, "DISP:WAV:%s:YSO", displayTraceArr[trace]);
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          ySource,
                          displayMultichannelYSourceArr,
                          VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Display Waveform Multichannel Y Source
 * Purpose:  This function returns which measurement results obtained with
 *           the additional measurement function Waveform are to be collected
 *           as TRACE A or TRACE B and displayed graphically on the left-hand
 *           or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayWaveformMultichannelYSource (ViSession instrSession,
                                                      ViInt32 trace,
                                                      ViInt32 *ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 2);
            
    sprintf (buffer, "DISP:WAV:%s:YSO?", displayTraceArr[trace]);
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          ySource,
                          displayMultichannelYSourceArr,
                          VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Display Bargraph Y-Source
 * Purpose:  This function selects whether the measurement result for channel 
 *           1 or channel 2 obtained with the analyzer measurement function THD, 
 *           THD+N & SINAD, Mod Dist, DFD Third Octave or 1/n Octave is to be
 *           collected as TRACE A or TRACE B and displayed graphically on the
 *           left-hand or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayBargraphYSource (ViSession instrSession,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViInt32 ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
    CHECKVIINT32 (ySource, RSUPV_DISP_Y_SRC_OFF, RSUPV_DISP_Y_SRC_FILEB, 4);
    if ((ySource > RSUPV_DISP_Y_SRC_FUNC2) && (ySource < RSUPV_DISP_Y_SRC_FILEA)) 
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "DISP:BAR%ld:%s:YSO", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Bargraph Y-Source
 * Purpose:  This function returns whether the measurement result for channel 
 *           1 or channel 2 obtained with the analyzer measurement function THD, 
 *           THD+N & SINAD, Mod Dist, DFD Third Octave or 1/n Octave is to be
 *           collected as TRACE A or TRACE B and displayed graphically on the
 *           left-hand or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayBargraphYSource (ViSession instrSession,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViInt32 *ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
    
    sprintf (buffer, "DISP:BAR%ld:%s:YSO?", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Bargraph Multichannel Y Source
 * Purpose:  This function selects whether the measurement result for
 *           specified channel obtained with the analyzer measurement
 *           function THD, THD+N & SINAD, Mod Dist, DFD Third Octave or 1/n
 *           Octave is to be collected as TRACE A or TRACE B and displayed
 *           graphically on the left-hand or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayBargraphMultichannelYSource (ViSession instrSession,
                                                      ViInt32 subsystemNumber,
                                                      ViInt32 trace,
                                                      ViInt32 ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
    
    sprintf (buffer, "DISP:BAR%ld:%s:YSO", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          ySource,
                          displayMultichannelYSourceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Bargraph Multichannel Y Source
 * Purpose:  This function returns whether the measurement result for
 *           specified channel obtained with the analyzer measurement
 *           function THD, THD+N & SINAD, Mod Dist, DFD Third Octave or 1/n
 *           Octave is to be collected as TRACE A or TRACE B and displayed
 *           graphically on the left-hand or right-hand Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayBargraphMultichannelYSource (ViSession instrSession,
                                                      ViInt32 subsystemNumber,
                                                      ViInt32 trace,
                                                      ViInt32 *ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
    
    sprintf (buffer, "DISP:BAR%ld:%s:YSO?", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          ySource,
                          displayMultichannelYSourceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display PESQ Y Source
 * Purpose:  This function select which measurement results of the PESQ
 *           measurement function are to be collected as TRACE A or as TRACE
 *           B and displayed graphically on the left-hand or right-hand
 *           y-axis.
 *           Perceptual evaluation of speech quality (PESQ) is a method for
 *           assessing the speech quality of transmission systems.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayPESQYSource (ViSession instrSession,
                                      ViInt32 subsystemNumber, ViInt32 trace,
                                      ViInt32 ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
    
    sprintf (buffer, "DISP:PESQ%ld:%s:YSO", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display PESQ Y Source
 * Purpose:  This function queries which measurement results of the PESQ
 *           measurement function are to be collected as TRACE A or as TRACE
 *           B and displayed graphically on the left-hand or right-hand
 *           y-axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayPESQYSource (ViSession instrSession,
                                      ViInt32 subsystemNumber, ViInt32 trace,
                                      ViInt32 *ySource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    CHECKVIINT32 (subsystemNumber, 1, 2, 2);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 3);
    
    sprintf (buffer, "DISP:PESQ%ld:%s:YSO?", subsystemNumber, displayTraceArr[trace]);
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          ySource,
                          displayYSourceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Y-Source File
 * Purpose:  This function loads a Y-Source trace from a file for the desired 
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayYSourceFile (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace,
                                               ViString ySourceFile)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:YSO:FILE", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:YSO:FILE", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:YSO:FILE", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:YSO:FILE", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:YSO:FILE", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:YSO:FILE", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    ySourceFile,
                                    VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Reference
 * Purpose:  This function selects the reference for referenced units for Trace 
 *           A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayReference (ViSession instrSession,
                                             ViInt32 subsystem,
                                             ViInt32 subsystemNumber,
                                             ViInt32 trace, ViInt32 reference)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    if ((rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPP) == VI_SUCCESS) &&
        (reference == RSUPV_DISP_REF_CH1M || reference == RSUPV_DISP_REF_CH2M))
        return VI_ERROR_PARAMETER5;
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            CHECKVIINT32 (reference,RSUPV_DISP_REF_MEAS, RSUPV_DISP_REF_HOLD, 5);
            sprintf (buffer, "DISP:SWE%ld:%s:REF", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            CHECKVIINT32 (reference,RSUPV_DISP_REF_MEAS, RSUPV_DISP_REF_HOLD, 5);
            sprintf (buffer, "DISP:MON:%s:REF", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:REF", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            if ((reference != RSUPV_DISP_REF_VAL) && (reference != RSUPV_DISP_REF_HOLD))
                return VI_ERROR_PARAMETER5;
            sprintf (buffer, "DISP:WAV:%s:REF", displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          reference,
                          displayReferenceArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Reference
 * Purpose:  This function returns the reference for referenced units for Trace 
 *           A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayReference (ViSession instrSession,
                                             ViInt32 subsystem,
                                             ViInt32 subsystemNumber,
                                             ViInt32 trace, ViInt32 *reference)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:REF?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:REF?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:REF?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:REF?", displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          reference,
                          displayReferenceArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Reference Value
 * Purpose:  This function sets the reference value for Trace A or Trace B for 
 *           the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayReferenceValue (ViSession instrSession,
                                                  ViInt32 subsystem,
                                                  ViInt32 subsystemNumber,
                                                  ViInt32 trace,
                                                  ViReal64 referenceValue,
                                                  ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:REF:VAL", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:REF:VAL", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:REF:VAL", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:REF:VAL", displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                                  buffer,
                                  referenceValue,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Reference Value
 * Purpose:  This function gets the reference value for Trace A or Trace B for 
 *           the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayReferenceValue (ViSession instrSession,
                                                  ViInt32 subsystem,
                                                  ViInt32 subsystemNumber,
                                                  ViInt32 trace,
                                                  ViReal64 *referenceValue,
                                                  ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:REF:VAL?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:REF:VAL?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:REF:VAL?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:REF:VAL?", displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  referenceValue,
                                  units,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Reference File
 * Purpose:  This function loads a reference trace from a file for the desired 
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayReferenceFile (ViSession instrSession,
                                                 ViInt32 subsystem,
                                                 ViInt32 subsystemNumber,
                                                 ViInt32 trace,
                                                 ViString referenceFile)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:REF:FILE", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:REF:FILE", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:REF:FILE", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:REF:FILE", displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    referenceFile,
                                    VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Normalization
 * Purpose:  This function sets the normalization for Trace A or Trace B for
 *           the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayNormalization (ViSession instrSession,
                                        ViInt32 subsystem,
                                        ViInt32 subsystemNumber, ViInt32 trace,
                                        ViInt32 normalization)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:NORM", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:NORM", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:NORM", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          normalization,
                          displayNormalizationArr,
                          VI_ERROR_PARAMETER5);
    
}

/*****************************************************************************
 * Function: Get Display Normalization
 * Purpose:  This function gets the normalization for Trace A or Trace B for
 *           the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayNormalization (ViSession instrSession,
                                        ViInt32 subsystem,
                                        ViInt32 subsystemNumber, ViInt32 trace,
                                        ViInt32 *normalization)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:NORM?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:NORM?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:NORM?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          normalization,
                          displayNormalizationArr,
                          VI_ERROR_PARAMETER5);
    
}

/*****************************************************************************
 * Function: Set Display Normalize Value
 * Purpose:  This function sets the normalization value for Trace A or Trace
 *           B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayNormalizeValue (ViSession instrSession,
                                         ViInt32 subsystem,
                                         ViInt32 subsystemNumber, ViInt32 trace,
                                         ViReal64 normalizeValue)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:NORM:VAL", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:NORM:VAL", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:NORM:VAL", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64 (instrSession, 
                              buffer,
                              normalizeValue,
                              0.0,0.0,
                              VI_ERROR_PARAMETER5);
    
}

/*****************************************************************************
 * Function: Get Display Normalize Value
 * Purpose:  This function returns the normalization value for Trace A or
 *           Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayNormalizeValue (ViSession instrSession,
                                         ViInt32 subsystem,
                                         ViInt32 subsystemNumber, ViInt32 trace,
                                         ViReal64 *normalizeValue)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:NORM:VAL?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:NORM:VAL?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:NORM:VAL?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64 (instrSession, 
                              buffer,
                              normalizeValue,
                              VI_ERROR_PARAMETER5);
    
}

/*****************************************************************************
 * Function: Set Display Copy Reference Settings
 * Purpose:  This function sets whether the copying of the reference settings 
 *           from trace A to trace B for the desired subsystem is on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayCopyReferenceSettings (ViSession instrSession,
                                                         ViInt32 subsystem,
                                                         ViInt32 subsystemNumber,
                                                         ViBoolean copyReferenceSettings)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:TRAC:REF", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:TRAC:REF");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:TRAC:REF", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:TRAC:REF");
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               copyReferenceSettings,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Display Copy Reference Settings
 * Purpose:  This function gets whether the copying of the reference settings 
 *           from trace A to trace B for the desired subsystem is on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayCopyReferenceSettings (ViSession instrSession,
                                                         ViInt32 subsystem,
                                                         ViInt32 subsystemNumber,
                                                         ViBoolean *copyReferenceSettings)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:TRAC:REF?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:TRAC:REF?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:TRAC:REF?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:TRAC:REF?");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:TRAC:REF?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               copyReferenceSettings,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Copy Scaling Settings
 * Purpose:  This function sets whether the copying of the scaling settings 
 *           from trace A to trace B for the desired subsystem is on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayCopyScalingSettings (ViSession instrSession,
                                                       ViInt32 subsystem,
                                                       ViInt32 subsystemNumber,
                                                       ViBoolean copyScalingSettings)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:TRAC:SCAL", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:TRAC:SCAL");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:TRAC:SCAL", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:TRAC:SCAL");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:TRAC:SCAL", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               copyScalingSettings,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Copy Scaling Settings
 * Purpose:  This function gets whether the copying of the scaling settings 
 *           from trace A to trace B for the desired subsystem is on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayCopyScalingSettings (ViSession instrSession,
                                                       ViInt32 subsystem,
                                                       ViInt32 subsystemNumber,
                                                       ViBoolean *copyScalingSettings)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:TRAC:SCAL?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:TRAC:SCAL?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:TRAC:SCAL?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:TRAC:SCAL?");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:TRAC:SCAL?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               copyScalingSettings,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Trace Update
 * Purpose:  This function selects the trace update type for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayTraceUpdate (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace,
                                               ViInt32 traceUpdateType)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UPD", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:UPD", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UPD", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:UPD", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UPD", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:UPD", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          traceUpdateType,
                          displayTraceUpdateTypeArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Trace Update
 * Purpose:  This function returns the trace update type for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayTraceUpdate (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace,
                                               ViInt32 *traceUpdateType)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UPD?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:UPD?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UPD?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:UPD?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UPD?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:UPD?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          traceUpdateType,
                          displayTraceUpdateTypeArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Unit Function Track
 * Purpose:  This function selects whether the unit is the same as that set 
 *           in the analyzer or whether it can be user-selected for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayUnitFunctionTrack (ViSession instrSession,
                                                     ViInt32 subsystem,
                                                     ViInt32 subsystemNumber,
                                                     ViInt32 trace,
                                                     ViBoolean unitFunctionTrack)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UNIT:TRAC", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UNIT:TRAC", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UNIT:TRAC", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               unitFunctionTrack,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Unit Function Track
 * Purpose:  This function returns whether the unit is the same as that set 
 *           in the analyzer or whether it can be user-selected for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayUnitFunctionTrack (ViSession instrSession,
                                                     ViInt32 subsystem,
                                                     ViInt32 subsystemNumber,
                                                     ViInt32 trace,
                                                     ViBoolean *unitFunctionTrack)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UNIT:TRAC?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UNIT:TRAC?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UNIT:TRAC?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               unitFunctionTrack,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Unit
 * Purpose:  This function selects the unit for Trace A or Trace B for the 
 *           desired subsystem.
 *
 *           Note(s):
 *
 *           (1) This function is available only when Unit Function Track is
 *           set to OFF - function SetDisplayUnitFunctionTrack.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayUnit (ViSession instrSession,
                                        ViInt32 subsystem,
                                        ViInt32 subsystemNumber, ViInt32 trace,
                                        ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UNIT", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:UNIT", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UNIT", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:UNIT", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UNIT", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          units,
                          unitsArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Unit
 * Purpose:  This function returns the unit for Trace A or Trace B for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayUnit (ViSession instrSession,
                                        ViInt32 subsystem,
                                        ViInt32 subsystemNumber, ViInt32 trace,
                                        ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UNIT?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:UNIT?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UNIT?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:UNIT?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UNIT?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          units,
                          unitsArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Y-Axis Spacing
 * Purpose:  This function selects the linear or logarithmic spacing of the Y 
 *           axis for Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayYAxisSpacing (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 trace, ViInt32 spacing)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:SPAC", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:SPAC", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:SPAC", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:SPAC", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:SPAC", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:SPAC", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          spacing,
                          displaySpacingArr,
                          VI_ERROR_PARAMETER5);     
}

/*****************************************************************************
 * Function: Get Display Y-Axis Spacing
 * Purpose:  This function returns the linear or logarithmic spacing of the Y 
 *           axis for Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayYAxisSpacing (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 trace, ViInt32 *spacing)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:SPAC?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:SPAC?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:SPAC?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:SPAC?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:SPAC?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:SPAC?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          spacing,
                          displaySpacingArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Y-Axis Top
 * Purpose:  This function sets the top value of the Y axis for Trace A or 
 *           Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayYAxisTop (ViSession instrSession,
                                            ViInt32 subsystem,
                                            ViInt32 subsystemNumber,
                                            ViInt32 trace, ViReal64 top,
                                            ViInt32 units)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:TOP", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:TOP", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:TOP", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:TOP", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:TOP", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:TOP", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                              buffer,
                              top,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Y-Axis Top
 * Purpose:  This function gets the top value of the Y axis for Trace A or 
 *           Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayYAxisTop (ViSession instrSession,
                                            ViInt32 subsystem,
                                            ViInt32 subsystemNumber,
                                            ViInt32 trace, ViReal64 *top,
                                            ViInt32 *units)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:TOP?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:TOP?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:TOP?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:TOP?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:TOP?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:TOP?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                              buffer,
                              top,
                              units,
                              VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Y-Axis Bottom
 * Purpose:  This function sets the bottom value of the Y axis for Trace A or 
 *           Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayYAxisBottom (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace, ViReal64 bottom,
                                               ViInt32 units)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:BOTT", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:BOTT", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:BOTT", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:BOTT", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:BOTT", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:BOTT", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                              buffer,
                              bottom,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Y-Axis Bottom
 * Purpose:  This function gets the bottom value of the Y axis for Trace A or 
 *           Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayYAxisBottom (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace, ViReal64 *bottom,
                                               ViInt32 *units)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:BOTT?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:BOTT?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:BOTT?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:BOTT?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:BOTT?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:BOTT?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                              buffer,
                              bottom,
                              units,
                              VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Unit Auto
 * Purpose:  This function selects whether the unit label displayed on the Y 
 *           axis for Trace A or Trace B is the same as the unit set in the 
 *           "Unit Funct Track" or "Unit" parameter field or whether it can be
 *           user-selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayUnitAuto (ViSession instrSession,
                                            ViInt32 subsystem,
                                            ViInt32 subsystemNumber,
                                            ViInt32 trace, ViBoolean unitAuto)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UNIT:AUT", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:UNIT:AUT", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UNIT:AUT", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:UNIT:AUT", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UNIT:AUT", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               unitAuto,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Unit Auto
 * Purpose:  This function returns whether the unit label displayed on the Y 
 *           axis for Trace A or Trace B is the same as the unit set in the 
 *           "Unit Funct Track" or "Unit" parameter field or whether it can be
 *           user-selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayUnitAuto (ViSession instrSession,
                                            ViInt32 subsystem,
                                            ViInt32 subsystemNumber,
                                            ViInt32 trace, ViBoolean *unitAuto)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UNIT:AUT?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:UNIT:AUT?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UNIT:AUT?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:UNIT:AUT?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UNIT:AUT?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               unitAuto,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Unit User String
 * Purpose:  This function sets the user-defined unit designation if Unit 
 *           Auto is not selected. The specified designation appears on the 
 *           Y-axis label instead of the usual units.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayUnitUserString (ViSession instrSession,
                                                  ViInt32 subsystem,
                                                  ViInt32 subsystemNumber,
                                                  ViInt32 trace,
                                                  ViString unitUserString)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UNIT:USER", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:UNIT:USER", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UNIT:USER", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:UNIT:USER", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UNIT:USER", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    unitUserString,
                                    VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Unit User String
 * Purpose:  This function gets the user-defined unit designation if Unit 
 *           Auto is not selected. The specified designation appears on the 
 *           Y-axis label instead of the usual units.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayUnitUserString (ViSession instrSession,
                                                  ViInt32 subsystem,
                                                  ViInt32 subsystemNumber,
                                                  ViInt32 trace,
                                                  ViInt32 bufferSize,
                                                  ViChar unitUserString[])
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:UNIT:USER?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:UNIT:USER?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:UNIT:USER?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:UNIT:USER?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:UNIT:USER?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    bufferSize,
                                    unitUserString,
                                    VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Axis Label Auto
 * Purpose:  This function selects whether the unit label displayed on the Y
 *           axis for Trace A or Trace B or X axis is oriented in the setting
 *           in the Y-Source parameter field, or is freely configurable.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayAxisLabelAuto (ViSession instrSession,
                                                 ViInt32 subsystem,
                                                 ViInt32 subsystemNumber,
                                                 ViInt32 trace,
                                                 ViBoolean axisLabelAuto)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LAB:AUTO", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LAB:AUTO", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LAB:AUTO", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LAB:AUTO", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LAB:AUTO", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               axisLabelAuto,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Axis Label Auto
 * Purpose:  This function queries whether the unit label displayed on the Y
 *           axis for Trace A or Trace B or X axis is oriented in the setting
 *           in the Y-Source parameter field, or is freely configurable.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayAxisLabelAuto (ViSession instrSession,
                                                 ViInt32 subsystem,
                                                 ViInt32 subsystemNumber,
                                                 ViInt32 trace,
                                                 ViBoolean *axisLabelAuto)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LAB:AUTO?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LAB:AUTO?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LAB:AUTO?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LAB:AUTO?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LAB:AUTO?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               axisLabelAuto,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Axis Label User String
 * Purpose:  This function sets the user-defined axis label.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayAxisLabelUserString (ViSession instrSession,
                                                       ViInt32 subsystem,
                                                       ViInt32 subsystemNumber,
                                                       ViInt32 trace,
                                                       ViString axisLabelUserString)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LAB:USER", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LAB:USER", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LAB:USER", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LAB:USER", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LAB:USER", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    axisLabelUserString,
                                    VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Axis Label User String
 * Purpose:  This function sets the user-defined axis label.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayAxisLabelUserString (ViSession instrSession,
                                                       ViInt32 subsystem,
                                                       ViInt32 subsystemNumber,
                                                       ViInt32 trace,
                                                       ViInt32 bufferSize,
                                                       ViChar axisLabelUserString[])
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LAB:USER?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LAB:USER?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LAB:USER?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LAB:USER?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LAB:USER?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    bufferSize,
                                    axisLabelUserString,
                                    VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Legend State
 * Purpose:  This function selects whether the legend will be shown.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLegendState (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace, ViBoolean legend)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LEG:SHOW", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LEG:SHOW", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LEG:SHOW", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LEG:SHOW", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LEG:SHOW", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               legend,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Legend State
 * Purpose:  This function queries whether the legend will be shown.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLegendState (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace, ViBoolean *legend)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LEG:SHOW?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LEG:SHOW?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LEG:SHOW?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LEG:SHOW?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LEG:SHOW?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               legend,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Legend
 * Purpose:  This function sets the user-defined legend.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLegend (ViSession instrSession,
                                          ViInt32 subsystem,
                                          ViInt32 subsystemNumber, ViInt32 trace,
                                          ViString legendString)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LEG:DESC", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LEG:DESC", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LEG:DESC", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LEG:DESC", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LEG:DESC", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    legendString,
                                    VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Legend
 * Purpose:  This function queries the user-defined legend.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLegend (ViSession instrSession,
                                          ViInt32 subsystem,
                                          ViInt32 subsystemNumber, ViInt32 trace,
                                          ViInt32 bufferSize, ViChar legend[])
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_X_AXIS, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LEG:DESC?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LEG:DESC?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LEG:DESC?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LEG:DESC?", displayTraceUnitArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LEG:DESC?", subsystemNumber, displayTraceUnitArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    bufferSize,
                                    legend,
                                    VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Copy Limit Settings
 * Purpose:  This function sets whether the copying of the limit settings from 
 *           trace A to trace B for the desired subsystem is on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayCopyLimitSettings (ViSession instrSession,
                                                     ViInt32 subsystem,
                                                     ViInt32 subsystemNumber,
                                                     ViBoolean copyLimitSettings)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:TRAC:LIM", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:TRAC:LIM");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:TRAC:LIM", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:TRAC:LIM");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:TRAC:LIM", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               copyLimitSettings,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Copy Limit Settings
 * Purpose:  This function gets whether the copying of the limit settings from 
 *           trace A to trace B for the desired subsystem is on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayCopyLimitSettings (ViSession instrSession,
                                                     ViInt32 subsystem,
                                                     ViInt32 subsystemNumber,
                                                     ViBoolean *copyLimitSettings)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:TRAC:LIM?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:TRAC:LIM?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:TRAC:LIM?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:TRAC:LIM?");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:TRAC:LIM?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               copyLimitSettings,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Limit State
 * Purpose:  This function sets the state of upper or lower limit for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLimitState (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 trace, ViInt32 limitType,
                                              ViBoolean limitState)
{
    ViStatus  rsupv_status;
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    CHECKERR (rsupv_checkIndexInList(instrSession, limitType, displayLimitTypeArr, VI_ERROR_PARAMETER5));

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:%s", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:%s", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:%s", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:%s", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:%s", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:%s", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               limitState,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Limit State
 * Purpose:  This function gets the state of upper or lower limit for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLimitState (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 trace, ViInt32 limitType,
                                              ViBoolean *limitState)
{
    ViStatus  rsupv_status;
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    CHECKERR (rsupv_checkIndexInList(instrSession, limitType, displayLimitTypeArr, VI_ERROR_PARAMETER5));

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:%s?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:%s?", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:%s?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:%s?", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:%s?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:%s?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               limitState,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Limit Source
 * Purpose:  This function selects how the upper or lower limit is set for 
 *           Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLimitSource (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace, ViInt32 limitType,
                                               ViInt32 limitSource)
{
    ViStatus  rsupv_status;
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    CHECKERR (rsupv_checkIndexInList(instrSession, limitType, displayLimitTypeArr, VI_ERROR_PARAMETER5));
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:%s:SOUR", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:%s:SOUR", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:%s:SOUR", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            if ((limitSource != RSUPV_DISP_LIM_SRC_VAL) && (limitSource != RSUPV_DISP_LIM_SRC_HOLD))
                return VI_ERROR_PARAMETER6;
            sprintf (buffer, "DISP:WAV:%s:%s:SOUR", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:%s:SOUR", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:%s:SOUR", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          limitSource,
                          displayLimitSourceArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Limit Source
 * Purpose:  This function returns how the upper or lower limit is set for 
 *           Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLimitSource (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace, ViInt32 limitType,
                                               ViInt32 *limitSource)
{
    ViStatus  rsupv_status;
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);
    
    CHECKERR (rsupv_checkIndexInList(instrSession, limitType, displayLimitTypeArr, VI_ERROR_PARAMETER5));

    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:%s:SOUR?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:%s:SOUR?", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:%s:SOUR?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:%s:SOUR?", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:%s:SOUR?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:%s:SOUR?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          limitSource,
                          displayLimitSourceArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Limit Source Value
 * Purpose:  This function sets the upper or lower limit value if "Value" has 
 *           been selected under "Source" (function rsupv_SetDisplayLimitSource) 
 *           for Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLimitSourceValue (ViSession instrSession,
                                                    ViInt32 subsystem,
                                                    ViInt32 subsystemNumber,
                                                    ViInt32 trace,
                                                    ViInt32 limitType,
                                                    ViReal64 limitSourceValue,
                                                    ViInt32 units)
{
    ViStatus  rsupv_status;
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    CHECKERR (rsupv_checkIndexInList(instrSession, limitType, displayLimitTypeArr, VI_ERROR_PARAMETER5));

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:%s:SOUR:VAL", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:%s:SOUR:VAL", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:%s:SOUR:VAL", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:%s:SOUR:VAL", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:%s:SOUR:VAL", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:%s:SOUR:VAL", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  limitSourceValue,
                                  units, 
                                  0, 0,
                                  VI_ERROR_PARAMETER6);
}

/*****************************************************************************
 * Function: Get Display Limit Source Value
 * Purpose:  This function gets the upper or lower limit value if "Value" has 
 *           been selected under "Source" (function rsupv_SetDisplayLimitSource) 
 *           for Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLimitSourceValue (ViSession instrSession,
                                                    ViInt32 subsystem,
                                                    ViInt32 subsystemNumber,
                                                    ViInt32 trace,
                                                    ViInt32 limitType,
                                                    ViReal64 *limitSourceValue,
                                                    ViInt32 *units)
{
    ViStatus  rsupv_status;
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    CHECKERR (rsupv_checkIndexInList(instrSession, limitType, displayLimitTypeArr, VI_ERROR_PARAMETER5));

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:%s:SOUR:VAL?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:%s:SOUR:VAL?", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:%s:SOUR:VAL?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:%s:SOUR:VAL?", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:%s:SOUR:VAL?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:%s:SOUR:VAL?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  limitSourceValue,
                                  units, 
                                  VI_ERROR_PARAMETER6);    
}

/*****************************************************************************
 * Function: Set Display Limit Source Filename
 * Purpose:  This function sets the file name for an upper or lower limit 
 *           curve if "File" has been selected under "Source". (function
 *           rsupv_SetDisplayLimitSource) for Trace A or Trace B for the
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLimitSourceFilename (ViSession instrSession,
                                                       ViInt32 subsystem,
                                                       ViInt32 subsystemNumber,
                                                       ViInt32 trace,
                                                       ViInt32 limitType,
                                                       ViString limitSourceFilename)
{
    ViStatus  rsupv_status;
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    CHECKERR (rsupv_checkIndexInList(instrSession, limitType, displayLimitTypeArr, VI_ERROR_PARAMETER5));

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:%s:SOUR:FILE", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:%s:SOUR:FILE", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:%s:SOUR:FILE", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:%s:SOUR:FILE", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:%s:SOUR:FILE", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    limitSourceFilename,
                                    VI_ERROR_PARAMETER6);    
}

/*****************************************************************************
 * Function: Get Display Limit Source Filename
 * Purpose:  This function gets the file name for an upper or lower limit 
 *           curve if "File" has been selected under "Source". (function
 *           rsupv_SetDisplayLimitSource) for Trace A or Trace B for the
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLimitSourceFilename (ViSession instrSession,
                                                       ViInt32 subsystem,
                                                       ViInt32 subsystemNumber,
                                                       ViInt32 trace,
                                                       ViInt32 limitType,
                                                       ViInt32 bufferSize,
                                                       ViChar limitSourceFilename[])
{
    ViStatus  rsupv_status;
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    CHECKERR (rsupv_checkIndexInList(instrSession, limitType, displayLimitTypeArr, VI_ERROR_PARAMETER5));

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:%s:SOUR:FILE?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:%s:SOUR:FILE?", displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:%s:SOUR:FILE?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:%s:SOUR:FILE?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:%s:SOUR:FILE?", subsystemNumber, displayTraceArr[trace], displayLimitTypeArr[limitType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    bufferSize,
                                    limitSourceFilename,
                                    VI_ERROR_PARAMETER6);    
}

/*****************************************************************************
 * Function: Set Display Limit Shift
 * Purpose:  This function enables limit lines or limit curves to be shifted 
 *           for Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLimitShift (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 trace,
                                              ViBoolean limitShift)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LIMS", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LIMS", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LIMS", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LIMS", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LIMS", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession,
                               buffer,
                               limitShift,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Limit Shift
 * Purpose:  This function returns whether the limit lines or limit curves are 
 *           shifted for Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLimitShift (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 trace,
                                              ViBoolean *limitShift)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LIMS?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LIMS?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LIMS?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LIMS?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LIMS?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession,
                               buffer,
                               limitShift,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Limit Shift Parallel
 * Purpose:  This function enables limit lines or limit curves to be parallel 
 *           shifted for Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLimitShiftParallel (ViSession instrSession,
                                                      ViInt32 subsystem,
                                                      ViInt32 subsystemNumber,
                                                      ViInt32 trace,
                                                      ViReal64 limitShiftParallel,
                                                      ViInt32 units)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LIMS:PAR", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LIMS:PAR", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LIMS:PAR", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LIMS:PAR", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LIMS:PAR", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  limitShiftParallel,
                                  units, 
                                  0, 0,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Limit Shift Parallel
 * Purpose:  This function returns limit lines or limit curves to be parallel 
 *           shifted for Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLimitShiftParallel (ViSession instrSession,
                                                      ViInt32 subsystem,
                                                      ViInt32 subsystemNumber,
                                                      ViInt32 trace,
                                                      ViReal64 *limitShiftParallel,
                                                      ViInt32 *units)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LIMS:PAR?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LIMS:PAR?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LIMS:PAR?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LIMS:PAR?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LIMS:PAR?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  limitShiftParallel,
                                  units, 
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Limit Shift Symmetrical
 * Purpose:  This function enables a tolerance band formed from two limit 
 *           lines or limit curves to be made broader or narrower for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLimitShiftSymmetrical (ViSession instrSession,
                                                         ViInt32 subsystem,
                                                         ViInt32 subsystemNumber,
                                                         ViInt32 trace,
                                                         ViReal64 limitShiftSymmetrical,
                                                         ViInt32 units)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LIMS:SYMM", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LIMS:SYMM", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LIMS:SYMM", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LIMS:SYMM", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LIMS:SYMM", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  limitShiftSymmetrical,
                                  units, 
                                  0, 0,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Limit Shift Symmetrical
 * Purpose:  This function returns a tolerance band formed from two limit 
 *           lines or limit curves to be made broader or narrower for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLimitShiftSymmetrical (ViSession instrSession,
                                                         ViInt32 subsystem,
                                                         ViInt32 subsystemNumber,
                                                         ViInt32 trace,
                                                         ViReal64 *limitShiftSymmetrical,
                                                         ViInt32 *units)
{
     ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:LIMS:SYMM?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:LIMS:SYMM?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:LIMS:SYMM?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:LIMS:SYMM?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:LIMS:SYMM?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  limitShiftSymmetrical,
                                  units, 
                                  VI_ERROR_PARAMETER5);   
}

/*****************************************************************************
 * Function: Set Display Sweep Graph X-Source
 * Purpose:  This function selects the x-source type for the desired sweep graph.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplaySweepGraphXSource (ViSession instrSession,
                                                     ViInt32 sweepGraphNumber,
                                                     ViInt32 xSource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (sweepGraphNumber, 1, 4, 2);
    sprintf (buffer, "DISP:SWE%ld:X:SOUR", sweepGraphNumber);
    
    return rsupv_setEnum (instrSession,
                          buffer,
                          xSource,
                          displayXSourceArr,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Display Sweep Graph X-Source
 * Purpose:  This function returns the x-source type for the desired sweep graph.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplaySweepGraphXSource (ViSession instrSession,
                                                     ViInt32 sweepGraphNumber,
                                                     ViInt32 *xSource)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (sweepGraphNumber, 1, 4, 2);
    sprintf (buffer, "DISP:SWE%ld:X:SOUR?", sweepGraphNumber);
    
    return rsupv_getEnum (instrSession,
                          buffer,
                          xSource,
                          displayXSourceArr,
                          VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Display Sweep Graph X-Axis
 * Purpose:  This function selects the x-axis type for the desired sweep graph.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplaySweepGraphXAxis (ViSession instrSession,
                                                   ViInt32 sweepGraphNumber,
                                                   ViInt32 xAxis)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (sweepGraphNumber, 1, 4, 2);
    sprintf (buffer, "DISP:SWE%ld:X:AXIS", sweepGraphNumber);
    
    return rsupv_setEnum (instrSession,
                          buffer,
                          xAxis,
                          displayXAxisArr,
                          VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Display Sweep Graph X-Axis
 * Purpose:  This function returns the x-axis type for the desired sweep graph.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplaySweepGraphXAxis (ViSession instrSession,
                                                   ViInt32 sweepGraphNumber,
                                                   ViInt32 *xAxis)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (sweepGraphNumber, 1, 4, 2);
    sprintf (buffer, "DISP:SWE%ld:X:AXIS?", sweepGraphNumber);
    
    return rsupv_getEnum (instrSession,
                          buffer,
                          xAxis,
                          displayXAxisArr,
                          VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Display X-Axis Reference Value
 * Purpose:  This function sets the x axis reference value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayXAxisReferenceValue (ViSession instrSession,
                                                       ViInt32 subsystem,
                                                       ViInt32 subsystemNumber,
                                                       ViReal64 referenceValue,
                                                       ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:REF:VAL", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:REF:VAL");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:REF:VAL", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:REF:VAL");
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                                  buffer,
                                  referenceValue,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display X-Axis Reference Value
 * Purpose:  This function gets the x axis reference value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayXAxisReferenceValue (ViSession instrSession,
                                                       ViInt32 subsystem,
                                                       ViInt32 subsystemNumber,
                                                       ViReal64 *referenceValue,
                                                       ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:REF:VAL?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:REF:VAL?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:REF:VAL?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:REF:VAL?");
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  referenceValue,
                                  units,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display X-Axis Scaling
 * Purpose:  This function selects the manual or automatic scaling of the X 
 *           axis for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayXAxisScaling (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 scaling)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:SCAL", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:SCAL");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:SCAL", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:SCAL");
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:X:SCAL", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          scaling,
                          displayScalingArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display X-Axis Scaling
 * Purpose:  This function returns the manual or automatic scaling of the X 
 *           axis for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayXAxisScaling (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 *scaling)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:SCAL?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:SCAL?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:SCAL?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:SCAL?");
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:X:SCAL?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          scaling,
                          displayScalingArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display X-Axis Spacing
 * Purpose:  This function selects the linear or logarithmic spacing of the 
 *           X axis for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayXAxisSpacing (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 spacing)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:SPAC", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:SPAC");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:SPAC", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:SPAC");
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:X:SPAC", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          spacing,
                          displaySpacingArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display X-Axis Spacing
 * Purpose:  This function returns the linear or logarithmic spacing of the 
 *           X axis for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayXAxisSpacing (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 *spacing)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:SPAC?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:SPAC?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:SPAC?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:SPAC?");
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:X:SPAC?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          spacing,
                          displaySpacingArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display X-Axis Left
 * Purpose:  This function sets the left-hand value on the X axis for the desired
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayXAxisLeft (ViSession instrSession,
                                             ViInt32 subsystem,
                                             ViInt32 subsystemNumber,
                                             ViReal64 left, ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:LEFT", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:LEFT");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:LEFT", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:LEFT");
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:X:LEFT", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                              buffer,
                              left,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER4);     
}

/*****************************************************************************
 * Function: Get Display X-Axis Left
 * Purpose:  This function gets the left-hand value on the X axis for the desired
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayXAxisLeft (ViSession instrSession,
                                             ViInt32 subsystem,
                                             ViInt32 subsystemNumber,
                                             ViReal64 *left, ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:LEFT?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:LEFT?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:LEFT?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:LEFT?");
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:X:LEFT?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                              buffer,
                              left,
                              units,
                              VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display X-Axis Right
 * Purpose:  This function sets the right-hand value on the X axis for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayXAxisRight (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViReal64 right, ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:RIGH", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:RIGH");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:RIGH", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:RIGH");
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:X:RIGH", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                              buffer,
                              right,
                              units,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display X-Axis Right
 * Purpose:  This function gets the right-hand value on the X axis for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayXAxisRight (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViReal64 *right, ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:X:RIGH?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:X:RIGH?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:X:RIGH?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:X:RIGH?");
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:X:RIGH?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                              buffer,
                              right,
                              units,
                              VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Cursor State
 * Purpose:  This function sets the O or X cursor state for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayCursorState (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 cursorType,
                                               ViInt32 cursorState)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:STAT", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:STAT", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:STAT", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:STAT", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:STAT", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:STAT", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          cursorState,
                          displayCursorStateArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Cursor State
 * Purpose:  This function gets the O or X cursor state for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayCursorState (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 cursorType,
                                               ViInt32 *cursorState)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:STAT?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:STAT?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:STAT?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:STAT?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:STAT?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:STAT?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          cursorState,
                          displayCursorStateArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Cursor Mode
 * Purpose:  This function sets the O or X cursor mode for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayCursorMode (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 cursorType,
                                              ViInt32 cursorMode)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:MODE", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:MODE", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:MODE", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:MODE", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:MODE", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:MODE", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          cursorMode,
                          displayCursorModeArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Cursor Mode
 * Purpose:  This function gets the O or X cursor mode for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayCursorMode (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 cursorType,
                                              ViInt32 *cursorMode)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:MODE?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:MODE?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:MODE?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:MODE?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:MODE?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:MODE?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          cursorMode,
                          displayCursorModeArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Cursor Amplitude
 * Purpose:  This function gets the O or X cursor amplitude for the desired
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayCursorAmplitude (ViSession instrSession,
                                                   ViInt32 subsystem,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 cursorType,
                                                   ViReal64 *cursorAmplitude)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:Y?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:Y?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:Y?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:Y?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:Y?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:Y?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64 (instrSession, 
                              buffer,
                              cursorAmplitude,
                              VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Cursor Move To
 * Purpose:  This function sets the O or X cursor to the desired position of 
 *           the trace for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayCursorMoveTo (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 cursorType,
                                                ViInt32 setCursorTo)
{
    ViStatus  rsupv_status;
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKERR (rsupv_checkIndexInList(instrSession, cursorType, displayCursorTypeArr, VI_ERROR_PARAMETER4));

    CHECKERR (rsupv_checkIndexInList(instrSession, setCursorTo, displaySetCursorToArr, VI_ERROR_PARAMETER5));

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:SETT:%s", subsystemNumber, displayCursorTypeArr[cursorType], displaySetCursorToArr[setCursorTo]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:SETT:%s", displayCursorTypeArr[cursorType], displaySetCursorToArr[setCursorTo]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:SETT:%s", subsystemNumber, displayCursorTypeArr[cursorType], displaySetCursorToArr[setCursorTo]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:SETT:%s", displayCursorTypeArr[cursorType], displaySetCursorToArr[setCursorTo]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            CHECKVIINT32 (setCursorTo, 0, 1, 5);
            sprintf (buffer, "DISP:BAR%ld:%s:SETT:%s", subsystemNumber, displayCursorTypeArr[cursorType], displaySetCursorToArr[setCursorTo]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:SETT:%s", subsystemNumber, displayCursorTypeArr[cursorType], displaySetCursorToArr[setCursorTo]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_sendCmd (instrSession, buffer);    
}

/*****************************************************************************
 * Function: Set Display Cursor X Position
 * Purpose:  This function sets the O or X cursor to the defined value for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayCursorXPosition (ViSession instrSession,
                                                   ViInt32 subsystem,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 cursorType,
                                                   ViReal64 cursorXPosition,
                                                   ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:SETT:XPOS", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:SETT:XPOS", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:SETT:XPOS", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:SETT:XPOS", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:SETT:XPOS", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:SETT:XPOS", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                                  buffer,
                                  cursorXPosition,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER5);     
}

/*****************************************************************************
 * Function: Get Display Cursor X Position
 * Purpose:  This function gets the O or X cursor to the defined value for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayCursorXPosition (ViSession instrSession,
                                                   ViInt32 subsystem,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 cursorType,
                                                   ViReal64 *cursorXPosition,
                                                   ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:SETT:XPOS?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:SETT:XPOS?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:SETT:XPOS?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:SETT:XPOS?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:SETT:XPOS?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:SETT:XPOS?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  cursorXPosition,
                                  units,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Cursor Y Position
 * Purpose:  This function sets the O or X cursor Y position for the desired 
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayCursorYPosition (ViSession instrSession,
                                                   ViInt32 subsystem,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 cursorType,
                                                   ViReal64 cursorYPosition,
                                                   ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:SETT:YPOS", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:SETT:YPOS", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:SETT:YPOS", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:SETT:YPOS", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:SETT:YPOS", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:SETT:YPOS", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                                  buffer,
                                  cursorYPosition,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER5);     
}

/*****************************************************************************
 * Function: Get Display Cursor Y Position
 * Purpose:  This function gets the O or X cursor Y position for the desired 
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayCursorYPosition (ViSession instrSession,
                                                   ViInt32 subsystem,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 cursorType,
                                                   ViReal64 *cursorYPosition,
                                                   ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:SETT:YPOS?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:SETT:YPOS?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:SETT:YPOS?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:SETT:YPOS?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:SETT:YPOS?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:SETT:YPOS?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  cursorYPosition,
                                  units,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Cursor Movement 
 * Purpose:  This function sets the O or X cursor position mode for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayCursorMovement (ViSession instrSession,
                                                  ViInt32 subsystem,
                                                  ViInt32 subsystemNumber,
                                                  ViInt32 cursorType,
                                                  ViInt32 cursorPositionMode)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            CHECKVIINT32 (cursorPositionMode, 0, 2, 5);
            sprintf (buffer, "DISP:SWE%ld:%s:POSM", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            CHECKVIINT32 (cursorPositionMode, 0, 2, 5);
            sprintf (buffer, "DISP:MON:%s:POSM", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:POSM", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            CHECKVIINT32 (cursorPositionMode, 0, 1, 5);
            sprintf (buffer, "DISP:WAV:%s:POSM", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            CHECKVIINT32 (cursorPositionMode, 1, 1, 5);
            sprintf (buffer, "DISP:BAR%ld:%s:POSM", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            CHECKVIINT32 (cursorPositionMode, 0, 2, 5);
            sprintf (buffer, "DISP:PESQ%ld:%s:POSM", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          cursorPositionMode,
                          displayCursorPositionModeArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Cursor Movement 
 * Purpose:  This function gets the O or X cursor position mode for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayCursorMovement (ViSession instrSession,
                                                  ViInt32 subsystem,
                                                  ViInt32 subsystemNumber,
                                                  ViInt32 cursorType,
                                                  ViInt32 *cursorPositionMode)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (cursorType, RSUPV_DISP_CURS_TYPE_O, RSUPV_DISP_CURS_TYPE_X, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:POSM?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:POSM?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:POSM?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:POSM?", displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:POSM?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:POSM?", subsystemNumber, displayCursorTypeArr[cursorType]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          cursorPositionMode,
                          displayCursorPositionModeArr,
                          VI_ERROR_PARAMETER5);      
}

/*****************************************************************************
 * Function: Set Display Marker Mode 
 * Purpose:  This function sets the marker mode for Trace A or Trace B for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayMarkerMode (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 trace, ViInt32 markerMode)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:MARK:MODE", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:MARK:MODE", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:MARK:MODE", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:MARK:MODE", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:MARK:MODE", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:MARK:MODE", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          markerMode,
                          displayMarkerModeArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Marker Mode 
 * Purpose:  This function gets the marker mode for Trace A or Trace B for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayMarkerMode (ViSession instrSession,
                                              ViInt32 subsystem,
                                              ViInt32 subsystemNumber,
                                              ViInt32 trace, ViInt32 *markerMode)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:MARK:MODE?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:MARK:MODE?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:MARK:MODE?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:MARK:MODE?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:MARK:MODE?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:MARK:MODE?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          markerMode,
                          displayMarkerModeArr,
                          VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Marker Harmonics 
 * Purpose:  This function sets the state of the marker harmonics for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayMarkerHarmonics (ViSession instrSession,
                                                   ViInt32 subsystem,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViBoolean harmonics)
{
    ViChar buffer[BUFFER_SIZE] = "";

    CHECKINSTR (RSUPV_INSTR_UPV);
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:MARK:HARM", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:MARK:HARM", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:MARK:HARM", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:MARK:HARM", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               harmonics,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Marker Harmonics 
 * Purpose:  This function gets the state of the marker harmonics for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayMarkerHarmonics (ViSession instrSession,
                                                   ViInt32 subsystem,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViBoolean *harmonics)
{
    ViChar buffer[BUFFER_SIZE] = "";

    CHECKINSTR (RSUPV_INSTR_UPV);
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:MARK:HARM?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:MARK:HARM?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:MARK:HARM?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:MARK:HARM?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               harmonics,
                               VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Marker Move To 
 * Purpose:  This function sets the marker to the desired position of the trace 
 *           for Trace A or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayMarkerMoveTo (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 trace, ViInt32 moveTo)
{
    ViStatus  rsupv_status;
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    CHECKERR (rsupv_checkIndexInList(instrSession, moveTo, displayCursorTypeArr, VI_ERROR_PARAMETER5));

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:MARK:SETT:%s ONCE", subsystemNumber, displayTraceArr[trace], displayCursorTypeArr[moveTo]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:MARK:SETT:%s ONCE", displayTraceArr[trace], displayCursorTypeArr[moveTo]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:MARK:SETT:%s ONCE", subsystemNumber, displayTraceArr[trace], displayCursorTypeArr[moveTo]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:MARK:SETT:%s ONCE", displayTraceArr[trace], displayCursorTypeArr[moveTo]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:MARK:SETT:%s ONCE", subsystemNumber, displayTraceArr[trace], displayCursorTypeArr[moveTo]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:MARK:SETT:%s ONCE", subsystemNumber, displayTraceArr[trace], displayCursorTypeArr[moveTo]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_sendCmd (instrSession, buffer);    
}

/*****************************************************************************
 * Function: Set Display Marker X Position 
 * Purpose:  This function sets marker to a position on the X-axis for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayMarkerXPosition (ViSession instrSession,
                                                   ViInt32 subsystem,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViReal64 markerXPosition,
                                                   ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:MARK:SETT:XPOS", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:MARK:SETT:XPOS", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:MARK:SETT:XPOS", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:MARK:SETT:XPOS", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:MARK:SETT:XPOS", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:MARK:SETT:XPOS", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                                  buffer,
                                  markerXPosition,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Get Display Marker X Position 
 * Purpose:  This function gets marker to a position on the X-axis for Trace A 
 *           or Trace B for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayMarkerXPosition (ViSession instrSession,
                                                   ViInt32 subsystem,
                                                   ViInt32 subsystemNumber,
                                                   ViInt32 trace,
                                                   ViReal64 *markerXPosition,
                                                   ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (trace, RSUPV_TRACE_A, RSUPV_TRACE_B, 4);

    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "DISP:SWE%ld:%s:MARK:SETT:XPOS?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "DISP:MON:%s:MARK:SETT:XPOS?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:FFT%ld:%s:MARK:SETT:XPOS?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "DISP:WAV:%s:MARK:SETT:XPOS?", displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:BAR%ld:%s:MARK:SETT:XPOS?", subsystemNumber, displayTraceArr[trace]);
        break;
        case RSUPV_DISP_SUBSYS_PESQ:
            CHECKINSTR (RSUPV_INSTR_UPV);
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "DISP:PESQ%ld:%s:MARK:SETT:XPOS?", subsystemNumber, displayTraceArr[trace]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  markerXPosition,
                                  units,
                                  VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Display Store Trace As 
 * Purpose:  This function sets the format in which the current trace is to be 
 *           stored.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayStoreTraceAs (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 storeTraceAs)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:STAS", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:STAS");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:STAS", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            if (storeTraceAs != RSUPV_DISP_STORE_TRACE_AS_TRCLIST)
                return VI_ERROR_PARAMETER4;
            sprintf (buffer, "MMEM:WAV:STAS");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:STAS", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          storeTraceAs,
                          displayStoreTraceAsArr,
                          VI_ERROR_PARAMETER4);     
}

/*****************************************************************************
 * Function: Get Display Store Trace As 
 * Purpose:  This function gets the format in which the current trace is to be 
 *           stored.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayStoreTraceAs (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 *storeTraceAs)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:STAS?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:STAS?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:STAS?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "MMEM:WAV:STAS?");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:STAS?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          storeTraceAs,
                          displayStoreTraceAsArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Trace Select
 * Purpose:  This function sets whether the data set of Trace A or Trace B 
 *           will be used for creating the corresponding lists for the desired 
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayTraceSelect (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 trace)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:TRAC", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:TRAC");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:TRAC", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:TRAC", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setEnum (instrSession, 
                          buffer,
                          trace,
                          displayTraceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Trace Select
 * Purpose:  This function gets whether the data set of Trace A or Trace B 
 *           will be used for creating the corresponding lists for the desired 
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayTraceSelect (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 *trace)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:TRAC?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:TRAC?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:TRAC?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:TRAC?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getEnum (instrSession, 
                          buffer,
                          trace,
                          displayTraceArr,
                          VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Normalization Frequency
 * Purpose:  This function sets the the frequency for the desired subsystem 
 *           whose voltage value is used for normalization. If the frequency 
 *           selected is that at which the frequency response curve is at its 
 *           maximum, the precorrector file contains only values from 0.0 to 
 *           1.0. If a different frequency is selected, factors greater than 
 *           1.0 also occur.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayNormalizationFrequency (ViSession instrSession,
                                                          ViInt32 subsystem,
                                                          ViInt32 subsystemNumber,
                                                          ViReal64 normalizationFrequency,
                                                          ViInt32 units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:EQU:NORM", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:EQU:NORM");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:EQU:NORM", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                                  buffer,
                                  normalizationFrequency,
                                  units,
                                  0.0,0.0,
                                  VI_ERROR_PARAMETER4);     
}

/*****************************************************************************
 * Function: Get Display Normalization Frequency
 * Purpose:  This function gets the the frequency for the desired subsystem 
 *           whose voltage value is used for normalization. If the frequency 
 *           selected is that at which the frequency response curve is at its 
 *           maximum, the precorrector file contains only values from 0.0 to 
 *           1.0. If a different frequency is selected, factors greater than 
 *           1.0 also occur.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayNormalizationFrequency (ViSession instrSession,
                                                          ViInt32 subsystem,
                                                          ViInt32 subsystemNumber,
                                                          ViReal64 *normalizationFrequency,
                                                          ViInt32 *units)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:EQU:NORM?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:EQU:NORM?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:EQU:NORM?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  normalizationFrequency,
                                  units,
                                  VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Modify Equ
 * Purpose:  This function sets the state of the modify equalization for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayModifyEqu (ViSession instrSession,
                                             ViInt32 subsystem,
                                             ViInt32 subsystemNumber,
                                             ViBoolean modifyEqulization)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:EQU:MOD", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:EQU:MOD");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:EQU:MOD", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               modifyEqulization,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Modify Equ
 * Purpose:  This function gets the state of the modify equalization for the 
 *           desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayModifyEqu (ViSession instrSession,
                                             ViInt32 subsystem,
                                             ViInt32 subsystemNumber,
                                             ViBoolean *modifyEqulization)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:EQU:MOD?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:EQU:MOD?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:EQU:MOD?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               modifyEqulization,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Invert Equ
 * Purpose:  This function activate/desactivate the invert equalization.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayInvertEqu (ViSession instrSession,
                                    ViInt32 subsystem, ViInt32 subsystemNumber,
                                    ViBoolean invertEqualization)
{
 
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:EQU:INV", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:EQU:INV");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:EQU:INV", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               invertEqualization,
                               VI_ERROR_PARAMETER4); 
}

/*****************************************************************************
 * Function: Get Display Invert Equ
 * Purpose:  This function queries the state of the invert equalization.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayInvertEqu (ViSession instrSession,
                                    ViInt32 subsystem, ViInt32 subsystemNumber,
                                    ViBoolean *invertEqualization)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:EQU:INV?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:EQU:INV?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:EQU:INV?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               invertEqualization,
                               VI_ERROR_PARAMETER4); 

}
/*****************************************************************************
 * Function: Set Display Limit Offset
 * Purpose:  This function sets the offset state to the limit trace data for 
 *           the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLimitOffset (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViBoolean limitOffset)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:LIM:OFFS", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:LIM:OFFS");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:LIM:OFFS", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:LIM:OFFS", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViBoolean (instrSession, 
                               buffer,
                               limitOffset,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Limit Offset
 * Purpose:  This function gets the offset state to the limit trace data for 
 *           the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLimitOffset (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViBoolean *limitOffset)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:LIM:OFFS?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:LIM:OFFS?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:LIM:OFFS?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:LIM:OFFS?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViBoolean (instrSession, 
                               buffer,
                               limitOffset,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Limit Offset Value
 * Purpose:  This function sets the offset value to the limit trace data for 
 *           the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayLimitOffsetValue (ViSession instrSession,
                                                    ViInt32 subsystem,
                                                    ViInt32 subsystemNumber,
                                                    ViReal64 limitOffsetValue)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:LIM:OFFS:VAL", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:LIM:OFFS:VAL");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:LIM:OFFS:VAL", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:LIM:OFFS:VAL", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Unit (instrSession, 
                                  buffer,
                                  limitOffsetValue,
                                  RSUPV_UNIT_DBFS,
                                  0.0,0.0,
                                  VI_ERROR_PARAMETER4);     
}

/*****************************************************************************
 * Function: Get Display Limit Offset Value
 * Purpose:  This function gets the offset value to the limit trace data for 
 *           the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayLimitOffsetValue (ViSession instrSession,
                                                    ViInt32 subsystem,
                                                    ViInt32 subsystemNumber,
                                                    ViReal64 *limitOffsetValue)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:LIM:OFFS:VAL?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:LIM:OFFS:VAL?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:LIM:OFFS:VAL?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:LIM:OFFS:VAL?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Unit (instrSession, 
                                  buffer,
                                  limitOffsetValue,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Display Store Trace To File
 * Purpose:  This function stores the current trace, complete with all scans, 
 *           in the format specified by Set Display Store Trace As, under the 
 *           specified file name for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetDisplayStoreTraceToFile (ViSession instrSession,
                                                    ViInt32 subsystem,
                                                    ViInt32 subsystemNumber,
                                                    ViString storeTraceToFile)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:STOR", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:STOR");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:STOR", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "MMEM:WAV:STOR");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:STOR", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    storeTraceToFile,
                                    VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Display Store Trace To File
 * Purpose:  This function returns the current trace file name for the desired 
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetDisplayStoreTraceToFile (ViSession instrSession,
                                                    ViInt32 subsystem,
                                                    ViInt32 subsystemNumber,
                                                    ViInt32 bufferSize,
                                                    ViChar storeTraceToFile[])
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "MMEM:SWE%ld:STOR?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "MMEM:MON:STOR?");
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:FFT%ld:STOR?", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "MMEM:WAV:STOR?");
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "MMEM:BAR%ld:STOR?", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViStringQuoted (instrSession, 
                                    buffer,
                                    "'",
                                    bufferSize,
                                    storeTraceToFile,
                                    VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Set Measurement Mode
 * Purpose:  This function selects whether is instrument in continuous or
 *           single measurement mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMeasurementMode(
    ViSession   instrSession,
    ViInt32   measurementMode
)
{
    return rsupv_setEnum (instrSession,
                          "INIT:CONT",
                          measurementMode,
                          measurementModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Measurement Mode
 * Purpose:  This function returns whether is instrument in continuous or
 *           single measurement mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMeasurementMode(
    ViSession   instrSession,
    ViInt32*  measurementMode
)
{
    return rsupv_getEnum (instrSession,
                          "INIT:CONT?",
                          measurementMode,
                          measurementModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Analyzer Notch Filter
 * Purpose:  This function configures the analyzer notch filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureAnalyzerNotchFilter(
    ViSession   instrSession,
    ViInt32     notchGain,
    ViInt32     notchFrequency,
    ViReal64    notchFrequencyValue
)
{
    ViStatus    rsupv_status;
    ViInt32     instrument = 0;
    ViInt32     function = 0;

    CHECKINSTR (RSUPV_INSTR_UPV);
    rsupv_status = rsupv_GetAnalyzerInstrument(instrSession, &instrument);
    rsupv_status = rsupv_GetAnalyzerFunction(instrSession, &function);

    if (((function == RSUPV_ANALYZER_FUNC_RMS) |
        (function == RSUPV_ANALYZER_FUNC_FFT)) &&
        (instrument == RSUPV_ANALYZER_ANALOG))
    {
        CHECKERR (rsupv_SetAnalyzerNotchGain(instrSession, notchGain));

        if (notchGain != RSUPV_ANALYZER_NOTCH_GAIN_OFF)
        {
            rsupv_status = rsupv_SetAnalyzerNotchFrequency(instrSession, notchFrequency);
            REMAP (2, 3);

            if (notchFrequency == RSUPV_ANALYZER_NOTCH_FREQ_FIX)
            {
                rsupv_status = rsupv_SetAnalyzerNotchCenterFrequency(instrSession, notchFrequencyValue);
                REMAP (2, 4);
            }
        }
    }

    else 
        rsupv_status = RSUPV_ERROR_INSTRUMENT_ERROR;
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Analyzer Notch Gain
 * Purpose:  This function activates/deactivates the analog notch filter and
 *           select the postgain. Only with AC level measurements in the analog
 *           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerNotchGain(
    ViSession   instrSession,
    ViInt32     notchGain
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                        "SENS:NOTC",
                        notchGain,
                        anNotchGainArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Notch Gain
 * Purpose:  This function gets the analog notch filter and the postgain. Only
 *           with AC level measurements in the analog analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerNotchGain(
    ViSession   instrSession,
    ViInt32*    notchGain
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                        "SENS:NOTC?",
                        notchGain,
                        anNotchGainArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Notch Frequency
 * Purpose:  This function selects the center frequency of the notch filter
 *           (notch frequency); only with activated analog notch filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerNotchFrequency(
    ViSession   instrSession,
    ViInt32     notchFrequency
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                        "SENS:NOTC:FREQ:MODE",
                        notchFrequency,
                        anNotchFrequencyArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Notch Frequency
 * Purpose:  This function gets the center frequency of the notch filter
 *           (notch frequency); only with activated analog notch filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerNotchFrequency(
    ViSession   instrSession,
    ViInt32*    notchFrequency
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                        "SENS:NOTC:FREQ:MODE?",
                        notchFrequency,
                        anNotchFrequencyArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Notch Center Frequency
 * Purpose:  This function sets the notch-filter center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerNotchCenterFrequency(
    ViSession   instrSession,
    ViReal64    notchFrequencyValue
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                                    "SENS:NOTC:FREQ",
                                    notchFrequencyValue,
                                    RSUPV_UNIT_HZ,
                                    10.0, 115.0e3,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Notch Center Frequency
 * Purpose:  This function gets the notch-filter center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerNotchCenterFrequency(
    ViSession   instrSession,
    ViReal64*   notchFrequencyValue
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                                    "SENS:NOTC:FREQ?",
                                    notchFrequencyValue,
                                    VI_NULL,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer DC Suppression
 * Purpose:  This function activates/deactivates DC suppression (only with RMS
 *           measurements in the digital analyzer).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerDCSuppression(
    ViSession   instrSession,
    ViBoolean   dcSuppression
)
{
    return rsupv_setViBoolean (instrSession,
                               "SENS:FUNC:DCS",
                               dcSuppression,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer DC Suppression
 * Purpose:  This function gets the DC suppression state (only with RMS
 *           measurements in the digital analyzer).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerDCSuppression(
    ViSession   instrSession,
    ViBoolean*  dcSuppression
)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS:FUNC:DCS?",
                               dcSuppression,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Filter
 * Purpose:  This function activates a digital filter in the measurement path.
 *           Depending on the selected level measurement method, up to 3
 *           user-defined filters or weighting filters can be looped into the
 *           measurement path:
 *
 *           - RMS: max. 3 filters
 *           - RMS Selective: max. 1 filter in addition to selection filter
 *           - Peak: max. 3 filters
 *           - Quasi Peak: max. 3 filters
 *           - DC: no filters
 *           - FFT: max. 3 filters
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFilter(
    ViSession   instrSession,
    ViInt32     filterNumber,
    ViInt32     filter
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPV) == VI_SUCCESS)
    {
        CHECKVIINT32 (filterNumber, 1, 3, 2);
    }
    else
    {
        CHECKVIINT32 (filterNumber, 1, 2, 2);
    }

    sprintf (buffer, "SENS:FILT%ld", filterNumber);

    return rsupv_setEnum (instrSession,
                        buffer,
                        filter,
                        anFilterArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analyzer Filter
 * Purpose:  This function gets a digital filter in the measurement path.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFilter(
    ViSession   instrSession,
    ViInt32     filterNumber,
    ViInt32*    filter
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPV) == VI_SUCCESS)
    {
        CHECKVIINT32 (filterNumber, 1, 3, 2);
    }
    else
    {
        CHECKVIINT32 (filterNumber, 1, 2, 2);
    }

    sprintf (buffer, "SENS:FILT%ld?", filterNumber);

    return rsupv_getEnum (instrSession,
                        buffer,
                        filter,
                        anFilterArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Analyzer Level Meas Time
 * Purpose:  This function selects the measurement time for adapting the
 *           measurement speed to the signal shape or (with single-tone
 *           signals) to the signal frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMeasTime(
    ViSession   instrSession,
    ViInt32     measurementTime
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:APER:MODE",
                        measurementTime,
                        anLevelMeasTimeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Level Meas Time
 * Purpose:  This function gets the measurement time for adapting the
 *           measurement speed to the signal shape or (with single-tone
 *           signals) to the signal frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMeasTime(
    ViSession   instrSession,
    ViInt32*    measurementTime
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:APER:MODE?",
                        measurementTime,
                        anLevelMeasTimeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Level Meas Time Value
 * Purpose:  This function sets the value of the measurement time.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMeasTimeValue(
    ViSession   instrSession,
    ViReal64    measurementTimeValue
)
{
    return rsupv_setViReal64Unit (instrSession,
                                    "SENS:VOLT:APER",
                                    measurementTimeValue,
                                    RSUPV_UNIT_S,
                                    100.0e-6, 10.0,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Level Meas Time Value
 * Purpose:  This function gets the value of the measurement time.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMeasTimeValue(
    ViSession   instrSession,
    ViReal64*   measurementTimeValue
)
{
    return rsupv_getViReal64Unit (instrSession,
                                    "SENS:VOLT:APER?",
                                    measurementTimeValue,
                                    VI_NULL,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Measurement Functions Settling
 * Purpose:  This function configures the measurement settling.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureMeasurementFunctionsSettling (ViSession instrSession,
                                                               ViInt32 fnctSettling,
                                                               ViInt32 samples,
                                                               ViReal64 timeout,
                                                               ViReal64 resolution,
                                                               ViReal64 tolerance)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetMeasurementFunctionsSettling (instrSession, fnctSettling));
    
    if (fnctSettling != RSUPV_MEAS_FUNC_SETTLING_OFF)
    {

        rsupv_status = rsupv_SetMeasurementFunctionSettlingCount (instrSession, samples);
        REMAP (2, 3);
    
        rsupv_status = rsupv_SetMeasurementFunctionSettlingTimeout (instrSession, timeout);
        REMAP (2, 4);
    
        if ((fnctSettling == RSUPV_MEAS_FUNC_SETTLING_EXP) || (fnctSettling == RSUPV_MEAS_FUNC_SETTLING_FLAT))
        {    
            rsupv_status = rsupv_SetMeasurementFunctionSettlingResolution(instrSession, resolution);
            REMAP (2, 5);
        
            rsupv_status = rsupv_SetMeasurementFunctionSettlingTolerance (instrSession, tolerance);
            REMAP (2, 6);
        }
    }

    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Analyzer FFT Frequency Limit
 * Purpose:  This function configures the frequency limit settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureAnalyzerFFTFrequencyLimit (ViSession instrSession,
                                                         ViBoolean limitEnable,
                                                         ViReal64 freqLimLow,
                                                         ViReal64 freqLimUpp)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetAnalyzerFFTFrequencyLimitState (instrSession, limitEnable));

    if (limitEnable == VI_TRUE)
    {    
        rsupv_status = rsupv_SetAnalyzerFFTFrequencyLimitLow(instrSession, freqLimLow);
        REMAP (2, 3);
        
        rsupv_status = rsupv_SetAnalyzerFFTFrequencyLimitUpp (instrSession, freqLimUpp);
        REMAP (2, 4);
    }

    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Analyzer Sweep
 * Purpose:  This function configures the sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureAnalyzerSweep(
    ViSession   instrSession,
    ViInt32     sweepControl,
    ViInt32     spacing,
    ViInt32     points,
    ViReal64    start,
    ViInt32     startUnits,
    ViReal64    stop,
    ViInt32     stopUnits,
    ViReal64    step,
    ViInt32     stepUnits
)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetAnalyzerSweepControl(instrSession, sweepControl));

    rsupv_status = rsupv_SetAnalyzerSweepSpacing(instrSession, spacing);
    REMAP (2, 3);
    
    if ((spacing == RSUPV_ANALYZER_SWEEP_SPAC_LINP) || (spacing == RSUPV_ANALYZER_SWEEP_SPAC_LOGP))
    {    
        rsupv_status = rsupv_SetAnalyzerSweepPoints(instrSession, points);
        REMAP (2, 4);
    }
    rsupv_status = rsupv_SetAnalyzerSweepStart(instrSession, start, startUnits);
    REMAP (2, 5);
    rsupv_status = rsupv_SetAnalyzerSweepStop(instrSession, stop, stopUnits);
    REMAP (2, 7);
    
    if ((spacing == RSUPV_ANALYZER_SWEEP_SPAC_LINS) || (spacing == RSUPV_ANALYZER_SWEEP_SPAC_LOGS))
    {    
        rsupv_status = rsupv_SetAnalyzerSweepSteps(instrSession, step, stepUnits);
        REMAPINSTR (2, 9);
    }

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Analyzer Sweep Control
 * Purpose:  This function activates and configures or deactivates the
 *           band-pass sweep / band-stop sweep in the analyzer sweep system.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSweepControl(
    ViSession   instrSession,
    ViInt32     control
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:SWE:CONT",
                        control,
                        anControlArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Sweep Control
 * Purpose:  This function gets the band-pass sweep / band-stop sweep in the
 *           analyzer sweep system.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSweepControl(
    ViSession   instrSession,
    ViInt32*    control
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:SWE:CONT?",
                        control,
                        anControlArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Sweep Spacing
 * Purpose:  This function selects the sweep range spacing, i.e. the way in
 *           which the sweep points are distributed on the sweep axis. Only
 *           visible for activated band-pass / band-stop sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSweepSpacing(
    ViSession   instrSession,
    ViInt32     spacing
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:SWE:SPAC",
                        spacing,
                        anSpacingArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Sweep Spacing
 * Purpose:  This function gets the sweep range spacing, i.e. the way in which
 *           the sweep points are distributed on the sweep axis. Only visible
 *           for activated band-pass / band-stop sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSweepSpacing(
    ViSession   instrSession,
    ViInt32*    spacing
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:SWE:SPAC?",
                        spacing,
                        anSpacingArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Sweep Start
 * Purpose:  This function sets the start value for the band-pass / band-stop
 *           center frequency. Only visible for activated band-pass / band-stop
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSweepStart(
    ViSession   instrSession,
    ViReal64    start,
    ViInt32     units
)
{
    return rsupv_setViReal64Unit (instrSession,
                                    "SENS:FREQ:STAR",
                                    start,
                                    units,
                                    0.0, 0.0,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Sweep Start
 * Purpose:  This function gets the start value for the band-pass / band-stop
 *           center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSweepStart(
    ViSession   instrSession,
    ViReal64*   start,
    ViInt32*    units
)
{
    return rsupv_getViReal64Unit (instrSession,
                                    "SENS:FREQ:STAR?",
                                    start,
                                    units,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Sweep Stop
 * Purpose:  This function sets the stop value for the band-pass / band-stop
 *           center frequency. Only visible for activated band-pass / band-stop
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSweepStop(
    ViSession   instrSession,
    ViReal64    stop,
    ViInt32     units
)
{
    return rsupv_setViReal64Unit (instrSession,
                                    "SENS:FREQ:STOP",
                                    stop,
                                    units,
                                    0.0, 0.0,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Sweep Stop
 * Purpose:  This function gets the stop value for the band-pass / band-stop
 *           center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSweepStop(
    ViSession   instrSession,
    ViReal64*   stop,
    ViInt32*    units
)
{
    return rsupv_getViReal64Unit (instrSession,
                                    "SENS:FREQ:STOP?",
                                    stop,
                                    units,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Sweep Points
 * Purpose:  This function sets the number of sweep points for the band-pass /
 *           bandstop center frequency. Only visible for activated band-pass /
 *           bandstop sweep if "Lin Points" or "Log Points" is set for
 *           "Spacing".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSweepPoints(
    ViSession   instrSession,
    ViInt32     points
)
{
    return rsupv_setViInt32 (instrSession,
                            "SENS:SWE:POIN",
                            points,
                            2, 1024,
                            VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Sweep Points
 * Purpose:  This function gets the number of sweep points for the band-pass /
 *           bandstop center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSweepPoints(
    ViSession   instrSession,
    ViInt32*    points
)
{
    return rsupv_getViInt32 (instrSession,
                            "SENS:SWE:POIN?",
                            points,
                            VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Sweep Steps
 * Purpose:  This function sets the (linear or logarithmic) step size for the
 *           band-pass / band-stop center frequency. Only visible for activated
 *           band-pass / band-stop sweep if "Lin Steps" or "Log Steps" is set
 *           for "Spacing".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSweepSteps(
    ViSession   instrSession,
    ViReal64    step,
    ViInt32     units
)
{
    return rsupv_setViReal64Unit (instrSession,
                                    "SENS:SWE:STEP",
                                    step,
                                    units,
                                    0.0, 0.0,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Sweep Steps
 * Purpose:  This function gets the (linear or logarithmic) step size for the
 *           band-pass / band-stop center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSweepSteps(
    ViSession   instrSession,
    ViReal64*   step,
    ViInt32*    units
)
{
    return rsupv_getViReal64Unit (instrSession,
                                    "SENS:SWE:STEP?",
                                    step,
                                    units,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Analyzer Peak Measurement
 * Purpose:  This function configures analyzer peak measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureAnalyzerPeakMeasurement (
    ViSession   instrSession,
    ViInt32     measMode,
    ViInt32     intvTime,
    ViReal64    intvTimeValue
)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetAnalyzerPeakMeasMode(instrSession, measMode));

    rsupv_status = rsupv_SetAnalyzerPeakMeasIntervalTime(instrSession, intvTime);
    REMAP (2, 3);

    if (intvTime == RSUPV_ANALYZER_PEAK_MEAS_INTV_TIME_VAL)
    {
        rsupv_status = rsupv_SetAnalyzerPeakMeasIntervalTimeValue(instrSession, intvTimeValue);
        REMAP (2, 4);
    }

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Analyzer Peak Meas Mode
 * Purpose:  This function selects the measurement mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPeakMeasMode (
    ViSession   instrSession,
    ViInt32     measMode
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:MMOD",
                        measMode,
                        anPeakMeasModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Peak Meas Mode
 * Purpose:  This function gets the measurement mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPeakMeasMode(
    ViSession   instrSession,
    ViInt32*    measMode
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:MMOD?",
                        measMode,
                        anPeakMeasModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Peak Meas Interval Time
 * Purpose:  This function selects the measurement interval within which the
 *           peak search is to be performed. When the interval time has
 *           expired, the peak value memory is reset and the next peak value is
 *           sought.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPeakMeasIntervalTime(
    ViSession   instrSession,
    ViInt32     intervalTime
)
{
    if (intervalTime == RSUPV_ANALYZER_PEAK_MEAS_INTV_TIME_FIX)
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
    return rsupv_setEnum (instrSession,
                        "SENS:VOLT:INTV:MODE",
                        intervalTime,
                        anIntervalTimeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Peak Meas Interval Time
 * Purpose:  This function gets the measurement interval within which the peak
 *           search is to be performed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPeakMeasIntervalTime(
    ViSession   instrSession,
    ViInt32*    intervalTime
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:VOLT:INTV:MODE?",
                        intervalTime,
                        anIntervalTimeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Peak Meas Interval Time Value
 * Purpose:  This function sets the peak measurement interval time value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPeakMeasIntervalTimeValue(
    ViSession   instrSession,
    ViReal64    intervalTimeValue
)
{
    return rsupv_setViReal64Unit (instrSession,
                                    "SENS:VOLT:INTV",
                                    intervalTimeValue,
                                    RSUPV_UNIT_S,
                                    0.0, 0.0,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Peak Meas Interval Time Value
 * Purpose:  This function gets the peak measurement interval time value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPeakMeasIntervalTimeValue(
    ViSession   instrSession,
    ViReal64*   intervalTimeValue
)
{
    return rsupv_getViReal64Unit (instrSession,
                                    "SENS:VOLT:INTV?",
                                    intervalTimeValue,
                                    VI_NULL,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Level Meas Bandwidth
 * Purpose:  This function selects the bandwidth and filter type (band-pass 
 *           filter BP... or band-stop filter BS...).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMeasBandwidth (ViSession instrSession,
                                                       ViInt32 bandwidth)
{
    return rsupv_setEnum (instrSession,
                        "SENS:BAND:MODE",
                        bandwidth,
                        anRMSBandwidthArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Level Meas Bandwidth
 * Purpose:  This function returns the bandwidth and filter type (band-pass 
 *           filter BP... or band-stop filter BS...).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMeasBandwidth (ViSession instrSession,
                                                       ViInt32 *bandwidth)
{
    return rsupv_getEnum (instrSession,
                        "SENS:BAND:MODE?",
                        bandwidth,
                        anRMSBandwidthArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Level Meas Bandwidth Value
 * Purpose:  This function sets the bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMeasBandwidthValue (ViSession instrSession,
                                                            ViReal64 bandwidthValue)
{
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:BAND",
                                  bandwidthValue,
                                  RSUPV_UNIT_HZ,
                                  20.0, 16.0e3,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Level Meas Bandwidth Value
 * Purpose:  This function gets the bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMeasBandwidthValue (ViSession instrSession,
                                                            ViReal64 *bandwidthValue)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:BAND?",
                                  bandwidthValue,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Level Meas Frequency Mode
 * Purpose:  This function selects the band-pass / band-stop center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMeasFrequencyMode (ViSession instrSession,
                                                           ViInt32 frequencyMode)
{
    if ((frequencyMode == RSUPV_ANALYZER_MEAS_FMODE_CH1F) ||
        (frequencyMode == RSUPV_ANALYZER_MEAS_FMODE_CH2F))
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
    return rsupv_setEnum (instrSession,
                        "SENS:FREQ:SEL",
                        frequencyMode,
                        anRMSFrequencyModeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Level Meas Frequency Mode
 * Purpose:  This function returns the band-pass / band-stop center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMeasFrequencyMode (ViSession instrSession,
                                                           ViInt32 *frequencyMode)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FREQ:SEL?",
                        frequencyMode,
                        anRMSFrequencyModeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Level Meas Frequency Value
 * Purpose:  This function sets the frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMeasFrequencyValue (ViSession instrSession,
                                                            ViReal64 frequencyValue)
{
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:FREQ",
                                  frequencyValue,
                                  RSUPV_UNIT_HZ,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Level Meas Frequency Value
 * Purpose:  This function gets the frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMeasFrequencyValue (ViSession instrSession,
                                                            ViReal64 *frequencyValue)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FREQ?",
                                  frequencyValue,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Level Meas Frequency Factor
 * Purpose:  This function sets the frequency factor.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMeasFrequencyFactor (ViSession instrSession,
                                                             ViReal64 frequencyFactor)
{
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:FREQ:FACT",
                                  frequencyFactor,
                                  RSUPV_UNIT_MLT,
                                  1.0, 20.0,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Level Meas Frequency Factor
 * Purpose:  This function gets the frequency factor.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMeasFrequencyFactor (ViSession instrSession,
                                                             ViReal64 *frequencyFactor)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FREQ:FACT?",
                                  frequencyFactor,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Level Meas Frequency Start
 * Purpose:  This function sets the start value for the band-pass / band-stop 
 *           center frequency. Only visible for activated band-pass / band-stop
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMeasFrequencyStart (ViSession instrSession,
                                                            ViReal64 frequencyStart)
{
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:FREQ:STAR",
                                  frequencyStart,
                                  RSUPV_UNIT_HZ,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Level Meas Frequency Start
 * Purpose:  This function gets the start value for the band-pass / band-stop 
 *           center frequency. Only visible for activated band-pass / band-stop
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMeasFrequencyStart (ViSession instrSession,
                                                            ViReal64 *frequencyStart)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FREQ:STAR?",
                                  frequencyStart,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Level Meas Frequency Stop
 * Purpose:  This function sets the stop value for the band-pass / band-stop 
 *           center frequency. Only visible for activated band-pass / band-stop
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMeasFrequencyStop (ViSession instrSession,
                                                           ViReal64 frequencyStop)
{
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:FREQ:STOP",
                                  frequencyStop,
                                  RSUPV_UNIT_HZ,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Level Meas Frequency Stop
 * Purpose:  This function gets the stop value for the band-pass / band-stop 
 *           center frequency. Only visible for activated band-pass / band-stop
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMeasFrequencyStop (ViSession instrSession,
                                                           ViReal64 *frequencyStop)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FREQ:STOP?",
                                  frequencyStop,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);   
}

/*****************************************************************************
 * Function: Set Analyzer Level DC Meas Time
 * Purpose:  This function selects the measurement time to adapt the measurement 
 *           speed to the signal shape or to the signal frequency of a 
 *           superimposed AC component.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelDCMeasTime (ViSession instrSession,
                                                    ViInt32 measTime)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:APER:MODE",
                        measTime,
                        anLevelMeasTimeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Level DC Meas Time
 * Purpose:  This function returns the measurement time to adapt the measurement 
 *           speed to the signal shape or to the signal frequency of a 
 *           superimposed AC component.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelDCMeasTime (ViSession instrSession,
                                                    ViInt32 *measTime)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:APER:MODE?",
                        measTime,
                        anLevelMeasTimeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Dynamic Mode
 * Purpose:  This function adapts the attainable dynamic range to the quality
 *           of the DUT. Only available in the analog analyzer; the digital
 *           analyzer always performs measurements with the full dynamic range
 *           (without any reduction in speed).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerDynamicMode(
    ViSession   instrSession,
    ViInt32 dynamicMode
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:DMOD",
                        dynamicMode,
                        anDynamicModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Dynamic Mode
 * Purpose:  This function gets the attainable dynamic range to the quality of
 *           the DUT. Only available in the analog analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerDynamicMode(
    ViSession   instrSession,
    ViInt32*    dynamicMode
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:DMOD?",
                        dynamicMode,
                        anDynamicModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Bargraph State
 * Purpose:  This function sets the state for the bargraph computation.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerBargraphState (ViSession instrSession,
                                                  ViBoolean bargraphState)
{
    return rsupv_setViBoolean (instrSession,
                               "SENS:FUNC:BARG",
                               bargraphState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Bargraph State
 * Purpose:  This function gets the state for the bargraph computation.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerBargraphState (ViSession instrSession,
                                                  ViBoolean *bargraphState)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS:FUNC:BARG?",
                               bargraphState,
                               VI_ERROR_PARAMETER2);    
}


/*****************************************************************************
 * Function: Set Analyzer Refinement
 * Purpose:  Improvement of the signal-to-noise ratio.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerRefinement(
    ViSession   instrSession,
    ViInt32 refinement
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:REFN",
                        refinement,
                        refinementArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Refinement 
 * Purpose:  This function returns the refinement value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerRefinement(
    ViSession   instrSession,
    ViInt32*    refinement
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:REFN?",
                        refinement,
                        refinementArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Analyzer Fundamental
 * Purpose:  This function configures the analyzer fundamental.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureAnalyzerFundamental(
    ViSession   instrSession,
    ViInt32     thdFundamental,
    ViReal64    thdFundamentalValue
)
{
    ViStatus    rsupv_status;
    ViInt32     function = 0;

    rsupv_status = rsupv_GetAnalyzerFunction(instrSession, &function);

    if ((function == RSUPV_ANALYZER_FUNC_THD) | (function == RSUPV_ANALYZER_FUNC_THDN))
    {
        CHECKERR (rsupv_SetAnalyzerTHDFundamental(instrSession, thdFundamental));

        if (thdFundamental == RSUPV_ANALYZER_THD_FUND_VAL)
        {
            rsupv_status = rsupv_SetAnalyzerTHDFundamentalValue(instrSession, thdFundamentalValue);
            REMAP (2, 3);
        }
    }

    else 
        rsupv_status = RSUPV_ERROR_INSTRUMENT_ERROR;
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Analyzer THD, THDN Fundamental
 * Purpose:  This function determines the fundamental frequency. With
 *           precision measurement (in the analog analyzer only), this field is
 *           also used to determine the notch frequency of the automatically
 *           activated analog notch filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDFundamental(
    ViSession   instrSession,
    ViInt32 thdFundamental
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:VOLT:FUND:MODE",
                        thdFundamental,
                        anThdFundamentalArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer THD, THDN Fundamental
 * Purpose:  This function gets the fundamental frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDFundamental(
    ViSession   instrSession,
    ViInt32*    thdFundamental
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:VOLT:FUND:MODE?",
                        thdFundamental,
                        anThdFundamentalArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer THD, THDN Fundamental Value
 * Purpose:  This function sets the THD fundamental value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDFundamentalValue(
    ViSession   instrSession,
    ViReal64    thdFundamentalValue
)
{
    return rsupv_setViReal64Unit (instrSession,
                                "SENS:VOLT:FUND",
                                thdFundamentalValue,
                                RSUPV_UNIT_HZ,
                                0.0, 0.0,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer THD, THDN Fundamental Value
 * Purpose:  This function gets the THD fundamental value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDFundamentalValue(
    ViSession   instrSession,
    ViReal64*   thdFundamentalValue
)
{
    return rsupv_getViReal64Unit (instrSession,
                                    "SENS:VOLT:FUND?",
                                    thdFundamentalValue,
                                    VI_NULL,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Analyzer THD
 * Purpose:  This function configures the analyzer THD measurement mode and
 *           THD harmonic state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureAnalyzerTHD(
    ViSession   instrSession,
    ViInt32     thdMeasurementMode,
    ViInt32     harmonic,
    ViBoolean   harmonicState
)
{
    ViStatus    rsupv_status;
    ViInt32     function = 0;

    rsupv_status = rsupv_GetAnalyzerFunction(instrSession, &function);

    if (function == RSUPV_ANALYZER_FUNC_THD)
    {
        CHECKERR (rsupv_SetAnalyzerTHDMeasMode(instrSession, thdMeasurementMode));

        if ((thdMeasurementMode == RSUPV_ANALYZER_THD_MMODE_SEL) |
                         (thdMeasurementMode == RSUPV_ANALYZER_THD_MMODE_LSEL))
        {
            rsupv_status = rsupv_SetAnalyzerTHDHarmonicState(instrSession, harmonic, harmonicState);
            REMAP (2, 3);
            REMAP (3, 4);
        }
    }

    else 
        rsupv_status = RSUPV_ERROR_INSTRUMENT_ERROR;
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Analyzer THD Meas Mode
 * Purpose:  This function selects the harmonics to be taken into
 *           consideration in the THD measurement value and selects the
 *           calculation formula for the measurement value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDMeasMode(
    ViSession   instrSession,
    ViInt32 thdMeasurementMode
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:MMOD",
                        thdMeasurementMode,
                        anThdMeasurementModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer THD Meas Mode
 * Purpose:  This function gets the harmonics to be taken into consideration
 *           in the THD measurement value and the calculation formula for the
 *           measurement value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDMeasMode(
    ViSession   instrSession,
    ViInt32*    thdMeasurementMode
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:MMOD?",
                        thdMeasurementMode,
                        anThdMeasurementModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer THD Harmonic State
 * Purpose:  This function selects the desired harmonics; only available if
 *           "(Level) Select di" is selected under "Meas Mode".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDHarmonicState(
    ViSession   instrSession,
    ViInt32     harmonic,
    ViBoolean   harmonicState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (harmonic, 2, 9, 2);

    sprintf (buffer, "SENS:FUNC:DIST%ld", harmonic);

    return rsupv_setViBoolean (instrSession,
                               buffer,
                               harmonicState,
                               VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Analyzer THD Harmonic State
 * Purpose:  This function gets the desired harmonics; only available if
 *           "(Level) Select di" is selected under "Meas Mode".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDHarmonicState(
    ViSession   instrSession,
    ViInt32     harmonic,
    ViBoolean*  harmonicState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (harmonic, 2, 9, 2);

    sprintf (buffer, "SENS:FUNC:DIST%ld?", harmonic);

    return rsupv_getViBoolean (instrSession,
                               buffer,
                               harmonicState,
                               VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Analyzer THDN Measurement Mode
 * Purpose:  This function selects the THD+N measurement method and the
 *           calculation formula for the measurement value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDNMeasurementMode(
    ViSession   instrSession,
    ViInt32     thdnMeasurementMode
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:MMOD",
                        thdnMeasurementMode,
                        anThdnMeasurementModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer THDN Measurement Mode
 * Purpose:  This function gets the THD+N measurement method and the
 *           calculation formula for the measurement value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDNMeasurementMode(
    ViSession   instrSession,
    ViInt32*    thdnMeasurementMode
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:MMOD?",
                        thdnMeasurementMode,
                        anThdnMeasurementModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer THDN Rejection
 * Purpose:  This function selects the fundamental rejection. Only available
 *           in "Dynamic Mode Fast" or in the digital analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDNRejection(
    ViSession   instrSession,
    ViInt32     thdnRejection
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:THDN:REJ",
                        thdnRejection,
                        anThdnRejectionArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer THDN Rejection
 * Purpose:  This function gets the fundamental rejection. Only available in
 *           "Dynamic Mode Fast" or in the digital analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDNRejection(
    ViSession   instrSession,
    ViInt32*    thdnRejection
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:THDN:REJ?",
                        thdnRejection,
                        anThdnRejectionArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer THDN Reject Bandwidth
 * Purpose:  This function selects the rejection bandwidth by implicitly
 *           entering the initial FFT value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDNRejectBandwidth(
    ViSession   instrSession,
    ViInt32     thdnRejectBandwidth
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:APER:MODE",
                        thdnRejectBandwidth,
                        anThdnRejectBandwidthArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer THDN Reject Bandwidth
 * Purpose:  This function gets the rejection bandwidth by implicitly entering
 *           the initial FFT value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDNRejectBandwidth(
    ViSession   instrSession,
    ViInt32*    thdnRejectBandwidth
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:APER:MODE?",
                        thdnRejectBandwidth,
                        anThdnRejectBandwidthArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer THDN Equalizer
 * Purpose:  This function activates an equalization table which contains
 *           frequency information and the associated voltage gain factors.
 *           This table can be used to, for example, equalize the frequency
 *           response of a transmission path so that the measurement point can
 *           be transformed to a different reference point.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDNEqualizer(
    ViSession   instrSession,
    ViBoolean   thdnEqualizer
)
{
    return rsupv_setViBoolean (instrSession,
                               "SENS:VOLT:EQU",
                               thdnEqualizer,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer THDN Equalizer
 * Purpose:  This function gets an equalization table which contains frequency
 *           information and the associated voltage gain factors.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDNEqualizer(
    ViSession   instrSession,
    ViBoolean*  thdnEqualizer
)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS:VOLT:EQU?",
                               thdnEqualizer,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer THDN Equalizer File
 * Purpose:  This function selects the equalization file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDNEqualizerFile(
    ViSession   instrSession,
    ViString    thdnEqualizerFile
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:IEQ",
                                    "'",
                                    thdnEqualizerFile,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer THDN Frequency Lim Low
 * Purpose:  This function sets the lower band limit for calculating the
 *           distortion voltage; this limit does not affect calculation of the
 *           total RMS.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDNFrequencyLimLow(
    ViSession   instrSession,
    ViReal64    frequencyLimitLower
)
{
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:FREQ:LIM:LOW",
                                  frequencyLimitLower,
                                  RSUPV_UNIT_HZ,
                                  10.0, 21.9331e+3,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer THDN Frequency Lim Low
 * Purpose:  This function gets the lower band limit for calculating the
 *           distortion voltage; this limit does not affect calculation of the
 *           total RMS.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDNFrequencyLimLow(
    ViSession   instrSession,
    ViReal64*   frequencyLimitLower
)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FREQ:LIM:LOW?",
                                  frequencyLimitLower,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer THDN Frequency Lim Upp
 * Purpose:  This function sets the upper band limit for calculating the
 *           distortion voltage and the total RMS.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerTHDNFrequencyLimUpp(
    ViSession   instrSession,
    ViReal64    frequencyLimitUpper
)
{
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:FREQ:LIM:UPP",
                                  frequencyLimitUpper,
                                  RSUPV_UNIT_HZ,
                                  18.7891, 260.0e+3,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer THDN Frequency Lim Upp
 * Purpose:  This function gets the upper band limit for calculating the
 *           distortion voltage and the total RMS.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerTHDNFrequencyLimUpp(
    ViSession   instrSession,
    ViReal64*   frequencyLimitUpper
)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FREQ:LIM:UPP?",
                                  frequencyLimitUpper,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer DFD Measurement Mode
 * Purpose:  This function selects the order of the intermodulation products
 *           and the measurement standard.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerDFDMeasurementMode(
    ViSession   instrSession,
    ViInt32 dfdMeasurementMode
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:MMOD",
                        dfdMeasurementMode,
                        anDfdMeasurementModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer DFD Measurement Mode
 * Purpose:  This function gets the order of the intermodulation products and
 *           the measurement standard.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerDFDMeasurementMode(
    ViSession   instrSession,
    ViInt32*    dfdMeasurementMode
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:MMOD?",
                        dfdMeasurementMode,
                        anDfdMeasurementModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Size
 * Purpose:  This function sets the size of the Function FFT.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTSize(
    ViSession   instrSession,
    ViInt32     fftSize
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:FFT:SIZE",
                        fftSize,
                        anFftSizeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Size
 * Purpose:  This function gets the size of the Function FFT.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTSize(
    ViSession   instrSession,
    ViInt32*    fftSize
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:FFT:SIZE?",
                        fftSize,
                        anFftSizeArr,
                        VI_ERROR_PARAMETER2);
}


/*****************************************************************************
 * Function: Set Analyzer FFT Max Size
 * Purpose:  This function sets the max size of the Function FFT.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTMaxSize(
    ViSession   instrSession,
    ViInt32     fftSize
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:MAX:FFT:SIZE",
                        fftSize,
                        anFftSizeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Max Size
 * Purpose:  This function gets the max size of the Function FFT.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTMaxSize(
    ViSession   instrSession,
    ViInt32*    fftSize
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:MAX:FFT:SIZE?",
                        fftSize,
                        anFftSizeArr,
                        VI_ERROR_PARAMETER2);
}   
    
/*****************************************************************************
 * Function: Get Analyzer FFT Resolution
 * Purpose:  This function displays the frequency resolution.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTResolution(
    ViSession   instrSession,
    ViReal64*   fftResolution
)
{
    return rsupv_getViReal64Unit (instrSession,
                                    "SENS:FUNC:FFT:RES?",
                                    fftResolution,
                                    VI_NULL,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Measurement Time
 * Purpose:  This function displays the integration time.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTMeasurementTime(
    ViSession   instrSession,
    ViReal64*   fftMeasurementTime
)
{
    return rsupv_getViReal64Unit (instrSession,
                                    "SENS:FUNC:FFT:MTIM?",
                                    fftMeasurementTime,
                                    VI_NULL,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Window
 * Purpose:  This function selects the window function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTWindow(
    ViSession   instrSession,
    ViInt32 fftWindow
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:FFT:WIND",
                        fftWindow,
                        anFftWindowArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Window
 * Purpose:  This function gets the window function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTWindow(
    ViSession   instrSession,
    ViInt32*    fftWindow
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:FFT:WIND?",
                        fftWindow,
                        anFftWindowArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Delay Ch1
 * Purpose:  This function activates the delay compensation.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTDelayCh1(
    ViSession   instrSession,
    ViReal64    fftDelayChannel1
)
{
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:CHAN:DEL",
                                  fftDelayChannel1,
                                  RSUPV_UNIT_S,
                                  -10.0, 10.0,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Delay Ch1
 * Purpose:  This function gets the delay compensation.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTDelayCh1(
    ViSession   instrSession,
    ViReal64*   fftDelayChannel1
)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:CHAN:DEL?",
                                  fftDelayChannel1,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Avg Mode
 * Purpose:  This function activates the averaging function and select the
 *           averaging method.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTAvgMode(
    ViSession   instrSession,
    ViInt32 fftAverageMode
)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:FFT:AVER:MODE",
                        fftAverageMode,
                        anFftAverageModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Avg Mode
 * Purpose:  This function gets the averaging function and the averaging
 *           method.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTAvgMode(
    ViSession   instrSession,
    ViInt32*    fftAverageMode
)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:FFT:AVER:MODE?",
                        fftAverageMode,
                        anFftAverageModeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Avg Count
 * Purpose:  This function selects the average count; only if averaging is
 *           activated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTAvgCount(
    ViSession   instrSession,
    ViInt32 fftAverageCount
)
{
    return rsupv_setViInt32 (instrSession,
                              "SENS:FUNC:FFT:AVER",
                              fftAverageCount,
                              1, 10000,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Avg Count
 * Purpose:  This function gets the average count; only if averaging is
 *           activated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTAvgCount(
    ViSession   instrSession,
    ViInt32*    fftAverageCount
)
{
    return rsupv_getViInt32 (instrSession,
                              "SENS:FUNC:FFT:AVER?",
                              fftAverageCount,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Undersample State
 * Purpose:  This function activates/deactivates the Undersample FFT.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTUndersampleState (ViSession instrSession,
                                                        ViBoolean undersample)
{
    return rsupv_setViBoolean (instrSession,
                               "SENS:FUNC:FFT:USAM",
                               undersample,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Undersample State
 * Purpose:  This function gets state of the Undersample FFT.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTUndersampleState (ViSession instrSession,
                                                        ViBoolean *undersample)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS:FUNC:FFT:USAM?",
                               undersample,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Comp Factor
 * Purpose:  This function sets compression factor in compressed recording
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTCompFactor(
    ViSession   instrSession,
    ViInt32 compFactor
)
{
    return rsupv_setViInt32 (instrSession,
                              "SENS:FUNC:FFT:CMPF",
                              compFactor,
                              1, 1024,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Comp Factor
 * Purpose:  This function gets compression factor in compressed recording.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTCompFactor(
    ViSession   instrSession,
    ViInt32*    compFactor
)
{
    return rsupv_getViInt32 (instrSession,
                              "SENS:FUNC:FFT:CMPF?",
                              compFactor,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Zoom Center
 * Purpose:  This function sets the FFT center frequency; only if the Zoom FFT
 *           is activated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTZoomCenter(
    ViSession   instrSession,
    ViReal64    fftZoomCenter
)
{
    return rsupv_setViReal64Unit(instrSession,
                                  "SENS:FUNC:FFT:CENT",
                                  fftZoomCenter,
                                  RSUPV_UNIT_HZ,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Zoom Center
 * Purpose:  This function gets the FFT center frequency; only if the Zoom FFT
 *           is activated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTZoomCenter(
    ViSession   instrSession,
    ViReal64*   fftZoomCenter
)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FUNC:FFT:CENT?",
                                  fftZoomCenter,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Zoom Start
 * Purpose:  This function gets the start value of the FFT frequency band.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTZoomStart(
    ViSession   instrSession,
    ViReal64*   fftZoomStart
)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FUNC:FFT:STAR?",
                                  fftZoomStart,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Zoom Stop
 * Purpose:  This function gets the end value of the FFT frequency band.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTZoomStop(
    ViSession   instrSession,
    ViReal64*   fftZoomStop
)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FUNC:FFT:STOP?",
                                  fftZoomStop,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Equalizer
 * Purpose:  This function activates/deactivates a precorrector table for the
 *           FFT bins.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTEqualizer (ViSession instrSession,
                                                 ViBoolean equalizer)
{
  return rsupv_setViBoolean (instrSession,
                               "SENS:VOLT:EQU",
                               equalizer,
                               VI_ERROR_PARAMETER2); 
    
    
}

/*****************************************************************************
 * Function: Get Analyzer FFT Equalizer
 * Purpose:  This function returns the precorrector table for the
 *           FFT bins.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTEqualizer (ViSession instrSession,
                                                 ViBoolean *equalizer)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS:VOLT:EQU?",
                               equalizer,
                               VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Analyzer FFT Equalizer File
 * Purpose:  This function selects the equalization file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTEqualizerFile (ViSession instrSession,
                                                     ViString equalFile)
{
   return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:IEQ",
                                    "'",
                                    equalFile,
                                    VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Analyzer Level Monitor
 * Purpose:  This function activates the level monitor and selects the
 *           measurement method.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerLevelMonitor(
    ViSession   instrSession,
    ViInt32 levelMonitor
)
{
    return rsupv_setEnum (instrSession,
                        "SENS6:FUNC",
                        levelMonitor,
                        anLevelMonitorArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Level Monitor
 * Purpose:  This function gets the level monitor and the measurement method.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerLevelMonitor(
    ViSession   instrSession,
    ViInt32*    levelMonitor
)
{
    return rsupv_getEnum (instrSession,
                        "SENS6:FUNC?",
                        levelMonitor,
                        anLevelMonitorArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Input Monitor
 * Purpose:  This function activates the input level meter and selects the
 *           measurement method.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerInputMonitor(
    ViSession   instrSession,
    ViInt32 inputMonitor
)
{
    if ((inputMonitor == RSUPV_ANALYZER_INPUT_MON_DIG) ||
        (inputMonitor == RSUPV_ANALYZER_INPUT_MON_PHAS))
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
    return rsupv_setEnum (instrSession,
                        "SENS2:FUNC",
                        inputMonitor,
                        anInputMonitorArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Input Monitor
 * Purpose:  This function gets the input level meter and the measurement
 *           method.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerInputMonitor(
    ViSession   instrSession,
    ViInt32*    inputMonitor
)
{
    return rsupv_getEnum (instrSession,
                        "SENS2:FUNC?",
                        inputMonitor,
                        anInputMonitorArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Monitor State
 * Purpose:  This function activates/deactivates the FFT Monitor.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTMonitorState(
    ViSession   instrSession,
    ViBoolean   fftMonitorState
)
{
    return rsupv_setViBoolean (instrSession,
                               "SENS5:FUNC",
                               fftMonitorState,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Monitor State
 * Purpose:  This function gets the FFT Monitor state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTMonitorState(
    ViSession   instrSession,
    ViBoolean*  fftMonitorState
)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS5:FUNC?",
                               fftMonitorState,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Post FFT State
 * Purpose:  This function activates/deactivates the Post FFT.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPostFFTState(
    ViSession   instrSession,
    ViBoolean   postFFTState
)
{
    return rsupv_setViBoolean (instrSession,
                               "SENS:FUNC:FFT:STAT",
                               postFFTState,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Post FFT State
 * Purpose:  This function gets the Post FFT state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPostFFTState(
    ViSession   instrSession,
    ViBoolean*  postFFTState
)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS:FUNC:FFT:STAT?",
                               postFFTState,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Triggered State
 * Purpose:  This function activates or deactivates the triggered FFT
 *           analysis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTTriggeredState (ViSession instrSession,
                                             ViBoolean triggeredFFTEnable)
{
   return rsupv_setViBoolean (instrSession,
                               "SENS:FUNC:FFT:TRIG",
                               triggeredFFTEnable,
                               VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Analyzer FFT Triggered State
 * Purpose:  This function queries the state of the FFT analysis.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTTriggeredState (ViSession instrSession,
                                             ViBoolean *triggeredFFTEnable)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS:FUNC:FFT:TRIG?",
                               triggeredFFTEnable,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Frequency Limit State
 * Purpose:  This function activates/deactivates the frequency limit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTFrequencyLimitState (ViSession instrSession,
                                                  ViBoolean limitEnable)
{
   return rsupv_setViBoolean (instrSession,
                               "SENS:FREQ:LIM",
                               limitEnable,
                               VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Analyzer FFT Frequency Limit State
 * Purpose:  This function queries the state of frequency limit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTFrequencyLimitState (ViSession instrSession,
                                                  ViBoolean *limitEnable)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS:FREQ:LIM?",
                               limitEnable,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer FFT Frequency Limit Low
 * Purpose:  This function sets the lower band limit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTFrequencyLimitLow (ViSession instrSession,
                                                ViReal64 freqLimLow)
{
     return rsupv_setViReal64Unit (instrSession,
                                   "SENS:FREQ:LIM:LOW",
                                    freqLimLow,
                                    RSUPV_UNIT_HZ,
                                    10.0, 21.9331e+3,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer FFT Frequency Limit Low
 * Purpose:  This function queries the lower band limit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTFrequencyLimitLow (ViSession instrSession,
                                                ViReal64 *freqLimLow)
{
   return rsupv_getViReal64Unit (instrSession,
                                   "SENS:FREQ:LIM:LOW?",
                                    freqLimLow,
                                    VI_NULL,
                                    VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Analyzer FFT Frequency Limit Upp
 * Purpose:  This function sets the upper band limit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFFTFrequencyLimitUpp (ViSession instrSession,
                                              ViReal64 freqLimUpp)
{
   return rsupv_setViReal64Unit (instrSession,
                                   "SENS:FREQ:LIM:UPP",
                                    freqLimUpp,
                                    RSUPV_UNIT_HZ,
                                    18.7891, 22.5e+3,
                                    VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Analyzer FFT Frequency Limit Upp
 * Purpose:  This function gets the upper band limit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFFTFrequencyLimitUpp (ViSession instrSession,
                                                 ViReal64 *freqLimUpp)
{
   return rsupv_getViReal64Unit (instrSession,
                                   "SENS:FREQ:LIM:UPP?",
                                    freqLimUpp,
                                    VI_NULL,
                                    VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Set Analyzer Waveform Monitor Trace Length
 * Purpose:  This function sets the recording length (memory depth), including 
 *           an optional pretrigger.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerWaveformMonitorTraceLength (ViSession instrSession,
                                                               ViReal64 traceLength)
{
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS7:TRIG:TRCL",
                                  traceLength,
                                  RSUPV_UNIT_S,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Waveform Monitor Trace Length
 * Purpose:  This function gets the recording length (memory depth), including 
 *           an optional pretrigger.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerWaveformMonitorTraceLength (ViSession instrSession,
                                                               ViReal64 *traceLength)
{
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS7:TRIG:TRCL?",
                                  traceLength,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Waveform Monitor Meas Mode
 * Purpose:  This function selects the operating mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerWaveformMonitorMeasMode (ViSession instrSession,
                                                            ViInt32 measMode)
{
    return rsupv_setEnum (instrSession,
                          "SENS7:MMOD",
                          measMode,
                          anWavMeasModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Waveform Monitor Meas Mode
 * Purpose:  This function returns the operating mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerWaveformMonitorMeasMode (ViSession instrSession,
                                                            ViInt32 *measMode)
{
    return rsupv_getEnum (instrSession,
                          "SENS7:MMOD?",
                          measMode,
                          anWavMeasModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Waveform Monitor Compression Factor
 * Purpose:  This function sets the compression factor in compressed recording.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerWaveformMonitorCompressionFactor (ViSession instrSession,
                                                                     ViInt32 compressionFactor)
{
    return rsupv_setViInt32 (instrSession,
                             "SENS7:COMP",
                             compressionFactor,
                             2, 1024,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Waveform Monitor Compression Factor
 * Purpose:  This function gets the compression factor in compressed recording.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerWaveformMonitorCompressionFactor (ViSession instrSession,
                                                                     ViInt32 *compressionFactor)
{
    return rsupv_getViInt32 (instrSession,
                             "SENS7:COMP?",
                             compressionFactor,
                             VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Waveform Monitor Trigger Source
 * Purpose:  This function activates the trigger and selects the source 
 *           (usually one of the measurement channels) which is to trigger 
 *           measurement recording if the trigger threshold is overranged or 
 *           underranged.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerWaveformMonitorTriggerSource (ViSession instrSession,
                                                                 ViInt32 triggerSource)
{
    if ((triggerSource == RSUPV_ANALYZER_WAV_TRG_SRC_CH1) ||
        (triggerSource == RSUPV_ANALYZER_WAV_TRG_SRC_CH2))
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
    return rsupv_setEnum (instrSession,
                          "SENS7:TRIG:SOUR",
                          triggerSource,
                          anWavTriggerSourceArr,
                          VI_ERROR_PARAMETER2);    
}


/*****************************************************************************
 * Function: Get Analyzer Waveform Monitor Trigger Source
 * Purpose:  This function returns the trigger source (usually one of the 
 *           measurement channels) which is to trigger measurement recording 
 *           if the trigger threshold is overranged or underranged.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerWaveformMonitorTriggerSource (ViSession instrSession,
                                                                 ViInt32 *triggerSource)
{
    return rsupv_getEnum (instrSession,
                          "SENS7:TRIG:SOUR?",
                          triggerSource,
                          anWavTriggerSourceArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Waveform Monitor Trigger Level
 * Purpose:  This function sets the trigger threshold.
 *
 *           Note(s):
 *
 *           (1) This function is available only if the trigger source is set
 *           to Channel 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerWaveformMonitorTriggerLevel (ViSession instrSession,
                                                                ViReal64 triggerLevel,
                                                                ViInt32     units)
{
    return rsupv_setViReal64Unit (instrSession, 
                                  "SENS7:TRIG:LEV",
                                  triggerLevel,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Waveform Monitor Trigger Level
 * Purpose:  This function gets the trigger threshold.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerWaveformMonitorTriggerLevel (ViSession instrSession,
                                                                ViReal64 *triggerLevel,
                                                                ViInt32*    units)
{
    return rsupv_getViReal64Unit (instrSession, 
                              "SENS7:TRIG:LEV?",
                              triggerLevel,
                              units,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Waveform Monitor Trigger Slope
 * Purpose:  This function selects the signal edge on which measurement is to 
 *           be triggered (trigger slope).
 *
 *           Note(s):
 *
 *           (1) This function is available only if the trigger source is set
 *           to Channel 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerWaveformMonitorTriggerSlope (ViSession instrSession,
                                                                ViInt32 triggerSlope)
{
    return rsupv_setEnum (instrSession,
                          "SENS7:TRIG:SLOP",
                          triggerSlope,
                          anWavTriggerSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Waveform Monitor Trigger Slope
 * Purpose:  This function returns the signal edge on which measurement is to 
 *           be triggered (trigger slope).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerWaveformMonitorTriggerSlope (ViSession instrSession,
                                                                ViInt32 *triggerSlope)
{
    return rsupv_getEnum (instrSession,
                          "SENS7:TRIG:SLOP?",
                          triggerSlope,
                          anWavTriggerSlopeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Waveform Monitor Pretrigger
 * Purpose:  This function activates the pretrigger and sets the pretrigger 
 *           interval in seconds. The pretrigger is deactivated by entering a 
 *           pretrigger time of 0.0 s.
 *
 *           Note(s):
 *
 *           (1) This function is available only if the trigger source is set
 *           to Channel 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerWaveformMonitorPretrigger (ViSession instrSession,
                                                              ViReal64 pretrigger)
{
    return rsupv_setViReal64 (instrSession,
                              "SENS7:TRIG:PRE",
                              pretrigger,
                              0.0, 0.4,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Waveform Monitor Pretrigger
 * Purpose:  This function gets the pretrigger and sets the pretrigger 
 *           interval in seconds.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerWaveformMonitorPretrigger (ViSession instrSession,
                                                              ViReal64 *pretrigger)
{
    return rsupv_getViReal64 (instrSession,
                              "SENS7:TRIG:PRE?",
                              pretrigger,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Waveform Monitor Autotrigger
 * Purpose:  This function activates the autotrigger, that is, forced 
 *           triggering, when the trigger condition is not fulfilled ("true") 
 *           within the time selected as "Trace Length".
 *
 *           Note(s):
 *
 *           (1) This function is available only if the trigger source is set
 *           to Channel 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerWaveformMonitorAutotrigger (ViSession instrSession,
                                                               ViBoolean autotrigger)
{
    return rsupv_setViBoolean (instrSession,
                               "SENS7:TRIG:AUTO",
                               autotrigger,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Waveform Monitor Autotrigger
 * Purpose:  This function returns the state of the autotrigger, that is, forced
 *           triggering, when the trigger condition is not fulfilled ("true")
 *           within the time selected as "Trace Length".
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerWaveformMonitorAutotrigger (ViSession instrSession,
                                                               ViBoolean *autotrigger)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS7:TRIG:AUTO?",
                               autotrigger,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Combined Measurement
 * Purpose:  This function activates the frequency measurement and selects the
 *           2nd frequency measurement value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerCombinedMeasurement(
    ViSession   instrSession,
    ViInt32 frequencytoPhase
)
{
    return rsupv_setEnum (instrSession,
                        "SENS3:FUNC",
                        frequencytoPhase,
                        anFrequencytoPhaseArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Combined Measurement
 * Purpose:  This function gets the frequency measurement and the 2nd
 *           frequency measurement value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerCombinedMeasurement(
    ViSession   instrSession,
    ViInt32*    frequencytoPhase
)
{
    return rsupv_getEnum (instrSession,
                        "SENS3:FUNC?",
                        frequencytoPhase,
                        anFrequencytoPhaseArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Measurement Time
 * Purpose:  This function selects the measurement speed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerMeasurementTime(
    ViSession   instrSession,
    ViInt32 measurementTime
)
{
    return rsupv_setEnum (instrSession,
                        "SENS3:FREQ:APER:MODE",
                        measurementTime,
                        anMeasurementTimeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Measurement Time
 * Purpose:  This function gets the measurement speed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerMeasurementTime(
    ViSession   instrSession,
    ViInt32*    measurementTime
)
{
    return rsupv_getEnum (instrSession,
                        "SENS3:FREQ:APER:MODE?",
                        measurementTime,
                        anMeasurementTimeArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Format Phase
 * Purpose:  This function selects the phase display format; only available if
 *           phase measurement is activated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFormatPhase(
    ViSession   instrSession,
    ViInt32 formatPhase
)
{
    return rsupv_setEnum (instrSession,
                        "SENS3:PHAS:FORM",
                        formatPhase,
                        anFormatPhaseArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Format Phase
 * Purpose:  This function gets the phase display format; only available if
 *           phase measurement is activated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFormatPhase(
    ViSession   instrSession,
    ViInt32*    formatPhase
)
{
    return rsupv_getEnum (instrSession,
                        "SENS3:PHAS:FORM?",
                        formatPhase,
                        anFormatPhaseArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Configure Analyzer PEAQ
 * Purpose:  This function configures audio quality measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureAnalyzerPEAQ (ViSession instrSession,
                                               ViInt32 modelVersion,
                                               ViInt32 measurementMode,
                                               ViString storeWAVToFilename)
{
    ViStatus rsupv_status;
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    CHECKERR (rsupv_SetAnalyzerPEAQModel (instrSession, modelVersion));
    
    switch (measurementMode)
    {
        case RSUPV_ANALYZER_PEAQ_MMODE_DUT:
            rsupv_status = rsupv_SetAnalyzerPEAQStoreWAVTo (instrSession,
                                 storeWAVToFilename);
            break;
        case RSUPV_ANALYZER_PEAQ_MMODE_OFFLINE:
            rsupv_status = rsupv_SetAnalyzerPEAQFilename (instrSession,
                                 storeWAVToFilename);
            break;
    }
    
    REMAP (2,4);
    
    rsupv_status = rsupv_SetAnalyzerPEAQMeasurementMode (instrSession,
                             measurementMode);
    REMAP (2,3);
    
    return rsupv_status;    
}

/*****************************************************************************
 * Function: Set Analyzer PEAQ Model
 * Purpose:  This function specifies whether the Basic or the Advanced model
 *           is to be used.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPEAQModel (ViSession instrSession,
                                              ViInt32 modelVersion)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "SENS:FUNC:PEAQ:VERS",
                          modelVersion,
                          anPEAQMeasurementModelArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer PEAQ Model
 * Purpose:  This function queries whether the Basic or the Advanced model is
 *           to be used.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPEAQModel (ViSession instrSession,
                                              ViInt32 *modelVersion)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SENS:FUNC:PEAQ:VERS?",
                          modelVersion,
                          anPEAQMeasurementModelArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer PEAQ Measurement Mode
 * Purpose:  This function selects the measurement mode for PEAQ measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPEAQMeasurementMode (ViSession instrSession,
                                                        ViInt32 measurementMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "SENS:FUNC:MMOD",
                          measurementMode,
                          anPEAQMeasurementModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer PEAQ Measurement Mode
 * Purpose:  This function queries the measurement mode for PEAQ measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPEAQMeasurementMode (ViSession instrSession,
                                                        ViInt32 *measurementMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SENS:FUNC:MMOD?",
                          measurementMode,
                          anPEAQMeasurementModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer PEAQ Store WAV To
 * Purpose:  This function select the WAV file to which measured signal will
 *           be stored.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPEAQStoreWAVTo (ViSession instrSession,
                                                   ViString storeWAVTo)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViStringQuoted (instrSession,
                              "MMEM:STOR:PWAV",
                              "'",
                              storeWAVTo,
                              VI_ERROR_PARAMETER2);
    
}

/*****************************************************************************
 * Function: Set Analyzer PEAQ Filename
 * Purpose:  This function select the WAV file containing measurement data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPEAQFilename (ViSession instrSession,
                                                 ViString filename)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViStringQuoted (instrSession,
                              "MMEM:LOAD:PWAV",
                              "'",
                              filename,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer PEAQ Reference Level
 * Purpose:  This function queries the peak value of the reference signal
 *           (after the measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPEAQReferenceLevel (ViSession instrSession,
                                                       ViReal64 *referenceLevel)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:PEAQ:REFL?",
                              referenceLevel,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer PEAQ Deg Level
 * Purpose:  This function queries the peak value of the measurement signal
 *           (after the measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPEAQDegLevel (ViSession instrSession,
                                                 ViReal64 *degLevel)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:PEAQ:DEGL?",
                              degLevel,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer PEAQ Avg Delay
 * Purpose:  This function queries the average time offset between the
 *           reference signal and the measurement signal (after the
 *           measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPEAQAvgDelay (ViSession instrSession,
                                                 ViReal64 *avgDelay)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
   return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:PEAQ:AVGD?",
                              avgDelay,
                              VI_ERROR_PARAMETER2);    
   
}

/*****************************************************************************
 * Function: Get Analyzer PEAQ Delay Detect
 * Purpose:  This function queries the average time offset between the
 *           reference signal and the measurement signal (after the
 *           measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPEAQDelayDetect (ViSession instrSession,
                                                    ViReal64 *delayDetect)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
   return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:PEAQ:DELD?",
                              delayDetect,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Configure Analyzer PESQ
 * Purpose:  This function configures speech quality measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureAnalyzerPESQ (ViSession instrSession,
                                               ViInt32 accordingTo,
                                               ViInt32 measurementMode,
                                               ViString storeWAVToFilename)
{
    ViStatus rsupv_status;
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    CHECKERR (rsupv_SetAnalyzerPESQAccordingTo (instrSession,
                             accordingTo));
    switch (measurementMode)
    {
        case RSUPV_ANALYZER_PESQ_MMODE_DUT:
            rsupv_status = rsupv_SetAnalyzerPESQStoreWAVTo (instrSession,
                                 storeWAVToFilename);
            break;
        case RSUPV_ANALYZER_PESQ_MMODE_OFFLINE:
            rsupv_status = rsupv_SetAnalyzerPESQFilename (instrSession,
                                 storeWAVToFilename);
            break;
    }
    
    REMAP (2,4);
    
    rsupv_status = rsupv_SetAnalyzerPESQMeasurementMode (instrSession,
                             measurementMode);
    REMAP (2,3);
    
    return rsupv_status;
    
}

/*****************************************************************************
 * Function: Set Analyzer PESQ According To
 * Purpose:  This function select the display mode for the PESQ or MOS value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPESQAccordingTo (ViSession instrSession,
                                                    ViInt32 accordingTo)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "SENS:FUNC:PESQ:ACC",
                          accordingTo,
                          anAccordingToArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer PESQ According To
 * Purpose:  This function queries the display mode for the PESQ or MOS
 *           value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPESQAccordingTo (ViSession instrSession,
                                                    ViInt32 *accordingTo)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SENS:FUNC:PESQ:ACC?",
                          accordingTo,
                          anAccordingToArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer PESQ Measurement Mode
 * Purpose:  This function selects the measurement mode for PESQ measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPESQMeasurementMode (ViSession instrSession,
                                                        ViInt32 measurementMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "SENS:FUNC:MMOD",
                          measurementMode,
                          anPESQMeasurementModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer PESQ Measurement Mode
 * Purpose:  This function queries the measurement mode for PESQ measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPESQMeasurementMode (ViSession instrSession,
                                                        ViInt32 *measurementMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SENS:FUNC:MMOD?",
                          measurementMode,
                          anPESQMeasurementModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer PESQ Store WAV To
 * Purpose:  This function select the WAV file to which measured signal will
 *           be stored.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPESQStoreWAVTo (ViSession instrSession,
                                                   ViString storeWAVTo)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViStringQuoted (instrSession,
                              "MMEM:STOR:PWAV",
                              "'",
                              storeWAVTo,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer PESQ Filename
 * Purpose:  This function select the WAV file containing measurement data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPESQFilename (ViSession instrSession,
                                                 ViString filename)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViStringQuoted (instrSession,
                              "MMEM:LOAD:PWAV",
                              "'",
                              filename,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer PESQ Reference Level
 * Purpose:  This function queries the peak value of the reference signal
 *           (after the measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPESQReferenceLevel (ViSession instrSession,
                                                       ViReal64 *referenceLevel)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:PESQ:REFL?",
                              referenceLevel,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer PESQ Deg Level
 * Purpose:  This function queries the peak value of the measurement signal
 *           (after the measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPESQDegLevel (ViSession instrSession,
                                                 ViReal64 *degLevel)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:PESQ:DEGL?",
                              degLevel,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer PESQ Avg Delay
 * Purpose:  This function queries the average time offset between the
 *           reference signal and the measurement signal (after the
 *           measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPESQAvgDelay (ViSession instrSession,
                                                 ViReal64 *avgDelay)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:PESQ:AVGD?",
                              avgDelay,
                              VI_ERROR_PARAMETER2);
}
                                                
/*****************************************************************************
 * Function: Set Analyzer POLQA Band
 * Purpose:  This function selects the analysis method for the POLQA algorithm.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPOLQABand (ViSession instrSession,
                                                        ViInt32 band)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "SENS:FUNC:POLQ:BAND",
                          band,
                          anDigitalPOLQABandArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer POLQA Band
 * Purpose:  This function returns the analysis method for the POLQA algorithm.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQABand (ViSession instrSession,
                                                        ViInt32 *band)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SENS:FUNC:POLQ:BAND?",
                          band,
                          anDigitalPOLQABandArr,
                          VI_ERROR_PARAMETER2);
}                                                    
                                                     

                                                     
/*****************************************************************************
 * Function: Set Analyzer POLQA Measurement Mode
 * Purpose:  This function selects the measurement mode for POLQA measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPOLQAMeasurementMode (ViSession instrSession,
                                                        ViInt32 measurementMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                          "SENS:FUNC:MMOD",
                          measurementMode,
                          anPEAQMeasurementModeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer POLQA Measurement Mode
 * Purpose:  This function queries the measurement mode for POLQA measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQAMeasurementMode (ViSession instrSession,
                                                        ViInt32 *measurementMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "SENS:FUNC:MMOD?",
                          measurementMode,
                          anPEAQMeasurementModeArr,
                          VI_ERROR_PARAMETER2);
}
    

/*****************************************************************************
 * Function: Get Analyzer POLQA Gain
 * Purpose:  This function returns compensating additional gains or
 *            attenuations of then transmission path.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQAGain(
    ViSession   instrSession,
    ViReal64*   gain
)
{
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:POLQ:GAIN?",
                              gain,
                              VI_ERROR_PARAMETER2);
}       

    
/*****************************************************************************
 * Function: Set Analyzer POLQA Gain
 * Purpose:  This function allows compensating additional gains or
 *           attenuations of then transmission path. See the description of
 *           the "Attenuation" display value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPOLQAGain(
    ViSession   instrSession,
    ViReal64    gain
)
{
    return rsupv_setViReal64Unit (instrSession,
                              "SENS:FUNC:POLQ:GAIN",
                              gain,
                              RSUPV_UNIT_DB,
                              0.0, 0.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer POLQA Reference Level
 * Purpose:  This function queries the peak value of the reference signal
 *           (after the measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQAReferenceLevel (ViSession instrSession,
                                                       ViReal64 *referenceLevel)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:PEAQ:REFL?",
                              referenceLevel,
                              VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer POLQA Deg Level
 * Purpose:  This function queries the peak value of the measurement signal
 *           (after the measurement).
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQADegLevel (ViSession instrSession,
                                                 ViReal64 *degLevel)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:PEAQ:DEGL?",
                              degLevel,
                              VI_ERROR_PARAMETER2);    
}


/*****************************************************************************
 * Function: Get Analyzer POLQA Attenuation
 * Purpose:  This function queries the gain between reference and measurement
 *           signal after measurement determined by the POLQA algorithm.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQAAttenuation(
    ViSession   instrSession,
    ViReal64*   attenuation
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:POLQ:ATT?",
                              attenuation,
                              VI_ERROR_PARAMETER2);
}       
        
/*****************************************************************************
 * Function: Get Analyzer POLQA SNR Ref
 * Purpose:  This function queries the S/N value in dB across the entire
 *           reference signal after the measurement
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQASNRRef(
    ViSession   instrSession,
    ViReal64*   SNRRef
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:POLQ:SNRR?",
                              SNRRef,
                              VI_ERROR_PARAMETER2);
}       
    
    
/*****************************************************************************
 * Function: Get Analyzer POLQA SNR Degraded
 * Purpose:  This function queries the S/N value in dB across the entire
 *           measurement signal after the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQASNRDegraded(
    ViSession   instrSession,
    ViReal64*   SNRDegraded
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:POLQ:SNRD?",
                              SNRDegraded,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer POLQA Ref Sp Ratio
 * Purpose:  This function returns the time ratio between voice/music and
 *           silence in the reference signal after the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQARefSpRatio(
    ViSession   instrSession,
    ViReal64*   refSpRatio
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:POLQ:REFS?",
                              refSpRatio,
                              VI_ERROR_PARAMETER2);
}       
    
/*****************************************************************************
 * Function: Get Analyzer POLQA Deg Sp Ratio
 * Purpose:  This function returns the time ratio between voice/music and
 *           silence in the measurement signal after the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQADegSpRatio(
    ViSession   instrSession,
    ViReal64*   degSpRatio
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:POLQ:DEGS?",
                              degSpRatio,
                              VI_ERROR_PARAMETER2);
}       

/*****************************************************************************
 * Function: Get Analyzer POLQA Min Delay
 * Purpose:  This function returns the minimum time offset between the
 *           reference and the measurement signals after measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQAMinDelay(
    ViSession   instrSession,
    ViReal64*   minDelay
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:POLQ:MIND?",
                              minDelay,
                              VI_ERROR_PARAMETER2);
}       

/*****************************************************************************
 * Function: Get Analyzer POLQA Max Delay
 * Purpose:  This function returns the maximum time offset between the
 *           reference and the measurement signals after measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPOLQAMaxDelay(
    ViSession   instrSession,
    ViReal64*   maxDelay
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    
    return rsupv_getViReal64 (instrSession,
                              "SENS:FUNC:POLQ:MAXD?",
                              maxDelay,
                              VI_ERROR_PARAMETER2);
}       

    
    
/*****************************************************************************
 * Function: Set Analyzer Waveform Monitor State
 * Purpose:  This function activates/deactivates the waveform monitor.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerWaveformMonitorState(
    ViSession   instrSession,
    ViBoolean   waveformState
)
{
    return rsupv_setViBoolean (instrSession,
                               "SENS7:FUNC",
                               waveformState,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Waveform Monitor State
 * Purpose:  This function gets the waveform monitor.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerWaveformMonitorState(
    ViSession   instrSession,
    ViBoolean*  waveformState
)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS7:FUNC?",
                               waveformState,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: MeasurementControl
 * Purpose:  This function selects the measurement control type.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_MeasurementControl (ViSession instrSession,
                                            ViInt32 measurement)
{
    return rsupv_setEnum (instrSession,
                          "INIT:FORC",
                          measurement,
                          MeasControlArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Unit
 * Purpose:  This function selects the units.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerUnit(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViInt32     units
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "SENS%ld:UNIT%ld", measurement, channel);

    return rsupv_setEnum (instrSession,
                        buffer,
                        units,
                        unitsArr,
                        VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Analyzer Unit
 * Purpose:  This function returns the units.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerUnit(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViInt32*    units
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (measurement, 1, 6, 3);
    
    sprintf (buffer, "SENS%ld:UNIT%ld?", measurement, channel);

    return rsupv_getEnum (instrSession,
                        buffer,
                        units,
                        unitsArr,
                        VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Analyzer Unit Auto
 * Purpose:  This function determines whether the unit label displayed on the
 *           Y axis for Trace A or Trace B is the same as the unit set in the
 *           Unit Funct Track parameter field or whether it can be
 *           user-selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerUnitAuto(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViBoolean   unitAuto
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (measurement, 1, 6, 3);
    
    sprintf (buffer, "SENS%ld:UNA%ld", measurement, channel);

    return rsupv_setViBoolean (instrSession,
                               buffer,
                               unitAuto,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Analyzer Unit Auto
 * Purpose:  This function gets whether the unit label displayed on the Y axis
 *           for Trace A or Trace B is the same as the unit set in the Unit
 *           Funct Track parameter field or whether it can be user-selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerUnitAuto(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViBoolean*  unitAuto
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (measurement, 1, 6, 2);
    CHECKVIINT32 (channel, 1, 2, 3);

    sprintf (buffer, "SENS%ld:UNA%ld?", measurement, channel);

    return rsupv_getViBoolean (instrSession,
                               buffer,
                               unitAuto,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Analyzer User Unit
 * Purpose:  This function sets an user-defined unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerUserUnit(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViString    userUnit
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (measurement, 1, 6, 3);
    
    sprintf (buffer, "SENS%ld:USER%ld", measurement, channel);

    return rsupv_setViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    userUnit,
                                    VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Analyzer User Unit
 * Purpose:  This function gets an user-defined unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerUserUnit(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViChar      userUnit[])
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (measurement, 1, 6, 3);
    
    sprintf (buffer, "SENS%ld:USER%ld?", measurement, channel);

    return rsupv_getViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    BUFFER_SIZE,
                                    userUnit,
                                    VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Analyzer Reference
 * Purpose:  This function sets the reference.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerReference(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViInt32     reference
)
{
    ViChar  buffer[BUFFER_SIZE];

    if ((reference == RSUPV_ANALYZER_FUNCTION_REF_CH1M) ||
        (reference == RSUPV_ANALYZER_FUNCTION_REF_CH2M))
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (measurement, 1, 6, 3);
    
    sprintf (buffer, "SENS%ld:REF:MODE%ld", measurement, channel);

    return rsupv_setEnum (instrSession,
                        buffer,
                        reference,
                        anReferenceArr,
                        VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Analyzer Reference
 * Purpose:  This function gets the reference.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerReference(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViInt32*    reference
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (measurement, 1, 6, 3);
    
    sprintf (buffer, "SENS%ld:REF:MODE%ld?", measurement, channel);

    return rsupv_getEnum (instrSession,
                        buffer,
                        reference,
                        anReferenceArr,
                        VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Analyzer Reference Value
 * Purpose:  This function sets the reference value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerReferenceValue(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViReal64    referenceValue,
    ViInt32     units
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (measurement, 1, 6, 3);

    sprintf (buffer, "SENS%ld:REF%ld", measurement, channel);

    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  referenceValue,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Analyzer Reference Value
 * Purpose:  This function gets the reference value.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerReferenceValue(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     measurement,
    ViReal64*   referenceValue,
    ViInt32*    units
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (measurement, 1, 6, 3);
    
    sprintf (buffer, "SENS%ld:REF%ld?", measurement, channel);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  referenceValue,
                                  units,
                                  VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Analyzer Frequency Unit
 * Purpose:  This function selects the frequency unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFrequencyUnit(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 frequencyUnit
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "SENS3:FREQ:UNIT%ld", channel);

    return rsupv_setEnum (instrSession,
                        buffer,
                        frequencyUnit,
                        unitsArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analyzer Frequency Unit
 * Purpose:  This function gets the frequency unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFrequencyUnit(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32*    frequencyUnit
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "SENS3:FREQ:UNIT%ld?", channel);

    return rsupv_getEnum (instrSession,
                        buffer,
                        frequencyUnit,
                        unitsArr,
                        VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Analyzer Phase Unit
 * Purpose:  This function selects the phase unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerPhaseUnit(
    ViSession   instrSession,
    ViInt32 phaseUnit
)
{
    return rsupv_setEnum (instrSession,
                        "SENS3:PHAS:UNIT",
                        phaseUnit,
                        unitsArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Phase Unit
 * Purpose:  This function gets the phase unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerPhaseUnit(
    ViSession   instrSession,
    ViInt32*    phaseUnit
)
{
    return rsupv_getEnum (instrSession,
                        "SENS3:PHAS:UNIT?",
                        phaseUnit,
                        unitsArr,
                        VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analyzer Frequency Phase Unit Auto
 * Purpose:  This function sets frequency and phase unit auto.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFrequencyPhaseUnitAuto(
    ViSession instrSession,
    ViInt32   channel,
    ViBoolean freqUnitAuto,
    ViBoolean phaseUnitAuto
)
{
    ViStatus    rsupv_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "SENS3:FREQ:UNA%ld", channel);

    CHECKERR (rsupv_setViBoolean (instrSession,
                               buffer,
                               freqUnitAuto,
                               VI_ERROR_PARAMETER3));

    CHECKERR (rsupv_setViBoolean (instrSession,
                               "SENS3:PHAS:UNA",
                               phaseUnitAuto,
                               VI_ERROR_PARAMETER4));
    return rsupv_status;
}

/*****************************************************************************
 * Function: Get Analyzer Frequency Phase Unit Auto
 * Purpose:  This function gets frequency and phase unit auto.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFrequencyPhaseUnitAuto(
    ViSession instrSession,
    ViInt32   channel,
    ViBoolean *freqUnitAuto,
    ViBoolean *phaseUnitAuto
)
{
    ViStatus    rsupv_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "SENS3:FREQ:UNA%ld?", channel);

    CHECKERR (rsupv_getViBoolean (instrSession,
                               buffer,
                               freqUnitAuto,
                               VI_ERROR_PARAMETER2));

    CHECKERR (rsupv_getViBoolean (instrSession,
                               "SENS3:PHAS:UNA?",
                               phaseUnitAuto,
                               VI_ERROR_PARAMETER3));
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Analyzer Frequency Phase User Unit
 * Purpose:  This function sets the frequency and phase user unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFrequencyPhaseUserUnit(
    ViSession instrSession,
    ViInt32   channel,
    ViString  freqUserUnit,
    ViString  phaseUserUnit
)
{
    ViStatus    rsupv_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "SENS3:FREQ:USER%ld", channel);

    CHECKERR (rsupv_setViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    freqUserUnit,
                                    VI_ERROR_PARAMETER2));

    CHECKERR (rsupv_setViStringQuoted (instrSession,
                                    "SENS3:PHAS:USER",
                                    "'",
                                    phaseUserUnit,
                                    VI_ERROR_PARAMETER3));

    return rsupv_status;
}

/*****************************************************************************
 * Function: Get Analyzer Frequency Phase User Unit
 * Purpose:  This function gets the frequency and phase user unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFrequencyPhaseUserUnit(
    ViSession instrSession,
    ViInt32   channel,
    ViChar freqUserUnit[],
    ViChar phaseUserUnit[]
)
{
    ViStatus    rsupv_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (channel, 1, 2, 2);

    sprintf (buffer, "SENS3:FREQ:USER%ld?", channel);

    CHECKERR(rsupv_getViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    255,
                                    freqUserUnit,
                                    VI_ERROR_PARAMETER3));
    CHECKERR(rsupv_getViStringQuoted (instrSession,
                                    "SENS3:PHAS:USER?",
                                    "'",
                                    255,
                                    phaseUserUnit,
                                    VI_ERROR_PARAMETER4));
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Analyzer Frequency Phase Reference
 * Purpose:  This function selects the frequency and phase reference.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFrequencyPhaseReference(
    ViSession   instrSession,
    ViInt32 frequencyReference,
    ViInt32 phaseReference
)
{
    ViStatus    rsupv_status;
    ViInt32     instrument = -1;

    if (frequencyReference == RSUPV_ANALYZER_FUNCTION_REF_STOR)
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
    CHECKERR (rsupv_setEnum (instrSession,
                        "SENS3:FREQ:REF:MODE",
                        frequencyReference,
                        anFreqReferenceArr,
                        VI_ERROR_PARAMETER2));

    CHECKERR (rsupv_GetAnalyzerInstrument (instrSession, &instrument));
    
    if (instrument == RSUPV_ANALYZER_ANALOG)
    {
    CHECKERR (rsupv_setEnum (instrSession,
                        "SENS3:PHAS:REF:MODE",
                        phaseReference,
                        anPhaseReferenceArr,
                        VI_ERROR_PARAMETER3));
    }
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Get Analyzer Frequency Phase Reference
 * Purpose:  This function gets the frequency and phase reference.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFrequencyPhaseReference(
    ViSession   instrSession,
    ViInt32*    frequencyReference,
    ViInt32*    phaseReference
)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_getEnum (instrSession,
                        "SENS3:FREQ:REF:MODE?",
                        frequencyReference,
                        anFreqReferenceArr,
                        VI_ERROR_PARAMETER2));

     CHECKERR (rsupv_getEnum (instrSession,
                        "SENS3:PHAS:REF:MODE?",
                        phaseReference,
                        anPhaseReferenceArr,
                        VI_ERROR_PARAMETER3));

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Analyzer Frequency Phase Reference Value
 * Purpose:  This function sets the frequency, phase and group delay values.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerFrequencyPhaseReferenceValue(
    ViSession   instrSession,
    ViReal64    frequencyRefVal,
    ViInt32     frequencyUnits,
    ViReal64    phaseRefVal,
    ViInt32     phaseUnits,
    ViReal64    groupDelayRefVal,
    ViInt32     groupDelayUnits
)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViInt32        state = 0;
    
    CHECKERR (rsupv_GetAnalyzerCombinedMeasurement (instrSession, &state));
                                                                          
    CHECKERR (rsupv_setViReal64Unit (instrSession,
                                      "SENS3:FREQ:REF",
                                      frequencyRefVal,
                                      frequencyUnits,
                                      0.0, 0.0,
                                      VI_ERROR_PARAMETER2));
    
    if (state == RSUPV_ANALYZER_FREQ_PHASE_FQPH)
        CHECKERR (rsupv_setViReal64Unit (instrSession,
                                          "SENS3:PHAS:REF",
                                          phaseRefVal,
                                          phaseUnits,
                                          0.0, 0.0,
                                          VI_ERROR_PARAMETER4));

    if (state == RSUPV_ANALYZER_FREQ_PHASE_FQGR)
        CHECKERR (rsupv_setViReal64Unit (instrSession,
                                          ":SENS3:GRO:REF",
                                          groupDelayRefVal,
                                          groupDelayUnits,
                                          0.0, 0.0,
                                          VI_ERROR_PARAMETER6));

    return rsupv_status;
}

/*****************************************************************************
 * Function: Get Analyzer Frequency Phase Reference Value
 * Purpose:  This function sets the frequency, phase and group delay values.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerFrequencyPhaseReferenceValue(
    ViSession   instrSession,
    ViReal64*   frequencyRefVal,
    ViInt32*    frequencyUnits,
    ViReal64*   phaseRefVal,
    ViInt32*    phaseUnits,
    ViReal64*   groupDelayRefVal,
    ViInt32*    groupDelayUnits
)
{
    ViStatus    rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_getViReal64Unit (instrSession,
                                      "SENS3:FREQ:REF?",
                                      frequencyRefVal,
                                      frequencyUnits,
                                      VI_ERROR_PARAMETER2));
    
    CHECKERR (rsupv_getViReal64Unit (instrSession,
                                      "SENS3:PHAS:REF?",
                                      phaseRefVal,
                                      phaseUnits,
                                      VI_ERROR_PARAMETER4));

    CHECKERR (rsupv_getViReal64Unit (instrSession,
                                      ":SENS3:GRO:REF?",
                                      groupDelayRefVal,
                                      groupDelayUnits,
                                      VI_ERROR_PARAMETER6));

    return rsupv_status;
}

/*****************************************************************************
 * Function: Read Measurement Result
 * Purpose:  This function gets the analyzer measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ReadMeasurementResult (ViSession instrSession,
                                               ViInt32 channel,
                                               ViInt32 measurement,
                                               ViReal64 *measurementResult,
                                               ViChar units[])
{
    ViChar      buffer[BUFFER_SIZE];
    ViStatus    rsupv_status;
    ViInt32     local_units;

    CHECKVIINT32 (channel, 1, 16, 2);

    switch (measurement)
        {
        case RSUPV_MEASUREMENT_FUNCTION:        /* Measurement Function */
            sprintf (buffer, "SENS1:DATA%ld?", channel);
            break;
        case RSUPV_MEASUREMENT_INPUT_MONITOR:   /* Input Monitor */
            sprintf (buffer, "SENS2:DATA%ld?", channel);
            break;
        case RSUPV_MEASUREMENT_LEVEL_MONITOR:   /* Level Monitor */
            sprintf (buffer, "SENS6:DATA%ld?", channel);
            break;
        case RSUPV_MEASUREMENT_FREQUENCY:       /* Frequency */
            sprintf (buffer, "SENS3:DATA%ld?", channel);
            break;
        case RSUPV_MEASUREMENT_PHASE_GRP_DELAY: /* Phase or Group Delay */
            sprintf (buffer, "SENS4:DATA%ld?", channel);
            break;
        default:
            return VI_ERROR_PARAMETER3;
        }

    CHECKERR (rsupv_getViReal64Unit (instrSession,
                              buffer,
                              measurementResult,
                              &local_units,
                              VI_ERROR_PARAMETER4));

    if (units)
        strcpy (units, unitsArr[local_units]);
    
    return rsupv_status;
}
/*****************************************************************************
 * Function: Set Analyzer SN Sequence State
 * Purpose:  This function activates/deactivates the S/N sequence in order to
 * determine the signal to noise.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSNSequenceState (ViSession instrSession,
                                           ViBoolean SNEnable)
{
    return rsupv_setViBoolean (instrSession,
                               "SENS:FUNC:SNS",
                               SNEnable,
                               VI_ERROR_PARAMETER2); 
}

/*****************************************************************************
 * Function: Get Analyzer SN Sequence State
 * Purpose:  This function returns the S/N state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSNSequenceState (ViSession instrSession,
                                           ViBoolean *SNEnable)
{
    return rsupv_getViBoolean (instrSession,
                               "SENS:FUNC:SNS?",
                               SNEnable,
                               VI_ERROR_PARAMETER2);
}
/*****************************************************************************
 * Function: Set Analyzer SN Meas Mode
 * Purpose:  This function selects the level weighting method for signal 
 *           measurement and noise measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSNMeasMode (ViSession instrSession,
                                               ViInt32 measMode)
{
    if (measMode == RSUPV_ANALYZER_SN_MEAS_MODE_QPE)
    {
        CHECKINSTR (RSUPV_INSTR_UPV);
    }
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:MMOD",
                        measMode,
                        anSNMeasModeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer SN Meas Mode
 * Purpose:  This function returns the level weighting method for signal 
 *           measurement and noise measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSNMeasMode (ViSession instrSession,
                                               ViInt32 *measMode)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:MMOD?",
                        measMode,
                        anSNMeasModeArr,
                        VI_ERROR_PARAMETER2);     
}

/*****************************************************************************
 * Function: Set Analyzer SN Meas Time
 * Purpose:  This function selects the measurement time for the purpose of 
 *           adapting the measurement speed to the waveform or, in the case of 
 *           single tone signals, to the signal frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerSNMeasTime (ViSession instrSession,
                                               ViInt32 measTime)
{
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:APER:MODE",
                        measTime,
                        anLevelMeasTimeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer SN Meas Time
 * Purpose:  This function returns the measurement time for the purpose of 
 *           adapting the measurement speed to the waveform or, in the case of 
 *           single tone signals, to the signal frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerSNMeasTime (ViSession instrSession,
                                               ViInt32 *measTime)
{
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:APER:MODE?",
                        measTime,
                        anLevelMeasTimeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer NOCT Meas Mode
 * Purpose:  This function selects the measurement mode and thus implicitly the
 *           center frequencies and relative bandwidth of the bandpass filters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerNOCTMeasMode (ViSession instrSession,
                                                 ViInt32 measMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:MMOD",
                        measMode,
                        anNOCTMeasModeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer NOCT Meas Mode
 * Purpose:  This function returns the measurement mode and thus implicitly the
 *           center frequencies and relative bandwidth of the bandpass filters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerNOCTMeasMode (ViSession instrSession,
                                                 ViInt32 *measMode)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:MMOD?",
                        measMode,
                        anNOCTMeasModeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Record File
 * Purpose:  This function is used for entering the drive, path and file name 
 *           of the WAV file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerRecordFile (ViSession instrSession,
                                               ViString recordFile)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViStringQuoted (instrSession,
                                    "SENS:FUNC:REC:FILE",
                                    "'",
                                    recordFile,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Record Length
 * Purpose:  This function sets the recording time.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerRecordLength (ViSession instrSession,
                                                 ViReal64 recordLength,
                                                 ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:FUNC:REC:LENG",
                                  recordLength,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analyzer Record Length
 * Purpose:  This function gets the recording time.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerRecordLength (ViSession instrSession,
                                                 ViReal64 *recordLength,
                                                 ViInt32 *units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FUNC:REC:LENG?",
                                  recordLength,
                                  units,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Record File Lengthh
 * Purpose:  This function gets the file length in bytes.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerRecordFileLength (ViSession instrSession,
                                                     ViInt32 *fileLength)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar        buffer[BUFFER_SIZE] = "";
    
    CHECKINSTR (RSUPV_INSTR_UPV);
    rsupv_status = rsupv_getViStringQuoted (instrSession,
                                            "SENS:FUNC:REC:FLEN?",
                                            "'",
                                            256,
                                            buffer,
                                            VI_ERROR_PARAMETER2);
    *fileLength = atoi (buffer);
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Analyzer Record Trigger Source 
 * Purpose:  This function selects the trigger source.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerRecordTriggerSource (ViSession instrSession,
                                                        ViInt32 triggerSource)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:REC:TRIG:SOUR",
                        triggerSource,
                        anWavTriggerSourceArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Record Trigger Source 
 * Purpose:  This function returns the trigger source.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerRecordTriggerSource (ViSession instrSession,
                                                        ViInt32 *triggerSource)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:REC:TRIG:SOUR?",
                        triggerSource,
                        anWavTriggerSourceArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Record Trigger Level 
 * Purpose:  This function sets the trigger threshold; appears only when the 
 *           trigger system is active.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerRecordTriggerLevel (ViSession instrSession,
                                                       ViReal64 triggerLevel,
                                                       ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:FUNC:REC:TRIG:LEV",
                                  triggerLevel,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Record Trigger Level 
 * Purpose:  This function gets the trigger threshold; appears only when the 
 *           trigger system is active.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerRecordTriggerLevel (ViSession instrSession,
                                                       ViReal64 *triggerLevel,
                                                       ViInt32 *units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FUNC:REC:TRIG:LEV?",
                                  triggerLevel,
                                  units,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Record Trigger Slope 
 * Purpose:  This function selects signal edge on which it is intended 
 *           triggering shall take place (Trigger Slope); appears only when 
 *           the trigger system is active.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerRecordTriggerSlope (ViSession instrSession,
                                                       ViInt32 triggerSlope)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                        "SENS:FUNC:REC:TRIG:SLOP",
                        triggerSlope,
                        anRECTriggerSlopeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Record Trigger Slope 
 * Purpose:  This function returns signal edge on which it is intended 
 *           triggering shall take place (Trigger Slope); appears only when 
 *           the trigger system is active.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerRecordTriggerSlope (ViSession instrSession,
                                                       ViInt32 *triggerSlope)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                        "SENS:FUNC:REC:TRIG:SLOP?",
                        triggerSlope,
                        anRECTriggerSlopeArr,
                        VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analyzer Record Pretrigger
 * Purpose:  This function sets the pretrigger interval in seconds.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAnalyzerRecordPretrigger (ViSession instrSession,
                                                     ViReal64 recordPretrigger,
                                                     ViInt32 units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:FUNC:REC:TRIG:PRE",
                                  recordPretrigger,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analyzer Record Pretrigger
 * Purpose:  This function gets the pretrigger interval in seconds.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAnalyzerRecordPretrigger (ViSession instrSession,
                                                     ViReal64 *recordPretrigger,
                                                     ViInt32 *units)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:FUNC:REC:TRIG:PRE?",
                                  recordPretrigger,
                                  units,
                                  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Protocol Analyzer Channel Byte
 * Purpose:  This function returns decimal value for the byte selected in 
 *           channel 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolAnalysisChannelByte (ViSession instrSession,
                                                        ViInt32 channel,
                                                        ViInt32 byte,
                                                        ViInt32 *value)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (channel, 1, 2, 2);
    CHECKVIINT32 (byte, 0, 4, 3);
    
    sprintf (buffer, "SOURce:PROTocol:CH%ld:BYTE%ld?", channel, byte);
    
    return rsupv_getViInt32 (instrSession, 
                             buffer,
                             value,
                             VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Protocol Analysis Error
 * Purpose:  This function returns protocol error code and message.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetProtocolAnalysisError (ViSession instrSession,
                                         ViInt32 *errorCode, ViInt32 arraySize,
                                         ViChar errorMessage[])
{
    ViStatus rsupv_status = 0;
    ViChar buffer[BUFFER_SIZE] = "";
    ViChar *p2buf = NULL;
    
    CHECKERR (rsupv_getViString (instrSession,
                                 "SENS8:PROT:ERR?",
                                 buffer,
                                 BUFFER_SIZE,
                                 VI_ERROR_PARAMETER3));
    sscanf (buffer, "%ld", errorCode);
    p2buf = strtok (buffer, "\"");
    strncpy (errorMessage, p2buf, arraySize);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Lip Sync Analysis Active Color High
 * Purpose:  This function sets option for "Active Color High" in text form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetLipSyncAnalysisActiveColorHigh (ViSession instrSession,
                                             ViString rgbString)
{
    CHECKOPT ("K45"); 
    
    return rsupv_setViStringQuoted (instrSession,
                                    "SENS:LIPS:COL:THR:HIGH:STR",
                                    "'",
                                    rgbString,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Lip Sync Analysis Active Color High 
 * Purpose:  This function gets the option for "Active Color High" in text
 *           form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetLipSyncAnalysisActiveColorHigh (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar rgbString[])
{
    CHECKOPT ("K45");
    return rsupv_getViStringQuoted (instrSession,
                              "SENS:LIPS:COL:THR:HIGH:STR?",
                              "'",
                              bufferSize,
                              rgbString,
                              VI_ERROR_PARAMETER3);    
}   

/*****************************************************************************
 * Function: Set Lip Sync Analysis Active Color Low
 * Purpose:  This function sets option for "Active Color Low" in text form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetLipSyncAnalysisActiveColorLow (ViSession instrSession,
                                             ViString rgbString)
{
    CHECKOPT ("K45");
    return rsupv_setViStringQuoted (instrSession,
                                    "SENS:LIPS:COL:THR:LOW:STR",
                                    "'",
                                    rgbString,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Lip Sync Analysis Active Color Low 
 * Purpose:  This function gets the option for "Active Color Low" in text
 *           form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetLipSyncAnalysisActiveColorLow (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar rgbString[])
{
    CHECKOPT ("K45");
    return rsupv_getViStringQuoted (instrSession,
                              "SENS:LIPS:COL:THR:LOW:STR?",
                              "'",
                              bufferSize,
                              rgbString,
                              VI_ERROR_PARAMETER3);    
}   

/*****************************************************************************
 * Function: Set Lip Sync Analysis Active Pattern Color
 * Purpose:  This function sets option for "Active Color" in text form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetLipSyncAnalysisActivePatternColor (ViSession instrSession,
                                             ViString rgbString)
{
    CHECKOPT ("K45");
    return rsupv_setViStringQuoted (instrSession,
                                    "SOUR:LIPS:PATT:AUD:COL:STR",
                                    "'",
                                    rgbString,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Lip Sync Analysis Active Pattern Color
 * Purpose:  This function gets the option for "Active Color" in text
 *           form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetLipSyncAnalysisActivePatternColor (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar rgbString[])
{
    CHECKOPT ("K45");
    return rsupv_getViStringQuoted (instrSession,
                              "SOUR:LIPS:PATT:AUD:COL:STR?",
                              "'",
                              bufferSize,
                              rgbString,
                              VI_ERROR_PARAMETER3);    
}       

/*****************************************************************************
 * Function: Set Lip Sync Analysis Mute Pattern Color
 * Purpose:  This function sets option for "Active Color" in text form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetLipSyncAnalysisMutePatternColor (ViSession instrSession,
                                             ViString rgbString)
{
    CHECKOPT ("K45");
    return rsupv_setViStringQuoted (instrSession,
                                    "SOUR:LIPS:PATT:MUTE:COL:STR",
                                    "'",
                                    rgbString,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Lip Sync Analysis Mute Pattern Color
 * Purpose:  This function gets the option for "Active Color" in text
 *           form.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetLipSyncAnalysisMutePatternColor (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar rgbString[])
{
    CHECKOPT ("K45");
    return rsupv_getViStringQuoted (instrSession,
                              "SOUR:LIPS:PATT:MUTE:COL:STR?",
                              "'",
                              bufferSize,
                              rgbString,
                              VI_ERROR_PARAMETER3);    
}   


/*****************************************************************************
 * Function: Set Lip Sync Analysis Audio Trigger Threshold
 * Purpose:  This function defines the condition of the audio trigger for the
 *           Lip Sync measurement 
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetLipSyncAnalysisAudioTriggerThreshold (ViSession instrSession,
                                                       ViReal64 low,
                                                       ViInt32 units)
{
    CHECKOPT ("K45");
    return rsupv_setViReal64Unit (instrSession,
                                  "SENS:LIPS:AUD:THR:LOW",
                                  low,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);  
}

/*****************************************************************************
 * Function: Get Lip Sync Analysis Audio Trigger Threshold
 * Purpose:  This function returns the condition of the audio trigger for the
 *           Lip Sync measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetLipSyncAnalysisAudioTriggerThreshold (ViSession instrSession,
                                                      ViReal64 *low,ViInt32 *units)
{
    CHECKOPT ("K45");
    return rsupv_getViReal64Unit (instrSession,
                                  "SENS:LIPS:AUD:THR:LOW?",
                                  low,
                                  units,
                                  VI_ERROR_PARAMETER2);     
}   
    
    

/*****************************************************************************
 * Function: Configure LP or HP Filter
 * Purpose:  This function configures the Low-Pass or High-Pass Filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureLPorHPFilter(
    ViSession   instrSession,
    ViInt32     filterNumber,
    ViInt32     order,
    ViReal64    passband,
    ViReal64    attenuation
)
{
    ViStatus    rsupv_status;
    ViInt32     filterType = 0;

    rsupv_status = rsupv_GetFilterType(instrSession, filterNumber, &filterType);

    if ((filterType == RSUPV_USER_DEF_FILTER_LPAS) |
        (filterType == RSUPV_USER_DEF_FILTER_HPAS))
        {
        CHECKERR(rsupv_SetFilterOrder(instrSession, filterNumber, order));

        rsupv_status = rsupv_SetFilterPassband(instrSession, filterNumber, passband);
        REMAP(3,4);
        
        rsupv_status = rsupv_SetFilterAttenuation(instrSession, filterNumber, attenuation);
        REMAP(3,5);
        }
    else
        rsupv_status = RSUPV_ERROR_SETTINGS_CONFLICT;

    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure BP or BS Filter
 * Purpose:  This function configures the Band-Pass or Band-Stop Filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureBPorBSFilter(
    ViSession   instrSession,
    ViInt32     filterNumber,
    ViReal64    passbandLow,
    ViReal64    passbandUpp,
    ViReal64    attenuation
)
{
    ViStatus    rsupv_status;
    ViInt32     filterType = 0;

    rsupv_status = rsupv_GetFilterType(instrSession, filterNumber, &filterType);

    if ((filterType == RSUPV_USER_DEF_FILTER_BPAS) |
        (filterType == RSUPV_USER_DEF_FILTER_BSTO))
        {
        CHECKERR(rsupv_SetFilterPassbandLow(instrSession, filterNumber, passbandLow));

        rsupv_status = rsupv_SetFilterPassbandUpp(instrSession, filterNumber, passbandUpp);
        REMAP(3,4);
        
        rsupv_status = rsupv_SetFilterAttenuation(instrSession, filterNumber, attenuation);
        REMAP(3,5);
        }
    else
        rsupv_status = RSUPV_ERROR_SETTINGS_CONFLICT;
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Notch Filter
 * Purpose:  This function configures the Notch Filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureNotchFilter(
    ViSession   instrSession,
    ViInt32     filterNumber,
    ViReal64    centerFrequency,
    ViReal64    width,
    ViReal64    attenuation
)
{
    ViStatus    rsupv_status;
    ViInt32     filterType = 0;

    rsupv_status = rsupv_GetFilterType(instrSession, filterNumber, &filterType);

    if (filterType == RSUPV_USER_DEF_FILTER_NOTC)
        {
        CHECKERR(rsupv_SetFilterCenterFreq(instrSession, filterNumber, centerFrequency));

        rsupv_status = rsupv_SetFilterWidth(instrSession, filterNumber, width);
        REMAP(3,4);

        rsupv_status = rsupv_SetFilterAttenuation(instrSession, filterNumber, attenuation);
        REMAP(3,5);
        }
    else
        rsupv_status = RSUPV_ERROR_SETTINGS_CONFLICT;
        
    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure Octave Filter
 * Purpose:  This function configures the Third Octave and Octave Filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureOctaveFilter(
    ViSession   instrSession,
    ViInt32     filterNumber,
    ViReal64    centerFrequency,
    ViReal64    attenuation
)
{
    ViStatus    rsupv_status;
    ViInt32     filterType = 0;

    rsupv_status = rsupv_GetFilterType(instrSession, filterNumber, &filterType);

    if ((filterType == RSUPV_USER_DEF_FILTER_TOCT) |
        (filterType == RSUPV_USER_DEF_FILTER_OCT))
        {
        CHECKERR(rsupv_SetFilterCenterFreq(instrSession, filterNumber, centerFrequency));

        rsupv_status = rsupv_SetFilterAttenuation(instrSession, filterNumber, attenuation);
        REMAP(3,4);
        }
    else
        rsupv_status = RSUPV_ERROR_SETTINGS_CONFLICT;

    return rsupv_status;
}

/*****************************************************************************
 * Function: Configure File Defined Filter
 * Purpose:  This function configures the File Defined Filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureFileDefinedFilter(
    ViSession   instrSession,
    ViInt32     filterNumber,
    ViString    fileDef,
    ViReal64    delay
)
{
    ViStatus    rsupv_status;
    ViInt32     filterType = 0;

    rsupv_status = rsupv_GetFilterType(instrSession, filterNumber, &filterType);

    if (filterType == RSUPV_USER_DEF_FILTER_FILE)
        {
        CHECKERR(rsupv_SetFilterFileDefined(instrSession, filterNumber, fileDef));

        rsupv_status = rsupv_SetFilterDelay(instrSession, filterNumber, delay);
        REMAP(3,4);

        }
    else
        rsupv_status = RSUPV_ERROR_SETTINGS_CONFLICT;

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Filter Type
 * Purpose:  This function selects the filter type for Filter 1 to 9.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterType(
    ViSession   instrSession,
    ViInt32     filter,
    ViInt32     filterType
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS1:UFIL%ld", filter);

    return rsupv_setEnum (instrSession,
                          buffer,
                          filterType,
                          filterTypeArr,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Type
 * Purpose:  This function gets the filter type for Filter 1 to 9.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterType(
    ViSession   instrSession,
    ViInt32     filter,
    ViInt32*    filterType
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS1:UFIL%ld?", filter);

    return rsupv_getEnum (instrSession,
                          buffer,
                          filterType,
                          filterTypeArr,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Filter Attenuation
 * Purpose:  This function specifies the desired filter attenuation of the
 *           stopband in dB.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterAttenuation(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64    attenuation
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS1:UFIL%ld:ATT", filter);

    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  attenuation,
                                  RSUPV_UNIT_DB,
                                  3.0, 120.0,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Attenuation
 * Purpose:  This function gets the desired filter attenuation of the stopband
 *           in dB.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterAttenuation(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64*   attenuation
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS1:UFIL%ld:ATT?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                    buffer,
                                    attenuation,
                                    VI_NULL,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Filter Delay
 * Purpose:  This function sets the filter delay.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterDelay(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64    filterDelay
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS1:UFIL%ld:DEL", filter);

    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  filterDelay,
                                  RSUPV_UNIT_S,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Delay
 * Purpose:  This function gets the filter delay.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterDelay(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64*   filterDelay
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS1:UFIL%ld:DEL?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  filterDelay,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Filter Order
 * Purpose:  This function specifeis the filter order.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterOrder(
    ViSession   instrSession,
    ViInt32 filter,
    ViInt32 fitlerOrder
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:ORD", filter);

    return rsupv_setEnum (instrSession,
                          buffer,
                          fitlerOrder,
                          fitlerOrderArr,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Order
 * Purpose:  This function gets the filter order.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterOrder(
    ViSession   instrSession,
    ViInt32 filter,
    ViInt32*    fitlerOrder
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:ORD?", filter);

    return rsupv_getEnum (instrSession,
                          buffer,
                          fitlerOrder,
                          fitlerOrderArr,
                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Filter Passband
 * Purpose:  This function specifies the cutoff frequency (-0.1 dB) of the
 *           passband in the case of low-pass and high-pass filters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterPassband(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64    filterPassband
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:PASS", filter);

    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  filterPassband,
                                  RSUPV_UNIT_HZ,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Passband
 * Purpose:  This function gets the cutoff frequency (-0.1 dB) of the passband
 *           in the case of low-pass and high-pass filters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterPassband(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64*   filterPassband
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:PASS?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  filterPassband,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Stopband
 * Purpose:  This function displays the cutoff frequency of the stopband in
 *           the case of low-pass and high-pass filters; the cutoff frequency
 *           is derived from the selected passband and the stopband
 *           attenuation. This value is only updated after the filter is
 *           activated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterStopband(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64*   filterStopband
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:STOP?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  filterStopband,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Filter Passband Low
 * Purpose:  This function specifies the lower cutoff frequency (-0.1 dB) of
 *           the passband in the case of bandpass and band-stop filters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterPassbandLow(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64    filterPassbandLow
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:PASS:LOW", filter);

    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  filterPassbandLow,
                                  RSUPV_UNIT_HZ,
                                  1.0, 1.0e+6,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Passband Low
 * Purpose:  This function gets the lower cutoff frequency (-0.1 dB) of the
 *           passband in the case of bandpass and band-stop filters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterPassbandLow(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64*   filterPassbandLow
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:PASS:LOW?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  filterPassbandLow,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Filter Passband Upp
 * Purpose:  This function specifies the upper cutoff frequency (-0.1 dB) of
 *           the passband in the case of bandpass and band-stop filters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterPassbandUpp(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64    filterPassbandUpp
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:PASS:UPP", filter);

    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  filterPassbandUpp,
                                  RSUPV_UNIT_HZ,
                                  1.0, 1.0e+6,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Passband Upp
 * Purpose:  This function specifies the upper cutoff frequency (-0.1 dB) of
 *           the passband in the case of bandpass and band-stop filters.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterPassbandUpp(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64*   filterPassbandUpp
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:PASS:UPP?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  filterPassbandUpp,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Stopband Low
 * Purpose:  This function displays the lower cutoff frequency of the stopband
 *           in the case of bandpass and band-stop filters; the cutoff
 *           frequency is derived from the selected passband and the stopband
 *           attenuation. This value is only updated after the filter is
 *           activated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterStopbandLow(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64*   filterStopbandLow
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:STOP:LOW?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  filterStopbandLow,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Stopband Upp
 * Purpose:  This function displays the upper cutoff frequency of the stopband
 *           in the case of bandpass and band-stop filters; the cutoff
 *           frequency is derived from the selected passband and the stopband
 *           attenuation. This value is only updated after the filter is
 *           activated.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterStopbandUpp(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64*   filterStopbandUpp
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:STOP:UPP?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  filterStopbandUpp,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Filter Center Freq
 * Purpose:  This function specifies the center frequency. The notch filter
 *           has a transmission zero at the center frequency; the attenuation
 *           at this frequency is considerably greater than the specified
 *           stopband attenuation.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterCenterFreq(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64    filterCenterFrequency
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:CENT", filter);

    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  filterCenterFrequency,
                                  RSUPV_UNIT_HZ,
                                  1.0, 1.0e+6,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Center Freq
 * Purpose:  This function gets the center frequency. The notch filter has a
 *           transmission zero at the center frequency; the attenuation at this
 *           frequency is considerably greater than the specified stopband
 *           attenuation.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterCenterFreq(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64*   filterCenterFrequency
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:CENT?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  filterCenterFrequency,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Filter Width
 * Purpose:  This function specifies the difference between the upper and
 *           lower cutoff frequency (-0.1 dB) of the passband for notch
 *           filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterWidth(
    ViSession   instrSession,
    ViInt32 filter,
    ViReal64    filterWidth
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:WIDT", filter);

    return rsupv_setViReal64Unit (instrSession,
                                  buffer,
                                  filterWidth,
                                  RSUPV_UNIT_HZ,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Filter Width
 * Purpose:  This function gets the difference between the upper and
 *           lower cutoff frequency (-0.1 dB) of the passband for notch
 *           filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetFilterWidth(
    ViSession   instrSession,
    ViInt32     filter,
    ViReal64*   filterWidth
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS:UFIL%ld:WIDT?", filter);

    return rsupv_getViReal64Unit (instrSession,
                                  buffer,
                                  filterWidth,
                                  VI_NULL,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Filter File Defined
 * Purpose:  This function reads in a filter from a file. Two file formats are
 *           supported in which the filter is described either by means of the
 *           coefficients of the transmission function (*.coe) or using the
 *           poles and zeros (*.zpz or *.npz) on the Z level.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetFilterFileDefined(
    ViSession   instrSession,
    ViInt32     filter,
    ViString    filterFileDef
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (filter, 1, 9, 2);

    sprintf (buffer, "SENS1:UFIL%ld:FNAM", filter);

    return rsupv_setViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    filterFileDef,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Configure Audio Monitoring
 * Purpose:  This function configures the audio monitorin function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ConfigureAudioMonitoring(
    ViSession   instrSession,
    ViInt32     signalSource,
    ViInt16     monitoringChannel,
    ViBoolean   speaker,
    ViInt32     phoneOutput,
    ViReal64    volume
)
{
    ViStatus    rsupv_status;

    CHECKERR (rsupv_SetAuxSignalSource(instrSession, signalSource));
    
    if (rsupv_checkInstrument (instrSession, RSUPV_INSTR_UPV) == VI_SUCCESS)
    {
        rsupv_status = rsupv_SetAuxMonitoringChannel(instrSession, monitoringChannel);
        REMAP(2,3);
        rsupv_status = rsupv_SetAuxSpeaker(instrSession, speaker);
        REMAP(2,4);
    }
    
    phoneOutput;
#if 0    
    rsupv_status = rsupv_SetAuxPhoneOutput(instrSession, phoneOutput);
    REMAP(2,5);
#endif    
    rsupv_status = rsupv_SetAuxVolume(instrSession, volume);
    REMAP(2,6);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Aux Audio Monitor
 * Purpose:  This function activates/deactivates the audio monitoring function.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxAudioMonitor (ViSession instrSession,
                                         ViBoolean audioMonitor)
{
    return rsupv_setViBoolean (instrSession,
                               "AUX:AUDM",
                               audioMonitor,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Aux Audio Monitor
 * Purpose:  This function gets the audio monitoring state.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxAudioMonitor (ViSession instrSession,
                                         ViBoolean *audioMonitor)
{
    return rsupv_getViBoolean (instrSession,
                               "AUX:AUDM?",
                               audioMonitor,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Aux Speaker
 * Purpose:  This function sets the internal loudspeaker playback.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxSpeaker(
    ViSession   instrSession,
    ViBoolean   auxSpeaker
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViBoolean (instrSession,
                               "AUX:SPE",
                               auxSpeaker,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Aux Speaker
 * Purpose:  This function gets the internal loudspeaker playback.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxSpeaker(
    ViSession   instrSession,
    ViBoolean*  auxSpeaker
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViBoolean (instrSession,
                               "AUX:SPE?",
                               auxSpeaker,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Aux Signal Source
 * Purpose:  This function selects the audio monitoring point along the signal
 *           path.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxSignalSource(
    ViSession   instrSession,
    ViInt32 auxSignalSource
)
{
    switch (auxSignalSource)
    {
        case RSUPV_AUX_SIGNAL_SOURCE_MON:
        case RSUPV_AUX_SIGNAL_SOURCE_GEN:
            CHECKINSTR (RSUPV_INSTR_UPV);
            break;
        case RSUPV_AUX_SIGNAL_SOURCE_DC:
            CHECKINSTR (RSUPV_INSTR_UPP);
            break;          
    }

    return rsupv_setEnum (instrSession,
                         "AUX:SPE:SOUR",
                          auxSignalSource,
                          auxSignalSourceArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Aux Signal Source
 * Purpose:  This function gets the audio monitoring point along the signal
 *           path.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxSignalSource(
    ViSession   instrSession,
    ViInt32*    auxSignalSource
)
{
    return rsupv_getEnum (instrSession,
                          "AUX:SPE:SOUR?",
                          auxSignalSource,
                          auxSignalSourceArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Aux Monitoring Channel
 * Purpose:  This function selects the audio monitoring channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxMonitoringChannel(
    ViSession   instrSession,
    ViInt32 auxChannel
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "AUX:SPE:CHAN",
                          auxChannel,
                          auxChannelArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Aux Monitoring Channel
 * Purpose:  This function gets the audio monitoring channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxMonitoringChannel(
    ViSession   instrSession,
    ViInt32*    auxChannel
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "AUX:SPE:CHAN?",
                          auxChannel,
                          auxChannelArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Aux Volume
 * Purpose:  This function sets the volume.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxVolume(
    ViSession   instrSession,
    ViReal64    auxVolume
)
{
    return rsupv_setViInt32(instrSession,
                              "AUX:SPE:VOL",
                            (ViInt32) ceil (auxVolume),
                              -120, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Aux Volume
 * Purpose:  This function gets the volume.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxVolume(
    ViSession   instrSession,
    ViReal64*   auxVolume
)
{
    ViStatus    rsupv_status;
    ViInt32     local_volume = 0;
    
    CHECKERR (rsupv_getViInt32(instrSession,
                              "AUX:SPE:VOL?",
                            &local_volume,
                            VI_ERROR_PARAMETER2));

    *auxVolume = local_volume;

    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Aux Analog Output
 * Purpose:  This function selects the auxiliary BNC output at the rear of the
 *           R&S UPV.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxAnalogOutput(
    ViSession   instrSession,
    ViInt32 auxAnalogAuxOut
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setEnum (instrSession,
                         "AUX:AAUX",
                          auxAnalogAuxOut,
                          auxAnalogAuxOutArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Aux Analog Output
 * Purpose:  This function gets the auxiliary BNC output at the rear of the
 *           R&S UPV.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxAnalogOutput(
    ViSession   instrSession,
    ViInt32*    auxAnalogAuxOut
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getEnum (instrSession,
                          "AUX:AAUX?",
                          auxAnalogAuxOut,
                          auxAnalogAuxOutArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Aux DC Voltage
 * Purpose:  This function sets the magnitude of the DC voltage level at the
 *           rear BNC socket.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxDCVoltage(
    ViSession   instrSession,
    ViReal64    auxDCValue,
    ViInt32     units
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViReal64Unit (instrSession,
                                  "AUX:DCV",
                                  auxDCValue,
                                  units,
                                  0.0, 0.0,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Aux DC Voltage
 * Purpose:  This function gets the magnitude of the DC voltage level at the
 *           rear BNC socket.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxDCVoltage(
    ViSession   instrSession,
    ViReal64*   auxDCValue,
    ViInt32*    units
)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViReal64Unit (instrSession,
                                  "AUX:DCV?",
                                  auxDCValue,
                                  units,
                                  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Aux Speaker Monitor
 * Purpose:  This function selects the monitored channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxSpeakerMonitor (ViSession instrSession,
                                     ViInt32 monitorNumber,
                                     ViInt32 monitoredChannel)
{
    ViChar buffer[BUFFER_SIZE] = "";

    CHECKINSTR (RSUPV_INSTR_UPP);

    CHECKVIINT32 (monitorNumber, 1, 2, 2);
    CHECKVIINT32 (monitoredChannel, 0, 16, 3);
    
    if (monitoredChannel == 0)
        sprintf (buffer, "AUX:SPE:MON%ld OFF", monitorNumber);
    else
        sprintf (buffer, "AUX:SPE:MON%ld CH%ld", monitorNumber, monitoredChannel);
    
    return rsupv_sendCmd (instrSession, buffer);
  
}

/*****************************************************************************
 * Function: Get Aux Speaker Monitor
 * Purpose:  This function queries the selected monitored channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxSpeakerMonitor (ViSession instrSession,
                                     ViInt32 monitorNumber,
                                     ViInt32 *monitoredChannel)
{
    ViChar buffer[BUFFER_SIZE] = "";

    CHECKINSTR (RSUPV_INSTR_UPP);
    
    sprintf (buffer, "AUX:SPE:MON%ld?", monitorNumber);
    
    return rsupv_getEnum (instrSession,
                        buffer,
                        monitoredChannel,
                        MonitoredChannelArr,
                        VI_ERROR_PARAMETER3);   
}

/*****************************************************************************
 * Function: Set Aux Phone State
 * Purpose:  This function activates/deactivates the front phone jack.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxPhoneState (ViSession instrSession,
                                       ViBoolean phoneState)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViBoolean (instrSession,
                               "AUX:PHON",
                               phoneState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Aux Phone State
 * Purpose:  This function returns the state of the front phone jack.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxPhoneState (ViSession instrSession,
                                          ViBoolean *phoneState)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_getViBoolean (instrSession,
                               "AUX:PHON?",
                               phoneState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Aux Phone Permanent State
 * Purpose:  This function activates/deactivates the Phone Out and Anlg Aux Out.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxPhonePermanentState (ViSession instrSession,
                                                   ViBoolean auxPhonePermanentState)
{
    CHECKINSTR (RSUPV_INSTR_UPV);
    return rsupv_setViBoolean (instrSession,
                               "AUX:PHP",
                               auxPhonePermanentState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Aux Phone Permanent State
 * Purpose:  This function returns the state of the Phone Out and Anlg Aux Out.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxPhonePermanentState (ViSession instrSession,
                                                   ViBoolean *auxPhonePermanentState)
{
    CHECKINSTR(RSUPV_INSTR_UPV);
    return rsupv_getViBoolean (instrSession,
                               "AUX:PHP?",
                               auxPhonePermanentState,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Aux Phone Output
 * Purpose:  This function selects the phone out.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetAuxPhoneOutput(
    ViSession   instrSession,
    ViInt32     auxPhoneOut
)
{
    instrSession;
    auxPhoneOut;
#if 0    
    return rsupv_setEnum (instrSession,
                         "AUX:PHON",
                          auxPhoneOut,
                          auxPhoneOutArr,
                          VI_ERROR_PARAMETER2);
#endif
    
    return RSUPV_ERROR_INSTRUMENT_NSUP_FUNC;
}

/*****************************************************************************
 * Function: Get Aux Phone Output
 * Purpose:  This function gets the phone out.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetAuxPhoneOutput(
    ViSession   instrSession,
    ViInt32*    auxPhoneOut
)
{
    instrSession;
    auxPhoneOut;
#if 0
    return rsupv_getEnum (instrSession,
                          "AUX:PHON?",
                          auxPhoneOut,
                          auxPhoneOutArr,
                          VI_ERROR_PARAMETER2);
#endif
    
    return RSUPV_ERROR_INSTRUMENT_NSUP_FUNC;
}

/*****************************************************************************
 * Function: Set Calibration
 * Purpose:  This function selects the calibration.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetCalibration(
    ViSession   instrSession,
    ViInt32     calibration
)
{
    return rsupv_setEnum (instrSession,
                         "DIAG:CAL",
                          calibration,
                          calibrationArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Calibration
 * Purpose:  This function gets the calibration.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetCalibration(
    ViSession   instrSession,
    ViInt32*    calibration
)
{
    return rsupv_getEnum (instrSession,
                          "DIAG:CAL?",
                          calibration,
                          calibrationArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Calibration Address
 * Purpose:  This function sets the calibration address.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetCalibrationAddress(
    ViSession   instrSession,
    ViInt32     calibrationAddress
)
{
    return rsupv_setViInt32 (instrSession,
                              "DIAG:CAL:ADDR",
                              calibrationAddress,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Calibration Address
 * Purpose:  This function gets the calibration address.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetCalibrationAddress(
    ViSession   instrSession,
    ViInt32*    calibrationAddress
)
{
    return rsupv_getViInt32 (instrSession,
                              "DIAG:CAL:ADDR?",
                              calibrationAddress,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Calibration Data
 * Purpose:  This function sets the calibration data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetCalibrationData(
    ViSession   instrSession,
    ViInt32     calibrationData
)
{
    return rsupv_setViInt32 (instrSession,
                              "DIAG:CAL:FDAT",
                              calibrationData,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Calibration Data
 * Purpose:  This function gets the calibration data.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetCalibrationData(
    ViSession   instrSession,
    ViInt32*    calibrationData
)
{
    return rsupv_getViInt32 (instrSession,
                              "DIAG:CAL:FDAT?",
                              calibrationData,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Hardcopy Source
 * Purpose:  This function selects the screen content to be output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHardcopySource (ViSession instrSession,
                                           ViInt32 source)
{
    return rsupv_setEnum (instrSession,
                         "HCOP:SOUR",
                          source,
                          hcopySourceArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Hardcopy Source
 * Purpose:  This function returns the screen content to be output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHardcopySource (ViSession instrSession,
                                           ViInt32 *source)
{
    return rsupv_getEnum (instrSession,
                         "HCOP:SOUR?",
                          source,
                          hcopySourceArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Hardcopy Destination
 * Purpose:  This function selects the destination for the screen content to 
 *           be output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHardcopyDestination (ViSession instrSession,
                                                ViInt32 destination)
{
    return rsupv_setEnum (instrSession,
                         "HCOP:DEST",
                          destination,
                          hcopyDestinationArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Hardcopy Destination
 * Purpose:  This function returns the destination for the screen content to 
 *           be output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHardcopyDestination (ViSession instrSession,
                                                ViInt32 *destination)
{
    return rsupv_getEnum (instrSession,
                         "HCOP:DEST?",
                          destination,
                          hcopyDestinationArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Hardcopy Orientation
 * Purpose:  This function selects the orientation wished on the Windows 
 *           printer (only if "Destination" is configured as "Printer").
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHardcopyOrientation (ViSession instrSession,
                                                ViInt32 orientation)
{
    return rsupv_setEnum (instrSession,
                         "HCOP:PRIN:ORI",
                          orientation,
                          hcopyOrientationArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Hardcopy Orientation
 * Purpose:  This function returns the orientation wished on the Windows 
 *           printer (only if "Destination" is configured as "Printer").
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHardcopyOrientation (ViSession instrSession,
                                                ViInt32 *orientation)
{
    return rsupv_getEnum (instrSession,
                         "HCOP:PRIN:ORI?",
                          orientation,
                          hcopyOrientationArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Hardcopy Store Mode
 * Purpose:  This function selects the mode for output to a file (only if 
 *           "Destination" is configured as "File").
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHardcopyStoreMode (ViSession instrSession,
                                              ViInt32 storeMode)
{
    return rsupv_setEnum (instrSession,
                         "HCOP:FILE:MOD",
                          storeMode,
                          hcopyStoreModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Hardcopy Store Mode
 * Purpose:  This function returns the mode for output to a file (only if 
 *           "Destination" is configured as "File").
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHardcopyStoreMode (ViSession instrSession,
                                              ViInt32 *storeMode)
{
    return rsupv_getEnum (instrSession,
                         "HCOP:FILE:MOD?",
                          storeMode,
                          hcopyStoreModeArr,
                          VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Hardcopy File Name
 * Purpose:  This function sets the name of the file for output to a file 
 *           (only if "Destination" is configured as "File" and "Store Mode" 
 *           is not "New").
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHardcopyFileName (ViSession instrSession,
                                             ViString fileName)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "HCOP:FILE",
                                    "'",
                                    fileName,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Hardcopy File Name
 * Purpose:  This function gets the name of the file for output to a file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHardcopyFileName (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar fileName[])
{
    return rsupv_getViStringQuoted (instrSession,
                              "HCOP:FILE?",
                              "'",
                              bufferSize,
                              fileName,
                              VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Hardcopy Header Footer State
 * Purpose:  This function selects the required header and footer in the output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHardcopyHeaderFooterState (ViSession instrSession,
                                                      ViBoolean headerFooter)
{
    return rsupv_setViBoolean (instrSession,
                               "HCOP:PRIN:ADD",
                               headerFooter,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Hardcopy Header Footer State
 * Purpose:  This function returns the required header and footer in the output.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHardcopyHeaderFooterState (ViSession instrSession,
                                                      ViBoolean *headerFooter)
{
    return rsupv_getViBoolean (instrSession,
                               "HCOP:PRIN:ADD?",
                               headerFooter,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Hardcopy Define Header
 * Purpose:  This function selects the required text for the header (only if
 *           "Header/Footer" is "On"). This text forms the title in the
 *           middle of the header line.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHardcopyDefineHeader (ViSession instrSession,
                                                 ViString defineHeader)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "HCOP:PRIN:HEAD",
                                    "'",
                                    defineHeader,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Hardcopy Define Header
 * Purpose:  This function returns the required text for the header.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHardcopyDefineHeader (ViSession instrSession,
                                                 ViInt32 bufferSize,
                                                 ViChar defineHeader[])
{
    return rsupv_getViStringQuoted (instrSession,
                                    "HCOP:PRIN:HEAD?",
                                    "'",
                                    bufferSize,
                                    defineHeader,
                                    VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Hardcopy Define Footer
 * Purpose:  This function selects the required text for the footer (only if
 *           "Header/Footer" is "On"). This text forms the legend in the
 *           middle of the footer line.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHardcopyDefineFooter (ViSession instrSession,
                                                 ViString defineFooter)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "HCOP:PRIN:FOOT",
                                    "'",
                                    defineFooter,
                                    VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Hardcopy Define Footer
 * Purpose:  This function returns the required text for the footer.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHardcopyDefineFooter (ViSession instrSession,
                                                 ViInt32 bufferSize,
                                                 ViChar defineFooter[])
{
    return rsupv_getViStringQuoted (instrSession,
                                    "HCOP:PRIN:FOOT?",
                                    "'",
                                    bufferSize,
                                    defineFooter,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Hardcopy Size
 * Purpose:  This function sets the name of the file for output to a file
 *           (only if "Destination" is configured as "File" and "Store Mode"
 *           is not "New").
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetHardcopySize (ViSession instrSession,
                                         ViString size)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "HCOP:GSIZ",
                                    "'",
                                    size,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Hardcopy Size
 * Purpose:  This function gets the name of the file for output to a file.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetHardcopySize (ViSession instrSession,
                                         ViInt32 bufferSize, ViChar size[])
{
    return rsupv_getViStringQuoted (instrSession,
                                    "HCOP:GSIZ?",
                                    "'",
                                    bufferSize,
                                    size,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Hardcopy Immediate
 * Purpose:  This function triggers the function for output of measured results. 
 *           The required action is immediately executed according to the 
 *           settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_HardcopyImmediate (ViSession instrSession)
{
    return rsupv_sendCmd (instrSession, "HCOP:IMM");    
}

/*****************************************************************************
 * Function: Set Window Style
 * Purpose:  This function selects the style of file selector window.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetWindowStyle (ViSession instrSession,
                                        ViInt32 fileSelectorWindowStyle)
{
    return rsupv_setViBoolean (instrSession,
                               "SYST:WINS",
                               (ViBoolean) fileSelectorWindowStyle,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Window Style
 * Purpose:  This function queries the style of file selector window.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetWindowStyle (ViSession instrSession,
                                        ViInt32 *fileSelectorWindowStyle)
{
    ViBoolean bool = 0;
    ViStatus  rsupv_status;
    CHECKERR (rsupv_getViBoolean (instrSession,
                               "SYST:WINS?",
                               &bool,
                               VI_ERROR_PARAMETER2));
    *fileSelectorWindowStyle = bool != 0;
    return rsupv_status;
}

/*****************************************************************************
 * Function: Set Memory String
 * Purpose:  This function defines the string to be cached.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMemoryString (ViSession instrSession,
                                         ViInt32 stringNumber,
                                         ViString string)
{
    ViChar buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (stringNumber, 1, 1024, 2);
    
    sprintf (buffer, "SYST:MEM:STR%ld", stringNumber);
    
    return rsupv_setViStringQuoted (instrSession,
                                    buffer,
                                    "'",
                                    string,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Memory String
 * Purpose:  This function returns the cached string.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMemoryString (ViSession instrSession,
                                         ViInt32 stringNumber,
                                         ViInt32 bufferSize, ViChar string[])
{
    CHECKVIINT32 (stringNumber, 1, 1024, 2);
    
    if (bufferSize > 540)
        return VI_ERROR_PARAMETER3;

    return rsupv_getViStringWindow (instrSession,
                              "SYST:MEM:STR%ld?",
                              stringNumber,
                              string,
                              bufferSize,
                              VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: StoreMemoryDataNumeric
 * Purpose:  This function stores up to 16 data sets with a length limited 
 *           only by the storage capacity made available by the operating system.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_StoreMemoryDataNumeric (ViSession instrSession,
                                                ViInt32 stringNumber,
                                                ViInt32 numberOfData,
                                                ViReal64 data[])
{
    ViChar    buffer[256];
    ViChar    *p2buf;
    ViInt32    i;
    
    CHECKVIINT32 (stringNumber, 1, 16, 2);

    p2buf = buffer;
    p2buf += sprintf(buffer, "SYST:MEM:DATA%ld ", stringNumber);
    
    for (i = 0; i < numberOfData; i++)
        p2buf += sprintf (p2buf, "%Lf,", data[i]);

    *--p2buf = '\0';

    return rsupv_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Store Memory Data Binary
 * Purpose:  This function stores up to 16 data sets with a length limited only 
 *           by the storage capacity made available by the operating system.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_StoreMemoryDataBinary (ViSession instrSession,
                                               ViInt32 stringNumber,
                                               ViInt32 binaryLength,
                                               ViChar binaryBlockData[])
{
    ViStatus    rsupv_status    = VI_SUCCESS;
    ViChar*     pbuffer;
    ViChar*     p2buf;

    if (!binaryBlockData)
        return VI_ERROR_PARAMETER4;

    p2buf = pbuffer = malloc (binaryLength + 1000);

    p2buf += sprintf (pbuffer, "SYST:MEM:DATA%ld #%ld%ld",
                               stringNumber,
                               (ViInt32)(floor(log10(binaryLength)) + 1),
                               binaryLength);

    memcpy (p2buf, binaryBlockData, binaryLength);
//    p2buf[binaryBlockLength] = '\n';

    rsupv_status = viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32)((p2buf - pbuffer) + binaryLength), VI_NULL);

    if (pbuffer)
        free (pbuffer);

    return rsupv_status;    
}

/*****************************************************************************
 * Function: Reset Memory Buffers
 * Purpose:  This function returns memory space of the available data buffers 
 *           to the operating system.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ResetMemoryBuffers (ViSession instrSession,
                                            ViInt32 freeBuffers)
{
    return rsupv_setEnum (instrSession,
                          "SYST:MEM:FREE",
                          freeBuffers,
                          freeBuffersArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: System Shutdown
 * Purpose:  This function returns memory space of the available data buffers 
 *           to the operating system.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SystemShutdown (ViSession instrSession, ViInt32 time)
{
    if (time < 0)
        return VI_ERROR_PARAMETER2;
    
    return rsupv_setViInt32 (instrSession,
                             "SYST:SHU",
                             time,
                             0,0,
                             VI_ERROR_PARAMETER2);
                             
}

/*****************************************************************************
 * Function: Load Setup
 * Purpose:  This function selects the setup file to be loaded.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_LoadSetup(
    ViSession   instrSession,
    ViString    setup
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:LOAD:STAT",
                                    "\"",
                                    setup,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Save Setup
 * Purpose:  This function selects the setup file to be saved.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SaveSetup(
    ViSession   instrSession,
    ViString    setup
)
{
    return rsupv_setViStringQuoted (instrSession,
                                    "MMEM:STOR:STAT",
                                    "\"",
                                    setup,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set GPIB Address
 * Purpose:  This function sets the GPIB address.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetGPIBAddress(
    ViSession   instrSession,
    ViInt32 gpibAddress
)
{
    return rsupv_setViInt32 (instrSession,
                              "SYST:COMM:GPIB:ADDR",
                              gpibAddress,
                              0, 0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Measurement Timeout
 * Purpose:  This function defines mesurement timeout. If timeout occurs
 *           without a new measurement result being available, the last
 *           measurement result that was read out is returned.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_SetMeasurementTimeout (ViSession instrSession,
                                      ViReal64 measurementTimeout)
{
    return rsupv_setViReal64 (instrSession,
                              "INIT:CONT:TIM",
                              measurementTimeout,
                              0.0,1000.0,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Measurement Timeout
 * Purpose:  This function queries mesurement timeout.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_GetMeasurementTimeout (ViSession instrSession,
                                      ViReal64 *measurementTimeout)
{
    return rsupv_getViReal64 (instrSession,
                              "INIT:CONT:TIM?",
                              measurementTimeout,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Start Measurement
 * Purpose:  Starts a single measurement. This function has no effect if it is
 *           output while a continuous measurement is being performed.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_StartMeasurement(
    ViSession   instrSession
)
{
    return rsupv_sendCmd (instrSession, ":INIT");
}

/*****************************************************************************
 * Function: Start Measurement and Wait for OPC
 * Purpose:  This function triggers all actions waiting for a trigger event
 *           and waits for operation completed (OPC) before returning the
 *           status code.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_StartMeasurementWaitOPC(
    ViSession   instrSession,
    ViInt32     timeout
)
{
    ViChar  buffer[BUFFER_SIZE];

    sprintf (buffer, ":INIT;*OPC?\n");  // Changed *TRG  to  INIT 

    return rsupv_waitOPC (instrSession, buffer, buffer, timeout);
}

/*****************************************************************************
 * Function: Read Measurement Result Min Max
 * Purpose:  This function is reading minimum or maximum of measurement
 *           results.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ReadMeasurementResultMinMax (ViSession instrSession,
                                                     ViInt32 channel,
                                                     ViInt32 measurement,
                                                     ViInt32 modifier,
                                                     ViReal64 *measurementResult,
                                                     ViChar units[])
{
    ViChar      buffer[BUFFER_SIZE] = "";
    ViStatus    rsupv_status = VI_SUCCESS;
    ViInt32     local_units = 0;
    ViChar      *p2buf = NULL;

    CHECKVIINT32 (channel, 1, 16, 2);
    CHECKVIINT32 (modifier, 0, 1, 4);

    p2buf = buffer;
    switch (measurement)
        {
        case RSUPV_MEASUREMENT_FUNCTION:        /* Measurement Function */
            p2buf += sprintf (buffer, "SENS1:DATA%ld?", channel);
            break;
        case RSUPV_MEASUREMENT_INPUT_MONITOR:   /* Input Monitor */
            p2buf += sprintf (buffer, "SENS2:DATA%ld?", channel);
            break;
        case RSUPV_MEASUREMENT_LEVEL_MONITOR:   /* Level Monitor */
            p2buf += sprintf (buffer, "SENS6:DATA%ld?", channel);
            break;
        case RSUPV_MEASUREMENT_FREQUENCY:       /* Frequency */
            p2buf += sprintf (buffer, "SENS3:DATA%ld?", channel);
            break;
        case RSUPV_MEASUREMENT_PHASE_GRP_DELAY: /* Phase or Group Delay */
            p2buf += sprintf (buffer, "SENS4:DATA%ld?", channel);
            break;
        default:
            return VI_ERROR_PARAMETER3;
        }

    sprintf (p2buf," %s", measModifierArr[modifier]);
    CHECKERR (rsupv_getViReal64Unit (instrSession,
                              buffer,
                              measurementResult,
                              &local_units,
                              VI_ERROR_PARAMETER4));

    if (units)
        strcpy (units, unitsArr[local_units]);
    
    return rsupv_status;
}

/*****************************************************************************
 * Function: Read Trace Data Sets
 * Purpose:  This function outputs trace data sets for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ReadTraceDataSets (ViSession instrSession,
                                           ViInt32 subsystem,
                                           ViInt32 subsystemNumber,
                                           ViInt32 dataSet,
                                           ViInt32 numberOfResults,
                                           ViReal64 output[])
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (dataSet, RSUPV_DATA_SET_AX, RSUPV_DATA_SET_BY, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "TRAC:SWE%ld:LOAD:%s?", subsystemNumber, dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "TRAC:MON:LOAD:%s?", dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "TRAC:FFT%ld:LOAD:%s?", subsystemNumber, dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "TRAC:WAV:LOAD:%s?", dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "TRAC:BAR%ld:LOAD:%s?", subsystemNumber, dataSetArr[dataSet]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Array (instrSession,
                                   buffer,
                                      output,
                                   numberOfResults,
                                   VI_ERROR_PARAMETER6);
}

/*****************************************************************************
 * Function: Write Trace Data Sets
 * Purpose:  This function saves sweep graph trace data sets.
 *
 *           Note(s):
 *
 *           (1) The transfer of data for the X-axis (...:AX and ...:BX) is
 *           reserved exclusively for the sweep subsystem! "X source" must be
 *           set to "Manual" - use function SetDisplaySweepGraphXSource.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_WriteTraceDataSets (ViSession instrSession,
                                            ViInt32 subsystem,
                                            ViInt32 subsystemNumber,
                                            ViInt32 dataSet,
                                            ViInt32 numberOfDataSets,
                                            ViReal64 dataSets[])
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (dataSet, RSUPV_DATA_SET_AX, RSUPV_DATA_SET_BY, 4);
    if (numberOfDataSets <= 0) 
        return VI_ERROR_PARAMETER5;
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "TRAC:SWE%ld:STOR:%s", subsystemNumber, dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            if ((dataSet == RSUPV_DATA_SET_AX) || (dataSet == RSUPV_DATA_SET_BX))
                return VI_ERROR_PARAMETER4;
            sprintf (buffer, "TRAC:MON:STOR:%s", dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            if ((dataSet == RSUPV_DATA_SET_AX) || (dataSet == RSUPV_DATA_SET_BX))
                return VI_ERROR_PARAMETER4;
            sprintf (buffer, "TRAC:FFT%ld:STOR:%s", subsystemNumber, dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            if ((dataSet == RSUPV_DATA_SET_AX) || (dataSet == RSUPV_DATA_SET_BX))
                return VI_ERROR_PARAMETER4;
            sprintf (buffer, "TRAC:WAV:STOR:%s", dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            if ((dataSet == RSUPV_DATA_SET_AX) || (dataSet == RSUPV_DATA_SET_BX))
                return VI_ERROR_PARAMETER4;
            sprintf (buffer, "TRAC:BAR%ld:STOR:%s", subsystemNumber, dataSetArr[dataSet]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_setViReal64Array (instrSession,
                                   buffer,
                                      dataSets,
                                   numberOfDataSets,
                                   VI_ERROR_PARAMETER6);    
}

/*****************************************************************************
 * Function: Read Trace Data Set Count
 * Purpose:  This function outputs trace data set count for the desired 
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ReadTraceDataSetCount (ViSession instrSession,
                                               ViInt32 subsystem,
                                               ViInt32 subsystemNumber,
                                               ViInt32 dataSet,
                                               ViInt32 *outputCount)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (dataSet, RSUPV_DATA_SET_AX, RSUPV_DATA_SET_BY, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "TRAC:SWE%ld:LOAD:COUN:%s?", subsystemNumber, dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "TRAC:MON:LOAD:COUN:%s?", dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "TRAC:FFT%ld:LOAD:COUN:%s?", subsystemNumber, dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "TRAC:WAV:LOAD:COUN:%s?", dataSetArr[dataSet]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "TRAC:BAR%ld:LOAD:COUN:%s?", subsystemNumber, dataSetArr[dataSet]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViInt32 (instrSession, 
                             buffer,
                             outputCount,
                             VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Read Trace Data List
 * Purpose:  This function outputs trace data list for the desired subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ReadTraceDataList (ViSession instrSession,
                                           ViInt32 subsystem,
                                           ViInt32 subsystemNumber,
                                           ViInt32 dataList,
                                           ViInt32 numberOfResults,
                                           ViReal64 output[])
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (dataList, RSUPV_DATA_SET_AX, RSUPV_DATA_SET_BY, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "TRAC:SWE%ld:LDL:%s?", subsystemNumber, dataSetArr[dataList]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "TRAC:MON:LDL:%s?", dataSetArr[dataList]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "TRAC:FFT%ld:LDL:%s?", subsystemNumber, dataSetArr[dataList]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "TRAC:WAV:LDL:%s?", dataSetArr[dataList]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "TRAC:BAR%ld:LDL:%s?", subsystemNumber, dataSetArr[dataList]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViReal64Array (instrSession,
                                   buffer,
                                      output,
                                   numberOfResults,
                                   VI_ERROR_PARAMETER6);    
}

/*****************************************************************************
 * Function: Read Trace Data List Count
 * Purpose:  This function outputs trace data list count for the desired 
 *           subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ReadTraceDataListCount (ViSession instrSession,
                                                ViInt32 subsystem,
                                                ViInt32 subsystemNumber,
                                                ViInt32 dataList,
                                                ViInt32 *outputCount)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32 (dataList, RSUPV_DATA_SET_AX, RSUPV_DATA_SET_BY, 4);
    
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            CHECKVIINT32 (subsystemNumber, 1, 4, 3);
            sprintf (buffer, "TRAC:SWE%ld:LDL:COUN:%s?", subsystemNumber, dataSetArr[dataList]);
        break;
        case RSUPV_DISP_SUBSYS_MON:
            sprintf (buffer, "TRAC:MON:LDL:COUN:%s?", dataSetArr[dataList]);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "TRAC:FFT%ld:LDL:COUN:%s?", subsystemNumber, dataSetArr[dataList]);
        break;
        case RSUPV_DISP_SUBSYS_WAV:
            sprintf (buffer, "TRAC:WAV:LDL:COUN:%s?", dataSetArr[dataList]);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            CHECKVIINT32 (subsystemNumber, 1, 2, 3);
            sprintf (buffer, "TRAC:BAR%ld:LDL:COUN:%s?", subsystemNumber, dataSetArr[dataList]);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    
    return rsupv_getViInt32 (instrSession, 
                             buffer,
                             outputCount,
                             VI_ERROR_PARAMETER5);    
}

/*****************************************************************************
 * Function: Read Axis Data
 * Purpose:  This function reads the x-axis data or the y-axis data of
 *           selected channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ReadAxisData (ViSession instrSession,
                                      ViInt32 subsystem, ViInt32 subsystemNumber,
                                      ViInt32 axis, ViInt32 channel,
                                      ViInt32 numberOfResults, ViReal64 output[])
{
    ViChar    buffer[BUFFER_SIZE] = "";
    ViChar    *p2buf;
    
    CHECKVIINT32 (subsystemNumber, 1, 2, 3);
    CHECKVIINT32 (channel, 1, 16, 5);
    
    p2buf = buffer;
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            p2buf += sprintf (buffer, "DATA:SWE%ld:", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            p2buf += sprintf (buffer, "DATA:FFT%ld:", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            p2buf += sprintf (buffer, "DATA:BAR%ld:", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    switch (axis)
    {
        case RSUPV_AXIS_X:
            p2buf += sprintf (p2buf, "X?");
        break;
        case RSUPV_AXIS_Y:
            p2buf += sprintf (p2buf, "Y%ld?", channel);
        break;
        default:
            return VI_ERROR_PARAMETER4;
    }
    
    return rsupv_getViReal64Array (instrSession,
                                   buffer,
                                   output,
                                   numberOfResults,
                                   VI_ERROR_PARAMETER7);    
}

/*****************************************************************************
 * Function: Read Axis Data Count
 * Purpose:  This function reads count of the x-axis data or the y-axis data 
 *           of selected channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsupv_ReadAxisDataCount (ViSession instrSession,
                                           ViInt32 subsystem,
                                           ViInt32 subsystemNumber, ViInt32 axis,
                                           ViInt32 channel, ViInt32 *outputCount)
{
    ViChar    buffer[BUFFER_SIZE] = "";
    ViChar    *p2buf;
    
    CHECKVIINT32 (subsystemNumber, 1, 2, 3);
    CHECKVIINT32 (channel, 1, 16, 5);
    
    p2buf = buffer;
    switch (subsystem)
    {
        case RSUPV_DISP_SUBSYS_SWE:
            p2buf += sprintf (buffer, "DATA:SWE%ld:COUN:", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_FFT:
            p2buf += sprintf (buffer, "DATA:FFT%ld:COUN:", subsystemNumber);
        break;
        case RSUPV_DISP_SUBSYS_BAR:
            p2buf += sprintf (buffer, "DATA:BAR%ld:COUN:", subsystemNumber);
        break;
        default:
            return VI_ERROR_PARAMETER2;
    }
    switch (axis)
    {
        case RSUPV_AXIS_X:
            p2buf += sprintf (p2buf, "X?");
        break;
        case RSUPV_AXIS_Y:
            p2buf += sprintf (p2buf, "Y%ld?", channel);
        break;
        default:
            return VI_ERROR_PARAMETER4;
    }
    
    return rsupv_getViInt32 (instrSession,
                             buffer,
                             outputCount,
                             VI_ERROR_PARAMETER7);    
}

/*===========================================================================*/
/* Function: Set Time Out                                                    */
/* Purpose:  Sets a minimum timeout value for driver I/O transactions in     */
/*           milliseconds. The timeout period may vary on computer platforms.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_setTimeOut (ViSession instrSession, ViInt32 timeout)
{
    ViStatus    rsupv_status = VI_SUCCESS;

    CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Get Time Out                                                    */
/* Purpose:  Returns the timeout value for driver I/O transactions in        */
/*           milliseconds.                                                   */
/*                                                                           */
/*           The timeout period may vary on computer platforms.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_getTimeOut (ViSession instrSession, ViPInt32 timeout)
{
    ViStatus    rsupv_status = VI_SUCCESS;

    if (!timeout)
        return RSUPV_ERROR_NULL_POINTER_VALUE;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout));

    return rsupv_status;
}

/*****************************************************************************/
/*-------- INSERT USER-CALLABLE INSTRUMENT-DEPENDENT ROUTINES HERE ----------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: State Checking                                                  */
/* Purpose:  This function switches ON/OFF state checking of the instrument  */
/*           (reading of the Standard Event Register and checking it for     */
/*           error). Nearly all driver function are using state checking.    */
/*           Switch this function to ON when debug your application. For     */
/*           better bus throughput and instruments performance switch it OFF.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_errorCheckState (ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    rsupv_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> errorChecking = state;

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Option Checking                                                 */
/* Purpose:  This function switches ON/OFF option checking of the instrument */
/*           The default setting is ON. If you switch it OFF,                */
/*			 the driver ignores current instrument or option information,    */
/*			 and proceedes to sending commands to the instrument.            */
/*           Switch this function to ON when debug your application. For     */
/*           better bus throughput and instruments performance switch it OFF.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_optionCheckState(ViSession instrSession,
										ViBoolean state)
{
	ViStatus    rsupv_status = VI_SUCCESS;
	rsupv_instrRange instrPtr;

	CHECKERR(viGetAttribute(instrSession, VI_ATTR_USER_DATA, &instrPtr));

	instrPtr->optionChecking = state;

	return rsupv_status;
}

/*===========================================================================*/
/* Function: Read To File From Instrument                                    */
/* Purpose:  This function is used to read data from the instrument and      */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_readToFile (ViSession instrSession, ViString source,
                                    ViString destination)
{
    ViStatus    rsupv_status    = VI_SUCCESS;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer, *p2buffer;
    FILE        *targetFile;

    sprintf (buffer, ":MMEM:DATA? '%s'\n", source);

    CHECKERR (viWrite (instrSession, (ViBuf)buffer, (ViUInt32)strlen(buffer), VI_NULL));
    CHECKERR (rsupv_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1d", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%ldd", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if ((rsupv_status = rsupv_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rsupv_status;
        }

    /* Store data from instrument to file on host computer */
    if ((targetFile = fopen (destination, "wb")) == VI_NULL)
        {
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != fwrite (p2buffer, sizeof (ViByte), (size_t)(count), targetFile))
        {
        fclose (targetFile);
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    fclose (targetFile);

    if (pbuffer) /* If not NULL pointer, then deallocate memory. */
        free (pbuffer);

    if ((rsupv_status = rsupv_checkStatus (instrSession)) < 0)
        return rsupv_status;

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Write From File To Instrument                                   */
/* Purpose:  This function is used to read data from a user specified file   */
/*           and write it to the instrument.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_writeFromFile (ViSession instrSession,
                                       ViString source, ViString destination)
{
    ViStatus    rsupv_status;
    ViUInt64     source_file;
    ViBoolean   src_eof = VI_FALSE;
    ViUInt32    offset = 0, block_length = 16000;
    
    
    CHECKERR (rsupv_writeHugeFileStart (instrSession, source, destination,
                              &source_file));
    
    do
    {
        CHECKERR (rsupv_contHugeFile (instrSession, source_file, offset, block_length, &src_eof));
        offset += block_length;
    } while (src_eof == VI_FALSE);

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Start Write Huge File To Instrument                             */
/* Purpose:  This function is used to read data from the host computer and   */
/*           write it to a user specified file in the instrument. It is used */
/*           for huge files. It sends only file header. To send file data,   */
/*           call into rsupv_contHugeFile in a loop.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_writeHugeFileStart (ViSession instrSession,
                                            ViString source,
                                            ViString destination,
                                            ViUInt64 *fileHandle)
{
    ViStatus    rsupv_status;
    FILE        *source_file;
    ViUInt32    file_size;
    ViChar      buffer [BUFFER_SIZE];
    
    if ((source_file = fopen (source, "rb")) == VI_NULL)
        return VI_ERROR_INSTR_FILE_OPEN;

    /* Get size of file */
    fseek (source_file, 0, SEEK_END);
    file_size = ftell (source_file);
    fseek (source_file, 0, SEEK_SET);

    CHECKERR (viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_FALSE));
    
    sprintf (buffer, ":MMEM:DATA '%s',#%ld%ld",
                               destination,
                               (ViInt32)(floor(log10(file_size)) + 1),
                               file_size);
    CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32)strlen(buffer), NULL));
    *fileHandle = (ViUInt64) source_file;
    
    return rsupv_status;
}

/*===========================================================================*/
/* Function: Continue Write Huge File To Instrument                          */
/* Purpose:  This function is used to read data from the host computer and   */
/*           write it to a user specified file in the instrument. It is used */
/*           for huge files. To start file transfer, call into               */
/*           rsupv_writeHugeFileStart prior to call into this function.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_contHugeFile (ViSession instrSession,
                                      ViUInt64 fileHandle, ViUInt32 offset,
                                      ViUInt32 blockLength, ViBoolean *EOFile)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar      *pbuffer;
    size_t      read;
    
    pbuffer = (ViChar*) malloc (blockLength);
    
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    if (fseek ((FILE*) fileHandle, offset, SEEK_SET) != 0)
    {
        rsupv_status = VI_ERROR_FILE_IO;
        goto cleanup;
    }
    
    read = fread (pbuffer, 1, blockLength, (FILE*) fileHandle);
    
    if (read == 0)
    {
        rsupv_status = VI_ERROR_FILE_IO;
        goto cleanup;
    }
    
    if (read != blockLength)
    {
        viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_TRUE);
    }
    
    rsupv_status = viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) read, NULL);
    if (read != blockLength)
    {
        *EOFile = VI_TRUE;
        fclose ((FILE*) fileHandle);
    }
    else
        *EOFile = VI_FALSE;
    
cleanup:
    if (pbuffer != NULL)
        free (pbuffer);
    return rsupv_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "%s\n", writeBuffer));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_readInstrData (ViSession instrSession,
                                        ViInt32 numberBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPInt32 numBytesRead)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViUInt32    tmpCnt;

    if (numBytesRead)
        *numBytesRead = 0L;

    CHECKERR (viRead (instrSession, (ViBuf)readBuffer, numberBytesToRead, &tmpCnt));

    if (numBytesRead)
        *numBytesRead = (ViInt32)tmpCnt;

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_reset (ViSession instrSession)
{
    ViStatus rsupv_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    CHECKERR (viPrintf (instrSession, "*RST\n"));
    CHECKERR (rsupv_defaultInstrSetup (instrSession));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_self_test (ViSession instrSession,
                                    ViInt16* testResult,
                                    ViChar _VI_FAR testMessage[])
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "*TST?\n"));
    CHECKERR (viScanf (instrSession, "%hd", testResult));

    sprintf (testMessage, "%s", (*testResult == 0) ? "Passed" : "Failed");

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_ERROR_QUERY.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_error_query (ViSession instrSession,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[])
{
    ViStatus rsupv_status = VI_SUCCESS;
    
    //viQueryf (instrSession, "SENS:DATA?\n","%*[^\r\n]");
    
    CHECKERR (viPrintf (instrSession, ":SYST:ERR?\n"));
    CHECKERR (viScanf (instrSession, "%ld,\"%[^\"]", errorCode, errorMessage));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_error_message (ViSession instrSession,
                                        ViStatus statusCode,
                                        ViChar _VI_FAR errMessage[])
{
    ViStatus rsupv_status = VI_SUCCESS;
    ViInt16 i;

    static rsupv_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,         "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,            "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,        "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,      "WARNING: Error Query not supported"},
        {VI_WARN_NSUP_REV_QUERY,        "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,           "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,           "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,           "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,           "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,           "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,           "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,           "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,           "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,        "ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,         "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,      "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,     "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,    "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,    "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,    "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,    "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,    "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,    "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,    "ERROR: Parameter 15 out of range"},

        {RSUPV_ERROR_INSTRUMENT_ERROR,              "ERROR: Instrument status error"},
        {RSUPV_ERROR_INVALID_CONFIGURATION,         "ERROR: Instrument configuration error"},
        {RSUPV_ERROR_INSTRUMENT_OPTION,             "ERROR: Option not installed"},
        {RSUPV_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE, "ERROR: Data are not available."},
        {RSUPV_ERROR_SETTINGS_CONFLICT,             "ERROR: Instrument settings conflict."},
        {RSUPV_ERROR_INSTRUMENT_NSUP_MODEL,         "ERROR: Required Instrument model is not available."},
        {RSUPV_ERROR_INSTRUMENT_NSUP_FUNC,          "ERROR: This function is not supported."},
        {RSUPV_ERROR_NULL_POINTER_VALUE,            "ERROR: NULL pointer passed as parameter."},
        
        {RSUPV_WARNING_SMALL_BUFFER,                "WARNING: Buffer too small."},
        
        {VI_NULL, VI_NULL}
    };

    rsupv_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (rsupv_status == VI_WARN_UNKNOWN_STATUS)
        {
        for (i = 0; statusDescArray[i].stringName; i++)
            {
            if (statusDescArray[i].stringVal == statusCode)
                {
                sprintf (errMessage, "%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
                }
            }
        sprintf (errMessage, "Unknown Error 0x%X", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
        }

    rsupv_status = VI_SUCCESS;
    return rsupv_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_revision_query (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "*IDN?\n"));
    CHECKERR (viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\r\n]", instrRev));

    sprintf (driverRev, "%s", RSUPV_REVISION);

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set Status Register                                             */
/* Purpose:  This function sets the Enable, NTransition, and PTransition bits*/
/* of status questionable and operating registers.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_setStatusRegister (ViSession instrSession,
                                  ViInt32 registerOper,
                                  ViInt32 selStatusReg, ViInt32 enable,
                                  ViInt32 PTransition, ViInt32 NTransition)
{
  ViStatus    rsupv_status    = VI_SUCCESS;
    
    if (rsupv_invalidViInt32Range (registerOper, 0, 3))
        return VI_ERROR_PARAMETER2;
    
    switch (registerOper) {
        case 0:
            CHECKERR (viPrintf (instrSession, ":STAT:PRES\n"));
        break;
        case 1:
            if (rsupv_invalidViInt32Range (selStatusReg, 0, 3))
                return VI_ERROR_PARAMETER3;
            
            if (rsupv_invalidViInt32Range (enable, 0, 65535))
                return VI_ERROR_PARAMETER4;
            if (rsupv_invalidViInt32Range (PTransition, 0, 65535))
                return VI_ERROR_PARAMETER5;
            if (rsupv_invalidViInt32Range (NTransition, 0, 65535))
                return VI_ERROR_PARAMETER6;

            CHECKERR (viPrintf (instrSession, ":STAT:QUES%s:ENAB %ld;:STAT:QUES%s:PTR %ld"
                ";:STAT:QUES%s:NTR %ld\n", selStatusRegArr[selStatusReg], enable, 
                selStatusRegArr[selStatusReg], PTransition,
                selStatusRegArr[selStatusReg], NTransition));
        break;
        case 2:
            if (rsupv_invalidViInt32Range (enable, 0, 65535))
                return VI_ERROR_PARAMETER4;
            if (rsupv_invalidViInt32Range (PTransition, 0, 65535))
                return VI_ERROR_PARAMETER5;
            if (rsupv_invalidViInt32Range (NTransition, 0, 65535))
                return VI_ERROR_PARAMETER6;
            
            CHECKERR (viPrintf (instrSession, ":STAT:OPER:ENAB %ld;:STAT:OPER:PTR %ld"
                      ";:STAT:OPER:NTR %ld\n", enable, PTransition, NTransition));
        break;
        case 3:
            if (rsupv_invalidViInt32Range (enable, 0, 65535))
                return VI_ERROR_PARAMETER4;
            if (rsupv_invalidViInt32Range (PTransition, 0, 65535))
                return VI_ERROR_PARAMETER5;
            if (rsupv_invalidViInt32Range (NTransition, 0, 65535))
                return VI_ERROR_PARAMETER6;

            CHECKERR (viPrintf (instrSession, ":STAT:XQUE:ENAB %ld;:STAT:XQUE:PTR %ld"
                ";:STAT:XQUE:NTR %ld\n", enable, PTransition, NTransition));
        break;
    }
    
    CHECKERR (rsupv_checkStatus (instrSession));
    
    return rsupv_status;  
}

/*===========================================================================*/
/* Function: Get Status Register                                             */
/* Purpose:  This function contains the commands for the querying status     */
/*           reporting system.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_getStatusRegister (ViSession instrSession,
                                  ViInt32 statusReg,
                                  ViInt32 *registerValue)
{
   ViStatus    rsupv_status   = VI_SUCCESS;
   
    if (rsupv_invalidViInt32Range (statusReg, 0, 28))
                return VI_ERROR_PARAMETER2;
    
    CHECKERR (viQueryf (instrSession, ":STAT:%s?\n", "%ld",
        statusRegArr[statusReg], registerValue));
    
    CHECKERR (rsupv_checkStatus (instrSession));
    
    return rsupv_status; 
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsupv_close (ViSession instrSession)
{
    rsupv_instrRange instrPtr;
    ViSession rmSession;
    ViUInt16 interface = VI_INTF_GPIB;
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    /* Interface specific operations */
    switch (interface)
        {
        case VI_INTF_ASRL: /* Serial Port. Go to local mode. */
            CHECKERR (viWrite (instrSession, (ViBuf) "@LOC\n", 5, VI_NULL));
            break;
        default:
            CHECKERR (viWrite (instrSession, (ViBuf) "&GTL\n", 5, VI_NULL));
            break;
/*
#ifdef VI_INTF_TCPIP

        case VI_INTF_TCPIP: // RSIB Interface. Go to local mode.
            CHECKERR (viSetAttribute (instrSession, RS_ATTR_RSIB_SRE, 0));
            break;

#endif
*/
        }

    if (instrPtr != NULL)
        free (instrPtr);

    rsupv_status = viClose (instrSession);
    viClose (rmSession);

    return rsupv_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Delay
/* Purpose : With a call to rsupv_delay, the current program is suspended
/*           from execution for the number of seconds specified by the
/*           argument seconds. rsupv_delay is accurate to a millisecond.
/*           This function is platform independent.
/*===========================================================================*/
void rsupv_delay (ViReal64 numberOfSeconds)
{
  clock_t StartTime = clock();

  while ((((double)(clock() - StartTime))/CLOCKS_PER_SEC) < (numberOfSeconds));

  return;
}

/*===========================================================================*/
/* Function: Array Item to Const                                             */
/* Purpose:  Search for item in array and return index. Last element of array*/
/*           must be NULL pointer.                                           */
/*===========================================================================*/
ViStatus rsupv_itemToConst (ViChar **array, ViString buffer, ViPInt32 itemNo)
{
    for ( *itemNo = 0; array[*itemNo]; (*itemNo)++ )
        if (!strcmp (buffer, array[*itemNo]))
            break;

    return (!array[*itemNo]) ? VI_ERROR_INSTR_INTERPRETING_RESPONSE : VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rsupv_invalidViBooleanRange (ViBoolean val)
{
    return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsupv_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsupv_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rsupv_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Check String                                                    */
/* Purpose:  This function checks a string value to see if it is             */
/*           of a defined length and initialized.                            */
/*===========================================================================*/
ViBoolean rsupv_invalidViString (ViString val, ViUInt32 maxLength)
{
    if (val)
        return (ViBoolean)((strlen (val) > maxLength) ? VI_TRUE : VI_FALSE);

    return VI_TRUE;
}

/*===========================================================================*/
/* Function: Wait for OPC                                                    */
/* Purpose:  This function sends a command to the instrument and waits until */
/*           the instrument response is available (queries MAV in STB).      */
/*===========================================================================*/
ViStatus rsupv_waitOPC (ViSession instrSession, ViPString command,
                ViPString instrResp, ViInt32 timeout)
{
    ViStatus rsupv_status = VI_SUCCESS;
    ViUInt16 response = 0;
    ViInt32 counter = 0;
    ViInt32 oldTimeout = 0;
    ViUInt16 interface = VI_INTF_GPIB;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    /* Interface specific operations */
    switch (interface)
        {
        case VI_INTF_ASRL: /* Serial interface */
            CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout));
             /* "\n" is appended in the passed buffer, do not include it here */
            if ((rsupv_status = viQueryf (instrSession, "%s", "%[^\r\n]", command, instrResp)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout);
                return rsupv_status;
                }
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));
        break;
        default: /* Default case */

            /* "\n" is appended in the passed buffer, do not include it here */
            CHECKERR (viPrintf (instrSession, "%s", command));

            while (!(response & 0x10)) /* (queries MAV in STB) */
                {

                CHECKERR (viReadSTB (instrSession, &response));

                rsupv_delay (0.005);
                counter+= 5;
                if (counter >= timeout)
                    return VI_ERROR_TMO;
                }

            CHECKERR (viScanf (instrSession, "%[^\r\n]", instrResp));
        break;
        }

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rsupv_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rsupv_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rsupv_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession,
                             ViStatus currentStatus)
{
    rsupv_instrRange instrPtr;

    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL)
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Read Data With Unknown length                                   */
/* Purpose:  This function allocates buffer and reads data from the          */
/*           instrument                                                      */
/* WARNING:  You must deallocate buffer, if any pointer has been returned.   */
/*           This function does not work correctly, when you read bin data   */
/*           via serial interface                                            */
/*===========================================================================*/
ViStatus rsupv_readDataUnknownLength(ViSession instrSession,
                                     ViChar **outBuffer,
                                     ViUInt32 *count)
{
    ViStatus rsupv_status = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = VI_NULL, *p2buffer;

    *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = BUFFER_SIZE;
    buffer = (ViChar *)calloc(actSize, sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
        {
        if ((rsupv_status = viRead (instrSession, (ViPBuf)p2buffer, BUFFER_SIZE, &retCnt)) < 0)
            {
            free(buffer);
            *outBuffer = VI_NULL;
            return rsupv_status;
            }
        if (retCnt == BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string at the end of data */
        }
       } while (rsupv_status == VI_SUCCESS_MAX_CNT); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count) *count = (actSize - BUFFER_SIZE) + retCnt;

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Send Command                                                    */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus rsupv_sendCmd (ViSession instrSession, ViString cmd)
{
    ViStatus    rsupv_status    = VI_SUCCESS;
    ViUInt32     length          = 0;
    ViChar      *pbuffer;

    length = (ViUInt32) strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf) pbuffer, length + 1, VI_NULL));
    GOCHECKERR (rsupv_checkStatus (instrSession));

ERROR:

    free(pbuffer);

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Send Command With Window                                        */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus rsupv_sendCmdWindow (ViSession instrSession, ViString cmd, ViInt32 window)
{
    ViStatus    rsupv_status    = VI_SUCCESS;
    size_t     length          = 0;
    ViChar      *pbuffer;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 2);
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';
    pbuffer[length + 1] = '\0';

    GOCHECKERR (viPrintf (instrSession, pbuffer, window));
    GOCHECKERR (rsupv_checkStatus (instrSession));

ERROR:

    free(pbuffer);

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set ViInt32                                                     */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsupv_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    if ((max - min) > 0)
        if (rsupv_invalidViInt32Range (value, min, max))
            return parameterErrorCode;

    sprintf (wrt_buf, "%s %ld", cmd, value);

    return rsupv_sendCmd (instrSession, wrt_buf);
}

/*===========================================================================*/
/* Function: Get ViInt32                                                     */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsupv_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR( viPrintf(instrSession, "*CLS;%s\n", cmd));
    CHECKERR( rsupv_checkBeforeRead (instrSession));
    CHECKERR( viScanf(instrSession, "%ld", value));
    CHECKERR( rsupv_checkStatus (instrSession));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set ViInt32 With Window                                         */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsupv_setViInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32  window,
                           ViInt32  value,
                           ViInt32  min,
                           ViInt32  max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    /* CHECKWINDOW; */

    sprintf (wrt_buf, cmd, window);

    return rsupv_setViInt32(instrSession,
                             wrt_buf,
                             value,
                             min,
                             max,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get ViInt32 With Window                                         */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsupv_getViInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE]="";

    /* CHECKWINDOW; */

    sprintf(wrt_buf, cmd, window);

    return rsupv_getViInt32(instrSession,
                            wrt_buf,
                            value,
                            parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set ViUInt32                                                    */
/* Purpose:  This function sets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsupv_setViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 value,
                           ViUInt32 min,
                           ViUInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    if ((max - min) > 0)
        if (rsupv_invalidViUInt32Range (value, min, max))
            return parameterErrorCode;

    sprintf (wrt_buf, "%s %lu", cmd, value);

    return rsupv_sendCmd (instrSession, wrt_buf);
}

/*===========================================================================*/
/* Function: Get ViUInt32                                                    */
/* Purpose:  This function gets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsupv_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR( viPrintf(instrSession, "*CLS;%s\n", cmd));
    CHECKERR( rsupv_checkBeforeRead (instrSession));
    CHECKERR( viScanf(instrSession, "%lu", value));
    CHECKERR( rsupv_checkStatus (instrSession));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set ViUInt32 With Window                                        */
/* Purpose:  This function sets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsupv_setViUInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViUInt32 value,
                           ViUInt32 min,
                           ViUInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    /* CHECKWINDOW; */

    sprintf (wrt_buf, cmd, window);

    return rsupv_setViUInt32(instrSession,
                             wrt_buf,
                             value,
                             min,
                             max,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get ViUInt32 With Window                                        */
/* Purpose:  This function gets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsupv_getViUInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE]="";

    /* CHECKWINDOW; */

    sprintf(wrt_buf, cmd, window);

    return rsupv_getViUInt32(instrSession,
                            wrt_buf,
                            value,
                            parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set ViBoolean                                                   */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsupv_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViStatus parameterErrorCode)
{
    if (rsupv_invalidViBooleanRange (value))
        return parameterErrorCode;

    return rsupv_setViString (instrSession,
                              cmd,
                              value ? "ON" : "OFF",
                              0);
}

/*===========================================================================*/
/* Function: Get ViBoolean                                                   */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsupv_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status;
    ViInt32     bool_val = 0;

    if (value == NULL)
        return parameterErrorCode;

#if 1    
    CHECKERR (rsupv_getEnum (instrSession,
                             cmd,
                             &bool_val,
                             offOnArr,
                             1));
#else
    CHECKERR (rsupv_getViInt32(instrSession,
                               cmd,
                               &bool_val,
                               parameterErrorCode));
#endif
    *value = (ViBoolean) bool_val;

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set ViBoolean With Window                                       */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsupv_setViBooleanWindow (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViBoolean value,
                           ViStatus parameterErrorCode)
{
    ViChar  wrt_buf[BUFFER_SIZE];
    ViChar* p2buf;

    /* CHECKWINDOW; */
    if (rsupv_invalidViBooleanRange (value))
        return parameterErrorCode;

    p2buf = wrt_buf + sprintf(wrt_buf, cmd, window);
    sprintf (p2buf, " %s", value ? "ON" : "OFF");

    return rsupv_sendCmd (instrSession, wrt_buf);
}

/*===========================================================================*/
/* Function: Get ViBoolean With Window                                       */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsupv_getViBooleanWindow (ViSession   instrSession,
                                    ViString    cmd,
                                    ViInt32     window,
                                    ViBoolean   *value,
                                    ViStatus    parameterErrorCode)
{
    ViStatus    rsupv_status;
    ViInt32     bool_val;

    if (value == NULL)
        return parameterErrorCode;

    CHECKERR (rsupv_getViInt32Window(instrSession,
                                   cmd,
                                   window,
                                   &bool_val,
                                   parameterErrorCode));
    *value = (ViBoolean) bool_val;

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set ViReal64                                                    */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsupv_setViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      buffer[BUFFER_SIZE];

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rsupv_invalidViReal64Range (value, min, max))
            return parameterErrorCode;

    sprintf(buffer,"%s %.12f", cmd, value);

    return rsupv_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Set ViReal64 with Units                                         */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsupv_setViReal64Unit (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value,
                            ViInt32  unit,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      buffer[BUFFER_SIZE];
    ViStatus    rsupv_status;

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rsupv_invalidViReal64Range (value, min, max))
            return parameterErrorCode;

    CHECKERR (rsupv_checkIndexInList (instrSession, unit, unitsArr, parameterErrorCode + 1));

    sprintf(buffer,"%s %.12f %s", cmd, value, unitsArr[unit]);

    return rsupv_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get ViReal64                                                    */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsupv_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR(viPrintf (instrSession, "*CLS;%s\n", cmd));
    CHECKERR( rsupv_checkBeforeRead (instrSession));
    CHECKERR(viScanf (instrSession, "%le", value));
    CHECKERR(rsupv_checkStatus (instrSession));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Get ViReal64 with Units                                         */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsupv_getViReal64Unit (ViSession instrSession,
                                ViString cmd,
                                ViReal64 *value,
                                ViInt32  *unit,
                                ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR(viPrintf (instrSession, "*CLS;%s\n", cmd));
    
    if (unit)
        {
        ViChar  ch_units[10];

        CHECKERR( rsupv_checkBeforeRead (instrSession));
        CHECKERR(viScanf (instrSession, "%le %s", value, ch_units));
        CHECKERR(rsupv_checkStatus (instrSession));
        rsupv_status = rsupv_getIndexFromString(instrSession, ch_units, unitsArr, unit);
        if (rsupv_status != VI_SUCCESS)
        {
            *unit = -1;
            rsupv_status = VI_SUCCESS;
        }
        }
    else
        {
        CHECKERR( rsupv_checkBeforeRead (instrSession));
        CHECKERR(viScanf (instrSession, "%le", value));
        CHECKERR(rsupv_checkStatus (instrSession));
        }

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 With Window                                        */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsupv_setViReal64Window (ViSession instrSession,
                            ViString cmd,
                            ViInt32  window,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    /* CHECKWINDOW; */

    sprintf (wrt_buf, cmd, window);

    return rsupv_setViReal64(instrSession,
                             wrt_buf,
                             value,
                             min,
                             max,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set ViReal64 With Window and Unit                               */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsupv_setViReal64WindowUnit (ViSession instrSession,
                                      ViString cmd,
                                      ViInt32  window,
                                      ViReal64 value,
                                      ViInt32  unit,
                                      ViReal64 min,
                                      ViReal64 max,
                                      ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    /* CHECKWINDOW; */

    sprintf (wrt_buf, cmd, window);
    
    return rsupv_setViReal64Unit(instrSession,
                                 wrt_buf,
                                 value,
                                 unit,
                                 min,
                                 max,
                                 parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get ViReal64 With Window                                        */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsupv_getViReal64Window (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    /* CHECKWINDOW;*/

    sprintf(wrt_buf, cmd, window);

    return rsupv_getViReal64(instrSession,
                             wrt_buf,
                             value,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get ViReal64 With Window and Unit                               */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsupv_getViReal64WindowUnit (ViSession instrSession,
                                      ViString cmd,
                                      ViInt32  window,
                                      ViReal64 *value,
                                      ViInt32  *unit,
                                      ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    /* CHECKWINDOW;*/

    sprintf(wrt_buf, cmd, window);

    return rsupv_getViReal64Unit(instrSession,
                                 wrt_buf,
                                 value,
                                 unit,
                                 parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set String                                                      */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rsupv_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 4 );
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, "%s %s\n", cmd, value);

    GOCHECKERR(viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32)(p2buf - wrt_buf), VI_NULL));
    GOCHECKERR(rsupv_checkStatus(instrSession));

ERROR:

    free(wrt_buf);

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Get String                                                      */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rsupv_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViInt32 buffer_size,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status    = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    ViUInt32    count           = 0,
                length          = 0;

    length = (ViUInt32) strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 10);
    sprintf (pbuffer, "*CLS;%s\n", cmd);
    
    if (!value)
        return parameterErrorCode;

    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) strlen (pbuffer), VI_NULL));
    GOCHECKERR (rsupv_checkBeforeRead (instrSession));
    
    free (pbuffer);

    pbuffer = VI_NULL;

    GOCHECKERR (rsupv_readDataUnknownLength (instrSession, &pbuffer, &count));
    GOCHECKERR (rsupv_status = rsupv_checkStatus (instrSession));

    if (count > (ViUInt32)buffer_size)
        {
        rsupv_status = RSUPV_WARNING_SMALL_BUFFER;
        count = buffer_size;
        memcpy (value, pbuffer, count);
        value[count] = '\0';
        }
    else
        if ((p2buffer = strpbrk (pbuffer, "\r\n")) != VI_NULL)
            {
            rsupv_status = VI_SUCCESS;
            memcpy (value, pbuffer, p2buffer - pbuffer);
            value[p2buffer - pbuffer] = '\0';
            }
         else
            {
            strcpy (value, pbuffer);
            }

ERROR:

    free (pbuffer);

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set Quoted String                                               */
/* Purpose:  This function sets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rsupv_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*)malloc(strlen(cmd) + strlen(value) + 2*strlen(quotes) + 4);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf,"%s %s%s%s\n", cmd, quotes, value, quotes);

    GOCHECKERR (viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32)(p2buf - wrt_buf), VI_NULL));
    GOCHECKERR (rsupv_checkStatus(instrSession));

ERROR:

    free (wrt_buf);

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Get Quoted String                                               */
/* Purpose:  This function gets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rsupv_getViStringQuoted (ViSession instrSession,
                                  ViString cmd,
                                  ViString quotes,
                                  ViInt32  arraySize,
                                  ViChar   value[],
                                  ViStatus parameterErrorValue)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar*     rd_buf;
    ViChar      fmt_string[BUFFER_SIZE];

    if (value == VI_NULL)
        return parameterErrorValue;

    rd_buf = (ViChar*)malloc(arraySize + 2*strlen(quotes) + 10);
    sprintf(fmt_string,"%s%%[^%s]", quotes, quotes);

    {
        ViUInt32     length      = 0;
        ViChar      *pbuffer;

        length = (ViUInt32) strlen (cmd);
        pbuffer = (ViChar*) malloc (length + 1);
        memcpy (pbuffer, cmd, length);
        pbuffer[length] = '\n';

        rsupv_status = viWrite (instrSession, (ViPBuf) cmd, length + 1, VI_NULL);

        free(pbuffer);
    }

    GOCHECKERR (rsupv_status);
    GOCHECKERR (rsupv_checkBeforeRead (instrSession));
    GOCHECKERR (viScanf (instrSession, fmt_string, value));
    GOCHECKERR (rsupv_checkStatus(instrSession));

ERROR:

    free (rd_buf);

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set String With Channel                                         */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rsupv_setViStringWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 11);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, cmd, window);
    p2buf += sprintf(p2buf," %s\n", value);

    rsupv_status = viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32)(p2buf - wrt_buf), VI_NULL);
    free(wrt_buf);

    CHECKERR(rsupv_status);
    CHECKERR(rsupv_checkStatus(instrSession));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Get String with Window                                          */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rsupv_getViStringWindow (ViSession instrSession,
                                ViString cmd,
                                ViInt32     window,
                                ViString value,
                                ViInt32 buffer_size,
                                ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    sprintf(wrt_buf, cmd, window);

    return rsupv_getViString(instrSession,
                             wrt_buf,
                             value,
                             buffer_size,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsupv_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_checkIndexInList (instrSession, value, list, parameterErrorCode));

    return rsupv_setViString (instrSession, cmd, list[value], parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsupv_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKERR( rsupv_getViString(instrSession, cmd, buffer, BUFFER_SIZE, parameterErrorCode));
    CHECKERR( rsupv_getIndexFromString (instrSession, buffer, list, value));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set Enum With Window                                              */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsupv_setEnumWindow (ViSession instrSession,
                        ViString cmd,
                        ViInt32     window,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus rsupv_status = VI_SUCCESS;

    CHECKERR (rsupv_checkIndexInList (instrSession, value, list, parameterErrorCode));

    return rsupv_setViStringWindow (instrSession, cmd, window, list[value], parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get Enum Window                                                   */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsupv_getEnumWindow (ViSession instrSession,
                        ViString cmd,
                        ViInt32     window,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKERR(rsupv_getViStringWindow(instrSession, cmd, window, buffer, BUFFER_SIZE, parameterErrorCode));
    CHECKERR(rsupv_getIndexFromString (instrSession, buffer, list, value));

    return rsupv_status;

}

/*===========================================================================*/
/* Function: Check Index List                                                */
/* Purpose:  This function checks if the passed index is valid for given list*/
/*===========================================================================*/
ViStatus rsupv_checkIndexInList (ViSession instrSession,
                                 ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError)

{
    ViInt32     i=0;
    instrSession;

    if (index < 0)
        return statusIfError;

    while (list[i++] != VI_NULL);
    if (index >= i)
        return statusIfError;
    else
        if (strlen (list[index]) == 0)
            return statusIfError;

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Get Index From String                                           */
/* Purpose:  This function returns index for string.                         */
/*===========================================================================*/
ViStatus rsupv_getIndexFromString (ViSession instrSession,
                                   ViString string,
                                   ViString *list,
                                   ViInt32 *index)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViUInt32    i;
    size_t      len;
    instrSession;

    for (i = 0; list[i]; i++)
        if ((len = strlen (string)) == strlen (list[i])) {
            if (strncmp(string, list[i], strlen (string)) == 0) break;
        }

    *index = i;

    return (list[i]) ? rsupv_status : VI_ERROR_INSTR_INTERPRETING_RESPONSE;
}

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rsupv_reset operation and by the     */
/*           rsupv_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/
/*===========================================================================*/
ViStatus rsupv_defaultInstrSetup (ViSession instrSession)
{
    ViStatus            rsupv_status = VI_SUCCESS;
    ViChar              buffer[BUFFER_SIZE];
    ViInt32             idx = 0, major_rev = 0, minor_rev = 0;
    rsupv_instrRange    instrPtr;
    ViChar*             p2buf;

    /* --- Set standard "C" locale --- */
    {
       char *locale;

       locale = setlocale(LC_ALL, "C");
    }

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /*
        Following two ViInt32 variables are used to store information about
        currently connected instrument and its options. Each bit represents
        one instrument model or option.
    */
    /* --- */
    instrPtr->errorChecking = VI_TRUE;
	instrPtr->optionChecking = VI_TRUE;
    strcpy (instrPtr->instrDriverRevision, RSUPV_REVISION);

    /*--- Recognize Instrument options (use *OPT? command) --- */
    /* Should return something like:
    K1(01.00),K21(01.00),K22(01.00),K4(01.00),B1(01.00),B2(01.00),B3(01.00),B41(01.00),B42(01.00),B43(01.00),B44(01.00),U1(01.00),Z1(01.00),S(01.00)\n
    */

    CHECKERR (viQueryf (instrSession, "*OPT?\n", "%[^\r\n]", buffer));
    /* If option K4 is not installed, *OPT? returns 0. In this case the
       instrument is not able to operate remotely */
    idx = 0;
    if (strncmp (buffer, "0", 1))
    {
        p2buf = strtok (buffer, ",");
        do
        {
            sscanf (p2buf, "%[^(](%ld.%ld", instrPtr -> instrOption[idx].optionName, &major_rev, &minor_rev);
            instrPtr -> instrOption[idx].optionRev = major_rev * 100 + minor_rev;
            idx++;
            p2buf = strtok (VI_NULL, ",");
        } while (p2buf);
    }

    // enable built-in options in case of UPP
    if (instrPtr->instrModel == RSUPV_INSTR_UPP)
    {
        strcpy (instrPtr->instrOption[idx].optionName,"B41");
        instrPtr->instrOption[idx++].optionRev = 100;
        strcpy (instrPtr->instrOption[idx].optionName, "B48");
        instrPtr->instrOption[idx++].optionRev = 100;
    }
    /* Default Instrument Setup String - Data Format */
    /* If not under development CHANGE to ":SYST:DISP:UPD OFF" */

    CHECKERR (viPrintf (instrSession, "*CLS;*SRE 32;SYST:QLON OFF\n"));

    /* Do the dummy STB read without error checking */
    {
    ViUInt16    response = 0;

    viReadSTB (instrSession, &response);
    }

    return rsupv_status;
}

/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads Event Status Register to check for        */
/*           possible errors.                                              */
/*=========================================================================*/
ViStatus rsupv_checkStatus (ViSession instrSession)
{
    ViStatus            rsupv_status = VI_SUCCESS;
    rsupv_instrRange   instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Error Checking Enabled? */
    if (instrPtr -> errorChecking)
        {
        ViInt32 esrByte = 0;

        /*--------------- Read the Standard Event Register ------------------*/

        CHECKERR (viQueryf (instrSession, "*STB?\n", "%ld", &esrByte));

        /*---------------- Check if any error or message bit was asserted ---*/

        rsupv_status = (esrByte & RSUPV_IEEE_ERROR_BITS) ? RSUPV_ERROR_INSTRUMENT_ERROR : VI_SUCCESS;
        }

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Check STB Before Read                                           */
/* Purpose:  This is a utility function which checks Error/Event Queue bit of*/
/*           status byte. Call this function before read result of query     */
/*           command. When error occurs, this function returns error code    */
/*           RSUPV_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE. Prior to calling    */
/*           this function is necessary to clear STB.                        */
/*===========================================================================*/
ViStatus rsupv_checkBeforeRead (ViSession instrSession)
{
    ViStatus rsupv_status = VI_SUCCESS;
    ViUInt16 response = 0;
    ViInt32 counter = 0, timeout = 0;
    ViUInt16 interface = VI_INTF_GPIB;
    rsupv_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    /* Is Error Checking Enabled? */
    if ((instrPtr -> errorChecking) && (RSUPV_CHECK_BEFORE_READ == VI_TRUE))
        {
        /* Interface specific operations */
        switch (interface)
            {
            case VI_INTF_ASRL: /* Serial interface */

                /* Not supported */

            break;
            default: /* Default case */

                CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &timeout));

#if 0 /* Removed from the build */

                /* Result of reading STB must be zero. Initial settings */
                do {
                    CHECKERR (rsupv_status = viReadSTB (instrSession, &response));

                    rsupv_delay (0.005);
                    counter += 5;
                    if (counter >= timeout)
                        return VI_ERROR_TMO;
                /* Wait until STB is non zero value. */
                } while (!response);
#endif
                /* Check STB for errors. */
                do {
                    if (viReadSTB (instrSession, &response) < VI_SUCCESS)
                        if (viReadSTB (instrSession, &response) < VI_SUCCESS) /* Workarounded */
                            return rsupv_status;

                    rsupv_delay (0.005);
                    counter += 5;
                    if (counter >= timeout)
                        return VI_ERROR_TMO;
                /* Wait while STB is non zero value. Only Error/Event Queue
                   and MAV bits are taken into consideration. */
                } while (!(response & 0x14));
#if 0
                if (response & 0x4)
                {
                    ViInt32 error;
                    
                    CHECKERR(viQueryf(instrSession, "*ESR?\n", "%ld", &error));
                    if (error == 4)
                        CHECKERR(viQueryf(instrSession, ":SYST:ERR?\n", "%*[^\r\n]"));
                }
#endif
                /* (check Error/Event Queue bit in STB) */
                return (response & 0x4) ?  RSUPV_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE : rsupv_status;
            }
        }

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Swap 32bit Integer                                              */
/* Purpose:  Byte order for binary transfers depends on MOTOROLA or INTEL    */
/*           formats.                                                        */
/*===========================================================================*/
void rsupv_SWAP_32(unsigned char *data, long points)
{
    /* MOTOROLA (byte order 0123) or INTEL (byte order 3210). */
    unsigned int    word = 0x1122;
    unsigned char   *p2word = (unsigned char*)&word;

    if (*p2word == 0x11) /* MOTOROLA format */
    {
       unsigned char    src[4];
       long             *p2data = (long*)data;
       int              idx;

       for (idx = 0; idx < points; idx++, p2data++)
          {
              *((long *)src) = *(p2data);
              ((unsigned char *)(p2data))[0] =  ((unsigned char*)(src))[3];
              ((unsigned char *)(p2data))[1] =  ((unsigned char*)(src))[2];
              ((unsigned char *)(p2data))[2] =  ((unsigned char*)(src))[1];
              ((unsigned char *)(p2data))[3] =  ((unsigned char*)(src))[0];
          }
    }
}

/*===========================================================================*/
/* Function: Get ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rsupv_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    GOCHECKERR (viPrintf(instrSession, "*CLS;%s\n",cmd));
    GOCHECKERR (rsupv_checkBeforeRead (instrSession));
    GOCHECKERR (rsupv_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while(p2buf && (i < count))
        {
        value[i++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
        };

    GOCHECKERR (rsupv_checkStatus (instrSession));

ERROR:

    free(pbuffer);

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rsupv_setViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    p2buf = pbuffer = (ViChar*) malloc (strlen (cmd) + 30 * count + 10);

    strcpy (pbuffer, cmd);
    strcat (pbuffer, " ");
    p2buf += strlen (pbuffer);

    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%.12f,", value[i]);

    *--p2buf = '\0';

    rsupv_status = viPrintf(instrSession, "%s\n", pbuffer);

    free(pbuffer);

    CHECKERR (rsupv_status);
    CHECKERR (rsupv_checkStatus (instrSession));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rsupv_setViReal64ArrayWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    p2buf = pbuffer = (ViChar*) malloc (strlen (cmd) + 20 * count + 10);

    p2buf += sprintf (pbuffer, cmd, window);
    strcat (pbuffer, " ");
    p2buf++;

    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%.12f,", value[i]);

    *--p2buf = '\0';

    rsupv_status = viPrintf(instrSession, "%s\n", pbuffer);

    free(pbuffer);

    CHECKERR (rsupv_status);
    CHECKERR (rsupv_checkStatus (instrSession));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Get ViReal64 Array Window                                         */
/*===========================================================================*/
ViStatus rsupv_getViReal64ArrayWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsupv_status;
    ViChar      *pbuffer;

    pbuffer = (ViChar*) malloc (strlen (cmd) + 10);
    sprintf (pbuffer, cmd, window);

    rsupv_status = rsupv_getViReal64Array (instrSession,
                                           pbuffer,
                                           value,
                                           count,
                                           parameterErrorCode);

    free (pbuffer);

    return rsupv_status;
}

/*****************************************************************************
 * Function: Get ViReal64 Binary Array
 *****************************************************************************/
ViStatus rsupv_getViReal64BinArray (ViSession instrSession,
                            ViString    cmd,
                            ViReal64    *value,
                            ViInt32     maxCount,
                            ViInt32     *retCount,
                            ViStatus    parameterErrorCode)
{
    ViStatus    rsupv_status   = VI_SUCCESS;
    ViUInt16    interface       = VI_INTF_GPIB;
    ViUInt32    retCnt          = 0;
    ViByte      *exBuf          = 0;
    ViReal32    *temp           = 0;
    ViInt32     lenLength       = 0,
                i               = 0,
                count           = 0;
    ViChar      buffer[BUFFER_SIZE];

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
    CHECKERR( viPrintf (instrSession, ":FORM REAL,32;*CLS;%s\n", cmd));
    CHECKERR( rsupv_checkBeforeRead (instrSession));

    /* Interface specific operations */
    switch (interface)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            /* Switch to Off using of termination character. */
            CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
            CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

            /* Parse binary data header. */
            if ((rsupv_status = viRead (instrSession, (ViPBuf)buffer, 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return rsupv_status;
                }
            buffer[retCnt] = '\0';
            if (sscanf (buffer, "#%1d", &lenLength) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Get number of points. */
            if ((rsupv_status = viRead (instrSession, (ViPBuf)buffer, lenLength, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return rsupv_status;
                }

            buffer[retCnt] = '\0';
            if (sscanf (buffer, "%ld", &count) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Allocate memory space */
            if ((exBuf = malloc (count + 2)) == NULL)
                return VI_ERROR_ALLOC;
            /* Read exact number of bytes equal to number of points. */
            if ((rsupv_status = viRead (instrSession, (ViPBuf)exBuf, count + 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsupv_status;
                }
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
        break;
        case VI_INTF_GPIB: default: /* GPIB Interface. Default case */
            CHECKERR( viRead (instrSession, (ViPBuf)buffer, 2, &retCnt));
            buffer[retCnt] = '\0';

            if (sscanf (buffer, "#%1d", &lenLength) != 1)
            {
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

            CHECKERR( viRead (instrSession, (ViPBuf)buffer, lenLength, &retCnt));
            buffer[retCnt] = '\0';

            if (sscanf (buffer, "%ld", &count) != 1)
            {
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

          /* Allocate memory space */
          if ((exBuf = malloc (count + 4)) == NULL)
              return VI_ERROR_ALLOC;

          if ((rsupv_status = viRead (instrSession, (ViPBuf)exBuf, count + 4, &retCnt)) < 0)
              {
              /* If not NULL pointer, then deallocate memory. */
              if (exBuf) free (exBuf);
              return rsupv_status;
              }
        break;
        }

    count /= 4; /* number samples */
    /* Binary data byte order (MOTOROLA or INTEL). */
    if (count > 0)
        rsupv_SWAP_32(exBuf, count);

    /* Convert binary data into array of ViReal64 types. */
    temp = (ViReal32*)(exBuf);
    *retCount = (count > maxCount) ? maxCount : count;

    for (i = 0; i < *retCount; i++)
        *value++ = (ViReal64)*temp++;

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    CHECKERR (rsupv_checkStatus (instrSession));

    return rsupv_status;
}

/*===========================================================================*/
/* Function: Coerce to nearest                                               */
/* Purpose:  This function is used to coerce entered value to the nearest    */
/*           value from the discreete table.                                 */
/*===========================================================================*/
void rsupv_coerceToNearest (void *table, ViPReal64 value, ViPInt16 index)
{
    ViReal64 *p2table = table;

    *index = 0;

    while ((*value > *p2table) && (fabs (*p2table) > NUMERIC_PRECISION))
        {
        *(++p2table);
        (*index)++;
        }

    if ((*value > NUMERIC_PRECISION) && (*index > 0))
        {
        if (fabs ((*value - *(--p2table)) / 2) < fabs ((*value - *(++p2table)) / 2))
            {
            *value = *(--p2table);
            (*index)--;
            }
        else
            {
            *value = *p2table;
            }
        }
    else
        {
        *value = *p2table;
        *index = 0;
        }

    return;
}

/*===========================================================================*/
/* Function: Check Option                                                    */
/* Purpose : This function tries to find the option in option list.          */
/*===========================================================================*/
ViStatus rsupv_checkOption (ViSession instrSession, ViString option)
{
    ViStatus rsupv_status = VI_SUCCESS;
    rsupv_instrRange instrPtr;
    ViInt32  idx = 0;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    if (instrPtr->errorChecking)
    {
        for (idx = 0; idx < RSUPV_MAX_OPTIONS; idx++)
        {
            if (strstr (instrPtr -> instrOption[idx].optionName, option))
                return VI_SUCCESS;
        }

        return RSUPV_ERROR_INSTRUMENT_OPTION;
    }
    else
        return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Check Instrument                                                */
/* Purpose : This function checks presence of the instrument.                */
/*===========================================================================*/
ViStatus rsupv_checkInstrument (ViSession instrSession, ViInt32 instrument)
{
    ViStatus rsupv_status = VI_SUCCESS;
    rsupv_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	if (instrPtr->errorChecking == VI_FALSE || instrPtr->optionChecking == VI_FALSE)
		return VI_SUCCESS;

    return instrPtr->instrModel != instrument ? RSUPV_ERROR_INSTRUMENT_NSUP_MODEL : VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Check Port                                                      */
/* Purpose : This function checks the range of port according to instr. type */
/*===========================================================================*/
ViStatus rsupv_checkPort (ViSession instrSession, ViInt32 port, ViStatus errorCode)
{
    ViStatus rsupv_status = VI_SUCCESS;
    rsupv_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (rsupv_invalidViInt32Range (port, 1, instrPtr->instrPorts))
        rsupv_status = errorCode;

    return rsupv_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
